#ifndef		EI_TYPES_H
#define		EI_TYPES_H

/*
	Version numbers:
*/

enum {
	cEI_InitialVersion = 1
};

enum {
	cEI_CurrentVersion = cEI_InitialVersion
};

/*
	Basic types:
*/

#ifdef		__cplusplus
extern "C" {
#endif

typedef struct {
	int		left;
	int		top;
	int		right;
	int		bottom;
} EI_Rect;

/*
	EI_ResourceFile types:
*/

typedef struct {
	int		data;
} EI_ResourceFile;

/*
	EI_DrawContext types:
*/

typedef struct {
	unsigned char		red;
	unsigned char		green;
	unsigned char		blue;
} EI_Color;

typedef enum {
	cEI_PenColor,
	cEI_PenXOR
} EI_PenMode;

enum {
	cEI_TextBold = 1 << 0,
	cEI_TextItalic = 1 << 1,
	cEI_TextUnderline = 1 << 2,
	cEI_TextOutline = 1 << 3,
	cEI_TextShadow = 1 << 4
};

typedef struct {
	int		data;
} EI_DrawContext;

/*
	EI_ImageBuffer types:
*/

typedef struct {
	int		data;
} EI_ImageBuffer;

/*
	EI_ImageBuffer pixel access constants:
*/

typedef unsigned long EI_32BitPixel;

#if			defined(__MACINTOSH__)

enum {
	cEI_AlphaShift = 24,
	cEI_RedShift = 16,
	cEI_GreenShift = 8,
	cEI_BlueShift = 0,

	cEI_AlphaOffset = 0,
	cEI_RedOffset = 1,
	cEI_GreenOffset = 2,
	cEI_BlueOffset = 3
};

#elif		defined(__WIN32__)

enum {
	cEI_AlphaShift = 24,
	cEI_RedShift = 16,
	cEI_GreenShift = 8,
	cEI_BlueShift = 0,

	cEI_AlphaOffset = 3,
	cEI_RedOffset = 2,
	cEI_GreenOffset = 1,
	cEI_BlueOffset = 0
};

#elif		defined(__SUN__)

enum {
	cEI_AlphaShift = 24,
	cEI_RedShift = 0,
	cEI_GreenShift = 8,
	cEI_BlueShift = 16,

	cEI_AlphaOffset = 0,
	cEI_RedOffset = 3,
	cEI_GreenOffset = 2,
	cEI_BlueOffset = 1
};

#elif		defined(__SGI__)

enum {
	cEI_AlphaShift = 24,
	cEI_RedShift = 0,
	cEI_GreenShift = 8,
	cEI_BlueShift = 16,

	cEI_AlphaOffset = 0,
	cEI_RedOffset = 3,
	cEI_GreenOffset = 2,
	cEI_BlueOffset = 1
};

#else

	This project does not define one of the known operating system macros.

#endif

/*
	EI_DiskImage types:
*/

typedef struct {
	int		data;
} EI_DiskImage;

/*
	EI_Dialog types:
*/

enum {
	cEI_ShiftKey = 1 << 0,
	cEI_ControlKey = 1 << 1,
	
	cEI_ModPrimary = 1 << 2,
	cEI_ModSecondary = 1 << 3,
	cEI_ModTernary = 1 << 4
};

typedef struct {
	int		data;
} EI_Dialog;

typedef void (*EI_DialogHitFunction)(EI_Dialog *iDialog);

typedef int (*EI_DialogKeyFilter)(EI_Dialog *iDialog,
									int iKey,
									int iModifiers);

typedef int (*EI_DialogValidationFunction)(EI_Dialog *ioDialog);

/*
	EI_Control types:
*/

typedef enum {
	cEI_NoControlType,
	cEI_PushButton,
	cEI_CheckBox,
	cEI_RadioGroup,
	cEI_DividerLine,
	cEI_GroupBox,
	cEI_TabGroup,
	cEI_ColorSlider,
	cEI_Scroller,
	cEI_StaticText,
	cEI_EditText,
	cEI_ColorButton,
	cEI_ListBox,
	cEI_PopupMenu,
	cEI_Picture,
	cEI_UserControl,
	cEI_OpenGLControl
} EI_ControlType;

typedef enum {
	cEI_NoFilter,
	cEI_UnsignedInt,
	cEI_SignedInt,
	cEI_UnsignedFloat,
	cEI_SignedFloat,
	cEI_Custom
} EI_EditFilterType;

enum {
	cEI_ListMultipleSelect = 1 << 0,
	cEI_ListDragReorder = 1 << 1
};

enum {
	cEI_Left,
	cEI_Top,
	cEI_Right,
	cEI_Bottom,
	cEI_Center
};

enum {
	cEI_MouseEnter = 1 << 0,
	cEI_MouseWithin = 1 << 1,
	cEI_MouseExit = 1 << 2
};

typedef struct {
	int		data;
} EI_Control;

/*
	Control callbacks:
*/

typedef void (*EI_ControlHitFunction)(EI_Control *iControl,
										EI_DrawContext *iContext);

typedef int (*EI_EditFilterFunction)(EI_Control *iControl,
										const char *iNewText);

typedef void (*EI_ListDoubleClickFunction)(EI_Control *iControl,
											int iDoubleClickedItem,
											EI_DrawContext *iContext);
typedef void (*EI_ListReorderFunction)(EI_Control *iControl,
										EI_DrawContext *iContext);

/*
	User control callbacks:
*/

typedef void (*EI_UserDrawFunction)(const EI_Control *iControl,
									EI_DrawContext *iContext);
typedef int (*EI_UserClickFunction)(EI_Control *iControl,
									EI_DrawContext *iContext,
									int iModifierKeys,
									int iClickH,
									int iClickV);
typedef void (*EI_UserMouseMovedFunction)(EI_Control *iControl,
											EI_DrawContext *iContext,
											int iModifierKeys,
											int iCursorH,
											int iCursorV,
											int iMoveType);

/*
	OpenGL control callbacks:
*/

typedef void (*EI_GLDrawFunction)(const EI_Control *iControl);
typedef int (*EI_GLClickFunction)(EI_Control *iControl,
									int iModifierKeys,
									int iClickH,
									int iClickV);
typedef void (*EI_GLMouseMovedFunction)(EI_Control *iControl,
										int iModifierKeys,
										int iClickH,
										int iClickV,
										int iMoveType);

/*
	EI_Stream types:
*/

typedef enum {
	cEI_ReadOnly,
	cEI_WriteOnly,
	cEI_ReadWrite
} EI_StreamAccess;

typedef struct {
	int		data;
} EI_DirectoryRef;

typedef struct {
	int		data;
} EI_FileRef;

typedef struct {
	int		data;
} EI_Stream;

/*
	EI_Timer types:
*/

typedef struct {
	int		data;
} EI_Timer;

typedef void (*EI_TimerFunction)(EI_Timer *iTimer);

/*
	EI_Cursor types:
*/

typedef struct {
	int		data;
} EI_Cursor;

#ifdef		__cplusplus
}
#endif

#endif
