/*
 *		Copyright (c) 1993-1998 Electric Image, Inc.
 *		This code is confidential and is a trade secret of Electric Image, Inc.
 */

#include "EIShader.h"
#ifdef __SGI__
	#include <stdio.h>
	#include <dlfcn.h>
#endif
#include "PString.h"
#ifdef __WIN32__
#include <windows.h>
#endif

#ifdef __MACINTOSH__
#include <CodeFragments.h>
#endif

/*
*  Global Functions
*/

/* EIShaderGetFunction will get a function from the shader or host */

void EIShaderGetFunction(long theConnectionID, PStrPtr theFunctionName, char **theFunction, long *theResult)
{
	long result = theResult != nil ? *theResult : 0;
	
	if (result == 0) {
#ifdef __MACINTOSH__
		CFragSymbolClass theSymClass;

		result = FindSymbol((CFragConnectionID)theConnectionID, theFunctionName, theFunction, &theSymClass);
		if ((result == noErr) && (theSymClass != kTVectorCFragSymbol)) {
			result = cfragNoSymbolErr;
			*theFunction = nil;
		}
#elif defined __WIN32__
		char cFunctionName[256];
		long i = theFunctionName[0];
		
		cFunctionName[i] = 0;
		while (i--)
			cFunctionName[i] = theFunctionName[i + 1];
		*theFunction = (char*)GetProcAddress((HMODULE)theConnectionID, cFunctionName);
		if (!(*theFunction))
			result = -1;
#elif defined __SGI__
		char cFunctionName[256];
		long i = theFunctionName[0];
		
		cFunctionName[i] = 0;
		while (i--)
			cFunctionName[i] = theFunctionName[i + 1];
		*theFunction = (char*)dlsym((char **)theConnectionID, cFunctionName);
		if (!(*theFunction))
			result = -1;
#else
		Ignore(theConnectionID);
		Ignore(theFunctionName);
		*theFunction = nil;
#endif
	}
	if (theResult != nil)
		*theResult = result;
}  /* EIShaderGetFunction */

/* __EISHADER_TEST__ is defined when a shader is being called directly as a library */

#ifdef __EISHADER_TEST__

EISHADER_FUNCTION long EIShaderInformation(EIShaderInformationRec *theInformationPtr);
EISHADER_FUNCTION long EIShaderInitialize(EIShaderInitializeRec *theInitializePtr);
EISHADER_FUNCTION long EIShaderFinish(EIShaderFinishRec *theFinishPtr);
EISHADER_FUNCTION long EIShaderShade(EIShaderShadeRec *theShadePtr);
EISHADER_FUNCTION long EIShaderSerial(EIShaderShadeRec *theShadePtr);
EISHADER_FUNCTION long EIShaderReinitialize(EIShaderReinitializeRec *theReinitializePtr);
EISHADER_FUNCTION long EIShaderHostInfo(EIShaderHostInfoRec *theHostInfoPtr);

long EIShaderOpenConnection(void *theFile, EIShaderInfoRec* theShaderInfo)
{
	Ignore(theFile);
	theShaderInfo->EIShaderInformation = EIShaderInformation;
	theShaderInfo->EIShaderInitialize = EIShaderInitialize;
	theShaderInfo->EIShaderFinish = EIShaderFinish;
	theShaderInfo->EIShaderShade = EIShaderShade;
	theShaderInfo->EIShaderSerial = EIShaderSerial;
	theShaderInfo->EIShaderReinitialize = EIShaderReinitialize;
	theShaderInfo->EIShaderHostInfo = EIShaderHostInfo;
	theShaderInfo->shaderConnectionID = 1;
	return 0;
}  /* EIShaderOpenConnection */

/* ClosePluginConnection will close commumications with the plugin code */

long EIShaderCloseConnection(EIShaderInfoRec* theShaderInfo)
{
	theShaderInfo->EIShaderInformation = nil;
	theShaderInfo->EIShaderInitialize = nil;
	theShaderInfo->EIShaderFinish = nil;
	theShaderInfo->EIShaderShade = nil;
	theShaderInfo->EIShaderSerial = nil;
	theShaderInfo->EIShaderReinitialize = nil;
	theShaderInfo->EIShaderHostInfo = nil;
	theShaderInfo->shaderConnectionID = 0;
	return 0;
}  /* EIShaderCloseConnection */

#else

long EIShaderOpenConnection(void *theFile, EIShaderInfoRec* theShaderInfo)
{
	long theResult = 0;
	
	theShaderInfo->EIShaderInformation = nil;
	theShaderInfo->EIShaderInitialize = nil;
	theShaderInfo->EIShaderFinish = nil;
	theShaderInfo->EIShaderShade = nil;
	theShaderInfo->EIShaderSerial = nil;
	theShaderInfo->EIShaderReinitialize = nil;
	theShaderInfo->EIShaderHostInfo = nil;
	theShaderInfo->shaderConnectionID = 0;
#ifdef __MACINTOSH__
	{
		FSSpec *theFSSpec = (FSSpec*)theFile;
		PStr255 errName;
		char *mainAddr;
		
		theResult = GetDiskFragment(theFSSpec, 0, kCFragGoesToEOF, theFSSpec->name, kLoadCFrag, (CFragConnectionID*)&theShaderInfo->shaderConnectionID, &mainAddr, errName);
	}
#elif defined __WIN32__
	{
		char *thePath = (char*)theFile;
		long i = CStrLength(thePath);

		if (i < _MAX_PATH) {
			unsigned int prevMode;
			char dllName[_MAX_PATH];
			long j;
			
			/* Make sure that the path has a file extension or ends in '.' */
			
			if ((i + 1 < _MAX_PATH) && (thePath[i - 4] != '.') && (thePath[i - 3] != '.') && (thePath[i - 2] != '.') && (thePath[i - 1] != '.')) {
				dllName[i] = '.';
				dllName[i + 1] = 0;
			} else
				dllName[i] = 0;
			while (i--)
				dllName[i] = thePath[i];
			
			/* get rid of extra charactors not supported in NT */
			
			i = 0;
			j = 0;
			while (dllName[i]) {
				if ((dllName[i] == '/') || (dllName[i] == '*') ||
/*					(dllName[i] == ':') || (dllName[i] == '\\') || */
					(dllName[i] == '?') || (dllName[i] == '"') ||
					(dllName[i] == '<') || (dllName[i] == '>') ||
					(dllName[i] == '|') ||
					(dllName[i] > 127) || (dllName[i] < 0))
					i++;
				else
					dllName[j++] = dllName[i++];
			}
			dllName[j] = 0;
			
			/* Save the current error mode while setting it to a value which will surpress the error dialog */
			
			prevMode = SetErrorMode(SEM_FAILCRITICALERRORS);

			theShaderInfo->shaderConnectionID = (long)LoadLibrary(dllName);
			if (!theShaderInfo->shaderConnectionID)
				theResult = -1;
			
			/* Restore the old error mode */
			
			SetErrorMode(prevMode);
		} else
			theResult = -1;
	}
#elif defined __SGI__
	theShaderInfo->shaderConnectionID = (long)dlopen((char*)theFile, RTLD_LAZY);
	if (!theShaderInfo->shaderConnectionID)
		theResult = -1;
#else
	Ignore(theFile);
	theResult = -1;
#endif
	EIShaderGetFunction(theShaderInfo->shaderConnectionID, EIShaderInformationName, (char**)&theShaderInfo->EIShaderInformation, &theResult);
	EIShaderGetFunction(theShaderInfo->shaderConnectionID, EIShaderInitializeName, (char**)&theShaderInfo->EIShaderInitialize, &theResult);
	EIShaderGetFunction(theShaderInfo->shaderConnectionID, EIShaderFinishName, (char**)&theShaderInfo->EIShaderFinish, &theResult);
	EIShaderGetFunction(theShaderInfo->shaderConnectionID, EIShaderShadeName, (char**)&theShaderInfo->EIShaderShade, &theResult);
	EIShaderGetFunction(theShaderInfo->shaderConnectionID, EIShaderSerialName, (char**)&theShaderInfo->EIShaderSerial, nil);
	EIShaderGetFunction(theShaderInfo->shaderConnectionID, EIShaderReinitializeName, (char**)&theShaderInfo->EIShaderReinitialize, nil);
	EIShaderGetFunction(theShaderInfo->shaderConnectionID, EIShaderHostInfoName, (char**)&theShaderInfo->EIShaderHostInfo, nil);
	return theResult;
}  /* EIShaderOpenConnection */

/* ClosePluginConnection will close commumications with the plugin code */

long EIShaderCloseConnection(EIShaderInfoRec* theShaderInfo)
{
	long theResult = 0;
	
#ifdef __MACINTOSH__
	if (theShaderInfo->shaderConnectionID)
		theResult = CloseConnection((CFragConnectionID*)&theShaderInfo->shaderConnectionID);
#elif defined __WIN32__
	if (theShaderInfo->shaderConnectionID)
	   if (!FreeLibrary((HMODULE)theShaderInfo->shaderConnectionID))
		   theResult = -1;
#elif defined __SGI__
	if (theShaderInfo->shaderConnectionID)
		theResult = dlclose((char**)theShaderInfo->shaderConnectionID);
#endif

	theShaderInfo->EIShaderInformation = nil;
	theShaderInfo->EIShaderInitialize = nil;
	theShaderInfo->EIShaderFinish = nil;
	theShaderInfo->EIShaderShade = nil;
	theShaderInfo->EIShaderSerial = nil;
	theShaderInfo->EIShaderReinitialize = nil;
	theShaderInfo->EIShaderHostInfo = nil;
	theShaderInfo->shaderConnectionID = 0;
	return theResult;
}  /* EIShaderCloseConnection */

#endif
