/*
 *		Copyright (c) 1987-1998 Electric Image, Inc.
 */

#include "Image.h"
#include "BufferedIO.h"
#include "FACTIO.h"
#include "GeneralIO.h"
#include "GeneralUtils.h"
#include "MemoryUtils.h"
#include "PString.h"
#include "StatusUtils.h"

/* Image global variables */

ImageVarsRec theImageVars;

static void Write1BitImage(char *theScanBuffer, char *theBase, long theScanSize, float theTime, long theXWidth, long theYWidth, long backColorIndex, ARGBPalettePtr thePalette);
static void Write2BitImage(char *theScanBuffer, char *theBase, long theScanSize, float theTime, long theXWidth, long theYWidth, long backColorIndex, ARGBPalettePtr thePalette);
static void Write4BitImage(char *theScanBuffer, char *theBase, long theScanSize, float theTime, long theXWidth, long theYWidth, long backColorIndex, ARGBPalettePtr thePalette);
static void Write8BitImage(char *theScanBuffer, char *theBase, long theScanSize, float theTime, long theXWidth, long theYWidth, long backColorIndex, ARGBPalettePtr thePalette);
static void Write16BitImage(char *theScanBuffer, char *theBase, long theScanSize, float theTime, long theXWidth, long theYWidth, ARGBPalettePtr thePalette);
static void Write32BitImage(char *theScanBuffer, char *theBase, long theScanSize, float theTime, long theXWidth, long theYWidth, ARGBPalettePtr thePalette);
static void Write32BitAlphaImage(char *theScanBuffer, char *theBase, long theScanSize, float theTime, long theXWidth, long theYWidth, ARGBPalettePtr thePalette);

/*
 *		Static Functions
 */

static void Write1BitImage(char *theScanBuffer, char *theBase, long theScanSize, float theTime, long theXWidth, long theYWidth, long backColorIndex, ARGBPalettePtr thePalette)
{
	typedef uchar* ScanLinePtr;				/* array of 0..32765 */
	
	ScanLinePtr yPtr;
	long x, y, xCount, xEnd, xMask, xBits;
	long packLoc;
	
	WriteReal(theTime);						/* Frame Time */
	WriteInt(0);							/* Image Rectangle  */
	WriteInt(0);
	WriteInt((short)theYWidth);
	WriteInt((short)theXWidth);
	WriteByte(1);							/* Image Depth -- 1 bits */
	WriteByte(1);							/* Image Type -- Palette Data */
	WriteInt(0);							/* Packing Rectangle  */
	WriteInt(0);
	WriteInt((short)theYWidth);
	WriteInt((short)theXWidth);
	WriteByte(1);							/* Pack Type -- RL Data */
	WriteByte(0);							/* Alpha Depth -- No Alpha plane */
	packLoc = outFilePos;
	WriteLong(0);							/* FileSize */
	WriteInt(2);							/* Palette Count */
	if ((backColorIndex < 0) || (backColorIndex > 1))
		backColorIndex = 1;
	WriteInt((short)backColorIndex);		/* Background Palette Color */
	WriteByte(thePalette[0].argb.r);
	WriteByte(thePalette[0].argb.g);
	WriteByte(thePalette[0].argb.b);
	WriteByte(thePalette[1].argb.r);
	WriteByte(thePalette[1].argb.g);
	WriteByte(thePalette[1].argb.b);
	oldColor = 0;
	runLength = 0;
	pixelBytes = 1;
	packetLength = 0;
	packSize = 0;
	xCount = theXWidth >> 3;
	xEnd = 8 - ((theXWidth & 7));
	xMask = 0xFF & (0xFF << xEnd);
	xBits = 0;
	for (; xEnd > 0; xEnd--)
		xBits = (xBits << 1) | backColorIndex;
	for (y = 0; (y < theYWidth) && (!errorFlag); y++) {
		if (!theScanBuffer)
			yPtr = (ScanLinePtr)(theBase + y * theScanSize);
		else {
			yPtr = (ScanLinePtr)theScanBuffer;
			CopyMem(theBase + y * theScanSize, theScanBuffer, theScanSize);
		}
		for (x = 0; (x < xCount) && (!errorFlag); x++) {
			newColor = yPtr[x];
			if (newColor != oldColor)
				RLEncode();
			runLength++;
		}
		if (xMask > 0) {
			newColor = (yPtr[x] & xMask) | xBits;
			if (newColor != oldColor)
				RLEncode();
			runLength++;
		}
		CHECKSTATUSEVENTS();
	}
	runLength = -runLength;
	RLEncode();
	InsertLong(packSize, packLoc);
}  /* Write1BitImage */

static void Write2BitImage(char *theScanBuffer, char *theBase, long theScanSize, float theTime, long theXWidth, long theYWidth, long backColorIndex, ARGBPalettePtr thePalette)
{
	typedef uchar* ScanLinePtr;				/* array of 0..32765 */
	
	ScanLinePtr yPtr;
	long i, x, y, xCount, xEnd, xMask, xBits;
	long packLoc;
	
	WriteReal(theTime);						/* Frame Time */
	WriteInt(0);							/* Image Rectangle  */
	WriteInt(0);
	WriteInt((short)theYWidth);
	WriteInt((short)theXWidth);
	WriteByte(2);							/* Image Depth -- 2 bits */
	WriteByte(1);							/* Image Type -- Palette Data */
	WriteInt(0);							/* Packing Rectangle  */
	WriteInt(0);
	WriteInt((short)theYWidth);
	WriteInt((short)theXWidth);
	WriteByte(1);							/* Pack Type -- RL Data */
	WriteByte(0);							/* Alpha Depth -- No Alpha plane */
	packLoc = outFilePos;
	WriteLong(0);							/* FileSize */
	WriteInt(4);							/* Palette Count */
	if ((backColorIndex < 0) || (backColorIndex > 3))
		backColorIndex = 3;
	WriteInt((short)backColorIndex);		/* Background Palette Color */
	for (i = 0; i <= 3; i++) {
		WriteByte(thePalette[i].argb.r);
		WriteByte(thePalette[i].argb.g);
		WriteByte(thePalette[i].argb.b);
	}
	oldColor = 0;
	runLength = 0;
	pixelBytes = 1;
	packetLength = 0;
	packSize = 0;
	xCount = theXWidth >> 2;
	xEnd = 4 - (theXWidth & 3);
	xMask = 0xFF & (0xFF << (2 * xEnd));
	xBits = 0;
	for (; xEnd > 0; xEnd--)
		xBits = (xBits << 2) | backColorIndex;
	for (y = 0; (y < theYWidth) && (!errorFlag); y++) {
		if (!theScanBuffer)
			yPtr = (ScanLinePtr)(theBase + y * theScanSize);
		else {
			yPtr = (ScanLinePtr)theScanBuffer;
			CopyMem(theBase + y * theScanSize, theScanBuffer, theScanSize);
		}
		for (x = 0; (x < xCount) && (!errorFlag); x++) {
			newColor = yPtr[x];
			if (newColor != oldColor)
				RLEncode();
			runLength++;
		}
		if (xMask > 0) {
			newColor = (yPtr[x] & xMask) | xBits;
			if (newColor != oldColor)
				RLEncode();
			runLength++;
		}
		CHECKSTATUSEVENTS();
	}
	runLength = -runLength;
	RLEncode();
	InsertLong(packSize, packLoc);
}  /* Write2BitImage */

static void Write4BitImage(char *theScanBuffer, char *theBase, long theScanSize, float theTime, long theXWidth, long theYWidth, long backColorIndex, ARGBPalettePtr thePalette)
{
	typedef uchar* ScanLinePtr;				/* array of 0..32765 */
	
	ScanLinePtr yPtr;
	long i, x, y, xCount, xEnd, xMask, xBits;
	long packLoc;
	
	WriteReal(theTime);						/* Frame Time */
	WriteInt(0);							/* Image Rectangle  */
	WriteInt(0);
	WriteInt((short)theYWidth);
	WriteInt((short)theXWidth);
	WriteByte(4);							/* Image Depth -- 4 bits */
	WriteByte(1);							/* Image Type -- Palette Data */
	WriteInt(0);							/* Packing Rectangle  */
	WriteInt(0);
	WriteInt((short)theYWidth);
	WriteInt((short)theXWidth);
	WriteByte(1);							/* Pack Type -- RL Data */
	WriteByte(0);							/* Alpha Depth -- No Alpha plane */
	packLoc = outFilePos;
	WriteLong(0);							/* FileSize */
	WriteInt(16);							/* Palette Count */
	if ((backColorIndex < 0) || (backColorIndex > 15))
		backColorIndex = 15;
	WriteInt((short)backColorIndex);		/* Background Palette Color */
	for (i = 0; i <= 15; i++) {
		WriteByte(thePalette[i].argb.r);
		WriteByte(thePalette[i].argb.g);
		WriteByte(thePalette[i].argb.b);
	}
	oldColor = 0;
	runLength = 0;
	pixelBytes = 1;
	packetLength = 0;
	packSize = 0;
	xCount = theXWidth >> 1;
	xEnd = 2 - (theXWidth & 1);
	xMask = 0xFF & (0xFF << (4 * xEnd));
	xBits = 0;
	for (; xEnd > 0; xEnd--)
		xBits = (xBits << 4) | backColorIndex;
	for (y = 0; (y < theYWidth) && (!errorFlag); y++) {
		if (!theScanBuffer)
			yPtr = (ScanLinePtr)(theBase + y * theScanSize);
		else {
			yPtr = (ScanLinePtr)theScanBuffer;
			CopyMem(theBase + y * theScanSize, theScanBuffer, theScanSize);
		}
		for (x = 0; (x < xCount) && (!errorFlag); x++) {
			newColor = yPtr[x];
			if (newColor != oldColor)
				RLEncode();
			runLength++;
		}
		if (xMask > 0) {
			newColor = (yPtr[x] & xMask) | xBits;
			if (newColor != oldColor)
				RLEncode();
			runLength++;
		}
		CHECKSTATUSEVENTS();
	}
	runLength = -runLength;
	RLEncode();
	InsertLong(packSize, packLoc);
}  /* Write4BitImage */

static void Write8BitImage(char *theScanBuffer, char *theBase, long theScanSize, float theTime, long theXWidth, long theYWidth, long backColorIndex, ARGBPalettePtr thePalette)
{
	typedef uchar* ScanLinePtr;				/* array of 0..8190 */
	
	ScanLinePtr yPtr;
	long i, x, y;
	long packLoc;
	
	WriteReal(theTime);						/* Frame Time */
	WriteInt(0);							/* Image Rectangle  */
	WriteInt(0);
	WriteInt((short)theYWidth);
	WriteInt((short)theXWidth);
	WriteByte(8);							/* Image Depth -- 8 bits */
	WriteByte(1);							/* Image Type -- Palette Data */
	WriteInt(0);							/* Packing Rectangle  */
	WriteInt(0);
	WriteInt((short)theYWidth);
	WriteInt((short)theXWidth);
	WriteByte(1);							/* Pack Type -- RL Data */
	WriteByte(0);							/* Alpha Depth -- No Alpha plane */
	packLoc = outFilePos;
	WriteLong(0);							/* FileSize */
	WriteInt(256);							/* Palette Count */
	if ((backColorIndex < 0) || (backColorIndex > 255))
		backColorIndex = 255;
	WriteInt((short)backColorIndex);		/* Background Palette Color */
	for (i = 0; i <= 255; i++) {
		WriteByte(thePalette[i].argb.r);
		WriteByte(thePalette[i].argb.g);
		WriteByte(thePalette[i].argb.b);
	}
	oldColor = 0;
	runLength = 0;
	pixelBytes = 1;
	packetLength = 0;
	packSize = 0;
	for (y = 0; (y < theYWidth) && (!errorFlag); y++) {
		if (!theScanBuffer)
			yPtr = (ScanLinePtr)(theBase + y * theScanSize);
		else {
			yPtr = (ScanLinePtr)theScanBuffer;
			CopyMem(theBase + y * theScanSize, theScanBuffer, theScanSize);
		}
		for (x = 0; (x < theXWidth) && (!errorFlag); x++) {
			newColor = yPtr[x];
			if (newColor != oldColor)
				RLEncode();
			runLength++;
		}
		CHECKSTATUSEVENTS();
	}
	runLength = -runLength;
	RLEncode();
	InsertLong(packSize, packLoc);
}  /* Write8BitImage */

static void Write16BitImage(char *theScanBuffer, char *theBase, long theScanSize, float theTime, long theXWidth, long theYWidth, ARGBPalettePtr thePalette)
{	
	short *yPtr, *yEnd;
	long y;
	long packLoc;
	
	WriteReal(theTime);						/* Frame Time */
	WriteInt(0);							/* Image Rectangle  */
	WriteInt(0);
	WriteInt((short)theYWidth);
	WriteInt((short)theXWidth);
	WriteByte(16);							/* Image Depth -- 16 bits */
	WriteByte(0);							/* Image Type -- Straight Data */
	WriteInt(0);							/* Packing Rectangle  */
	WriteInt(0);
	WriteInt((short)theYWidth);
	WriteInt((short)theXWidth);
	WriteByte(1);							/* Pack Type -- RL Data */
	WriteByte(0);							/* Alpha Depth -- No Alpha plane */
	packLoc = outFilePos;
	WriteLong(0);							/* FileSize */
	WriteInt(1);							/* Palette Count */
	WriteInt(0);							/* Background Palette Color */
	WriteByte(thePalette[0].argb.r);
	WriteByte(thePalette[0].argb.g);
	WriteByte(thePalette[0].argb.b);
	oldColor = 0;
	runLength = 0;
	pixelBytes = 2;
	packetLength = 0;
	packSize = 0;
	for (y = 0; (y < theYWidth) && (!errorFlag); y++) {
		if (!theScanBuffer)
			yPtr = (short*)(theBase + y * theScanSize);
		else {
			yPtr = (short*)theScanBuffer;
			CopyMem(theBase + y * theScanSize, theScanBuffer, theScanSize);
		}
		yEnd = (short*)((char*)yPtr + (theXWidth << 1));
		while ((yPtr != yEnd)  && (!errorFlag)) {
			newColor = 0x7FFF & *yPtr;
			if (newColor != oldColor)
				RLEncode();
			runLength++;
			yPtr++;
		}
		CHECKSTATUSEVENTS();
	}
	runLength = -runLength;
	RLEncode();
	InsertLong(packSize, packLoc);
}  /* Write16BitImage */

static void Write32BitImage(char *theScanBuffer, char *theBase, long theScanSize, float theTime, long theXWidth, long theYWidth, ARGBPalettePtr thePalette)
{
	long *yPtr, *yEnd;
	long y;
	long packLoc;
	
	WriteReal(theTime);						/* Frame Time */
	WriteInt(0);							/* Image Rectangle  */
	WriteInt(0);
	WriteInt((short)theYWidth);
	WriteInt((short)theXWidth);
	WriteByte(24);							/* Image Depth -- 24 bits */
	WriteByte(0);							/* Image Type -- Straight Data */
	WriteInt(0);							/* Packing Rectangle  */
	WriteInt(0);
	WriteInt((short)theYWidth);
	WriteInt((short)theXWidth);
	WriteByte(1);							/* Pack Type -- RL Data */
	WriteByte(0);							/* Alpha Depth -- No Alpha plane */
	packLoc = outFilePos;
	WriteLong(0);							/* FileSize */
	WriteInt(1);							/* Palette Count */
	WriteInt(0);							/* Background Palette Color */
	WriteByte(thePalette[0].argb.r);
	WriteByte(thePalette[0].argb.g);
	WriteByte(thePalette[0].argb.b);
	oldColor = 0;
	runLength = 0;
	pixelBytes = 3;
	packetLength = 0;
	packSize = 0;
	for (y = 0; (y < theYWidth) && (!errorFlag); y++) {
		if (!theScanBuffer)
			yPtr = (long*)(theBase + y * theScanSize);
		else {
			yPtr = (long*)theScanBuffer;
			CopyMem(theBase + y * theScanSize, theScanBuffer, theScanSize);
		}
		yEnd = (long*)((char*)yPtr + (theXWidth << 2));
		while ((yPtr != yEnd)  && (!errorFlag)) {
			newColor = 0x00FFFFFF & *yPtr;
			if (newColor != oldColor)
				RLEncode();
			runLength++;
			yPtr++;
		}
		CHECKSTATUSEVENTS();
	}
	runLength = -runLength;
	RLEncode();
	InsertLong(packSize, packLoc);
}  /* Write32BitImage */

static void Write32BitAlphaImage(char *theScanBuffer, char *theBase, long theScanSize, float theTime, long theXWidth, long theYWidth, ARGBPalettePtr thePalette)
{
	long *yPtr, *yEnd;
	long y;
	long packLoc;
	
	WriteReal(theTime);						/* Frame Time */
	WriteInt(0);							/* Image Rectangle  */
	WriteInt(0);
	WriteInt((short)theYWidth);
	WriteInt((short)theXWidth);
	WriteByte(24);							/* Image Depth -- 24 bits */
	WriteByte(0);							/* Image Type -- Straight Data */
	WriteInt(0);							/* Packing Rectangle  */
	WriteInt(0);
	WriteInt((short)theYWidth);
	WriteInt((short)theXWidth);
	WriteByte(1);							/* Pack Type -- RL Data */
	WriteByte(8);							/* Alpha Depth -- 8 Bit Alpha plane */
	packLoc = outFilePos;
	WriteLong(0);							/* FileSize */
	WriteInt(1);							/* Palette Count */
	WriteInt(0);							/* Background Palette Color */	
	WriteByte(thePalette[0].argb.r);
	WriteByte(thePalette[0].argb.g);
	WriteByte(thePalette[0].argb.b);
	oldColor = 0;
	runLength = 0;
	pixelBytes = 4;
	packetLength = 0;
	packSize = 0;
	for (y = 0; (y < theYWidth) && (!errorFlag); y++) {
		if (!theScanBuffer)
			yPtr = (long*)(theBase + y * theScanSize);
		else {
			yPtr = (long*)theScanBuffer;
			CopyMem(theBase + y * theScanSize, theScanBuffer, theScanSize);
		}
		yEnd = (long*)((char*)yPtr + (theXWidth << 2));
		while ((yPtr != yEnd)  && (!errorFlag)) {
			newColor = *yPtr;
			if (newColor != oldColor)
				RLEncode();
			runLength++;
			yPtr++;
		}
		CHECKSTATUSEVENTS();
	}
	runLength = -runLength;
	RLEncode();
	InsertLong(packSize, packLoc);
}  /* Write32BitAlphaImage */

/*
*  Global Routines
*/

/* Image file utility routines */

void StampImage(StringPtr theFileName, FolderRef theVolNum)
{
#ifdef __MACINTOSH__
	Handle theVersion;
	boolean saveHide;
	FileRef theFile;
	long bitDepth = 0;
	long xRes = 0;
	long yRes = 0;
	long frameCount = 0;
	ImageHeader theImageHeader;
	ImageFrame theImageFrame;
	
	if ((!errorFlag) && CheckFile(theFileName, anyCreator, anyType, theVolNum)) {
		saveHide = hideErrorFlag;
		hideErrorFlag = 1;
		OpenFileName(theFileName, theVolNum, &theFile);
		if ((!errorFlag) && (FileSize(theFile) > sizeof(ImageHeader) + sizeof(ImageFrame))) {
			ReadFileImageHeader(theFile, &theImageHeader);
			if (theImageHeader.imageVersion != 5)
				errorFlag = 1;
			else {
				frameCount = theImageHeader.imageFrames;
				ReadFileImageFrame(theFile, &theImageFrame);
				bitDepth = theImageFrame.frameBitDepth + theImageFrame.frameAlpha;
				xRes = (long)theImageFrame.frameRect.right - theImageFrame.frameRect.left;
				yRes = (long)theImageFrame.frameRect.bottom - theImageFrame.frameRect.top;
			}
		}
		CloseFile(&theFile);
		if (!errorFlag) {
			OpenTheResFile(theFileName, theVolNum);
			theVersion = Get1Resource(versType, 1);
			if (theVersion && (ResError() == noErr)) {
				RemoveResource(theVersion);
				DisposeHandle(theVersion);
			}
			theVersion = Get1Resource(versType, 2);
			if (theVersion && (ResError() == noErr)) {
				RemoveResource(theVersion);
				DisposeHandle(theVersion);
			}
			theVersion = MakeVersion(0, 0, 0, 0, 0, aEmptyPStr, PStrConcat(NumStr(bitDepth), PS(" Bit, "), NumStr(xRes), PS("x"), NumStr(yRes), PStrNULL));
			if (theVersion)
				AddResource(theVersion, versType, 1, aEmptyPStr);
			if (frameCount == 1) {
				theVersion = MakeVersion(0, 0, 0, 0, 0, aEmptyPStr, PS("1 Frame"));
				if (theVersion)
					AddResource(theVersion, versType, 2, aEmptyPStr);
			} else {
				theVersion = MakeVersion(0, 0, 0, 0, 0, aEmptyPStr, PStrConcat(NumStr(frameCount), PS(" Frames"), PStrNULL));
				if (theVersion)
					AddResource(theVersion, versType, 2, aEmptyPStr);
			}
			CloseTheResFile();
		}
		errorFlag = 0;
		hideErrorFlag = saveHide;
		lastErrorValue = 0;
		PStrMakeEmpty(lastErrorMessage);
	}
#else
	Ignore(theFileName);
	Ignore(theVolNum);
#endif
}  /* StampImage */

/* Image file input routines */

long SkipFrame(void)
{
	long packingSize;
	
	/* If the file is shorter than the current frame header, set the frame size to zero */
	
	if (inFilePos + 30 > inFileSize)
		return 0;
	else {
		SkipBytes(24);
		packingSize = ReadLong();
		packingSize += ReadInt() * 3 + 2;
		
		/* If the file is shorter than the current frame header indicates, set the input point at the frame header and */
		/* set the frame size to zero */
		
		if (inFilePos + packingSize > inFileSize) {
			SetInFilePos(inFilePos - 30);
			return 0;
		} else {
			
			/* Otherise, skip the entire frame and return the total size in bytes */
			
			SkipBytes(packingSize);
			return packingSize + 30;
		}
	}
}  /* SkipFrame */

/* CheckAnimation will return the number of frames in an animtion.  It will leave the file pointer positioned after */
/* the last valid frame */

long CheckAnimation(void)
{
	long version;
	long frameCount, i;
	
	frameCount = 0;
	SetInFilePos(0);
	if ((!errorFlag) && (inFileSize > 6)) {
		version = ReadInt();
		if (version == 5) {
			frameCount = ReadLong();
			if (frameCount > 0) {
				for (i = 0; (i < frameCount) && (!errorFlag);)
					if (SkipFrame() > 0)
						i++;
					else
						frameCount = i;
			} else
				frameCount = 0;
		} else
			SetInFilePos(0);
	}
	return errorFlag ? 0 : frameCount;
}  /* CheckAnimation */

void SkipImagePixel(void)
{
	if (theImageVars.theDepth < 8) {
		if (theImageVars.bitCount == 0) {
			theImageVars.colorBits = ReadRL();
			theImageVars.bitCount = 8;
		}
		theImageVars.colorBits = 0xFF & (theImageVars.colorBits << theImageVars.theDepth);
		theImageVars.bitCount -= theImageVars.theDepth;
	} else
		ReadRL();
}  /* SkipImagePixel */

void ReadImagePixel(ARGBPtr thePixel)
{
	long alpha1, alpha2;
	
	switch (theImageVars.theDepth) {
		case 1:
		case 2:
		case 4: 
			if (theImageVars.bitCount == 0) {
				theImageVars.colorBits = ReadRL();
				theImageVars.bitCount = 8;
			}
			*thePixel = theImageVars.thePalette[theImageVars.colorBits >> (8 - theImageVars.theDepth)];
			theImageVars.colorBits = 0xFF & (theImageVars.colorBits << theImageVars.theDepth);
			theImageVars.bitCount -= theImageVars.theDepth;
			break;
		case 8: 
			*thePixel = theImageVars.thePalette[ReadRL()];
			break;
		case 16: 
			alpha1 = ReadRL();
			thePixel->argb.a = 255;
			alpha2 = 31 & (alpha1 >> 10);
			thePixel->argb.r = (alpha2 << 3) | (alpha2 >> 2);
			alpha2 = 31 & (alpha1 >> 5);
			thePixel->argb.g = (alpha2 << 3) | (alpha2 >> 2);
			alpha2 = 31 & alpha1;
			thePixel->argb.b = (alpha2 << 3) | (alpha2 >> 2);
			break;
		case 24: 
			thePixel->colorValue = ReadRL();
			thePixel->argb.a = 255;
			break;
		case 32: 
			thePixel->colorValue = ReadRL();
			if (theImageVars.calcAlphaFlag && (thePixel->argb.a < 255))
				if (thePixel->argb.a == 0)
					*thePixel = theImageVars.theBackColor;
				else {
					alpha1 = thePixel->argb.a;
					alpha2 = 256 - alpha1;
					thePixel->argb.a = 255;
					thePixel->argb.r = (alpha1 * thePixel->argb.r + alpha2 * theImageVars.theBackColor.argb.r) >> 8;
					thePixel->argb.g = (alpha1 * thePixel->argb.g + alpha2 * theImageVars.theBackColor.argb.g) >> 8;
					thePixel->argb.b = (alpha1 * thePixel->argb.b + alpha2 * theImageVars.theBackColor.argb.b) >> 8;
				}
			break;
	}
}  /* ReadImagePixel */

void ReadImage(boolean alpha, long* xWidth, long *yWidth, char **thePtr)
{
	long i;
	long x, y;
	ARGBPtr thePixel;
	ImageFrame theImageFrame;
	
	*thePtr = NULL;
	ReadImageFrame(&theImageFrame);
	if (!errorFlag) {
		theImageVars.calcAlphaFlag = alpha;
		theImageVars.theDepth = theImageFrame.frameBitDepth;
		theImageVars.colorBits = 0;
		theImageVars.bitCount = 0;
		for (i = 0, thePixel = theImageVars.thePalette; (i < theImageFrame.framePalettes) && (!errorFlag); i++, thePixel++) {
			thePixel->argb.a = 255;
			thePixel->argb.r = ReadByte();
			thePixel->argb.g = ReadByte();
			thePixel->argb.b = ReadByte();
		}
		for (; i < 256; i++, thePixel++)
			thePixel->colorValue = 0;
		theImageVars.theBackColor = theImageVars.thePalette[theImageFrame.frameBackground];
		oldColor2 = 0;
		switch (theImageVars.theDepth) {
			case 1:
			case 2:
			case 4:
			case 8: 
				pixelBytes2 = 1;
				break;
			case 16: 
				pixelBytes2 = 2;
				break;
			case 24: 
				pixelBytes2 = 3;
				break;
			case 32: 
				pixelBytes2 = 4;
				break;
		}
		if (theImageFrame.framePacking == 0)
			packetLength2 = MAXINT;
		else
			packetLength2 = 0;
		*xWidth = (long)theImageFrame.framePackRect.right - theImageFrame.framePackRect.left;
		*yWidth = (long)theImageFrame.framePackRect.bottom - theImageFrame.framePackRect.top;
	}
	if (!errorFlag)
		*thePtr = CarefulNewPtr(*xWidth * *yWidth * sizeof(ARGB));
	if ((!errorFlag) && *thePtr) {
		thePixel = (ARGBPtr)*thePtr;
		for (y = 0; (y < *yWidth) && (!errorFlag); y++) {
			theImageVars.bitCount = 0;
			for (x = 0; (x < *xWidth) && (!errorFlag); x++, thePixel++)
				ReadImagePixel(thePixel);
		}
	}
	if (errorFlag) {
		CarefulDisposePtr(thePtr);
		*xWidth = 0;
		*yWidth = 0;
	}
}  /* ReadImage */

/* Image file output routines */

void WriteImageFileHeader(long totalFrames)
{
	WriteInt(5);						/* Version */
	WriteLong(totalFrames);				/* Number of frames */
}  /* WriteImageFileHeader */

void WriteImageFileFrame(char *theScanBuffer, char *theBase, long theScanSize, float theTime, long theXWidth, long theYWidth, long theDepth, long theAlpha, long backColorIndex, ARGBPalettePtr thePalette)
{
	switch (theDepth) {
	case 1: 
		Write1BitImage(theScanBuffer, theBase, theScanSize, theTime, theXWidth, theYWidth, backColorIndex, thePalette);
		break;
	case 2: 
		Write2BitImage(theScanBuffer, theBase, theScanSize, theTime, theXWidth, theYWidth, backColorIndex, thePalette);
		break;
	case 4: 
		Write4BitImage(theScanBuffer, theBase, theScanSize, theTime, theXWidth, theYWidth, backColorIndex, thePalette);
		break;
	case 8: 
		Write8BitImage(theScanBuffer, theBase, theScanSize, theTime, theXWidth, theYWidth, backColorIndex, thePalette);
		break;
	case 16: 
		Write16BitImage(theScanBuffer, theBase, theScanSize, theTime, theXWidth, theYWidth, thePalette);
		break;
	case 24:
	case 32: 
		if (theAlpha == 0)
			Write32BitImage(theScanBuffer, theBase, theScanSize, theTime, theXWidth, theYWidth, thePalette);
		else
			Write32BitAlphaImage(theScanBuffer, theBase, theScanSize, theTime, theXWidth, theYWidth, thePalette);
		break;
	}
}  /* WriteImageFileFrame */
