/*
Copyright 1996-1998 Electric Image, Inc. This source code may only be used by
registered owners of the ElectricImage Plug-In Kit. Owners of the Kit may
include the source code in their products (freeware, shareware, or commercial).
This copyright notice must be included in its original form in all such
products. This source code may not be offered for sale or distributed by any
entity without the written permission of Electric Image, Inc.
*/
#include "MemoryUtils.h"

/* Memory copying routines */

void CopySmallBlockForwards(void* sourcePtr, void* destPtr, unsigned long byteCount)
{
	if (byteCount & 3) 
		if (byteCount & 1) {
			char *srcPtr = (char*)sourcePtr;
			char *dstPtr = (char*)destPtr;
			char *endPtr = srcPtr + byteCount;
			while (srcPtr != endPtr)
				*dstPtr++ = *srcPtr++;
		} else {
			short* srcPtr = (short*)sourcePtr;
			short* dstPtr = (short*)destPtr;
			short* endPtr = srcPtr + (byteCount >> 1);
			while (srcPtr != endPtr)
				*dstPtr++ = *srcPtr++;
		}
	else {
		unsigned long* srcPtr = (unsigned long*)sourcePtr;
		unsigned long* dstPtr = (unsigned long*)destPtr;
		unsigned long* endPtr = srcPtr + (byteCount >> 2);
		while (srcPtr != endPtr)
			*dstPtr++ = *srcPtr++;
	}
}  /* CopySmallBlockForwards */

void CopySmallBlockBackwards(void* sourcePtr, void* destPtr, unsigned long byteCount)
{
	char *srcPtr = (char*)sourcePtr;
	char *dstPtr = (char*)destPtr + byteCount;
	char *endPtr = srcPtr + byteCount;
	while (srcPtr != endPtr)
		*--dstPtr = *srcPtr++;
}  /* CopySmallBlockBackwards */

/* Memory initialization routines */

#if powerc
asm unsigned long GetCR(void)
{
	mfcr r3							/* return value(r3) = CR */
	blr								/* return */
}  /* GetCR */

asm void SetCR(unsigned long)
{
	mtcr r3							/* CR = value */
	blr								/* return */
}  /* SetCR */

asm unsigned long GetMSR(void)
{
	mfmsr r3						/* return value = MSR */
	blr								/* return */
}  /* GetMSR */

asm void SetMSR(unsigned long)
{
	mtmsr r3						/* MSR = value */
	blr								/* return */
}  /* SetMSR */

asm void ZeroCache(void*, unsigned long, unsigned long)
{
			mtctr	r4				/* ctr = count */
Loop:		dcbz	r0,r3			/* Clear32Bytes(thePtr) */
			add		r3,r3,r5		/* thePtr += size; */
			bdnz	Loop			/* if (--ctr > 0) goto Loop1 */

			blr						/* return */
}  /* ZeroCache */

asm void TouchCache(void*, unsigned long, unsigned long)
{		
			mtctr	r4				/* ctr = count */
Loop:		dcbt	r0,r3			/* Touch32Bytes(thePtr) */
			add		r3,r3,r5		/* thePtr += size; */
			bdnz	Loop			/* if (--ctr > 0) goto Loop2 */

			blr						/* return */
}  /* TouchCache */

asm void TouchSCache(void*, unsigned long, unsigned long)
{
			mtctr	r4				/* ctr = count */
Loop:		dcbtst	r0,r3			/* TouchForStore32Bytes(thePtr) */
			add		r3,r3,r5		/* thePtr += size; */
			bdnz	Loop			/* if (--ctr > 0) goto Loop3 */

			blr						/* return */
}  /* TouchSCache */

asm void FlushCache(void*, unsigned long, unsigned long)
{
			mtctr	r4				/* ctr = count */
Loop:		dcbf	r0,r3			/* Flush32Bytes(thePtr) */
			add		r3,r3,r5		/* thePtr += size; */
			bdnz	Loop			/* if (--ctr > 0) goto Loop4 */

			blr						/* return */
}  /* FlushCache */

void ZeroMem(void* startNum, unsigned long rangeNum)
{
	unsigned long *thePtr;
	unsigned long *endPtr;
	unsigned long n = 0;
	
	rangeNum <<= 2;
	if (rangeNum >= 64) {
		unsigned long count;

		/* Clear the unaligned words at the beginning of the range */
		
		thePtr = (unsigned long*)startNum;
		count = (unsigned long)startNum & 31;
		if (count) {
			count = 32 - count;
			endPtr = (unsigned long*)((char*)startNum + count);
			while (thePtr != endPtr)
				*thePtr++ = n;
		}
		
		/* Clear all of the 32 byte aligned cache blocks in the range */
		
		count = (rangeNum - count) & 0xFFFFFFE0;
		ZeroCache(thePtr, count >> 5, 32);
		
		/* Clear the unaligned words at the end of the range */

		thePtr = (unsigned long*)((char*)thePtr + count);
		endPtr = (unsigned long*)((char*)startNum + rangeNum);
		while (thePtr != endPtr)
			*thePtr++ = n;
	} else {
		thePtr = (unsigned long*)startNum;
		endPtr = (unsigned long*)((char*)startNum + rangeNum);
		
		while (thePtr != endPtr)
			*thePtr++ = n;
	}
}  /* ZeroMem */

void FillMem(void* startNum, unsigned long rangeNum, unsigned long valueNum)
{
	unsigned long *thePtr;
	unsigned long *endPtr;
	
	rangeNum <<= 2;
	if (rangeNum >= 64) {
		unsigned long count;
		
		/* Set the unaligned words at the beginning of the range */
		
		thePtr = (unsigned long*)startNum;
		count = (unsigned long)startNum & 31;
		if (count) {
			count = 32 - count;
			endPtr = (unsigned long *)((char*)startNum + count);
			while (thePtr != endPtr)
				*thePtr++ = valueNum;
		}
		
		/* Set all of the 32 byte aligned cache blocks in the range */
		
		endPtr = (unsigned long*)((char*)thePtr + (0xFFFFFFE0 & (rangeNum - count)));
		while (thePtr != endPtr) {
			ZeroCache(thePtr, 1, 32);
			*thePtr++ = valueNum;
			*thePtr++ = valueNum;
			*thePtr++ = valueNum;
			*thePtr++ = valueNum;
			*thePtr++ = valueNum;
			*thePtr++ = valueNum;
			*thePtr++ = valueNum;
			*thePtr++ = valueNum;
		}
		
		/* Set the unaligned words at the end of the range */

		endPtr = (unsigned long*)((char*)startNum + rangeNum);
		while (thePtr != endPtr)
			*thePtr++ = valueNum;
	} else {
		thePtr = (unsigned long*)startNum;
		endPtr = (unsigned long*)((char*)startNum + rangeNum);
		
		while (thePtr != endPtr)
			*thePtr++ = valueNum;
	}
}  /* FillMem */

#else

void ZeroMem(void* startNum, unsigned long rangeNum)
{
	memset(startNum, 0, rangeNum << 2);
}  /* ZeroMem */

void FillMem(void* startNum, unsigned long rangeNum, unsigned long valueNum)
{
	unsigned long *thePtr = (unsigned long*)startNum;
	unsigned long *endPtr = thePtr + rangeNum;
	
	while (thePtr != endPtr)
		*thePtr++ = valueNum;
}  /* FillMem */

#endif
