/*
Copyright 1995-1998 Electric Image, Inc. This source code may only be used by
registered owners of the ElectricImage Plug-In Kit. Owners of the Kit may
include the source code in their products (freeware, shareware, or commercial).
This copyright notice must be included in its original form in all such
products. This source code may not be offered for sale or distributed by any
entity without the written permission of Electric Image, Inc.
*/

#include "PluginUtils.h"

#include <stdio.h>
#include "PluginClient.h"
#include "PluginFPU.h"
#include "PString.h"
#include <string.h>
#include <stdlib.h>

#include "EI_Control.h"
#include "EI_Dialog.h"
#include "EI_FileRef.h"
#include "EI_ResourceFile.h"

boolean errorFlag, hideErrorFlag, inBackground;

static void SetParamText(EI_Dialog *iDialog, const unsigned char* text0,
							const unsigned char* text1, const unsigned char* text2,
							const unsigned char* text3)

{
	int							numControls, size, i;
	EI_Control					*control;
	unsigned char				str[256];
	const unsigned char*		subStr;
	
	numControls = EI_CountControls(iDialog);
	for (i = 0; i < numControls; ++i) {
		control = EI_FindControlByIndex(iDialog, i);
		if (EI_GetControlType(control) == cEI_StaticText) {
			size = EI_GetControlTitleSize(control);
			if (size > 256)
				size = 256;
			EI_GetControlTitle(control, (char*) str, &size);
			PStrC2P((char*) str);
			
		//	Can use < here rather than <=, because the last character does not need to
		//	be checked (if it is a '^', it cannot have a digit after it).
			for (i = 1; i < str[0]; ) {
				if (str[i] == '^') {
					if (str[i + 1] == '0')
						subStr = text0;
					else if (str[i + 1] == '1')
						subStr = text1;
					else if (str[i + 1] == '2')
						subStr = text2;
					else if (str[i + 1] == '3')
						subStr = text3;
					else {
						++i;
						continue;
					}
					PStrDelete(str, i, 2);
					PStrInsert((unsigned char*) subStr, str, i);
					i += *subStr;
				} else
					++i;
			}
			
			PStrP2C(str);
			EI_SetControlTitle(control, (char*) str);
		}
	}
}

/*
 *		Static Functions
 */


/*
*  Global Functions
*/

int StandardDialog(EI_ResourceFile *iResFile, short iDialogID,
				   const unsigned char* iText0,
				   const unsigned char* iText1, const unsigned char* iText2,
				   const unsigned char* iText3)
{
	EI_Dialog		*dialog;
	int				result;
	
	dialog = EI_MakeDialog(iResFile, iDialogID);
	if (dialog == NULL)
		return 0;
	
	SetParamText(dialog, iText0, iText1, iText2, iText3);
	result = EI_ExecuteDialog(dialog);
	EI_DestroyDialog(dialog);
	return result;
}  /* StandardDialog */

/* Numeric conversion routines */

#ifndef __FPU_h__

double FixDouble(double theValue)
{
	DoubleRecordPtr d;
	
	d = (DoubleRecordPtr)&theValue;
	if (((d->l.l1 == (long)0x80000000) && (d->l.l2 == 0)) || ((0x7FF0 & d->i.i1) == 0x7FF0))
		return 0.0;
	else
		return theValue;
}  /* FixDouble */

#endif

PStrPtr CharStr(uchar theChar)
{
	PStrPtr theStr;
	
	theStr = PStrNext();
	PStrSetLength(theStr, sizeof(uchar));
	theStr[1] = theChar;
	return theStr;
}  /* CharStr */

PStrPtr NumStr(long theNum)
{
	return PStrNum(theNum);
}  /* NumStr */

double Str2Num(PStrPtr s)
{
	double theValue=0.0;
	CStrPtr cStr;
	
	cStr = PStrPToC(s, NULL);
	sscanf(cStr, "%lf", &theValue);
	return theValue;
}  /* Str2Num */

void Num2Str(long digits, double x, PStrPtr s)
{
	if (digits < 0)
		sprintf((CStrPtr)s, "%#1.*f", -digits, x);
	else
		sprintf((CStrPtr)s, "%1.*f", digits, x);
	s = PStrC2P((CStrPtr)s);
}  /* Num2Str */

PStrPtr RealTrimStr(double theNum, short theDigits)
{
	short theLength, i;
	PStrPtr theStr = PStrNext();
	
	Num2Str(theDigits, theNum, theStr);
	if (theDigits >= 0) {
		Num2Str(theDigits, theNum, theStr);
		theLength = PStrLength(theStr);
		for (i = 1; i <= theLength; i++)
			if (theStr[i] == '.')
				break;
		if (i <= theLength) {
			for (; theLength >= i; theLength--)
				if ((theStr[theLength] != '0') && (theStr[theLength] != '.'))
					break;
			PStrSetLength(theStr, theLength);
		}
		if ((PStrLength(theStr) == 2) && (theStr[1] == '-') && (theStr[2] == '0')) {
			PStrSetLength(theStr, sizeof(uchar));
			theStr[1] = '0';
		}			
	}
	return theStr;
}  /* RealTrimStr */

int Str2Int(PStrPtr s)

{
	int			theValue;
	CStrPtr		cStr;
	
	cStr = PStrPToC(s, NULL);
	sscanf(cStr, "%d", &theValue);
	return theValue;
}

/* Variable Setting Routines */

void SetFlag(ulong* theFlagFields, long theFlag, boolean theState)
{
	if (theState)
		BSET(*theFlagFields, theFlag);
	else
		BCLR(*theFlagFields, theFlag);
}  /* SetFlag */

void SetIntegerFlag(ushort* theFlagFields, long theFlag, boolean theState)
{
	if (theState)
		BSET(*theFlagFields, theFlag);
	else
		BCLR(*theFlagFields, theFlag);
}  /* SetIntegerFlag */

void SetBooleanFlag(ulong* dst, long theFlag, boolean src, boolean* flag)
{
	*flag = *flag || (BTST(*dst, theFlag) != src);
	if (src)
		BSET(*dst, theFlag);
	else
		BCLR(*dst, theFlag);
}  /* SetBooleanFlag */

void SetBoolean(boolean* dst, boolean src, boolean* flag)
{
	*flag = *flag || (src != *dst);
	*dst = src;
}  /* SetBoolean */

void SetByte(char* dst, short src, boolean* flag)
{
	*flag = *flag || (src != *dst);
	*dst = src;
}  /* SetByte */

void SetInteger(short* dst, short src, boolean* flag)
{
	*flag = *flag || (src != *dst);
	*dst = src;
}  /* SetInteger */

void SetLongInt(long* dst, long src, boolean* flag)
{
	*flag = *flag || (src != *dst);
	*dst = src;
}  /* SetLongInt */

void SetReal(float* dst, float src, boolean* flag)
{
	*flag = *flag || (src != *dst);
	*dst = src;
}  /* SetReal */

void SetDouble(double* dst, double src, boolean* flag)
{
	*flag = *flag || (src != *dst);
	*dst = src;
}  /* SetDouble */

void SetExtended(double* dst, double src, boolean* flag)
{
	*flag = *flag || (src != *dst);
	*dst = src;
}  /* SetExtended */

void SetRGBColor(RGB_Color* dst, RGB_Color* src, boolean* flag)
{
	*flag = *flag || (src->red != dst->red) || (src->green != dst->green) || (src->blue != dst->blue);
	*dst = *src;
}  /* SetRGB16Color */

void SetTheARGB(ARGB* dst, ARGB* src, boolean* flag)

{
	*flag = *flag || (src->argb.a != dst->argb.a) || (src->argb.r != dst->argb.r) || (src->argb.g != dst->argb.g) || (src->argb.b != dst->argb.b);
	*dst = *src;
}

/* Event Utilities */

#if 0
boolean OptionKeyDown(void)
{
	KeyMap theKeyMap;
	
	GetKeys(theKeyMap);
	return BitTst(theKeyMap, 58);
}  /* OptionKeyDown */

boolean CheckForAbort(void)
{
	QHdrPtr	theEventQue;
	EvQElPtr theQElemPtr;
	boolean result;
	
	result = 0;
	theEventQue = LMGetEventQueue();
	theQElemPtr = (EvQElPtr) theEventQue->qHead;
	while (theQElemPtr != NULL)
		if ((theQElemPtr->evtQWhat == keyDown) && (BAND(theQElemPtr->evtQModifiers, cmdKey) != 0) && (BAND(theQElemPtr->evtQMessage, charCodeMask) == '.')) {
			theQElemPtr->evtQWhat = nullEvent;
			theQElemPtr->evtQMessage = 0;
			theQElemPtr = (EvQElPtr)theEventQue->qHead;
			result = 1;
		} else
			theQElemPtr = (EvQElPtr)theQElemPtr->qLink;
	return result;
}  /* CheckForAbort */
#endif

/* Window And Dialog Utilities */

EI_ResourceFile *OpenResourceFile(const char *iResourceFileName)

{
	EI_ResourceFile			*resourceFile;
	EI_DirectoryRef			*homeDir;
	EI_FileRef				*fileRef;
	
	/* No need to check for NULL, since all EI_ functions deal with NULL correctly. */
	homeDir = EI_GetHomeDirectory();
	fileRef = EI_MakeFileRef(homeDir, iResourceFileName,
								EI_MAKE_FILE_TYPE('r', 's', 'r', 'c'), "rsc");
	
	resourceFile = EI_OpenResourceFile(fileRef);
	
	EI_DestroyFileRef(fileRef);
	EI_DestroyDirectoryRef(homeDir);
	
	return resourceFile;
}

void SetColorButtonIDARGB(const EI_Dialog *iDialog, int iControlID, const ARGB* iColor)

{
	EI_Control		*control;
	
	control = EI_FindControlByID(iDialog, iControlID);
	EI_SetColorButtonColor(control,
							iColor->argb.r,
							iColor->argb.g,
							iColor->argb.b,
							iColor->argb.a);
}

void GetColorButtonIDARGB(const EI_Dialog *iDialog, int iControlID, ARGB* oColor)

{
	EI_Control		*control;
	int				alpha, red, green, blue;
	
	control = EI_FindControlByID(iDialog, iControlID);
	EI_GetColorButtonColor(control, &red, &green, &blue, &alpha);
	
	oColor->argb.a = (unsigned char) alpha;
	oColor->argb.r = (unsigned char) red;
	oColor->argb.g = (unsigned char) green;
	oColor->argb.b = (unsigned char) blue;
}

void SetUserControlIDFunctions(EI_Dialog *iDialog, int iUserControlID,
								EI_UserDrawFunction iDraw, EI_UserClickFunction iClick)

{
	EI_Control*		control;
	
	control = EI_FindControlByID(iDialog, iUserControlID);
	if (control != NULL) {
		EI_SetUserControlDraw(control, iDraw);
		EI_SetUserControlClick(control, iClick);
	}
}

void SetControlIDHitFunction(EI_Dialog *iDialog, int iControlID,
								EI_ControlHitFunction iFunction)

{
	EI_Control*		control;
	
	control = EI_FindControlByID(iDialog, iControlID);
	if (control != NULL)
		EI_SetControlHitFunction(control, iFunction);
}

void SetControlIDVisible(EI_Dialog *iDialog, int iControlID, int iVisible)

{
	EI_Control*		control;
	
	control = EI_FindControlByID(iDialog, iControlID);
	EI_SetControlVisible(control, iVisible);
}

void SetControlIDTitle(EI_Dialog *iDialog, int iControlID, const char *iTitle)

{
	EI_Control*		control;
	
	control = EI_FindControlByID(iDialog, iControlID);
	EI_SetControlTitle(control, iTitle);
}

void SetEditTextIDString(EI_Dialog *iDialog, int iControlID, const char *iString)

{
	EI_Control*		control;
	
	control = EI_FindControlByID(iDialog, iControlID);
	EI_SetEditTextString(control, iString);
}

/* Old Fashioned Functions: */

void SetButton(const EI_Dialog *theDialog, int theItemID, int theValue)

{
	EI_Control		*control;
	
	control = EI_FindControlByID(theDialog, theItemID);
	if (control != NULL)
		EI_SetControlValue(control, theValue);
}

int GetButton(const EI_Dialog *theDialog, int theItemID)

{
	EI_Control		*control;
	
	control = EI_FindControlByID(theDialog, theItemID);
	if (control != NULL)
		return EI_GetControlValue(control);
	else
		return 0;
}

int ToggleButton(const EI_Dialog *theDialog, int theItemID)

{
	EI_Control		*control;
	
	control = EI_FindControlByID(theDialog, theItemID);
	if (control != NULL) {
		if (EI_GetControlValue(control)) {
			EI_SetControlValue(control, 0);
			return 0;
		} else {
			EI_SetControlValue(control, 1);
			return 1;
		}
	} else
		return 0;
}

void SetButtonName(const EI_Dialog *theDialog, int theItemID, PStrPtr theName)

{
	EI_Control		*control;
	char			str[256];
	
	control = EI_FindControlByID(theDialog, theItemID);
	if (control != NULL) {
		PStrPToC(theName, str);
		EI_SetControlTitle(control, str);
	}
}

PStrPtr GetButtonName(const EI_Dialog *theDialog, int theItemID, PStrPtr theName)

{
	EI_Control		*control;
	int				size;
	
	control = EI_FindControlByID(theDialog, theItemID);
	if (control != NULL) {
		size = EI_GetControlTitleSize(control);
		if (size > 256)
			size = 256;
		EI_GetControlTitle(control, (char*) theName, &size);
		PStrC2P((char*) theName);
	} else
		theName[0] = '\0';
	return theName;
}

void SelectItem(const EI_Dialog *theDialog, int theItemID)

{
	EI_Control		*control;
	
	control = EI_FindControlByID(theDialog, theItemID);
	if (control != NULL) {
		EI_SetKeyControl(control);
		if (EI_GetControlType(control) == cEI_EditText)
			EI_SetEditTextSelection(control, 0, 10000);
	}
}

void SetTheIText(const EI_Dialog *theDialog, int theItemID, PStrPtr theText)

{
	EI_Control		*control;
	char			str[256];
	
	control = EI_FindControlByID(theDialog, theItemID);
	if (control != NULL) {
		PStrPToC(theText, str);
		EI_SetEditTextString(control, str);
	}
}

void GetTheIText(const EI_Dialog *theDialog, int theItemID, PStrPtr theText)

{
	EI_Control		*control;
	int				size;
	
	control = EI_FindControlByID(theDialog, theItemID);
	if (control != NULL) {
		size = EI_GetEditTextStringSize(control);
		if (size > 256)
			size = 256;
		EI_GetEditTextString(control, (char*) theText, &size);
		PStrC2P((char*) theText);
	} else
		theText[0] = '\0';
}

void SetTheINum(const EI_Dialog *theDialog, int theItemID, long theNum)

{
	EI_Control		*control;
	char			theText[50];
	
	control = EI_FindControlByID(theDialog, theItemID);
	if (control != NULL) {
		sprintf(theText, "%ld", theNum);
		EI_SetEditTextString(control, theText);
	}
}

long GetTheINum(const EI_Dialog *theDialog, int theItemID)

{
	EI_Control		*control;
	char			str[50];
	int				size;
	
	control = EI_FindControlByID(theDialog, theItemID);
	if (control != NULL) {
		size = EI_GetEditTextStringSize(control);
		if (size > 50)	/*	Should never happen	*/
			size = 50;
		EI_GetEditTextString(control, (char*) str, &size);
		return (long) atoi(str);
	}
	return (long) 0;
}

void SetTheIReal(const EI_Dialog *theDialog, int theItemID, double theNum, short theDigits)

{
	SetTheIText(theDialog, theItemID, RealTrimStr(theNum, theDigits));
}

double GetTheIReal(const EI_Dialog *theDialog, int theItemID)

{
	PStrPtr theText;
	
	theText = PStrNext();
	GetTheIText(theDialog, theItemID, theText);
	return FixDouble(Str2Num(theText));
}

void SetTheIPopUp(const EI_Dialog *theDialog, int theItemID, short theValue)

{
	EI_Control		*control;
	
	control = EI_FindControlByID(theDialog, theItemID);
	if (control != NULL)
		EI_SetControlValue(control, theValue);
}

short GetTheIPopUp(const EI_Dialog *theDialog, int theItemID)

{
	EI_Control		*control;
	
	control = EI_FindControlByID(theDialog, theItemID);
	if (control != NULL)
		return EI_GetControlValue(control);
	else
		return 0;
}

EI_ControlType GetTheIType(const EI_Dialog *theDialog, int theItemID)

{
	EI_Control		*control;
	
	control = EI_FindControlByID(theDialog, theItemID);
	if (control != NULL)
		return EI_GetControlType(control);
	else
		return cEI_NoControlType;
}

void SetTheIRect(const EI_Dialog *theDialog, int theItemID, const EI_Rect *theRect)

{
	EI_Control		*control;
	
	if (theRect == NULL)
		return;
	
	control = EI_FindControlByID(theDialog, theItemID);
	if (control != NULL)
		EI_SetControlBounds(control, theRect);
}

void GetTheIRect(const EI_Dialog *theDialog, int theItemID, EI_Rect *theRect)

{
	EI_Control		*control;
	
	if (theRect == NULL)
		return;
	
	control = EI_FindControlByID(theDialog, theItemID);
	if (control != NULL)
		EI_GetControlBounds(control, theRect);
}

/* Error Utilities */

void Error(EI_ResourceFile *iResourceFile, PStrPtr theMessage, long theErr)
{
	if ((!errorFlag) && (!hideErrorFlag))
		StandardDialog(iResourceFile, errorDialogID,
						PStrConcat(theMessage, PS(": "), NumStr(theErr), NULL, NULL),
						aEmptyPStr, aEmptyPStr, aEmptyPStr);

	errorFlag = 1;
}  /* Error */

void StandardError(EI_ResourceFile *iResourceFile, PStrPtr theMessage)
{
	if ((!errorFlag) && (!hideErrorFlag) && PStrNotEmpty(theMessage))
		StandardDialog(iResourceFile, errorDialogID, theMessage, aEmptyPStr, aEmptyPStr, aEmptyPStr);
	errorFlag = 1;
}  /* StandardError */

void Stop(EI_ResourceFile *iResourceFile, PStrPtr theMessage)
{
	StandardDialog(iResourceFile, errorDialogID, theMessage, aEmptyPStr, aEmptyPStr, aEmptyPStr);
}  /* Stop */

void Caution(EI_ResourceFile *iResourceFile, PStrPtr theMessage)
{
	StandardDialog(iResourceFile, cautionDialogID, theMessage, aEmptyPStr, aEmptyPStr, aEmptyPStr);
}  /* Caution */

void Message(EI_ResourceFile *iResourceFile, PStrPtr theMessage)
{
	StandardDialog(iResourceFile, messageDialogID, theMessage, aEmptyPStr, aEmptyPStr, aEmptyPStr);
}  /* Message */

int Warning(EI_ResourceFile *iResourceFile, PStrPtr theMessage)
{
	return StandardDialog(iResourceFile, warningDialogID, theMessage, aEmptyPStr, aEmptyPStr, aEmptyPStr);
}  /* Warning */

/* UtilLibs Setup */

void InitUtils(void)
{
	/* Initialize global flags and variables */
	
	errorFlag = 0;
	hideErrorFlag = 0;
	inBackground = 0;
}  /* InitUtils */

void FinishUtils(void)
{
}  /* FinishUtils */
