/*
 *		Copyright (c)1991-1997 Electric Image, Inc.
 *		This code is confidential and is a trade secret of Electric Image, Inc.
 */

#include "ColorSpline.h"
#include "EIShader.h"
#include "FPU.h"
#include "Matrix.h"
#include "MemoryUtils.h"
#include "Noise.h"
#include "PString.h"
#include "System.h"

#define MIX(a, b, f) ((a) + (f) * ((b) - (a)))

#define shaderType				'CHRM'
#define shaderVersion			0x00000001
#define shaderRevVersion		0x01000000

#define defAltitudeFactor		0.5
#define defAffectSurface		0x80000000


typedef struct {
	unsigned long shaderOwnerType;
	unsigned long shaderVersionNum;

	long shaderAffectFlags;
	float shaderAltitudeFactor;
	ARGB chromeColors[13];
} ShaderInterfaceRec, *ShaderInterfacePtr;

typedef struct {
	long shaderAffectFlags;
	float shaderAltitudeFactor;
	ARGB_Real chromeColors[13];
} ShaderDataRec, *ShaderDataPtr;

/* Export function prototypes */

long EIShaderInformation(EIShaderInformationRec *theInformationPtr);
long EIShaderInitialize(EIShaderInitializeRec *theInitializePtr);
long EIShaderFinish(EIShaderFinishRec *theFinishPtr);
long EIShaderSerial(EIShaderSerialRec *theSerialPtr);
long EIShaderShade(EIShaderShadeRec *theShadePtr);

/* Static functions */

static void DefaultShaderInterface(ShaderInterfacePtr theShaderInterface)
{
	theShaderInterface->shaderOwnerType = shaderType;

	theShaderInterface->shaderAltitudeFactor = defAltitudeFactor;
	theShaderInterface->shaderAffectFlags = defAffectSurface;

	theShaderInterface->chromeColors[0].argb.a = 255;
	theShaderInterface->chromeColors[0].argb.r = 0.1307 * 255.0;
	theShaderInterface->chromeColors[0].argb.g = 0.0609 * 255.0;
	theShaderInterface->chromeColors[0].argb.b = 0.0355 * 255.0;

	theShaderInterface->chromeColors[1].argb.a = 255;
	theShaderInterface->chromeColors[1].argb.r = 0.1307 * 255.0;
	theShaderInterface->chromeColors[1].argb.g = 0.0609 * 255.0;
	theShaderInterface->chromeColors[1].argb.b = 0.0355 * 255.0;

	theShaderInterface->chromeColors[2].argb.a = 255;
	theShaderInterface->chromeColors[2].argb.r = 0.1307 * 255.0;
	theShaderInterface->chromeColors[2].argb.g = 0.0609 * 255.0;
	theShaderInterface->chromeColors[2].argb.b = 0.0355 * 255.0;

	theShaderInterface->chromeColors[3].argb.a = 255;
	theShaderInterface->chromeColors[3].argb.r = 0.1307 * 255.0;
	theShaderInterface->chromeColors[3].argb.g = 0.0609 * 255.0;
	theShaderInterface->chromeColors[3].argb.b = 0.0355 * 255.0;

	theShaderInterface->chromeColors[4].argb.a = 255;
	theShaderInterface->chromeColors[4].argb.r = 0.1307 * 255.0;
	theShaderInterface->chromeColors[4].argb.g = 0.0609 * 255.0;
	theShaderInterface->chromeColors[4].argb.b = 0.0355 * 255.0;

	theShaderInterface->chromeColors[5].argb.a = 255;
	theShaderInterface->chromeColors[5].argb.r = 0.1307 * 255.0;
	theShaderInterface->chromeColors[5].argb.g = 0.0609 * 255.0;
	theShaderInterface->chromeColors[5].argb.b = 0.0355 * 255.0;

	theShaderInterface->chromeColors[6].argb.a = 255;
	theShaderInterface->chromeColors[6].argb.r = 0.4274 * 255.0;
	theShaderInterface->chromeColors[6].argb.g = 0.5880 * 255.0;
	theShaderInterface->chromeColors[6].argb.b = 0.9347 * 255.0;

	theShaderInterface->chromeColors[7].argb.a = 255;
	theShaderInterface->chromeColors[7].argb.r = 0.1221 * 255.0;
	theShaderInterface->chromeColors[7].argb.g = 0.3794 * 255.0;
	theShaderInterface->chromeColors[7].argb.b = 0.9347 * 255.0;

	theShaderInterface->chromeColors[8].argb.a = 255;
	theShaderInterface->chromeColors[8].argb.r = 0.1090 * 255.0;
	theShaderInterface->chromeColors[8].argb.g = 0.3386 * 255.0;
	theShaderInterface->chromeColors[8].argb.b = 0.8342 * 255.0;

	theShaderInterface->chromeColors[9].argb.a = 255;
	theShaderInterface->chromeColors[9].argb.r = 0.0643 * 255.0;
	theShaderInterface->chromeColors[9].argb.g = 0.2571 * 255.0;
	theShaderInterface->chromeColors[9].argb.b = 0.6734 * 255.0;

	theShaderInterface->chromeColors[10].argb.a = 255;
	theShaderInterface->chromeColors[10].argb.r = 0.0513 * 255.0;
	theShaderInterface->chromeColors[10].argb.g = 0.2053 * 255.0;
	theShaderInterface->chromeColors[10].argb.b = 0.5377 * 255.0;

	theShaderInterface->chromeColors[11].argb.a = 255;
	theShaderInterface->chromeColors[11].argb.r = 0.0326 * 255.0;
	theShaderInterface->chromeColors[11].argb.g = 0.1591 * 255.0;
	theShaderInterface->chromeColors[11].argb.b = 0.4322 * 255.0;

	theShaderInterface->chromeColors[12].argb.a = 255;
	theShaderInterface->chromeColors[12].argb.r = 0;
	theShaderInterface->chromeColors[12].argb.g = 0;
	theShaderInterface->chromeColors[12].argb.b = 0;

}  /* DefaultShaderInterface */

static void FixShaderInterface(ShaderInterfacePtr theShaderInterface)
{
	FixField(theShaderInterface->shaderVersionNum);
	FixField(theShaderInterface->shaderAltitudeFactor);
	FixField(theShaderInterface->shaderAffectFlags);

}  /* FixShaderInterface */

static void GetShaderInterface(void* theInterface, long theSize, ShaderInterfacePtr theShaderInterface)
{
	/* Copy the shader interface data */
	
	if (theSize == sizeof(ShaderInterfaceRec))
		*theShaderInterface = *(ShaderInterfacePtr)theInterface;
	else {
		theShaderInterface->shaderOwnerType = 0;
		theShaderInterface->shaderVersionNum = 0;
	}
	
	/* Validate the shader interface data */
	
	if (theShaderInterface->shaderOwnerType != shaderType)
		DefaultShaderInterface(theShaderInterface);
	else if (theShaderInterface->shaderVersionNum == shaderRevVersion)
		FixShaderInterface(theShaderInterface);
	else if (theShaderInterface->shaderVersionNum != shaderVersion)
		DefaultShaderInterface(theShaderInterface);
}  /* GetShaderInterface */

/* Export functions */

long EIShaderInformation(EIShaderInformationRec *theInformationPtr)
{
	ShaderInterfaceRec theShaderInterface;
		
	/* Get the shader interface parameters */
		
	GetShaderInterface(theInformationPtr->shaderInterface, theInformationPtr->shaderInterfaceSize, &theShaderInterface);
	
	/* Setup the shader attribute access flags */	
	/* Setup the shader feature flags */
	
	BSET(theInformationPtr->shaderFeatureFlags, shaderAntialiasFlag);

	if (BTST(theShaderInterface.shaderAffectFlags, 31))
		BSET(theInformationPtr->shaderFeatureFlags, shaderColorFlag);

	if (BTST(theShaderInterface.shaderAffectFlags, 30))
		BSET(theInformationPtr->shaderFeatureFlags, shaderReflectionFlag);
		
		
	return shaderNoError;
}  /* EIShaderInformation */

long EIShaderInitialize(EIShaderInitializeRec *theInitializePtr)
{
	ShaderDataPtr theShaderData;
	short i;
	
	/* Allocate the shader's private data record */
	
	theShaderData = (ShaderDataPtr)theInitializePtr->HostMalloc(sizeof(ShaderDataRec));
	if (theShaderData) {
		ShaderInterfaceRec theShaderInterface;
		
		/* Get the shader interface parameters */
		
		GetShaderInterface(theInitializePtr->shaderInterface, theInitializePtr->shaderInterfaceSize, &theShaderInterface);

		/* Setup the shader's private data */
		
		theShaderData->shaderAltitudeFactor = theShaderInterface.shaderAltitudeFactor;
		theShaderData->shaderAffectFlags = theShaderInterface.shaderAffectFlags;

		for (i=0;i<13;i++) {
			theShaderData->chromeColors[i].a = theShaderInterface.chromeColors[i].argb.a / 255.0;
			theShaderData->chromeColors[i].r = theShaderInterface.chromeColors[i].argb.r / 255.0;
			theShaderData->chromeColors[i].g = theShaderInterface.chromeColors[i].argb.g / 255.0;
			theShaderData->chromeColors[i].b = theShaderInterface.chromeColors[i].argb.b / 255.0;
		}
		
		/* Return the pointer to the shader's private data to the host */
		
		theInitializePtr->shaderData = theShaderData;
		return shaderNoError;
	} else
		return shaderMemoryError;
}  /* EIShaderInitialize */

long EIShaderFinish(EIShaderFinishRec *theFinishPtr)
{
	/* Get rid of the shader's private data */
	
	theFinishPtr->HostFree(theFinishPtr->shaderData);
	
	return shaderNoError;
}  /* EIShaderFinish */

long EIShaderSerial(EIShaderSerialRec *theSerialPtr)
{	
	unsigned long theChallengeType = QUADWORD('SERL');
	unsigned char testString[8];

	theSerialPtr->HostChallenge(theChallengeType, &testString[0]);

	return shaderNoError;
}  /* EIShaderSerial */

long EIShaderShade(EIShaderShadeRec *theShadePtr)
{
	ShaderDataPtr theShaderData = (ShaderDataPtr)theShadePtr->shaderData;
	EIShaderMapPtr theMap = theShadePtr->shaderMap;
	EIShaderPixelPtr thePixel = theShadePtr->shaderPixel;
	float altitude, Rx, Ry, Rz, dotProd;
	double IwX, IwY, IwZ;
	float Vx, Vy, Vz;
	ARGB_Real chromeColor;
	
	Matrix4	cameraInvRotMatrix;
	
	
	Vx = thePixel->shadeXN;	Vy = thePixel->shadeYN;	Vz = thePixel->shadeZN;

	dotProd = (thePixel->shadeNormalA * Vx) + (thePixel->shadeNormalB * Vy) + (thePixel->shadeNormalC * Vz);

	Rx = 2.0 * thePixel->shadeNormalA * dotProd - Vx;
	Ry = 2.0 * thePixel->shadeNormalB * dotProd - Vy;
	Rz = 2.0 * thePixel->shadeNormalC * dotProd - Vz;

	
	
	CopyMem(thePixel->shadeCameraInvMatrix, &cameraInvRotMatrix, sizeof(Matrix4));
	cameraInvRotMatrix.m.m30 = 0.0;
	cameraInvRotMatrix.m.m31 = 0.0;
	cameraInvRotMatrix.m.m32 = 0.0;	
	
	IwX = Rx; IwY = Ry; IwZ = Rz;
	TransformPoint(&IwX, &IwY, &IwZ, &cameraInvRotMatrix);

	altitude = theShaderData->shaderAltitudeFactor * IwY + 0.5;



	ColorSpline(altitude, 13, &theShaderData->chromeColors[0], &chromeColor);

	if (BTST(theShaderData->shaderAffectFlags, 31)) {
		theShadePtr->shaderColor = chromeColor;
	}
	
	if (BTST(theShaderData->shaderAffectFlags, 30)) {
		theShadePtr->shaderReflection.r = chromeColor.r;
		theShadePtr->shaderReflection.g = chromeColor.g;
		theShadePtr->shaderReflection.b = chromeColor.b;
	}
	
	return shaderNoError;
}  /* EIShaderShade */
