#include "StubXP.hpp"

// global variables
OSType theCreator;
long theError;
PluginInfoRec * theInfo;
EIPluginRoutine theHostProc;

const char * theText[3] = {
 "Set Dst group to distance 100 from Src group",
 "Set Dst group at location of vertex 100 of Src group",
 "Set Dst group at location of Src group (- 5 frames)"
};
 
//---------------- Public ---------------------------

Boolean HostCall( OSType cmd, long size, void * data )
{
	static long debugTrap = 0;
	theError = theHostProc(cmd, size, data);
	if (theError)
	 ++debugTrap;
	return theError == 0;
}

void Information( void )
{
	PluginInformationPtr theParam  = PluginInformationPtr(theInfo);
	theParam->thePluginType        = pluginType;
	theParam->thePluginVersion     = pluginVersion;
	PStrCpy(theParam->thePluginName, PS(pluginName));
	PStrCpy(theParam->thePluginCopyright, pluginCopyright);
	PStrCpy(theParam->thePluginAuthor, pluginAuthor);
	PStrCpy(theParam->thePluginDescription, pluginDescription);
	theParam->thePluginPermission  = pluginPermission;
}

CControlData * GetControlData( void )
{
	CControlData * ctl;
	if (!theInfo->theControlData || theInfo->theControlDataSize < sizeof(CControlData)) {
	 HostAllocateRec har;
	 har.theReference = theInfo->theHostReference;
	 har.theControlData = theInfo->theControlData;	
	 har.theControlDataSize = sizeof(CControlData);
	 har.thePluginData = 0;
	 har.thePluginDataSize = 0;
	 if (!HostCall('ALOC', sizeof(har), &har)) return 0;
	
	 theInfo->thePluginData = har.thePluginData;
	 theInfo->theControlData = har.theControlData;
	 theInfo->thePluginDataSize = har.thePluginDataSize;
	 theInfo->theControlDataSize = har.theControlDataSize;
	 
	 ctl = (CControlData *) theInfo->theControlData;
	 ctl->mMode = TEST_MODE_0;
	} 
	else {
	 ctl = (CControlData *) theInfo->theControlData;
	 ctl->mMode = MAX(ctl->mMode, TEST_MODE_0);
	 ctl->mMode = MIN(ctl->mMode, TEST_MODE_2);
	} 
	return ctl;
}

void ModelSetup( void )
{
	GetControlData();
	if (!theError) {
	 HostModelResetRec hmr;
	 hmr.theReference = theInfo->theHostReference;
	 HostCall('RSET', sizeof(hmr), &hmr);
	} 
}	

static void ModeHit( EI_Control * ctl,	EI_DrawContext * )
{
	CControlData * ctlData = GetControlData();
	EI_Dialog * dlg = EI_GetControlDialog(ctl);
	ctlData->mMode = EI_GetControlValue(ctl);
	EI_SetControlTitle(EI_FindControlByID(dlg, ID_TEXT), theText[ctlData->mMode]);

// ask host to start pluginProjectInfo loop
// we must set pluginPermissionModeless flag to process this now
	HostProjectInfoRequestRec hrq;
	hrq.theReference = theInfo->theHostReference;
	HostCall(hostProjectInfoRequest, sizeof(hrq), &hrq);
}

void Interface( void )
{
	CControlData * ctl = GetControlData();
	if (!ctl) return;
	
	EI_ResourceFile * theRF = OpenResourceFile(resourceName);
	if (!theRF) theError = pluginResourceErr;
	else {
	 EI_Dialog * dlg = EI_MakeDialog(theRF, DLG_INTERFACE);
	 if (!dlg) theError = pluginResourceErr;
	 else {	 
	  EI_SetControlValue(EI_FindControlByID(dlg, ID_MODE), ctl->mMode);
	  EI_SetControlTitle(EI_FindControlByID(dlg, ID_TEXT), theText[ctl->mMode]);
	  EI_SetControlHitFunction(EI_FindControlByID(dlg, ID_MODE), ModeHit);
	  EI_SetDialogTitle(dlg, "StubXP test");
	  if (!EI_ExecuteDialog(dlg)) theError = pluginCancelErr;
	  else {
	   HostModelResetRec hmr;
	   hmr.theReference = theInfo->theHostReference;
	   HostCall('RSET', sizeof(hmr), &hmr);
	  } 
	 } 
	 EI_CloseResourceFile(theRF);
	}
} 


#ifdef __MACINTOSH__
pascal long Main( OSType theCommand, long, PluginInfoRec * info );
#endif

#ifdef __WIN32__
EXPDLL long Main( OSType theCommand, long, PluginInfoRec * info )
#else
pascal long Main( OSType theCommand, long, PluginInfoRec * info )
#endif
{
	theInfo = info;
	theHostProc = info->theHostProc;
	theError = 0;
	
	switch (theCommand) { 
	 case pluginInitialize: 	
	  Initialize();
	  break;
	  
	 case pluginInformation: 	
	  Information();	 
	  break;
	  
	 case pluginInterface:   	
	  Interface();	 
	  break;
	  
	 case pluginProjectInfo:	 	
	  ProjectInfo();	 
	  break;
	}
	
	return theError;
}  

// #pragma export list	main
