#include <stdlib.h>
#include <stdio.h>
#include "StubXP.hpp"
#include "FPU.h"
#include "PluginProjectInfo.h"
#include <string.h>
#include "Matrix.h"
#include "TrackCache.hpp"

#define UNIV_POS		1010
#define	UNIV_COL		2010

#define UNIV_ROT_X		1120
#define UNIV_ROT_Y		1130
#define UNIV_ROT_Z		1140

#define UNIV_SCALE_X	1200
#define UNIV_SCALE_Y	1210
#define UNIV_SCALE_Z	1220

#define MAX_NUM_OBJ		200
#define DIA_MAX_NUM		100

#define SYNC_CURRENT	0
#define SYNC_FRAME		1
#define SYNC_TIME		2

namespace {

typedef HostObjectInfoRec THif;

struct CData {
	CData( void ) : mGroup(0) {}
	
	Boolean Init ( PStrPtr name );
	
	void *  Read ( long ID, long type, long frame );
	Boolean Write( long ID, long type, long frame, void * data );

	Boolean Read ( long ID, long frame, DCoordinate & );
	Boolean Read ( long ID, long frame, double & d );

	Boolean ReadLocal( Matrix4 & M, long frame );
	Boolean ReadMatr ( Matrix4 & M );
	Boolean ReadPos  ( DCoordinate & );
	Boolean WritePos ( long frame, DCoordinate & );
	
// data members	
	Ptr mGroup;
};

struct CObjectArray {
	CObjectArray( void ) : mCount(0) {}
	
	Boolean AutoInit( void );
	void    Update  ( void );
	THif *  Ptr2Hif ( Ptr thePtr );
	Ptr     Name2Ptr( PStrPtr name );
	THif *  Name2Hif( PStrPtr name );
	void    Delete  ( Ptr thePtr );
	void    Add     ( const THif & src );

// data members	
	long mCount;
	THif mData[MAX_NUM_OBJ];
};	

CObjectArray theObject;
CTrackCache theTrackCache;

//----------- CObjectArray ------------

Boolean CObjectArray::AutoInit( void )
{
	if (mCount) return true;
	
	THif & hif = mData[0];
	hif.theReference = theInfo->theHostReference;
	hif.theReferenceIndex = -1;
	if (!HostCall('UOBJ', sizeof(hif), &hif)) return false;
	
	long i, limit = MIN(hif.theObjectIndex + 1, MAX_NUM_OBJ); 
	
// read loop	
	for (i = 1; i < limit; ++i) {
	 mData[i - 1].theReference = theInfo->theHostReference;
	 mData[i - 1].theReferenceIndex = i;
	 if (!HostCall('UOBJ', sizeof(THif), mData + i - 1)) return false;
	}
	
	mCount = limit;
	return true;
}

THif * CObjectArray::Name2Hif( PStrPtr name )
{
	for (long i = 0; i < mCount; ++i)
	 if (!PStrCompare(name, mData[i].theObjectName)) return mData + i;
	return 0;
}

THif * CObjectArray::Ptr2Hif( Ptr thePtr )
{
	for (long i = 0; i < mCount; ++i)
	 if (mData[i].theObject == thePtr) return mData + i;
	return 0;
}

Ptr CObjectArray::Name2Ptr( PStrPtr name )
{
	THif * hif = Name2Hif(name);
	return hif ? hif->theObject : 0;
}

void CObjectArray::Delete( Ptr thePtr )
{
	THif * hif = Ptr2Hif(thePtr);
	if (!hif) return;
	memmove(hif, hif + 1, sizeof(THif) * (mCount - (hif - mData) - 1));
	--mCount;
}

void CObjectArray::Add( const THif & src )
{
	if (mCount < MAX_NUM_OBJ) mData[mCount++] = src;
}

void CObjectArray::Update( void )
{
	long i;
	PluginProjectInfoRec * inf = (PluginProjectInfoRec *) theInfo;
	
// look for delete
	for (i = 0; i < inf->theActionNum; ++i)
	 if (inf->theAction[i].theAction == tsa_ObjectDelete) 
	  Delete(inf->theAction[i].theParent);

// look for add/change	
	for (i = 0; i < inf->theObjectNum; ++i) {
	 THif * hif = Ptr2Hif(inf->theObject[i].theObject);
	 if (!hif) Add(inf->theObject[i]);
	 else *hif = inf->theObject[i];
	} 
}
 
//--------------- CData -------------------

Boolean CData::Init( PStrPtr name )
{
	mGroup = theObject.Name2Ptr(name);
	return mGroup != 0;
}

void * CData::Read( long ID, long type, long frame )
{
	void * cache = theTrackCache.Get(mGroup, ID, frame);
	if (cache) return cache;
	
	HostGetChannelFrameRec gchf;
	gchf.theReference = theInfo->theHostReference;
	gchf.theChannelOwner = mGroup;
	gchf.theChannelReference = Ptr(ID);
	gchf.theChannelID = 0;
	gchf.theData = 0;			
	gchf.theFrameIndex = frame;					
	gchf.theDataType = type;
	if (!HostCall('GCHF', sizeof(gchf), &gchf)) return 0;
	return gchf.theData;
}

Boolean CData::Write( long ID, long type, long frame, void * data )
{
// check to write always (track just ended)
	Boolean writeAbsolute = !theTrackCache.IsActive() && theTrackCache.Look(mGroup, ID);
	
// check to avoid writing if channel value is match	
	if (!writeAbsolute && CompareChannelValues(data, Read(ID, type, frame + 1), type)) 
	  return true;
	
	HostAddChannelFrameRec achf;
	achf.theReference = theInfo->theHostReference;
	achf.theChannelOwner = mGroup;
	achf.theChannelReference = Ptr(ID);
	achf.theChannelID = 0;
	achf.theData = Ptr(data);			
	achf.theFrameIndex = frame;					
	achf.theDataType = type;
	if (!HostCall('ACHF', sizeof(achf), &achf)) return false;
	theTrackCache.Put(mGroup, ID, frame + 1, data, type);
	return true;
}

Boolean CData::Read( long ID, long frame, DCoordinate & d )
{
	void * tmp = Read(ID, channelCoordinate, frame);
	if (!tmp) return false;
	d = *(DCoordinate *) tmp;
	return true;	
}

Boolean CData::Read( long ID, long frame, double & d )
{
	void * tmp = Read(ID, channelCoordinate, frame);
	if (!tmp) return false;
	d = *(double *) tmp;
	return true;	
}

Boolean CData::ReadMatr( Matrix4 & matr )
{
	HostObjectTransformRec trans;
	trans.theReference = theInfo->theHostReference;
	trans.theTransformOwner = mGroup;

	if (!HostCall(hostObjectTransform, sizeof(trans), &trans)) return false;
	matr = trans.theTransform;
	return true;
}

Boolean CData::ReadPos( DCoordinate & pos )
{
	Matrix4 matr;
	if (!ReadMatr(matr)) return false;
	pos = *(DCoordinate *) &matr.m.m30;
	return true;
}

Boolean CData::ReadLocal( Matrix4 & M, long frame )
{
	InitMatrix4(&M);

// read channels	
	DCoordinate rotation, scale, offset;
	if (!Read(UNIV_ROT_X, frame, rotation.x)) return false;
	if (!Read(UNIV_ROT_Y, frame, rotation.y)) return false;
	if (!Read(UNIV_ROT_Z, frame, rotation.z)) return false;

	if (!Read(UNIV_SCALE_X, frame, scale.x)) return false;
	if (!Read(UNIV_SCALE_Y, frame, scale.y)) return false;
	if (!Read(UNIV_SCALE_Z, frame, scale.z)) return false;
	
	if (!Read(UNIV_POS, frame, offset)) return false;
	
// assume ZXY default rotation (use GLNK for more)
	rotation.x *= PI / 180;
	rotation.y *= PI / 180;
	rotation.z *= PI / 180;
	if (rotation.z) ZRotateMatrix4(&M, sin(rotation.z), cos(rotation.z));
	if (rotation.x) XRotateMatrix4(&M, sin(rotation.x), cos(rotation.x));
	if (rotation.y) YRotateMatrix4(&M, sin(rotation.y), cos(rotation.y));
	
// scale
	DCoordinate	* ortX = (DCoordinate *) &M.m.m00;
	DCoordinate	* ortY = (DCoordinate *) &M.m.m10;
	DCoordinate	* ortZ = (DCoordinate *) &M.m.m20;
	if (scale.x != 1) {
	 ortX->x *= scale.x;
	 ortX->y *= scale.x;
	 ortX->z *= scale.x;
	}
	if (scale.y != 1) {
	 ortY->x *= scale.y;
	 ortY->y *= scale.y;
	 ortY->z *= scale.y;
	}
	if (scale.x != 1) {
	 ortZ->x *= scale.z;
	 ortZ->y *= scale.z;
	 ortZ->z *= scale.z;
	}
	
// offset
	*(DCoordinate *) &M.m.m30 = offset;	
	return true;
}

Boolean CData::WritePos( long frame, DCoordinate & pos )
{
	Matrix4 globalPrev, globalChild, localChild, temp;

// get global transform
	if (!ReadMatr(globalChild)) return false;	

// get local child matrix
	if (!ReadLocal(localChild, frame)) return false;

// extract global "pre-matrix"
	InvertMatrix4(&temp, localChild);
	globalPrev = globalChild;
	AppendMatrix4(&globalPrev, &temp);
	
// the final globalChild should have the offset we need
	*(DCoordinate *) &globalChild.m.m30 = pos;
	
// extract local child matrix
	InvertMatrix4(&temp, globalPrev);
	AppendMatrix4(&temp, &globalChild);

// update channel	
	return Write(UNIV_POS, channelCoordinate, frame - 1, &temp.m.m30);
}

//-----------------------------------

Boolean Test100( long frame )
{
	CData srcData, dstData;
	if (!srcData.Init(PS("Src"))) return false;
	if (!dstData.Init(PS("Dst"))) return false;
	
	DCoordinate src, dst;
	if (!srcData.Read(UNIV_POS, frame, src)) return false;
	if (!dstData.Read(UNIV_POS, frame, dst)) return false;
	
// "dst" should be at distance 100 from "src"	
	DCoordinate vec;
	vec.x = dst.x - src.x;
	vec.y = dst.y - src.y;
	vec.z = dst.z - src.z;
	
	if (!vec.x && !vec.y && !vec.z) vec.x = 1;
	double len = SQRT(SQR(vec.x) + SQR(vec.y) + SQR(vec.z));
	len = 100.0 / len;
	
	dst.x = src.x + vec.x * len;
	dst.y = src.y + vec.y * len;
	dst.z = src.z + vec.z * len;

// NOTE: Write(ACHF) requires frame - 1	
	return dstData.Write(UNIV_POS, channelCoordinate, frame - 1, &dst);
}

Boolean Test100_2( long frame )
{
	CData srcData, dstData;
	if (!srcData.Init(PS("Src"))) return false;
	if (!dstData.Init(PS("Dst"))) return false;
	
	DCoordinate src, dst;
	if (!srcData.ReadPos(src)) return false;
	if (!dstData.ReadPos(dst)) return false;

// "dst" should be at distance 100 from "src"	
	DCoordinate vec;
	vec.x = dst.x - src.x;
	vec.y = dst.y - src.y;
	vec.z = dst.z - src.z;
	
	if (!vec.x && !vec.y && !vec.z) vec.x = 1;
	double len = SQRT(SQR(vec.x) + SQR(vec.y) + SQR(vec.z));
	len = 100.0 / len;
	
	dst.x = src.x + vec.x * len;
	dst.y = src.y + vec.y * len;
	dst.z = src.z + vec.z * len;

// test color	
	if (1) {
	 ARGB_Color color;
	 float x = FABS(dst.x);
	 float y = FABS(dst.y);
	 float z = FABS(dst.z);
	 float max = MAX3(x, y, z);
	 color.alpha = 0xFFFF;
	 color.red   = Byte(x / max * 255) << 8;
	 color.green = Byte(y / max * 255) << 8;
	 color.blue  = Byte(z / max * 255) << 8;
	 dstData.Write(UNIV_COL, channelARGB, frame - 1, &color);
	}

	return dstData.WritePos(frame, dst);
}

Boolean TestVer( const PluginProjectInfoRec * inf )
{
	long frame = inf->theFrameIndex;

	CData srcData, dstData;
	if (!srcData.Init(PS("Src"))) return false;
	if (!dstData.Init(PS("Dst"))) return false;

	DCoordinate src, dst;
	if (!srcData.Read(UNIV_POS, frame, src)) return false;
	if (!dstData.Read(UNIV_POS, frame, dst)) return false;
	
// "dst" is located at vertex 100 of "src"	

// ask host to perform all geometry transformations for "src"
	HostObjectSynchronizeRec hos;
	hos.theReference = theInfo->theHostReference;
	hos.theFrameIndex = inf->theFrameIndex;
	hos.theFrameTime = inf->theFrameTime;
	hos.theFrameTimeDelta = inf->theFrameTimeDelta;
	hos.theObject = srcData.mGroup;
	if (!HostCall(hostObjectSynchronize, sizeof(hos), &hos)) return false;
	
// use hostModelGINF to read vertex count
	HostModelGINFRec tginf;
	tginf.theReference = theInfo->theHostReference;
	tginf.theGroup = hos.theObject;
	tginf.theGINF = 0;	
	if (!HostCall(hostModelGINF, sizeof(tginf), &tginf)) return false;
	if (!tginf.theGINF || tginf.theGINF->totalCoordinates <= 100) return false;
	
// extract vertex 100 
	HostModelGetVertexRec tver;
	tver.theReference = theInfo->theHostReference;
	tver.theGroup = hos.theObject;
	tver.theVertexIndex = 100;
	if (!HostCall(hostModelGetVertex, sizeof(tver), &tver)) return false;
	
	dst = tver.thePosition;
		
// NOTE: Write(ACHF) requires frame - 1	
	return dstData.Write(UNIV_POS, channelCoordinate, frame - 1, &dst);
}

Boolean IgnoreExtra( const PluginProjectInfoRec * inf )
{
	long i, act;
	for (i = 0; i < inf->theActionNum; ++i) {
	 act = inf->theAction[i].theAction;
	 if (act < tsa_ObjectCreate) return false;
	 if (act == tsa_PluginRequest) return false;
	}
	return true;
}

Boolean Synchronize( long theFrame, double theTime, long theType )
{ 
	HostSynchronizeRec sync;
	sync.theReference = theInfo->theHostReference;
	sync.theTimingType = theType;
	
// Note: need frame - 1 here 	
	sync.theTimingFrame = theFrame - 1; 
	sync.theTimingTime = theTime;
	return HostCall(hostSynchronize, sizeof(sync), &sync);
}	

Boolean SynchronizeFrame( long theFrame )
{ 
	return Synchronize(theFrame, 0, SYNC_FRAME);
}

Boolean TestDelay( const PluginProjectInfoRec * inf )
{
// sorry, this thing is not for drag mode
 	if (inf->theProjectInfoMode == projectInfoMode_Track) return false;
	
	CData srcData, dstData;
	if (!srcData.Init(PS("Src"))) return false;
	if (!dstData.Init(PS("Dst"))) return false;
	
	DCoordinate pos;
	long frame = inf->theFrameIndex;

	if (!SynchronizeFrame(frame - 5)) return false;
	Boolean Ok = srcData.ReadPos(pos);

// synchronize back to actual frame, NOT to current frame (SYNC_CURRENT)
// (it's important for projectInfoMode_Animation)	
	if (!SynchronizeFrame(frame)) Ok = false;

	return Ok && dstData.WritePos(frame, pos);
}

} // namespace	


//--------- Public ---------------

void Initialize( void )
{
	theObject.mCount = 0;
	GetControlData();
}

void ProjectInfo( void )
{
	PluginProjectInfoRec * inf = (PluginProjectInfoRec *) theInfo;

// update objects
	theObject.AutoInit();
	theObject.Update();

// simple choice: just ignore secondary and further sessions	
	if (inf->theSessionID != 0) return;

// skip all Extra Actions except tsa_PluginRequest
	if (IgnoreExtra(inf)) return;
	
	CControlData * ctl = GetControlData();
	if (!ctl) return;

// take care about theTrackCache
	CTrackControl trackControl(&theTrackCache, inf->theProjectInfoMode == projectInfoMode_Track);
	
	switch (ctl->mMode) {
	 case TEST_MODE_0: 
	  Test100_2(inf->theFrameIndex);
	  break;

	 case TEST_MODE_1: 
	  TestVer(inf);
	  break;

	 case TEST_MODE_2: 
	  TestDelay(inf);
	  break;
	} 
}