/*
Copyright 1993-1998 Electric Image, Inc. This source code may only be used by
registered owners of the ElectricImage Plug-In Kit. Owners of the Kit may
include the source code in their products (freeware, shareware, or commercial).
This copyright notice must be included in its original form in all such
products. This source code may not be offered for sale or distributed by any
entity without the written permission of Electric Image, Inc.
*/

#ifndef __PluginProjectInfo_h__
#define __PluginProjectInfo_h__

#ifndef __PluginClient_h__
#include "PluginClient.h"
#endif

#define PROJECTINFO_VERSION		0x00010000			   	/* The actual version of PluginProjectInfoRec */

/* Host's Primarily Actions */

#define	tsa_Hierarchy				1					/* The Hierarchy dependency		*/
#define	tsa_IKInverse				2					/* The IK Handler				*/
#define	tsa_IKInverseEnd			3					/* The IK Handler End			*/
#define	tsa_Master					4					/* The Master Light, Material 	*/
#define	tsa_Deform					5					/* not used 					*/
#define	tsa_Skin					6					/* The Skinizer					*/
#define	tsa_Constraint				10000				/* The Constrain				*/
#define tsa_Expression				20000				/* not used 					*/

/* Host's Extra Actions */

#define tsa_ObjectCreate			20001				/* The object is created 		 	   */
#define tsa_ObjectDelete			20002				/* The object is removed		 	   */
#define tsa_ObjectRename			20003				/* The object is renamed		 	   */
#define tsa_ObjectDuplicate			20004				/* The object is duplicated		 	   */
#define tsa_PluginRequest			20005				/* The model plug-in finished		   */
#define tsa_ObjectVisibility		20006				/* The visibility is changed		   */
#define tsa_ObjectAnimation			20007				/* The object's animation is changed   */

/* Modes */

#define projectInfoMode_Update		0					/* The project's update/synchronization */
#define projectInfoMode_Track		1					/* The user's tracking mode				*/
#define projectInfoMode_Animation	2					/* The animation/preview is processing	*/

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __MACINTOSH__
#pragma options align=mac68k
#endif

typedef struct {
	char * theParent;										/* The parent node 	*/
	char * theChild;										/* The child node 	*/
	long   theAction;										/* The action 		*/
} TSActionRec, *TSActionPtr;

typedef struct {
	char * theChannel;									/* The channel reference	*/
	char * theOwner;										/* The channel's owner	 	*/
} TSChannelRec, *TSChannelPtr;

typedef struct TSUserListRec {
	IFFType thePluginID;								/* The ID of the plug-in adds this element	*/
	long    thePluginDataSize;							/* The size of private plug-in's data	 	*/
	struct  TSUserListRec * theNext;					/* The next data element 					*/
	char 	thePluginData[4];							/* The private plug-in's data starts here	*/
} TSUserListRec, *TSUserListPtr;

typedef struct PluginProjectInfoRec {
	PluginInfoRec theInfo;								/* <==  Sends the plugin data, control and host data references */
	long 	theVersion;									/* <==  The version of this PluginProjectInfoRec 				*/

	long    theFrameIndex;								/* <==  The current frame index number 							*/
	double  theFrameTime;								/* <==  The time of the current frame 							*/
	double  theFrameTimeDelta;							/* <==  The time per frame (1/fps) 								*/

	long 	theSessionID;								/* <==  The actual session ID 					 				*/
	long	theProjectInfoMode;							/* <==  The mode (see "Modes" above)							*/
	
	long 	theObjectNum;								/* <==  The count of objects in theObject array					*/
	HostObjectInfoRec * theObject;						/* <==  The array of objects									*/

	long	theActionNum;								/* <==  The count of actions in theAction array					*/
	TSActionRec * theAction;							/* <==  The actions array 										*/

	long	theChannelNum;								/* <==  The count of channels in theChannel array				*/
	TSChannelRec * theChannel;							/* <==  The input channels array 								*/

	TSUserListRec * theUserList;						/* <==  The list of data added by plug-ins						*/
} PluginProjectInfoRec, *PluginProjectInfoPtr;

#ifdef __MACINTOSH__
#pragma options align=reset
#endif

#ifdef __cplusplus
}
#endif

#endif // __PluginProjectInfo_h__
