#ifndef		EI_CONTROL_H
#define		EI_CONTROL_H

#ifndef		EI_TYPES_H
#include	"EI_Types.h"
#endif

#ifdef		__cplusplus
extern "C" {
#endif

int EI_GetControlID(const EI_Control *iControl);
void EI_SetControlID(EI_Control *ioControl,
						int iNewID);

int EI_CountControls(const EI_Dialog *iDialog);
EI_Control *EI_FindControlByIndex(const EI_Dialog *iDialog,
									int iControlNumber);

EI_Control *EI_FindControlByID(const EI_Dialog *iDialog,
								int iControlID);

void EI_InvalControl(const EI_Control *iControl);

EI_Dialog *EI_GetControlDialog(const EI_Control *iControl);

EI_ControlType EI_GetControlType(const EI_Control *iControl);

void EI_GetControlBounds(const EI_Control *iControl,
							EI_Rect *oBounds);
void EI_SetControlBounds(EI_Control *ioControl,
							const EI_Rect *iBounds);

void EI_SetControlDrawImmediate(EI_Control *ioControl,
								int iDrawImmediateFlag);
int EI_GetControlDrawImmediate(const EI_Control *iControl);

void EI_SetControlVisible(EI_Control *iControl,
							int iVisible);
int EI_GetControlVisible(const EI_Control *iControl);

int EI_GetControlEnable(const EI_Control *iControl);
void EI_SetControlEnable(EI_Control *iControl,
							int iEnabled);

void EI_SetControlTitle(EI_Control *ioControl,
						const char *iTitleString);
int EI_GetControlTitleSize(const EI_Control *iControl);
void EI_GetControlTitle(const EI_Control *iControl,
						char *oText,
						int *ioNumChars);

int EI_GetControlValue(const EI_Control *iControl);
void EI_SetControlValue(EI_Control *ioControl,
						int iNewValue);

int EI_GetControlMax(const EI_Control *iControl);
void EI_SetControlMax(EI_Control *ioControl,
						int iNewMax);

void *EI_GetControlExtraData(const EI_Control *iControl);
void EI_SetControlExtraData(EI_Control *ioControl,
							void *iExtraData);

void EI_SetKeyControl(EI_Control *iControl);
EI_Control *EI_GetKeyControl(const EI_Dialog *iDialog);

int EI_GetControlImageID(const EI_Control *iControl);
void EI_SetControlImageID(EI_Control *ioControl,
							int iNewImageID);

void EI_GetPushButtonLayout(const EI_Control *iControl,
							int *oLabelPosition,
							int *oIconPosition);
void EI_SetPushButtonLayout(EI_Control *ioControl,
							int iLabelPosition,
							int iIconPosition);

void EI_SetEditTextString(EI_Control *ioControl,
							const char *iEditString);
int EI_GetEditTextStringSize(const EI_Control *iControl);
void EI_GetEditTextString(const EI_Control *iControl,
							char *oText,
							int *ioNumChars);
void EI_SetEditTextSelection(EI_Control *ioControl,
								int iSelectionStart,
								int iSelectionEnd);
void EI_GetEditTextSelection(const EI_Control *ioControl,
								int *oSelectionStart,
								int *oSelectionEnd);

void EI_SetEditTextInternational (EI_Control *ioControl,
									int iInternational);
void EI_SetEditTextFilter(EI_Control *ioControl,
							EI_EditFilterType iFilterType,
							EI_EditFilterFunction iFilterFunction);

void EI_GetColorButtonColor(const EI_Control *iControl,
							int *oRed,
							int *oGreen,
							int *oBlue,
							int *oAlpha);
void EI_SetColorButtonColor(EI_Control *ioControl,
							int iRed,
							int iGreen,
							int iBlue,
							int iAlpha);

int EI_TrackMouseDown(const EI_Control* iControl,
						int* oMouseX,
						int* oMouseY);

void EI_SetListString(EI_Control *ioControl,
						int iWhichItem,
						const char *iItemString);
int EI_GetListStringSize(const EI_Control *iControl,
							int iWhichItem);
void EI_GetListString(const EI_Control *iControl,
						int iWhichItem,
						char *oText,
						int *ioNumChars);

void EI_SetListItemStyle(EI_Control *ioControl,
							int iWhichItem,
							int iStyleFlags);
int EI_GetListItemStyle(const EI_Control *iControl,
						int iWhichItem);

int EI_GetListSelectedItem(const EI_Control *iControl);
void EI_SelectListItem(EI_Control *ioControl,
						int iWhichItem,
						int iSelect);
int EI_IsListItemSelected(const EI_Control *iControl,
							int iWhichItem);
int EI_CountListItems(const EI_Control *iControl);
void EI_InsertListItem(EI_Control *ioControl,
						int iBeforeWhichItem,
						const char *iItemString);
void EI_RemoveListItem(EI_Control *ioControl,
						int iWhichItem);
void EI_SetListFlags(EI_Control *ioControl,
						int iListFlags);
int EI_GetListFlags(const EI_Control *iControl);
void EI_SetListDoubleClickFunction(EI_Control *ioControl,
									EI_ListDoubleClickFunction iFunction);
void EI_SetListReorderFunction(EI_Control *ioControl,
								EI_ListReorderFunction iFunction);

int EI_GetNumTabPanels(const EI_Control *iControl);
int EI_GetCurrentTabPanel(const EI_Control *iControl);
void EI_SetCurrentTabPanel(EI_Control *ioControl,
							int iPanelNumber);

/*
	Callback functions:
*/

void EI_SetControlHitFunction(EI_Control *ioControl,
								EI_ControlHitFunction iFunction);

void EI_SetUserControlDraw(EI_Control *ioControl,
							EI_UserDrawFunction iFunction);
void EI_SetUserControlClick(EI_Control *ioControl,
							EI_UserClickFunction iFunction);
void EI_SetUserControlMouseMoved(EI_Control *ioControl,
									EI_UserMouseMovedFunction iFunction);

#ifdef		__cplusplus
}
#endif

#endif
