#include "MMX_Noise.h"
#include "Noise.h"

#ifdef __MMX__

#ifndef _int64
	typedef long long _int64;	// BMB XCODE
#endif

static short *noiseData16;
static double xyz; // dummy variable to align constants on 64-bit boundaries
static _int64 const_noiseMask;
static _int64 const_noisePhi123 =   0x0000D1E717C77ED1;
static _int64 const_mask1 =         0x000000000000FFFF;

bool MMX_InitNoise(MMX_NoiseMallocFunction noiseMalloc, unsigned long seed)	// BMB XCODE was ulong
{
	unsigned long local_noiseMask = noiseMask;	// BMB XCODE was ulong

	noiseData16 = (short*)noiseMalloc((noisePoints + 1) * sizeof(short));
	if (!noiseData16) 
		return false;

	unsigned long i = 0;	// BMB XCODE added unsigned long
	while (i <= noisePoints) {
		RANDOMSEED(seed);
		noiseData16[i++] = seed >> 17;						/* yields a number between 0 and 32767 */
	}
	
	_asm{
		movd mm0, local_noiseMask	/* mm0 = x, x, x, noiseMask */
		punpcklwd mm0, mm0			/* mm0 = x, x, noiseMask, noiseMask */
		punpcklwd mm0, mm0			/* mm0 = noiseMask, noiseMask, noiseMask, noiseMask */
		movq const_noiseMask, mm0	/* const_noiseMask = mm0 */
		emms
	}
}  /* MMX_InitNoise */

void MMX_FinishNoise(MMX_NoiseFreeFunction noiseFree)
{
	if (noiseData16) {
		noiseFree((void*)noiseData16);
		noiseData16 = NULL;
	}
}  /* MMX_FinishNoise */

//;***************************************************************************/
//;*
//;*                INTEL Corporation Proprietary Information  
//;*
//;*     This listing is supplied under the terms of a CNDA agreement
//;*     with INTEL Corporation and may not be disclosed except 
//;*     in accordance with the terms of that agreement.
//;*      
//;*                  Copyright (c) 1996 Intel Corporation.
//;*                         All rights reserved.
//;*
//;*
//;*     Revision: 0.0       Completed Bilal Qureshi on 3/19/97
//;***************************************************************************/


long MMX_Noise(long x, long y, long z)
{    

	 unsigned int local_noiseMask = noiseMask;
	 unsigned int mmx_result;
     static char  var[72 + 8];         /* to store program variables, 8 for padding */
	 char *lp_var = (int*)(((int)(var) + 7) &~ 7);
     short *lp_noiseData = noiseData16;

     /* lp_var is a 8-byte aligned pointer to an array which will have: */
     /* x2x1x0 at offset 0-byte */
     /* y2y1y0 at offset 8-byte */
     /* z2z1z0 at offset 16-byte */

     /* zayaxa at offset 24-bute */
     /* zbybxb at offset 32-byte */
     /* zcyzxc at offset 40-byte */
     /* cabaaa at offset 48-byte */
     /* cbbbab at offset 56-byte */
     /* ccbcac at offset 64-byte */
	 
	 _asm{
          mov  edi, lp_var           /* edi is the pointer to the 8-byte aligned var array */
          mov  ebx, lp_noiseData     /* ebx points to noiseData array address */


          movd mm0, x
          movd mm1, y
          movd mm2, z

          /* compute xi, yi, zi and xf, yf, zf */

          punpcklwd mm0, mm2
          movq mm3, mm0
          psrlq mm3, 32
          punpcklwd mm0, mm1
          psrlw mm0, 1               /* mm0:  15-bit unsigned fractional portions of x,y,z */
          psrlq mm1, 16
          punpcklwd mm3, mm1         /* mm3: 16-bit unsigned integer portions of x,y,z */

          /* calculate xsqr, ysqr, zsqr */

          movq mm4, mm0
          movq mm5, mm0
          pmulhw mm5, mm5
          pmullw mm4, mm4
          psrlw mm4, 15
          psllw mm5, 1
          paddsw mm5, mm4
          movq mm4, mm5               /* mm4: xsqr, ysqr, zsqr */

          /* calculate x0, y0, z0 */

          psrlw mm5, 1
          mov  eax, 040004000h
          movd mm7, eax
          punpckldq mm7, mm7
          paddusw mm5, mm7
          psubusw mm5, mm0          /* mm5:  x0, y0, z0 */

          /* calculate x1, y1, z1 */

          movq mm6, mm0
          paddusw mm6, mm7
          psubusw mm6, mm4          /* mm6:  x1, y1, z1 */

          /* calculate x2, y2, z2 */

          movq mm2, mm5
          paddusw mm2, mm6

          mov  eax, 080008000h
          movd mm7, eax
          punpckldq mm7, mm7
          psubusw mm7,mm2           /* mm7:  x2, y2, z2 */


          pcmpeqw  mm4, mm4         /* mm4 = ffff,ffff,ffff,ffff */
          psrlq    mm4, 16          /* mm4 or the mask = 0000,ffff,ffff,ffff */

          /* doing conversion from the z0y0x0 -- x2x1x0 and so on... */

          movq     mm2, mm5
          punpcklwd  mm5, mm7          /* mm5 = y2, y0, x2, x0 */
          punpckhwd  mm2, mm7          /* mm2 = x, x, z2, z0 */
          psllq      mm2, 32           /* mm2 = z2, z0, 0, 0 */

          movq       mm1, mm5          /* mm1 = y2, y0, x2, x0 */
          punpcklwd  mm5, mm6          /* mm5 = x, x2, x1, x0 */
          punpckhwd  mm2, mm6          /* mm2 = x, z2, z1, z0 */
          psllq      mm6, 16           /* mm6 = z1, y1, x1, 0 */

          punpckhwd  mm1, mm6          /* mm1 = x, y2, y1, y0 */
          pand       mm5, mm4          /* mm5 = 0, x2, x1, x0 */
          pand       mm2, mm4          /* mm2 = 0, z2, z1, z0 */
          pand       mm1, mm4          /* mm1 = 0, y2, y1, y0 */

          movq       [edi + 8], mm1 
          movq       [edi + 16], mm2		  
		  movq       [edi], mm5

          /* calculate xa,ya,za */

          pmullw mm3, const_noisePhi123
          movq   [edi + 24], mm3     /* store mm3 or za, ya, xa  */

          /* calculate xb, yb, zb */

          movq mm6, mm3
          paddw mm6,const_noisePhi123
          movq   [edi + 32], mm6     /* store mm6 or zb, yb, xb */

          /* calculate xc, yc, zc */

          movq mm0, mm6
          paddw mm0, const_noisePhi123
          movq  [edi + 40], mm0     /* store mm0 or zc, yc, xc */

          /* pack xa, xb, xc into mmx register */

          movq mm2, mm3
          punpcklwd mm2, mm0
          punpcklwd mm2, mm6

		  /* pack 3 copies of yc into mmx register */

          psrlq mm0, 16
		  punpcklwd mm0,mm0
		  punpckldq mm0, mm0 
	      
          /* compute ac, bc, cc */

          paddw mm0, mm2            /* mm0 = x, cc, bc, ac */
          movq  [edi + 64], mm0     /* write x, cc, bc, ac */
          
          /* pack 3 copies of ya into register */

          psrlq mm3, 16
	      punpcklwd mm3, mm3
	      punpckldq mm3, mm3 
 
          /* compute aa, ba, ca */

          paddw mm3, mm2            /* mm7 = x, ca, ba, aa */
          movq  [edi + 48], mm3     /* write x, ca, ba, aa */

          /* pack 3 copies of yb into register */

          psrlq mm6, 16
		  punpcklwd mm6, mm6
		  punpckldq mm6, mm6
		  
          /* compute ab, bb, cb */

          paddw mm6, mm2            /* mm5 = x, cb, bb, ab */
          movq  [edi + 56], mm6     /* write x, cb, bb, ab */

          /* initialization code before calculating p00-p22 values */

          movq  mm0, [edi + 24]     /* mm0 = x, za, ya, xa */
          punpckhwd mm0, mm0        /* mm0 = x, x, za, za */
          punpcklwd mm0, mm0        /* mm0 = za, za, za, za */

          movq   mm1, mm0           /* mm1 = za, za, za, za */
          movq   mm2, mm0           /* mm2 = za, za, za, za */

          paddw  mm0, [edi + 48]
          paddw  mm1, [edi + 56]
          paddw  mm2, [edi + 64]

          movq  mm7, [edi]          /* mm7 = 0, x2, x1, x0 */

          /* below instruction assumes the noiseMask is defined as a 32-bit value */

          movd  mm6, local_noiseMask/* mm6 = x, x, x, noiseMask */
          punpcklwd  mm6, mm6       /* mm6 = x, x, noiseMask, noiseMask */
          punpcklwd  mm6, mm6       /* mm6 = noiseMask, noiseMask, noiseMask, noiseMask */

          pand      mm6, mm4        /* mm6 = 0, noiseMask, noiseMask, noiseMask */

          pand mm0, mm6
          pand mm1, mm6
          pand mm2, mm6

          /* Let's do the p000-p220 to obtain mult_z0 value */
          /* This chunk to obtain the p000-p200 values */

          psllw mm0, 1              /* to get the word offset */
          movd  ecx, mm0
          psrlq mm0, 16
          and   ecx, 0ffffh
          movd  edx, mm0
          psrlq mm0, 16
          and   edx, 0ffffh
          movd  esi, mm0
          movd mm0, [ecx + ebx]          /* mm0 = 0, 0, x, p000 */
          movd mm3, [esi + ebx]          /* mm3 = 0, 0, x, p200 */
          punpcklwd mm0, mm3             /* mm0 = x, x, p200, p000 */
		  movd mm3, [edx + ebx]          /* mm3 = 0, 0, x, p100 */
		  punpcklwd mm0, mm3             /* mm0 = x, p200, p100, p000 */
          pand mm0, mm4                  /* mm0 = 0, p200, p100, p000 */

         /* This chunk to obtain the p010-p210 values */

          psllw mm1, 1                   /* to get the word offset */
          movd  ecx, mm1
          psrlq mm1, 16
          and   ecx, 0ffffh
          movd  edx, mm1
          psrlq mm1, 16
          and   edx, 0ffffh
          movd  esi, mm1
          movd mm1, [ecx + ebx]          /* mm0 = 0, 0, x, p010 */
          movd mm3, [esi + ebx]          /* mm3 = 0, 0, x, p210 */
          punpcklwd mm1, mm3             /* mm0 = x, x, p210, p010 */
		  movd mm3, [edx + ebx]          /* mm3 = 0, 0, x, p110 */
          punpcklwd mm1, mm3             /* mm0 = x, p210, p110, p010 */
          pand mm1, mm4                  /* mm0 = 0, p210, p110, p010 */

          /* This chunk to obtain the p020-p220 values */

          psllw mm2, 1                   /* to get the word offset */
          movd  ecx, mm2
          psrlq mm2, 16
          and   ecx, 0ffffh
          movd  edx, mm2
          psrlq mm2, 16
          and   edx, 0ffffh
          movd  esi, mm2
          movd mm2, [ecx + ebx]          /* mm2 = 0, 0, x, p020 */
          movd mm3, [esi + ebx]          /* mm3 = 0, 0, x, p220 */
          punpcklwd mm2, mm3             /* mm2 = x, x, p220, p020 */
     	  movd mm3, [edx + ebx]          /* mm3 = 0, 0, x, p120 */
          punpcklwd mm2, mm3             /* mm2 = x, p220, p120, p020 */
      	  pand mm2, mm4                  /* mm2 = 0, p220, p120, p020 */
          
          /* Calculate mult_z0 value */

          pmaddwd mm0, mm7               /* mm0 = x2 * p200, x1 * p100 + x0 * p000 */
          pmaddwd mm1, mm7               /* mm1 = x2 * p210, x1 * p110 + x0 * p010 */
          pmaddwd mm2, mm7               /* mm2 = x2 * p220, x1 * p120 + x0 * p020 */

          movq    mm3, mm0
          psrlq   mm3, 32
          paddd   mm0, mm3
          psrld   mm0, 15                /* mm0 = n00 (lower 15 bits) */

          movq    mm3, mm1
          psrlq   mm3, 32
          paddd   mm1, mm3
          psrld   mm1, 15                /* mm1 = n10 (lower 15 bits) */

          movq    mm3, mm2
          psrlq   mm3, 32
          paddd   mm2, mm3
          psrld   mm2, 15                /* mm2 = n20 (lower 15 bits) */

          pand    mm0, const_mask1
          pand    mm1, const_mask1
          pand    mm2, const_mask1

          movq    mm5, mm0
          psllq   mm1, 16
          psllq   mm2, 32
          paddd   mm5, mm1
          paddd   mm5, mm2               /* mm5 = 0, n200, n100, n000 */
          pmaddwd mm5, [edi + 8]
          movq    mm1, mm5
          psrlq   mm1, 32
          paddd   mm5, mm1
          psrld   mm5, 15
          pand    mm5, const_mask1      /* mm5 = mult_z0 */

          /* cannot use mm5 -- it has the mult_z0 value */
          /* Let's do the p001-p221 to obtain mult_z1 value */
          /* Let mm0, mm1 and mm2 to be zb, zb, zb, zb */

          /* initialization code before calculating p00-p22 values */

          movq  mm0, [edi + 32]        /* mm0 = x, zb, yb, xb */
          punpckhwd mm0, mm0           /* mm0 = x, x, zb, zb */
          punpcklwd mm0, mm0           /* mm0 = zb, zb, zb, zb */

          movq   mm1, mm0              /* mm1 = zb, zb, zb, zb */
          movq   mm2, mm0              /* mm2 = zb, zb, zb, zb */

          paddw  mm0, [edi + 48]
          paddw  mm1, [edi + 56]
          paddw  mm2, [edi + 64]

          pand mm0, mm6
          pand mm1, mm6
          pand mm2, mm6
 
          /* This chunk to obtain the p001-p201 values */

          psllw mm0, 1                 /* to get the word offset */
          movd  ecx, mm0
          psrlq mm0, 16
          and   ecx, 0ffffh
          movd  edx, mm0
          psrlq mm0, 16
          and   edx, 0ffffh
          movd  esi, mm0
          movd mm0, [ecx + ebx]        /* mm0 = 0, 0, x, p001 */
          movd mm3, [esi + ebx]        /* mm3 = 0, 0, x, p201 */
          punpcklwd mm0, mm3           /* mm0 = x, x, p201, p001 */
          movd mm3, [edx + ebx]        /* mm3 = 0, 0, x, p101 */
          punpcklwd mm0, mm3           /* mm0 = x, p201, p101, p001 */
          pand mm0, mm4                /* mm0 = 0, p201, p101, p001 */

          /* This chunk to obtain the p011-p211 values */

          psllw mm1, 1                 /* to get the word offset */
          movd  ecx, mm1
          psrlq mm1, 16
          and   ecx, 0ffffh
          movd  edx, mm1
          psrlq mm1, 16
          and   edx, 0ffffh
          movd  esi, mm1
          movd mm1, [ecx + ebx]        /* mm1 = 0, 0, x, p011 */
          movd mm3, [esi + ebx]        /* mm3 = 0, 0, x, p211 */
          punpcklwd mm1, mm3           /* mm1 = x, x, p211, p011 */
          movd mm3, [edx + ebx]        /* mm3 = 0, 0, x, p111 */
          punpcklwd mm1, mm3           /* mm1 = x, p211, p111, p011 */
          pand mm1, mm4                /* mm1 = 0, p211, p111, p011 */

          /* This chunk to obtain the p021-p221 values */

          psllw mm2, 1                 /* to get the word offset */
          movd  ecx, mm2
          psrlq mm2, 16
          and   ecx, 0ffffh
          movd  edx, mm2
          psrlq mm2, 16
          and   edx, 0ffffh
          movd  esi, mm2
          movd mm2, [ecx + ebx]        /* mm2 = 0, 0, x, p021 */
          movd mm3, [esi + ebx]        /* mm3 = 0, 0, x, p221 */
          punpcklwd mm2, mm3           /* mm2 = x, x, p221, p021 */
          movd mm3, [edx + ebx]        /* mm3 = 0, 0, x, p121 */
          punpcklwd mm2, mm3           /* mm2 = x, p221, p121, p021 */
          pand mm2, mm4                /* mm2 = 0, p221, p121, p021 */

          /* Calculate mult_z1 value */

          pmaddwd mm0, mm7             /* mm0 = x2 * p201, x1 * p101 + x0 * p001 */

          pmaddwd mm1, mm7             /* mm1 = x2 * p211, x1 * p111 + x0 * p011 */

          pmaddwd mm2, mm7             /* mm2 = x2 * p221, x1 * p121 + x0 * p021 */


          movq    mm3, mm0
          psrlq   mm3, 32
          paddd   mm0, mm3
          psrld   mm0, 15              /* mm0 = n011 ( lower 15 bits) */

          movq    mm3, mm1
          psrlq   mm3, 32
          paddd   mm1, mm3
          psrld   mm1, 15              /* mm1 = n111 ( lower 15 bits) */

          movq    mm3, mm2
          psrlq   mm3, 32
          paddd   mm2, mm3
          psrld   mm2, 15              /* mm2 = n211( lower 15 bits) */

          pand    mm0, const_mask1
          pand    mm1, const_mask1
          pand    mm2, const_mask1

          movq    mm3, mm0
          psllq   mm1, 16
          psllq   mm2, 32
          paddd   mm3, mm1
          paddd   mm3, mm2             /* mm3 = 0, n210, n110, n010 */
          pmaddwd mm3, [edi + 8]
          movq    mm1, mm3
          psrlq   mm1, 32
          paddd   mm3, mm1
          psrld   mm3, 15
          pand    mm3, const_mask1     /* mm3 = mult_z1 */

          /* cannot use mm5 -- it has the mult_z0 value */
          /* cannot use mm3 -- it has the mult_z1 value */
          /* Let's do the p002-p222 to obtain mult_z2 value */
          /* Let mm0, mm1 and mm2 to be zc, zc, zc, zc */

          movq  mm0, [edi + 40]        /* mm0 = x, zc, yc, xc */
          punpckhwd mm0, mm0           /* mm0 = x, x, zc, zc */
          punpcklwd mm0, mm0           /* mm0 = zc, zc, zc, zc */

          movq   mm1, mm0              /* mm1 = zc, zc, zc, zc */
          movq   mm2, mm0              /* mm2 = zc, zc, zc, zc */

          paddw  mm0, [edi + 48]
          paddw  mm1, [edi + 56]
          paddw  mm2, [edi + 64]

          pand mm0, mm6
          pand mm1, mm6
          pand mm2, mm6

          /* This chunk to obtain the p002-p202 values */

          psllw mm0, 1                 /* to get the word offset */
          movd  ecx, mm0
          psrlq mm0, 16
          and   ecx, 0ffffh
          movd  edx, mm0
          psrlq mm0, 16
          and   edx, 0ffffh
          movd  esi, mm0
          movd mm0, [ecx + ebx]        /* mm0 = 0, 0, x, p002 */
          movd mm6, [esi + ebx]        /* mm6 = 0, 0, x, p202 */
          punpcklwd mm0, mm6           /* mm0 = x, x, p202, p002 */
          movd mm6, [edx + ebx]        /* mm6 = 0, 0, x, p102 */
          punpcklwd mm0, mm6           /* mm0 = x, p202, p102, p002 */
          pand mm0, mm4                /* mm0 = 0, p202, p102, p002 */

          /* This chunk to obtain the p012-p212 values */

          psllw mm1, 1                 /* to get the word offset */
          movd  ecx, mm1
          psrlq mm1, 16
          and   ecx, 0ffffh
          movd  edx, mm1
          psrlq mm1, 16
          and   edx, 0ffffh
          movd  esi, mm1
          movd mm1, [ecx + ebx]    /* mm1 = 0, 0, x, p012 */
          movd mm6, [esi + ebx]    /* mm6 = 0, 0, x, p212 */
          punpcklwd mm1, mm6      /* mm1 = x, x, p212, p012 */
          movd mm6, [edx + ebx]    /* mm6 = 0, 0, x, p112 */
          punpcklwd mm1, mm6       /* mm1 = x, p212, p112, p012 */
          pand mm1, mm4            /* mm1 = 0, p212, p112, p012 */

          /* This chunk to obtain the p022-p222 values */

          psllw mm2, 1               /* to get the word offset */
          movd  ecx, mm2
          psrlq mm2, 16
          and   ecx, 0ffffh
          movd  edx, mm2
          psrlq mm2, 16
          and   edx, 0ffffh
          movd  esi, mm2
          movd mm2, [ecx + ebx]     /* mm2 = 0, 0, x, p022 */
          movd mm6, [esi + ebx]     /* mm6 = 0, 0, x, p222 */
          punpcklwd mm2,  mm6       /* mm2 = x, x, p222, p022 */
          movd mm6, [edx + ebx]            /* mm6 = 0, 0, x, p122 */
          punpcklwd mm2, mm6       /* mm2 = x, p222, p122, p022 */
          pand mm2, mm4            /* mm2 = 0, p222, p122, p022 */

          // Calculate mult_z2 value

          pmaddwd mm0, mm7          /* mm0 = x2 * p202, x1 * p102 + x0 * p002 */

          pmaddwd mm1, mm7          /* mm1 = x2 * p212, x1 * p112 + x0 * p012 */

          pmaddwd mm2, mm7          /* mm2 = x2 * p222, x1 * p122 + x0 * p022 */


          movq    mm6, mm0
          psrlq   mm6, 32
          paddd   mm0, mm6
          psrld   mm0, 15               /* mm0 = n022 (lower 15 bits) */

          movq    mm6, mm1
          psrlq   mm6, 32
          paddd   mm1, mm6
          psrld   mm1, 15               /* mm1 = n122 (lower 15 bits) */

          movq    mm6, mm2
          psrlq   mm6, 32
          paddd   mm2, mm6
          psrld   mm2, 15           /* mm2 = n222 (lower 15 bits) */

          pand    mm0, const_mask1
          pand    mm1, const_mask1
          pand    mm2, const_mask1

          movq    mm6, mm0
          psllq   mm1, 16
          psllq   mm2, 32
          paddd   mm6, mm1
          paddd   mm6, mm2          /* mm6 = 0, n222, n122, n022 */
          pmaddwd mm6, [edi + 8]
          movq    mm1, mm6
          psrlq   mm1, 32
          paddd   mm6, mm1
          psrld   mm6, 15
          pand    mm6, const_mask1  /* mm6 = mult_z2 */

          /* at this stage: */
          /* mm5 = mult_z0 */
          /* mm3 = mult_z1 */
          /* mm6 = mult_z2 */

          psllq   mm3, 16
          psllq   mm6, 32
          paddd   mm5, mm3
          paddd   mm5, mm6          /* mm5 = 0, mult_z2, mult_z1, mult_z0 */

          pmaddwd mm5, [edi + 16]   /* mm5 = z2 * mult_z2, z1 * mult_z1 + z0 * mult_z0 */
          movq    mm4, mm5
          psrlq   mm4, 32
          paddd   mm5, mm4
          psrlq   mm5, 15
          pand    mm5, const_mask1

          movd    mmx_result, mm5
          //emms
     } /* _asm */
     return mmx_result;
}  /* MMX_Noise */

//;***************************************************************************/
//;*
//;*                INTEL Corporation Proprietary Information  
//;*
//;*     This listing is supplied under the terms of a CNDA agreement
//;*     with INTEL Corporation and may not be disclosed except 
//;*     in accordance with the terms of that agreement.
//;*      
//;*                  Copyright (c) 1996 Intel Corporation.
//;*                         All rights reserved.
//;*
//;*
//;*     Revision: 0.0       Completed Bilal Qureshi on 3/19/97
//;***************************************************************************/

long MMX_Noise2(long x, long y, long z)
{
     unsigned int mmx_result;
	 short *lp_noiseData = noiseData16;
	 
	 _asm{

		 mov ebx, lp_noiseData // ebx points to noiseData array address

		 // compute the xi, yi, zi, xf, yf, zf values
	     
		 movd mm0, x
		 movd mm1, y
		 movd mm2, z

		 punpcklwd mm0, mm2
		 movq mm3, mm0
		 psrlq mm3, 32
		 punpcklwd mm0, mm1
		 psrlw mm0, 1         // mm0 = x, zf, yf, xf
		 psrlq mm1, 16
		 punpcklwd mm3, mm1   // mm3 = x, zi, yi, xi
		 
		 // compute za, ya, xa, zb, yb and xb values

		 pmullw mm3, const_noisePhi123 // mm3 = x, za, ya, xa
		 movq mm4, mm3
		 paddw mm4, const_noisePhi123  // mm4 = x, zb, yb, xb

		 // compute aa, ab, ba, bb values

		 movq mm5, mm3                 
		 punpcklwd mm5, mm4            // mm5 = yb, ya, x, x
	     punpckhdq mm5, mm5            // mm5 = yb, ya, yb, ya
		 movq mm6, mm3
		 punpcklwd mm6, mm4            // mm6 = x, x, xb, xa
		 punpcklwd mm6, mm6            // mm6 = xb, xb, xa, xa
		 punpckhwd mm3, mm4            // mm3 = x, x, zb, za
		 punpcklwd mm3, mm3            // mm3 = zb, zb, za, za
		 paddw mm6, mm5                // mm6 = bb, ba, ab, aa 
	     movq  mm5, mm6
		 punpckldq mm6, mm6            // mm6 = ab, aa, ab, aa
		 punpckhdq mm5, mm5            // mm5 = bb, ba, bb, ba

		 // compute offsets for noiseData
		 
		 paddw mm6, mm3      
		 paddw mm5, mm3
		 pand  mm6, const_noiseMask    // this is for p1-p4
		 pand  mm5, const_noiseMask    // this is for p5-p8

		 // compute p1-p4 values:
		 //p1 = noiseData[(aa + za) & noiseMask];
	     //p2 = noiseData[(ab + za) & noiseMask];
		 //p3 = noiseData[(aa + zb) & noiseMask];
		 //p4 = noiseData[(ab + zb) & noiseMask];
	     
		 psllw mm6, 1                  // to get the word offset
		 movd ecx, mm6
		 psrlq mm6, 16
		 and ecx, 0ffffh
		 movd edx, mm6
		 psrlq mm6, 16
		 and edx, 0ffffh
		 movd esi, mm6
		 psrlq mm6, 16
		 and esi, 0ffffh
		 movd eax, mm6
		 movd mm1, [ecx + ebx]         // mm1 = 0, 0, x, p1
		 movd mm2, [esi + ebx]         // mm2 = 0, 0, x, p3
		 punpcklwd mm1, mm2            // mm1 = x, x, p3, p1
		 movd mm3, [edx + ebx]         // mm3 = 0, 0, x, p2
		 movd mm4, [eax + ebx]         // mm4 = 0, 0, x, p4
	     punpcklwd mm3, mm4            // mm3 = x, x, p4, p2
		 punpcklwd mm1, mm3            // mm1 = p4, p3, p2, p1
		 
		 // compute p5-p8:
		 //p5 = noiseData[(ba + za) & noiseMask];
		 //p6 = noiseData[(bb + za) & noiseMask];
		 //p7 = noiseData[(ba + zb) & noiseMask];
		 //p8 = noiseData[(bb + zb) & noiseMask];
		      
		 psllw mm5, 1                  // to get the word offset
		 movd ecx, mm5
		 psrlq mm5, 16
		 and ecx, 0ffffh
		 movd edx, mm5
		 psrlq mm5, 16
		 and edx, 0ffffh
		 movd esi, mm5
		 psrlq mm5, 16
		 and esi, 0ffffh
		 movd eax, mm5
		 movd mm2, [ecx + ebx]         // mm2 = 0, 0, x, p5
		 movd mm3, [esi + ebx]         // mm3 = 0, 0, x, p7
		 punpcklwd mm2, mm3            // mm2 = x, x, p7, p5
		 movd mm4, [edx + ebx]         // mm4 = 0, 0, x, p6
		 movd mm5, [eax + ebx]         // mm5 = 0, 0, x, p8
	     punpcklwd mm4, mm5            // mm4 = x, x, p8, p6
		 punpcklwd mm2, mm4            // mm2 = p8, p7, p6, p5
		 
		 // compute the following:
		 //n01 = p1 + xf * (p5 - p1);
	     //n02 = p2 + xf * (p6 - p2);
	     //n11 = p3 + xf * (p7 - p3);
	     //n12 = p4 + xf * (p8 - p4);
	     
		 psubw  mm2, mm1               // mm2 = p8-p4, p7-p3, p6-p2, p5-p1
		 movq mm7, mm0
		 punpcklwd  mm0, mm0           // mm0 = yf, yf, xf, xf
		 movq mm6, mm0         
		 punpckhdq mm6, mm6            // mm6 = yf, yf, yf, yf
		 punpckldq mm0, mm0            // mm0 = xf, xf, xf, xf
		 punpckhwd mm7, mm7            // mm7 = x, x, zf, zf
		 pmulhw mm2, mm0
		 paddw  mm2, mm1               // mm2 = n12, n11, n02, n01

		 // do the following:
		 //na = n01 + yf * (n02 - n01);
		 //nb = n11 + yf * (n12 - n11);
	     
		 movq mm0, mm2
		 psrlq mm0, 16
		 psubw mm0, mm2
		 pmulhw mm0, mm6
		 paddw mm0, mm2                // mm0 = x, nb, x, na

		 // do the following: 
		 // na + zf * (nb - na);

		 movq  mm1, mm0
	     psrlq mm1, 32                 // mm1 = x, x, x, nb
		 psubw mm1, mm0  
		 pmulhw mm1, mm7
		 paddw mm1, mm0                // mm1 = x, x, x, na + zf * (nb - na)
	     
		 movd  eax, mm1
		 and  eax, 0FFFFh
		 mov  mmx_result, eax
		 //emms
	} /* _asm */
	return mmx_result;
}  /* MMX_Noise2 */

long MMX_DNoise2(long x, long y, long z, long *aNum, long *bNum, long *cNum)
{    
	 unsigned int local_noiseMask;
	 unsigned int mmx_result;
	 int tempa, tempb, tempc;
     static char  var[120 + 8];         /* to store program variables, 8 for padding */
     char *lp_var;
     short *lp_noiseData = noiseData16;

	 static double xyz; // to align constants on 64-bit boundaries
     static _int64 const_noisePhi123 =   0x0000D1E717C77ED1;
     static _int64 const_mask1 =         0x000000000000FFFF;
	 static _int64 const_mask2 =		 0x0000FFFFFFFFFFFF;
	 static _int64 const_mask3 =		 0x00000000FFFFFFFF;
	 
	 lp_var = (int*)(((int)(var) + 7) &~ 7);
     /* lp_var is a 8-byte aligned pointer to an array which will have: */
     /* x2x1x0 at offset 0-byte */
     /* y2y1y0 at offset 8-byte */
     /* z2z1z0 at offset 16-byte */

     /* zayaxa at offset 24-bute */
     /* zbybxb at offset 32-byte */
     /* zcyzxc at offset 40-byte */
     /* cabaaa at offset 48-byte */
     /* cbbbab at offset 56-byte */
     /* ccbcac at offset 64-byte */
     /* xfxd1xd0 at offset 72-byte */
	 /* yfyd1yd0 at offset 80-byte */
	 /* zfzd1zd0 at offset 88-byte */
	 /* p20p10p00 at offset 96-byte */
	 /* p21p11p01 at offset 104-byte */
	 /* p22p12p02 at offset 112-byte */

	 local_noiseMask = noiseMask;
	 
	 _asm{
          mov  edi, lp_var           /* edi is the pointer to the 8-byte aligned var array */
          mov  ebx, lp_noiseData     /* ebx points to noiseData array address */


          movd mm0, x
          movd mm1, y
          movd mm2, z

          /* compute xi, yi, zi and xf, yf, zf */

          punpcklwd mm0, mm2
          movq mm3, mm0
          psrlq mm3, 32
          punpcklwd mm0, mm1
          psrlw mm0, 1               /* mm0:  15-bit unsigned fractional portions of x,y,z */
        
		  psrlq mm1, 16
          punpcklwd mm3, mm1         /* mm3: 16-bit unsigned integer portions of x,y,z */

          /* calculate xsqr, ysqr, zsqr */

          movq mm4, mm0
          movq mm5, mm0
          pmulhw mm5, mm5
          pmullw mm4, mm4
          psrlw mm4, 15
          psllw mm5, 1
          paddsw mm5, mm4
          movq mm4, mm5               /* mm4: xsqr, ysqr, zsqr */

          /* calculate x0, y0, z0 */

          psrlw mm5, 1
          mov  eax, 040004000h
          movd mm7, eax
          punpckldq mm7, mm7
          paddusw mm5, mm7
          psubusw mm5, mm0          /* mm5:  x0, y0, z0 */

          /* calculate x1, y1, z1 */

          movq mm6, mm0
          paddusw mm6, mm7
          psubusw mm6, mm4          /* mm6:  x1, y1, z1 */

          /* calculate x2, y2, z2 */

          movq mm2, mm5
          paddusw mm2, mm6

          mov  eax, 080008000h
          movd mm7, eax
          punpckldq mm7, mm7		/* mm7: 4 word copies of 32768 */
		  movq mm4, mm7				/* mm4: 4 word copies of 32768 */
          psubusw mm7,mm2           /* mm7:  x2, y2, z2 */

          /* doing conversion from the z0y0x0 -- x2x1x0 and so on... */

          movq       mm2, mm5
          punpcklwd  mm5, mm7          /* mm5 = y2, y0, x2, x0 */
          punpckhwd  mm2, mm7          /* mm2 = x, x, z2, z0 */
          psllq      mm2, 32           /* mm2 = z2, z0, 0, 0 */

          movq       mm1, mm5          /* mm1 = y2, y0, x2, x0 */
          punpcklwd  mm5, mm6          /* mm5 = x, x2, x1, x0 */
          punpckhwd  mm2, mm6          /* mm2 = x, z2, z1, z0 */
          psllq      mm6, 16           /* mm6 = z1, y1, x1, 0 */

          punpckhwd  mm1, mm6          /* mm1 = x, y2, y1, y0 */
          pand       mm5, const_mask2  /* mm5 = 0, x2, x1, x0 */
          pand       mm2, const_mask2  /* mm2 = 0, z2, z1, z0 */
          pand       mm1, const_mask2  /* mm1 = 0, y2, y1, y0 */

          movq       [edi + 8], mm1 
          movq       [edi + 16], mm2		  
		  movq       [edi], mm5

		  /* calculate xd0, yd0, zd0 */
		  movq mm5, mm0
		  psubw mm5, mm4			/* mm5:  xd0, yd0, zd0 */

		  /* calculate xd1, yd1, zd1 */
		  psubw mm4, mm0
		  psubw	mm4, mm0			/* mm4:  xd1, yd1, zd1 */

		  /* convert xd0yd0zd0 -- xfxd1xd0 etc. */
		  movq       mm2, mm5
          punpcklwd  mm5, mm0          /* mm5 = yf, yd0, xf, xd0 */
          punpckhwd  mm2, mm0          /* mm2 = x, x, zf, zd0    */
          psllq      mm2, 32           /* mm2 = zf, zd0, 0, 0    */

          movq       mm1, mm5          /* mm1 = yf, yd0, xf, xd0 */
          punpcklwd  mm5, mm4          /* mm5 = x, xf, xd1, xd0  */
          punpckhwd  mm2, mm4          /* mm2 = x, zf, zd1, zd0  */
          psllq      mm4, 16           /* mm4 = zd1, yd1, xd1, 0 */

          punpckhwd  mm1, mm4          /* mm1 = x, yf, yd1, yd0 */
          pand       mm5, const_mask2  /* mm5 = 0, xf, xd1, xd0 */
          pand       mm2, const_mask2  /* mm2 = 0, zf, zd1, zd0 */
          pand       mm1, const_mask2  /* mm1 = 0, yf, yd1, yd0 */

          movq       [edi + 80], mm1 
          movq       [edi + 88], mm2		  
		  movq       [edi + 72], mm5

          /* calculate xa,ya,za */
          
		  pmullw mm3, const_noisePhi123
          movq   [edi + 24], mm3     /* store mm3 or za, ya, xa  */

          /* calculate xb, yb, zb */

          movq mm6, mm3
          paddw mm6,const_noisePhi123
          movq   [edi + 32], mm6     /* store mm6 or zb, yb, xb */

          /* calculate xc, yc, zc */

          movq mm0, mm6
          paddw mm0, const_noisePhi123
          movq  [edi + 40], mm0     /* store mm0 or zc, yc, xc */

          /* pack xa, xb, xc into mmx register */

          movq mm2, mm3
          punpcklwd mm2, mm0
          punpcklwd mm2, mm6

		  /* pack 3 copies of yc into mmx register */

          psrlq mm0, 16
		  punpcklwd mm0,mm0
		  punpckldq mm0, mm0 
	      
          /* compute ac, bc, cc */

          paddw mm0, mm2            /* mm0 = x, cc, bc, ac */
          movq  [edi + 64], mm0     /* write x, cc, bc, ac */
          
          /* pack 3 copies of ya into register */

          psrlq mm3, 16
	      punpcklwd mm3, mm3
	      punpckldq mm3, mm3 
 
          /* compute aa, ba, ca */

          paddw mm3, mm2            /* mm3 = x, ca, ba, aa */
          movq  [edi + 48], mm3     /* write x, ca, ba, aa */

          /* pack 3 copies of yb into register */

          psrlq mm6, 16
		  punpcklwd mm6, mm6
		  punpckldq mm6, mm6
		  
          /* compute ab, bb, cb */

          paddw mm6, mm2            /* mm6 = x, cb, bb, ab */
          movq  [edi + 56], mm6     /* write x, cb, bb, ab */

          /* initialization code before calculating p00-p22 values */

          movq  mm0, [edi + 24]     /* mm0 = x, za, ya, xa */
          punpckhwd mm0, mm0        /* mm0 = x, x, za, za */
          punpcklwd mm0, mm0        /* mm0 = za, za, za, za */

          movq   mm1, mm0           /* mm1 = za, za, za, za */
          movq   mm2, mm0           /* mm2 = za, za, za, za */

          paddw  mm0, [edi + 48]
          paddw  mm1, [edi + 56]
          paddw  mm2, [edi + 64]

          /* below instruction assumes the noiseMask is defined as a 32-bit value */

          movd  mm6, local_noiseMask/* mm6 = x, x, x, noiseMask */
          punpcklwd  mm6, mm6       /* mm6 = x, x, noiseMask, noiseMask */
          punpcklwd  mm6, mm6       /* mm6 = noiseMask, noiseMask, noiseMask, noiseMask */

          pand mm6, const_mask2     /* mm6 = 0, noiseMask, noiseMask, noiseMask */

          pand mm0, mm6
          pand mm1, mm6
          pand mm2, mm6

         /* Let's do the p000-p220 to obtain mult_z0 value */
         /* This chunk to obtain the p000-p200 values */

		  movd edx, mm0
		  movzx ecx, dx
		  shr edx, 16
		  psrlq mm0, 32
          movd  esi, mm0

	      movzx eax,WORD PTR [ecx + ebx]     /* mm0 = 0, 0, x, p000 */
		  movd mm0, eax
          movzx ecx,WORD PTR [esi + ebx]     /* mm3 = 0, 0, x, p200 */
		  movd mm3, ecx
          punpcklwd mm0, mm3                 /* mm0 = x, x, p200, p000 */
		  movzx eax,WORD PTR [edx + ebx]     /* mm3 = 0, 0, x, p100 */
		  movd mm3, eax
		  punpcklwd mm0, mm3                 /* mm0 = 0, p200, p100, p000 */
 
         /* This chunk to obtain the p010-p210 values */

		  movd edx, mm1
		  movzx ecx, dx
		  shr edx, 16
		  psrlq mm1, 32
          movd  esi, mm1

          movzx eax,WORD PTR [ecx+ ebx]      /* mm1 = 0, 0, x, p010 */
		  movd mm1, eax
          movzx ecx,WORD PTR [esi + ebx]     /* mm3 = 0, 0, x, p210 */
		  movd mm3, ecx
          punpcklwd mm1, mm3                 /* mm1 = x, x, p210, p010 */
		  movzx eax,WORD PTR [edx + ebx]     /* mm3 = 0, 0, x, p110 */
		  movd mm3, eax
          punpcklwd mm1, mm3                 /* mm1 = 0, p210, p110, p010 */

          /* This chunk to obtain the p020-p220 values */

		  movd edx, mm2
		  movzx ecx, dx
		  shr edx, 16
		  psrlq mm2, 32
          movd  esi, mm2

          movzx eax,WORD PTR [ecx + ebx]      /* mm2 = 0, 0, x, p020 */
		  movd mm2, eax
          movzx ecx,WORD PTR [esi + ebx]      /* mm3 = 0, 0, x, p220 */
		  movd mm3, ecx
          punpcklwd mm2, mm3                  /* mm2 = x, x, p220, p020 */
     	  movzx eax,WORD PTR [edx + ebx]      /* mm3 = 0, 0, x, p120 */
		  movd mm3, eax
          punpcklwd mm2, mm3                  /* mm2 = 0, p220, p120, p020 */
          
		  movq [edi+96], mm0
		  movq [edi+104], mm1
		  movq [edi+112], mm2

          /* Calculate mult_z0 value */

          pmaddwd mm0, [edi]              /* mm0 = x2 * p200, x1 * p100 + x0 * p000 */
          pmaddwd mm1, [edi]              /* mm1 = x2 * p210, x1 * p110 + x0 * p010 */
          pmaddwd mm2, [edi]              /* mm2 = x2 * p220, x1 * p120 + x0 * p020 */

          movq    mm3, mm0
          psrlq   mm3, 32
          paddd   mm0, mm3
          psrld   mm0, 15                /* mm0 = n00 (lower 15 bits) */

          movq    mm3, mm1
          psrlq   mm3, 32
          paddd   mm1, mm3
          psrld   mm1, 15                /* mm1 = n10 (lower 15 bits) */

          movq    mm3, mm2
          psrlq   mm3, 32
          paddd   mm2, mm3
          psrld   mm2, 15                /* mm2 = n20 (lower 15 bits) */

          pand    mm0, const_mask1
          pand    mm1, const_mask1
          pand    mm2, const_mask1

          movq    mm5, mm0
          psllq   mm1, 16
          psllq   mm2, 32
          paddd   mm5, mm1
          paddd   mm5, mm2               /* mm5 = 0, n200, n100, n000 */
		  movq    mm4, mm5               /* mm4 = 0, n200, n100, n000 */
          pmaddwd mm5, [edi + 8]
          movq    mm1, mm5
          psrlq   mm1, 32
          paddd   mm5, mm1
          psrld   mm5, 15
          pand    mm5, const_mask1      /* mm5 = mult_z0 */

		  /* calculate sumd2_0 */
		  pmaddwd mm4, [edi + 80]
		  movq mm1, mm4
		  psrlq mm1, 32
		  paddd mm4, mm1
		  psrad mm4, 15
		  pand mm4, const_mask1			/* mm4 = sumd2_0 */

		  /* calculate sumd1_0 */
		  movq mm0, [edi+96]
		  movq mm1, [edi+104]
		  movq mm2, [edi+112]
          
		  pmaddwd mm0, [edi+72]          /* mm0 = xf * p200, xd1 * p100 + xd0 * p000 */
          pmaddwd mm1, [edi+72]          /* mm1 = xf * p210, xd1 * p110 + xd0 * p010 */
          pmaddwd mm2, [edi+72]          /* mm2 = xf * p220, xd1 * p120 + xd0 * p020 */

          movq    mm3, mm0
          psrlq   mm3, 32
          paddd   mm0, mm3
          psrad   mm0, 15                /* mm0 = d00 (lower 15 bits) */

          movq    mm3, mm1
          psrlq   mm3, 32
          paddd   mm1, mm3
          psrad   mm1, 15                /* mm1 = d10 (lower 15 bits) */

          movq    mm3, mm2
          psrlq   mm3, 32
          paddd   mm2, mm3
          psrad   mm2, 15                /* mm2 = d20 (lower 15 bits) */

          pand    mm0, const_mask1
          pand    mm1, const_mask1
          pand    mm2, const_mask1

          movq    mm7, mm0
          psllq   mm1, 16
          psllq   mm2, 32
          paddd   mm7, mm1
          paddd   mm7, mm2               /* mm7 = 0, d200, d100, d000 */

		  pmaddwd mm7, [edi + 8]
		  movq mm1, mm7
		  psrlq mm1, 32
		  paddd mm7, mm1
		  psrad mm7, 15
		  pand mm7, const_mask1			/* mm7 = sumd1_0 */	

         /* cannot use mm5, mm4, mm7 -- they contain mult_z0, sumd1_0,
		    and sumd2_0 */
         /* Let's find p001-p221 to obtain mult_z1, sumd1_1, sumd2_1
		    values */
         /* Let mm0, mm1 and mm2 to be zb, zb, zb, zb */

         /* initialization code before calculating p00-p22 values */

          movq  mm0, [edi + 32]        /* mm0 = x, zb, yb, xb */
          punpckhwd mm0, mm0           /* mm0 = x, x, zb, zb */
          punpcklwd mm0, mm0           /* mm0 = zb, zb, zb, zb */

          movq   mm1, mm0              /* mm1 = zb, zb, zb, zb */
          movq   mm2, mm0              /* mm2 = zb, zb, zb, zb */

          paddw  mm0, [edi + 48]
          paddw  mm1, [edi + 56]
          paddw  mm2, [edi + 64]

          pand mm0, mm6
          pand mm1, mm6
          pand mm2, mm6
 
          /* This chunk to obtain the p001-p201 values */

		  movd edx, mm0
		  movzx ecx, dx
		  shr edx, 16
		  psrlq mm0, 32
          movd  esi, mm0

          movzx eax,WORD PTR [ecx + ebx]   /* mm0 = 0, 0, x, p001 */
		  movd mm0, eax
          movzx ecx,WORD PTR [esi + ebx]   /* mm3 = 0, 0, x, p201 */
		  movd mm3, ecx
          punpcklwd mm0, mm3               /* mm0 = x, x, p201, p001 */
          movzx eax,WORD PTR [edx + ebx]   /* mm3 = 0, 0, x, p101 */
		  movd mm3, eax
          punpcklwd mm0, mm3               /* mm0 = x, p201, p101, p001 */

          /* This chunk to obtain the p011-p211 values */

		  movd edx, mm1
		  movzx ecx, dx
		  shr edx, 16
		  psrlq mm1, 32
          movd  esi, mm1

          movzx eax,WORD PTR [ecx + ebx]     /* mm1 = 0, 0, x, p011 */
		  movd mm1, eax
          movzx ecx,WORD PTR [esi + ebx]     /* mm3 = 0, 0, x, p211 */
		  movd mm3, ecx
          punpcklwd mm1, mm3                 /* mm1 = x, x, p211, p011 */
          movzx eax,WORD PTR [edx + ebx]     /* mm3 = 0, 0, x, p111 */
		  movd mm3, eax
          punpcklwd mm1, mm3                 /* mm1 = x, p211, p111, p011 */

          /* This chunk to obtain the p021-p221 values */

		  movd edx, mm2
		  movzx ecx, dx
		  shr edx, 16
		  psrlq mm2, 32
          movd  esi, mm2

          movzx eax,WORD PTR [ecx + ebx]     /* mm2 = 0, 0, x, p021 */
		  movd mm2, eax
          movzx ecx,WORD PTR [esi + ebx]     /* mm3 = 0, 0, x, p221 */
		  movd mm3, ecx
          punpcklwd mm2, mm3                 /* mm2 = x, x, p221, p021 */
          movzx eax,WORD PTR [edx + ebx]     /* mm3 = 0, 0, x, p121 */
		  movd mm3, eax
          punpcklwd mm2, mm3                 /* mm2 = x, p221, p121, p021 */

		  movq [edi+96], mm0
		  movq [edi+104], mm1
		  movq [edi+112], mm2

          /* Calculate mult_z1 value */

          pmaddwd mm0, [edi]           /* mm0 = x2 * p201, x1 * p101 + x0 * p001 */

          pmaddwd mm1, [edi]            /* mm1 = x2 * p211, x1 * p111 + x0 * p011 */

          pmaddwd mm2, [edi]            /* mm2 = x2 * p221, x1 * p121 + x0 * p021 */

          movq    mm3, mm0
          psrlq   mm3, 32
          paddd   mm0, mm3
          psrld   mm0, 15              /* mm0 = n011 ( lower 15 bits) */

          movq    mm3, mm1
          psrlq   mm3, 32
          paddd   mm1, mm3
          psrld   mm1, 15              /* mm1 = n111 ( lower 15 bits) */

          movq    mm3, mm2
          psrlq   mm3, 32
          paddd   mm2, mm3
          psrld   mm2, 15              /* mm2 = n211( lower 15 bits) */

          pand    mm0, const_mask1
          pand    mm1, const_mask1
          pand    mm2, const_mask1

          movq    mm3, mm0
          psllq   mm1, 16
          psllq   mm2, 32
          paddd   mm3, mm1
          paddd   mm3, mm2             /* mm3 = 0, n210, n110, n010 */
		  movq    mm2, mm3			   /* mm2 = 0, n210, n110, n010 */
          pmaddwd mm3, [edi + 8]
          movq    mm1, mm3
          psrlq   mm1, 32
          paddd   mm3, mm1
          psrld   mm3, 15
          pand    mm3, const_mask1     /* mm3 = mult_z1 */
		  psllq   mm3, 16
		  por	  mm5, mm3			  /* mm5 = 0, 0, mult_z1, mult_z0 */
		  
		  /* calculate sumd2_1 */
		  pmaddwd mm2, [edi + 80]
		  movq mm1, mm2
		  psrlq mm1, 32
		  paddd mm2, mm1
		  psrad mm2, 15
		  pand mm2, const_mask1		  /* mm2 = sumd2_1 */
		  psllq mm2, 16
		  por   mm4, mm2			  /* mm4 = 0, 0, sumd2_1, sumd2_0 */

		  /* calculate sumd1_1 */
		  movq mm0, [edi+96]
		  movq mm1, [edi+104]
		  movq mm2, [edi+112]
          
		  pmaddwd mm0, [edi+72]          /* mm0 = xf * p200, xd1 * p100 + xd0 * p000 */
          pmaddwd mm1, [edi+72]          /* mm1 = xf * p210, xd1 * p110 + xd0 * p010 */
          pmaddwd mm2, [edi+72]          /* mm2 = xf * p220, xd1 * p120 + xd0 * p020 */

          movq    mm3, mm0
          psrlq   mm3, 32
          paddd   mm0, mm3
          psrad   mm0, 15                /* mm0 = d00 (lower 15 bits) */

          movq    mm3, mm1
          psrlq   mm3, 32
          paddd   mm1, mm3
          psrad   mm1, 15                /* mm1 = d10 (lower 15 bits) */

          movq    mm3, mm2
          psrlq   mm3, 32
          paddd   mm2, mm3
          psrad   mm2, 15                /* mm2 = d20 (lower 15 bits) */

          pand    mm0, const_mask1
          pand    mm1, const_mask1
          pand    mm2, const_mask1

          movq    mm3, mm0
          psllq   mm1, 16
          psllq   mm2, 32
          paddd   mm3, mm1
          paddd   mm3, mm2               /* mm3 = 0, d200, d100, d000 */

		  pmaddwd mm3, [edi + 8]
		  movq mm1, mm3
		  psrlq mm1, 32
		  paddd mm3, mm1
		  psrad mm3, 15
		  pand mm3, const_mask1			/* mm3 = sumd1_1 */	
		  psllq mm3, 16
		  por mm7, mm3					/* mm7 = 0, 0, sumd1_1, sumd1_0 */

          /* cannot use mm5, mm4, mm7 -- they contain mult_z0, sumd1_0,
		      and sumd2_0 */
          /* Let's find p002-p222 to obtain mult_z2, sumd1_2, sumd2_2
		      values */
          /* Let mm0, mm1 and mm2 to be zc, zc, zc, zc */

          movq  mm0, [edi + 40]        /* mm0 = x, zc, yc, xc */
          punpckhwd mm0, mm0           /* mm0 = x, x, zc, zc */
          punpcklwd mm0, mm0           /* mm0 = zc, zc, zc, zc */

          movq   mm1, mm0              /* mm1 = zc, zc, zc, zc */
          movq   mm2, mm0              /* mm2 = zc, zc, zc, zc */

          paddw  mm0, [edi + 48]
          paddw  mm1, [edi + 56]
          paddw  mm2, [edi + 64]

          pand mm0, mm6
          pand mm1, mm6
          pand mm2, mm6

          /* This chunk to obtain the p002-p202 values */
          
		  movd edx, mm0
		  movzx ecx, dx
		  shr edx, 16
		  psrlq mm0, 32
          movd  esi, mm0

          movzx eax,WORD PTR [ecx + ebx]     /* mm0 = 0, 0, x, p002 */
		  movd mm0, eax
          movzx ecx,WORD PTR [esi + ebx]     /* mm6 = 0, 0, x, p202 */
		  movd mm6, ecx
          punpcklwd mm0, mm6                 /* mm0 = x, x, p202, p002 */
          movzx eax,WORD PTR [edx + ebx]     /* mm6 = 0, 0, x, p102 */
		  movd mm6, eax
          punpcklwd mm0, mm6                 /* mm0 = x, p202, p102, p002 */

         /* This chunk to obtain the p012-p212 values */

          movd edx, mm1
		  movzx ecx, dx
		  shr edx, 16
		  psrlq mm1, 32
          movd  esi, mm1

          movzx eax,WORD PTR [ecx + ebx]    /* mm1 = 0, 0, x, p012 */
		  movd mm1, eax
          movzx ecx,WORD PTR [esi + ebx]    /* mm6 = 0, 0, x, p212 */
		  movd mm6, ecx
          punpcklwd mm1, mm6                /* mm1 = x, x, p212, p012 */
          movzx eax,WORD PTR [edx + ebx]    /* mm6 = 0, 0, x, p112 */
		  movd mm6, eax
          punpcklwd mm1, mm6                /* mm1 = x, p212, p112, p012 */

          /* This chunk to obtain the p022-p222 values */

		  movd edx, mm2
		  movzx ecx, dx
		  shr edx, 16
		  psrlq mm2, 32
          movd  esi, mm2

          movzx eax,WORD PTR [ecx + ebx]     /* mm2 = 0, 0, x, p022 */
		  movd mm2, eax
          movzx ecx,WORD PTR [esi + ebx]     /* mm6 = 0, 0, x, p222 */
		  movd mm6, ecx
          punpcklwd mm2,  mm6                /* mm2 = x, x, p222, p022 */
          movzx eax,WORD PTR [edx + ebx]     /* mm6 = 0, 0, x, p122 */
		  movd mm6, eax
          punpcklwd mm2, mm6                 /* mm2 = x, p222, p122, p022 */
		  
		  movq [edi+96], mm0
		  movq [edi+104], mm1
		  movq [edi+112], mm2

          // Calculate mult_z2 value

          pmaddwd mm0, [edi]         /* mm0 = x2 * p202, x1 * p102 + x0 * p002 */

          pmaddwd mm1, [edi]         /* mm1 = x2 * p212, x1 * p112 + x0 * p012 */

          pmaddwd mm2, [edi]         /* mm2 = x2 * p222, x1 * p122 + x0 * p022 */


          movq    mm6, mm0
          psrlq   mm6, 32
          paddd   mm0, mm6
          psrld   mm0, 15               /* mm0 = n022 (lower 15 bits) */

          movq    mm6, mm1
          psrlq   mm6, 32
          paddd   mm1, mm6
          psrld   mm1, 15               /* mm1 = n122 (lower 15 bits) */

          movq    mm6, mm2
          psrlq   mm6, 32
          paddd   mm2, mm6
          psrld   mm2, 15               /* mm2 = n222 (lower 15 bits) */

          pand    mm0, const_mask1
          pand    mm1, const_mask1
          pand    mm2, const_mask1

          movq    mm6, mm0
          psllq   mm1, 16
          psllq   mm2, 32
          paddd   mm6, mm1
          paddd   mm6, mm2          /* mm6 = 0, n222, n122, n022 */
		  movq	  mm3, mm6			/* mm3 = 0, n222, n122, n022 */
          pmaddwd mm6, [edi + 8]
          movq    mm1, mm6
          psrlq   mm1, 32
          paddd   mm6, mm1
          psrld   mm6, 15
          pand    mm6, const_mask1  /* mm6 = mult_z2 */
		  psllq   mm6, 32
		  por	  mm5, mm6	/* mm5 = 0, mult_z2, mult_z1, mult_z0 */

		  /* calculate sumd2_2 */
		  pmaddwd mm3, [edi + 80]
		  movq mm1, mm3
		  psrlq mm1, 32
		  paddd mm3, mm1
		  psrad mm3, 15
		  pand mm3, const_mask1		  /* mm3 = sumd2_2 */
		  psllq mm3, 32
		  por   mm4, mm3	/* mm4 = 0, sumd2_2, sumd2_1, sumd2_0 */

		  /* calculate sumd1_2 */
		  movq mm0, [edi+96]
		  movq mm1, [edi+104]
		  movq mm2, [edi+112]

          pmaddwd mm0, [edi+72]          /* mm0 = xf * p200, xd1 * p100 + xd0 * p000 */
          pmaddwd mm1, [edi+72]          /* mm1 = xf * p210, xd1 * p110 + xd0 * p010 */
          pmaddwd mm2, [edi+72]          /* mm2 = xf * p220, xd1 * p120 + xd0 * p020 */

          movq    mm3, mm0
          psrlq   mm3, 32
          paddd   mm0, mm3
          psrad   mm0, 15                /* mm0 = d00 (lower 15 bits) */

          movq    mm3, mm1
          psrlq   mm3, 32
          paddd   mm1, mm3
          psrad   mm1, 15                /* mm1 = d10 (lower 15 bits) */

          movq    mm3, mm2
          psrlq   mm3, 32
          paddd   mm2, mm3
          psrad   mm2, 15                /* mm2 = d20 (lower 15 bits) */

          pand    mm0, const_mask1
          pand    mm1, const_mask1
          pand    mm2, const_mask1

          movq    mm3, mm0
          psllq   mm1, 16
          psllq   mm2, 32
          paddd   mm3, mm1
          paddd   mm3, mm2               /* mm3 = 0, d200, d100, d000 */

		  pmaddwd mm3, [edi + 8]
		  movq mm1, mm3
		  psrlq mm1, 32
		  paddd mm3, mm1
		  psrad mm3, 15
		  pand mm3, const_mask1			/* mm3 = sumd1_1 */	
		  psllq mm3, 32
		  por mm7, mm3	 /* mm7 = 0, sumd1_2, sumd1_1, sumd1_0 */


          /* at this stage: */
          /* mm5 = 0, mult_z2, mult_z1, mult_z0 */
          /* mm7 = 0, sumd1_2, sumd1_1, sumd1_0 */
		  /* mm4 = 0, sumd2_2, sumd2_1, sumd2_0 */
		
		  /* calculate return value */
	
		  movq	  mm6, mm5			/* mm6 = 0, mult_z2, mult_z1, mult_z0 */
          pmaddwd mm5, [edi + 16]   /* mm5 = z2 * mult_z2, z1 * mult_z1 + z0 * mult_z0 */
          movq    mm3, mm5
          psrlq   mm3, 32
          paddd   mm5, mm3
          psrlq   mm5, 15
          pand    mm5, const_mask1

          movd    mmx_result, mm5

		  /* calculate aNum value */

		  pmaddwd mm7, [edi + 16]   /* mm7 = z2 * sumd1_2, z1 * sumd1_1 + z0 * sumd1_0 */
          movq    mm2, mm7
          psrlq   mm2, 32
          paddd   mm7, mm2
          psrad   mm7, 15
          pand    mm7, const_mask3  /* use 32-bit mask to preserve sign for movd*/

          movd    tempa, mm7

		  /* calculate bNum value */
		  pmaddwd mm4, [edi + 16]   /* mm4 = z2 * sumd2_2, z1 * sumd2_1 + z0 * sumd2_0 */
          movq    mm2, mm4
          psrlq   mm2, 32
          paddd   mm4, mm2
          psrad   mm4, 15
          pand    mm4, const_mask3  /* use 32-bit mask to preserve sign for movd*/

          movd    tempb, mm4

		  /* calculate cNum value */
		  pmaddwd mm6, [edi + 88]   /* mm6 = zf * mult_z2, zd1 * mult_z1 + zd0 * mult_z0 */
          movq    mm3, mm6
          psrlq   mm3, 32
          paddd   mm6, mm3
          psrad   mm6, 15
          pand    mm6, const_mask3 /* use 32-bit mask to preserve sign for movd*/

          movd    tempc, mm6

	      emms

     }     /* _asm */

	 *aNum = tempa; 
	 *bNum = tempb;
	 *cNum = tempc;

     return mmx_result;
}  /* MMX_DNoise2 */

#endif

