/*
Copyright 1993-1998 Electric Image, Inc. This source code may only be used by
registered owners of the ElectricImage Plug-In Kit. Owners of the Kit may
include the source code in their products (freeware, shareware, or commercial).
This copyright notice must be included in its original form in all such
products. This source code may not be offered for sale or distributed by any
entity without the written permission of Electric Image, Inc.
*/

#include <stdarg.h>
#include "PString.h"
#include <stdio.h>
#include <time.h>

#define QUEUESIZE			20

/* PString global variables */

unsigned char aEmptyPStr[1] = {'\0'};

/*
 *		Global Functions
 */

unsigned long CStrLength(const char *theString)
{
	unsigned long count;
	
	for (count = 0; theString[count] != '\0'; count++) {}
	return count;
}  /* CStrLength */

long CStrIn(unsigned char ch, const char *theString)
{
	int found = 0;
	long i = 0;
	
	while (theString[i] && !found)
		if (theString[i] == ch)
			return i;
		else
			i++;
	return -1;
}  /* CStrIn */

void CStrCpy(char *dst, const char *src)
{
	char theChar;
	unsigned long i = 0;
	
	do {
		theChar = src[i];
		dst[i++] = theChar;
	} while (theChar);
}  /* CStrCpy */

PStrPtr PStrNext(void)
{
	static PStr255 pStrQueue[QUEUESIZE];
	static long pStrQueueIndex = 0;

	if (pStrQueueIndex == QUEUESIZE)
		pStrQueueIndex = 0;
	return pStrQueue[pStrQueueIndex++];
}  /* PStrNext */

PStrPtr PStrC2P(CStrPtr cStr)
{
	unsigned long length = CStrLength(cStr);
	unsigned long i;
	
	if (length > 255)
		length = 255;
	i = length;
	while (i) {
		cStr[i] = cStr[i - 1];
		i--;
	}
	PStrSetLength(cStr, length);
	return (PStrPtr)cStr;
}  /* PStrC2P */

CStrPtr PStrP2C(PStrPtr pStr)
{
	unsigned long length = PStrLength(pStr);
	unsigned long i = 1;
	
	while (i <= length) {
		pStr[i - 1] = pStr[i];
		i++;
	}
	pStr[length] = '\0';
	return (CStrPtr)pStr;
}  /* PStrP2C */

PStrPtr PStrCToP(CStrPtr cStr, PStrPtr pStr)
{
	unsigned long length = CStrLength((CStrPtr)cStr);
	unsigned long i;
	
	if (!pStr)
		pStr = PStrNext();
	if (length > 255)
		length = 255;
	i = length;
	while (i) {
		pStr[i] = (unsigned char)cStr[i - 1];
		i--;
	}
	PStrSetLength(pStr, length);
	return pStr;
}  /* PStrCToP */

CStrPtr PStrPToC(PStrPtr pStr, CStrPtr cStr)
{
	long length = PStrLength(pStr);
	long i = 1;

	if (!cStr)
		cStr = (CStrPtr)PStrNext();
	while (i <= length) {
		cStr[i - 1] = (char)pStr[i];
		i++;
	}
	cStr[length] = '\0';
	return cStr;
}  /* PStrPToC */

PStrPtr PS(const char *cStr)
{
	PStrPtr theString = PStrNext();
	unsigned long length = CStrLength(cStr);

	if (length > 255)
		length = 255;
	PStrSetLength(theString, length);
	while (length) {
		theString[length] = (unsigned char)cStr[length - 1];
		length--;
	}
	return theString;
}  /* PS */

CStrPtr CS(PStrPtr pStr)
{
	CStrPtr cStr = (CStrPtr)PStrNext();
	unsigned long length = PStrLength(pStr);
	unsigned long i = 1;
	
	while (i <= length) {
		cStr[i - 1] = (char)pStr[i];
		i++;
	}
	cStr[length] = '\0';
	return cStr;
}  /* CS */

PStrPtr PStrType(unsigned long theType)
{
	PStrPtr theStr = PStrNext();
	unsigned long typeNum = (unsigned long)theType;
	
	theStr[0] = 4;
	theStr[1] = (unsigned char)(typeNum >> 24);
	theStr[2] = (unsigned char)(typeNum >> 16);
	theStr[3] = (unsigned char)(typeNum >> 8);
	theStr[4] = (unsigned char)typeNum;
	return theStr;
}  /* PStrType */

PStrPtr PStrNum(long theNum)
{
	PStrPtr theStr = PStrNext();
	
	if (theNum == 0) {
		theStr[0] = 1;
		theStr[1] = '0';
	} else {
		long i, j;
		
		if (theNum < 0) {
			theStr[1] = '-';
			theNum = -theNum;
			i = 1;
			j = 2;
		} else {
			i = 0;
			j = 1;
		}
		while (theNum) {
			long k = ++i;
			
			while (k > j) {
				theStr[k] = theStr[k - 1];
				k--;
			}
			theStr[j] = (unsigned char)((theNum % 10) + '0');
			theNum /= 10;
			
		}
		theStr[0] = (unsigned char)i;
	}
	return theStr;
}  /* PStrNum */

long PStrCompare(PStrPtr a, PStrPtr b)
{
	long aLength, bLength, len, i;

	aLength = PStrLength(a);
	bLength = PStrLength(b);
	len = aLength < bLength ? aLength : bLength;
	for (i = 1; i <= len; i++) {
		if (a[i] < b[i])
			return -1;
		else if (a[i] > b[i])
			return +1;
	}
	return aLength - bLength;
}  /* PStrCompare */

PStrPtr PStrCpy(PStrPtr dst, PStrPtr src)
{
	unsigned long length = PStrLength(src);

	if (!dst)
		dst = PStrNext();
	PStrSetLength(dst, length);
	while (length) {
		dst[length] = src[length];
		length--;
	}
	return dst;
}  /* PStrCpy */

PStrPtr PStrAppend(PStrPtr dst, PStrPtr src)
{
	unsigned long srcLen, dstLen;
	
	if (!dst) {
		dst = PStrNext();
		PStrCpy(dst, src);
	} else {
		unsigned long i = 1;
		
		srcLen = PStrLength(src);
		dstLen = PStrLength(dst);
		if (srcLen + dstLen > 255)
			srcLen = 255 - dstLen;
		while (i <= srcLen) {
			dst[dstLen + i] = src[i];
			i++;
		}
		PStrSetLength(dst, srcLen + dstLen);
	}
	return dst;
}  /* PStrAppend */

PStrPtr PStrAppendChar(PStrPtr theStr, unsigned char theChar)
{
	unsigned long theLength;

	theLength = (unsigned long)(PStrLength(theStr) + 1);
	if (theLength <= 255) {
		PStrSetLength(theStr, theLength);
		theStr[theLength] = theChar;
	}
	return theStr;
}  /* PStrAppendChar */

PStrPtr PStrConcat(PStrPtr firstStr, ...)
{
	va_list ap;
	PStrPtr nextStr;
	PStrPtr finalStr = PStrCpy(PStrNULL, firstStr);
	
	va_start(ap, firstStr);
	nextStr = va_arg(ap, PStrPtr);
	
	while (nextStr) {
		PStrAppend(finalStr, nextStr);
		nextStr = va_arg(ap, PStrPtr);
	}
	va_end(ap);
	return finalStr;
}  /* PStrConcat */

unsigned long PStrIn(unsigned char ch, PStrPtr theString)
{
	unsigned long theLength, i;
	int found;
	
	theLength = PStrLength(theString);
	found = 0;
	i = 1;
	while ((i <= theLength) && (!found))
		if (ch == theString[i])
			return i;
		else
			i++;
	return 0;
}  /* PStrIn */

unsigned long PStrPos(PStrPtr subStr, PStrPtr theString)
{
	unsigned long subLen, strLen;
	unsigned long subIndex, strIndex, tmpIndex;

	subLen = PStrLength(subStr);
	strLen = PStrLength(theString);
	if (subLen <= strLen) {
		for (strIndex = 1; strIndex <= strLen; strIndex++) {
			for (subIndex = 1, tmpIndex = strIndex; (subIndex <= subLen) && (subStr[subIndex] == theString[tmpIndex]); subIndex++, tmpIndex++)
				;
			if (subIndex > subLen)
				return strIndex;
		}
	}
	return 0;
}  /* PStrPos */

PStrPtr PStrSub(PStrPtr source, unsigned long index, unsigned long count)
{
	PStrPtr subString;
	unsigned long srcSize;

	subString = PStrNext();
	srcSize = (unsigned long)PStrSize(source);
	if (index + count > srcSize)
		count = srcSize - index;
	if (count <= 0)
		PStrMakeEmpty(subString);
	else {
		PStrSetLength(subString, count);
		while (count) {
			subString[count] = source[index + count - 1];
			count--;
		}
	}
	return subString;
}  /* PStrSub */

void PStrDelete(PStrPtr dest, unsigned long index, unsigned long count)
{
	unsigned long dstSize, moveIndex;

	dstSize = (unsigned long)PStrSize(dest);
	if (index + count > dstSize)
		count = dstSize - index;
	if (count >= 1) {
		PStrSetLength(dest, PStrLength(dest) - count);
		moveIndex = index + count;
		if (moveIndex < dstSize) {
			dstSize -= moveIndex;
			while (dstSize--)
				dest[index++] = dest[moveIndex++];
		}
	}
}  /* PStrDelete */

PStrPtr PStrOmit(PStrPtr str, unsigned long index, unsigned long count)
{
	PStrPtr newStr;

	newStr = PStrCpy(PStrNULL, str);
	PStrDelete(newStr, index, count);
	return newStr;
}  /* PStrOmit */

void PStrInsert(PStrPtr source, PStrPtr dest, unsigned long index)
{
	unsigned long srcLength = PStrLength(source);
	unsigned long dstLength = PStrLength(dest);
	unsigned long i;
	
	if (srcLength + dstLength > 255)
		dstLength = 255 - srcLength;
	if (index < 1)
		index = 1;
	else if (index > dstLength)
		index = dstLength + 1;
	if (index <= dstLength) {
		i = dstLength - index + 1;
		while (i) {
			i--;
			dest[index + srcLength + i] = dest[index + i];
		}
	}
	i = srcLength;
	while (i) {
		dest[index + i - 1] = source[i];
		i--;
	}
	PStrSetLength(dest, srcLength + dstLength);
}  /* PStrInsert */

PStrPtr PStrLongStr(long num)
{
	PStrPtr newStr = PStrNext();
	unsigned long length = 0;
	unsigned long power = 1000000000;
	int found = 0;
	
	if (num == 0)
		newStr[++length] = '0';
	else {
		if (num < 0) {
			newStr[++length] = '-';
			num = -num;
		}
		while (power) {
			if ((unsigned long)num >= power)
				found = 1;
			if (found)
				newStr[++length] = (unsigned char)('0' + ((num / power) % 10));
			power /= 10;
		}
	}
	newStr[0] = (unsigned char)length;
	return newStr;
}  /* PStrLongStr */

PStrPtr PStrULongStr(unsigned long num)
{
	PStrPtr newStr = PStrNext();
	unsigned long length = 0;
	unsigned long power = 1000000000;
	int found = 0;
	
	while (power) {
		if (num >= power)
			found = 1;
		if (found)
			newStr[++length] = (unsigned char)('0' + ((num / power) % 10));
		power /= 10;
	}
	newStr[0] = (unsigned char)length;
	return newStr;
}  /* PStrULongStr */

PStrPtr PStrCharStr(unsigned char theChar)
{
	PStrPtr theStr = PStrNext();
	
	PStrSetLength(theStr, 1);
	theStr[1] = theChar;
	return theStr;
}  /* PStrCharStr */

PStrPtr PStrFloatStr(double theNum, long theDigits)
{
	unsigned long theLength, i;
	PStrPtr theStr = PStrNext();

	if (theDigits < 0)
		sprintf((char*)theStr, "%#1.*lf", -theDigits, theNum);
	else
		sprintf((char*)theStr, "%1.*lf", theDigits, theNum);
	PStrC2P((char*)theStr);
	if (theDigits >= 0) {
		theLength = PStrLength(theStr);
		for (i = 1; i <= theLength; i++)
			if (theStr[i] == '.')
				break;
		if (i <= theLength) {
			for (; theLength >= i; theLength--)
				if ((theStr[theLength] != '0') && (theStr[theLength] != '.'))
					break;
			PStrSetLength(theStr, theLength);
		}
		if ((PStrLength(theStr) == 2) && (theStr[1] == '-') && (theStr[2] == '0')) {
			PStrSetLength(theStr, sizeof(unsigned char));
			theStr[1] = '0';
		}			
	}
	return theStr;
}  /* PStrFloatStr */

PStrPtr PStrNumericalStr(long theNum)
{
	PStrPtr theString = PStrLongStr(theNum < 0 ? -theNum : theNum);
	unsigned long theLength = PStrLength(theString);
	
	if (theLength >= 4) {
		PStrPtr newString = PStrNext();
		unsigned long newLength = theLength + ((theLength - 1) / 3);
		unsigned long count = 0;

		if (theNum < 0) {
			newLength++;
			newString[1] = '-';
		}
		PStrSetLength(newString, newLength);
		while (theLength >= 1) {
			if (count == 3) {
				count = 0;
				newString[newLength] = ',';
			} else {
				count++;
				newString[newLength] = theString[theLength--];
			}
			newLength--;
		}
		return newString;
	}
	return theString;
}  /* PStrNumericStr */

PStrPtr PStrTimeStr(long seconds)
{
	long minutes, hours, days;
	unsigned char theSign[2];
	
	if (seconds < 0) {
		PStrSetLength(theSign, sizeof(unsigned char));
		theSign[1] = '-';
	} else
		PStrMakeEmpty(theSign);
	if (seconds < 0)
		seconds = -seconds;
	minutes = seconds / 60;
	seconds = seconds % 60;
	hours = minutes / 60;
	minutes = minutes % 60;
	days = hours / 24;
	hours = hours % 24;
	if (days > 0)
		return PStrConcat(theSign, PStrLongStr(days), PS("D "), PStrLongStr(hours), PS("H "), PStrLongStr(minutes), PS("M "), PStrLongStr(seconds), PS("S"), PStrNULL);
	else if (hours > 0)
		return PStrConcat(theSign, PStrLongStr(hours), PS("Hr "), PStrLongStr(minutes), PS("Min "), PStrLongStr(seconds), PS("Sec"), PStrNULL);
	else if (minutes > 0)
		return PStrConcat(theSign, PStrLongStr(minutes), PS(" Min.  "), PStrLongStr(seconds), PS(" Sec."), PStrNULL);
	else if (seconds == 1)
		return PStrConcat(theSign, PS("1 Second"), PStrNULL);
	else
		return PStrConcat(theSign, PStrLongStr(seconds), PS(" Seconds"), PStrNULL);
}  /* PStrTimeStr */

PStrPtr PStrDateStr(unsigned long theSecond, unsigned long curSecond)
{
	struct tm theDate, curDate;
	PStrPtr theString;
	unsigned char ampmString[3] = {0x2, 'X', 'M'};
	unsigned char colon[2] = {0x1, ':'};
	unsigned char space[2] = {0x1, ' '};
	unsigned char slash[2] = {0x1, '/'};
	
	if (curSecond == 1)
		time((time_t*)&curSecond);
	if (theSecond == 0)
		if (curSecond == 0)
			time((time_t*)&theSecond);
		else
			theSecond = curSecond;
	theDate = *localtime((time_t*)&theSecond);
	if ((theDate.tm_hour == 0) || (theDate.tm_hour == 12)) {
		theString = PStrLongStr(12);
		ampmString[1] = (unsigned char)(theDate.tm_hour == 0 ? 'A' : 'P');
	} else {
		if (theDate.tm_hour <= 11) {
			theString = PStrLongStr(theDate.tm_hour);
			ampmString[1] = 'A';
		} else {
			theString = PStrLongStr(theDate.tm_hour - 12);
			ampmString[1] = 'P';
		}
	}
	theString = PStrConcat(theString, colon, PStrSub(PStrLongStr(theDate.tm_min + 100), 2, 2), colon, PStrSub(PStrLongStr(theDate.tm_sec + 100), 2, 2), ampmString, PStrNULL);
	curDate = *localtime((time_t*)&curSecond);
	if ((theDate.tm_mon == curDate.tm_mon) && (theDate.tm_mday == curDate.tm_mday) && (theDate.tm_year == curDate.tm_year))
		return PStrAppend(theString, PS(" Today"));
	else {
		theSecond = curSecond + 86400;
		curDate = *localtime((time_t*)&theSecond);
		if ((theDate.tm_mon == curDate.tm_mon) && (theDate.tm_mday == curDate.tm_mday) && (theDate.tm_year == curDate.tm_year))
			return PStrAppend(theString, PS(" Tomorrow"));
		else {
			theSecond = curSecond - 86400;
			curDate = *localtime((time_t*)&theSecond);
			if ((theDate.tm_mon == curDate.tm_mon) && (theDate.tm_mday == curDate.tm_mday) && (theDate.tm_year == curDate.tm_year))
				return PStrAppend(theString, PS(" Yesterday"));
			else
				return PStrConcat(theString, space, PStrLongStr(theDate.tm_mon), slash, PStrLongStr(theDate.tm_mday), slash, PStrSub(PStrLongStr(theDate.tm_year), 3, 2), PStrNULL);
		}
	}
}  /* PStrDateStr */

PStrPtr PStrKStr(unsigned long theNum)
{
	return PStrAppendChar(PStrNumericalStr((long)(theNum >> 10)), 'K');
}  /* PStrKStr */

PStrPtr PStrKSStr(long theNum)
{
	return PStrAppendChar(PStrNumericalStr(theNum / 1024), 'K');	/* Cannot use >> because the sign must be preserved */
}  /* PStrKSStr */

void PStrUpperCase(PStrPtr theString)
{
	unsigned long i, j;
	
	for (i = 1, j = PStrLength(theString); i <= j; i++)
		if ((theString[i] >= 'a') && (theString[i] <= 'z'))
			theString[i] += (unsigned char)('A' - 'a');
}  /* PStrUpperCase */

PStrPtr PStrUpCase(PStrPtr theString)
{
	unsigned long i, theLength;
	PStrPtr newString;
	
	newString = PStrNext();
	theLength = PStrLength(theString);
	PStrSetLength(newString, theLength);
	for (i = 1; i <= theLength; i++)
		if ((theString[i] >= 'a') && (theString[i] <= 'z'))
			newString[i] = (unsigned char)(theString[i] + ('A' - 'a'));
		else
			newString[i] = theString[i];
	return newString;
}  /* PStrUpCase */

void PStrStrip(PStrPtr theString)
{
	unsigned long i, j, k;
	
	for (i = 1, j = PStrLength(theString), k = 0; i <= j; i++)
		if (theString[i] != ' ')
			theString[++k] = theString[i];
	theString[0] = (unsigned char)k;
}  /* PStrStrip */

PStrPtr PStrInclude(PStrPtr source, PStrPtr str, unsigned long index)
{
	PStrPtr newStr = PStrCpy(PStrNULL, str);

	PStrInsert(source, newStr, index);
	return newStr;
}  /* PStrInclude */

CStrPtr PrintPStr(PStrPtr pStr)
{
	return PStrPToC(pStr, PStrNULL);
}  /* PrintPStr */

CStrPtr PrintBoolean(unsigned char b)
{
	return (b ? (CStrPtr)"1" : (CStrPtr)"0");
}  /* PrintBoolean */

unsigned long MakeType(CStrPtr str)
{
	unsigned long theType;
	
	theType = *(unsigned long*)str;
	return theType;
}  /* MakeType */
