/*
Copyright 1993-1998 Electric Image, Inc. This source code may only be used by
registered owners of the ElectricImage Plug-In Kit. Owners of the Kit may
include the source code in their products (freeware, shareware, or commercial).
This copyright notice must be included in its original form in all such
products. This source code may not be offered for sale or distributed by any
entity without the written permission of Electric Image, Inc.
*/

/*
This file contains plugin definitions as needed by plugins.
*/

#include "PluginClient.h"
#ifdef __SGI__
	#include <stdio.h>
	#include <dlfcn.h>
#endif
#include <stddef.h>

static void CopyStr(unsigned char *dst, char *src);
static void CopyPStr(unsigned char *dst, unsigned char *src);

/*
*  Static Functions
*/

static void CopyStr(unsigned char *dst, char *src)
{
	long i;
	
	for (i = 0; (i < 255) && (src[i] != 0); i++)
		dst[i + 1] = src[i];
	dst[0] = i;
}  /* CopyStr */

static void CopyPStr(unsigned char *dst, unsigned char *src)
{
	long i;
	
	dst[0] = src[0];
	for (i = 1; i <= src[0]; i++)
		dst[i] = src[i];
}  /* CopyPStr */

/*
*  Global Functions
*/

/* CallPluginRoutine may be used to call a plugin or host routine. (Provided for Pascal) */

long CallPluginRoutine (IFFType command, long parameterSize, void* parameters, EIPluginRoutine theRoutine)
{
	return theRoutine(command, parameterSize, parameters);
}  /* CallPluginRoutine */

/* DescribePluginError will return a description of a plugin's result code as a string */

void DescribePluginError(long theResultCode, PStrPtr theString)
{
	switch (theResultCode) {
		case pluginNoErr:
			CopyStr(theString, "No plug-in error");
			break;
		case pluginMemoryErr: 
			CopyStr(theString, "Plug-in could not allocate needed memory");
			break;
		case pluginCommandErr: 
			CopyStr(theString, "Unsupported plug-in command was received");
			break;
		case pluginParameterErr: 
			CopyStr(theString, "Incompatible plug-in parameter record was received");
			break;
		case pluginCancelErr: 
			CopyStr(theString, "The user canceled the plug-in operation");
			break;
		case pluginSequenceErr: 
			CopyStr(theString, "A plug-in command was received out of sequence");
			break;
		case pluginResourceErr: 
			CopyStr(theString, "Plug-in could not get a required resource");
			break;
		case pluginProtectionErr: 
			CopyStr(theString, "Plug-in could not verify copy protection");
			break;
		case pluginFeatureErr: 
			CopyStr(theString, "A requested plug-in feature or mode is not supported");
			break;
		default:
			CopyStr(theString, "");
			break;
	}
}  /* DescribePluginError */

/* EIPluginMalloc is called by a plug-in to malloc or realloc a pointer */

void EIPluginMalloc(PluginInfoPtr theInfo, long *theResult, unsigned long theSize, void **thePtr)
{
	HostMemMallocRec theHostMemMalloc;

	if (*theResult == pluginNoErr) {
		theHostMemMalloc.theReference = theInfo->theHostReference;
		theHostMemMalloc.theSize = theSize;
		theHostMemMalloc.thePtr = *thePtr;
		*theResult = theInfo->theHostProc(hostMemMalloc, sizeof(HostMemMallocRec), &theHostMemMalloc);
		*thePtr = theHostMemMalloc.thePtr;
	}
}  /* EIPluginMalloc */

/* EIPluginFree is called by a plug-in to free a pointer */

void EIPluginFree(PluginInfoPtr theInfo, long *theResult, void *thePtr)
{
	HostMemFreeRec theHostMemFree;

	if (*theResult == pluginNoErr) {
		theHostMemFree.theReference = theInfo->theHostReference;
		theHostMemFree.thePtr = thePtr;
		*theResult = theInfo->theHostProc(hostMemFree, sizeof(HostMemFreeRec), &theHostMemFree);
	}
}  /* EIPluginFree */

/* EIPluginStatusCheck will check for user aborts */

void EIPluginStatusCheck(PluginInfoPtr theInfo, HostStatusRec *theStatus, long *theResult)
{
	theInfo;
	theStatus;
	theResult;

//
// Markus Nov 1, 2000
//
// After Removal of System.h, this function became platform dependent and thus is disabled
//
//	if ((*theResult == 0) && ((theStatus->theStatusTick == 0) || (TICKCOUNT() >= theStatus->theStatusTick)))
//		*theResult = theInfo->theHostProc(hostStatus, sizeof(HostStatusRec), theStatus);
}  /* EIPluginStatusCheck */

/* EIPluginStatusUpdate will update the status */

void EIPluginStatusUpdate(PluginInfoPtr theInfo, HostStatusRec *theStatus, float thePos, long theValue, long *theResult)
{
	if (*theResult == 0) {
		theStatus->theReference = theInfo->theHostReference;
		theStatus->theStatusValue = theValue;
		theStatus->theStatusPos = thePos;
		*theResult = theInfo->theHostProc(hostStatus, sizeof(HostStatusRec), theStatus);
	}
}  /* EIPluginStatusUpdate */

/* EIPluginStatusSetup will setup the status information */

void EIPluginStatusSetup(PluginInfoPtr theInfo, HostStatusRec *theStatus, PStrPtr theUnit, PStrPtr theHeading, long min, long max, float thePos, long theValue, long *theResult)
{
	theStatus->theReference = theInfo->theHostReference;
	theStatus->theStatusPos = thePos;
	CopyPStr(theStatus->theStatusUnit, theUnit);
	CopyPStr(theStatus->theStatusInfo, theHeading);
	theStatus->theStatusMin = min;
	theStatus->theStatusMax = max;
	theStatus->theStatusValue = theValue;
	theStatus->theStatusTick = 0;
	EIPluginStatusUpdate(theInfo, theStatus, thePos, theValue, theResult);
}  /* EIPluginStatusSetup */

/* EIPluginStatusFinish will finish updating a status box */

void EIPluginStatusFinish(PluginInfoPtr theInfo, HostStatusRec *theStatus, long *theResult)
{
	if ((*theResult == 0) && theStatus->theStatusTick)
		EIPluginStatusUpdate(theInfo, theStatus, 1.0, theStatus->theStatusMax, theResult);
}  /* EIPluginStatusFinish */
