#include <dlfcn.h>

#include "EI_Types.h"

#include "EI_API.h"
#include "EI_ColorPicker.h"
#include "EI_Control.h"
#include "EI_Dialog.h"
#include "EI_DiskImage.h"
#include "EI_DrawContext.h"
#include "EI_FileRef.h"
#include "EI_ImageBuffer.h"
#include "EI_OpenGL.h"
#include "EI_Rect.h"
#include "EI_ResourceFile.h"
#include "EI_Stream.h"
#include "EI_Timer.h"
//#include "Movies.h"

int EI_GetAPIVersion(void)
{
	int (*APICallback)(void);

	APICallback = (int (*)(void)) dlsym(RTLD_DEFAULT, "EI_GetAPIVersion");
	if (APICallback)
		return (APICallback());
	else
		return 0;
}

int EI_PickColor(const char *iPromptString,
				 const EI_Color *iOriginalColor,
				 EI_Color *oNewColor)
{
	int (*APICallback)(const char *, const EI_Color *, EI_Color *);

	APICallback = (int (*)(const char *, const EI_Color *, EI_Color *)) dlsym(RTLD_DEFAULT, "EI_PickColor");
	if (APICallback)
		return (APICallback(iPromptString, iOriginalColor, oNewColor));
	else
		return 0;
}

int EI_PickColorWithAlpha(const char *iPromptString,
						 	const EI_Color *iOriginalColor,
							unsigned char iOriginalAlpha,
							EI_Color *oNewColor,
							unsigned char *oNewAlpha)
{
	int (*APICallback)(const char *, const EI_Color *, unsigned char, EI_Color *, unsigned char *);

	APICallback = (int (*)(const char *, const EI_Color *, unsigned char, EI_Color *, unsigned char *)) dlsym(RTLD_DEFAULT, "EI_PickColorWithAlpha");
	if (APICallback)
		return (APICallback(iPromptString, iOriginalColor, iOriginalAlpha, oNewColor, oNewAlpha));
	else
		return 0;
}

int EI_GetControlID(const EI_Control *iControl)
{
	int (*APICallback)(const EI_Control *);

	APICallback = (int (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_GetControlID");
	if (APICallback)
		return (APICallback(iControl));
	else
		return 0;
}

//================

void EI_SetControlID(EI_Control *ioControl, int iNewID)
{
	void (*APICallback)(const EI_Control *, int);

	APICallback = (void (*)(const EI_Control *, int)) dlsym(RTLD_DEFAULT, "EI_SetControlID");
	if (APICallback)
		return (APICallback(ioControl, iNewID));
	else
		return;
}

int EI_CountControls(const EI_Dialog *iDialog)
{
	int (*APICallback)(const EI_Dialog *);

	APICallback = (int (*)(const EI_Dialog *)) dlsym(RTLD_DEFAULT, "EI_CountControls");
	if (APICallback)
		return (APICallback(iDialog));
	else
		return 0;
}

EI_Control *EI_FindControlByIndex(const EI_Dialog *iDialog, int iControlNumber)
{
	EI_Control* (*APICallback)(const EI_Dialog *, int iControlNumber);

	APICallback = (EI_Control* (*)(const EI_Dialog *, int iControlNumber)) dlsym(RTLD_DEFAULT, "EI_FindControlByIndex");
	if (APICallback)
		return (APICallback(iDialog, iControlNumber));
	else
		return NULL;
}

EI_Control *EI_FindControlByID(const EI_Dialog *iDialog, int iControlID)
{
	EI_Control* (*APICallback)(const EI_Dialog *, int);

	APICallback = (EI_Control* (*)(const EI_Dialog *, int)) dlsym(RTLD_DEFAULT, "EI_FindControlByID");
	if (APICallback)
		return (APICallback(iDialog, iControlID));
	else
		return NULL;
}

void EI_InvalControl(const EI_Control *iControl)
{
	void (*APICallback)(const EI_Control *);
	
	APICallback = (void (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_InvalControl");
	if (APICallback)
		return (APICallback(iControl));
	else
		return;
}

EI_Dialog *EI_GetControlDialog(const EI_Control *iControl)
{
	EI_Dialog* (*APICallback)(const EI_Control *);
	
	APICallback = (EI_Dialog* (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_GetControlDialog");
	if (APICallback)
		return (APICallback(iControl));
	else
		return NULL;
}

EI_ControlType EI_GetControlType(const EI_Control *iControl)
{
	EI_ControlType (*APICallback)(const EI_Control *);
	
	APICallback = (EI_ControlType (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_GetControlType");
	if (APICallback)
		return (APICallback(iControl));
	else
		return cEI_NoControlType;
}

void EI_GetControlBounds(const EI_Control *iControl, EI_Rect *oBounds)
{
	void (*APICallback)(const EI_Control *, EI_Rect*);
	
	APICallback = (void (*)(const EI_Control *, EI_Rect*)) dlsym(RTLD_DEFAULT, "EI_GetControlBounds");
	if (APICallback)
		return (APICallback(iControl, oBounds));
	else
		return;
}

void EI_SetControlBounds(EI_Control *ioControl, const EI_Rect *iBounds)
{
	void (*APICallback)(const EI_Control *, const EI_Rect*);
	
	APICallback = (void (*)(const EI_Control *, const EI_Rect*)) dlsym(RTLD_DEFAULT, "EI_SetControlBounds");
	if (APICallback)
		return (APICallback(ioControl, iBounds));
	else
		return;
}

void EI_SetControlDrawImmediate(EI_Control *ioControl, int iDrawImmediateFlag)
{
	void (*APICallback)(const EI_Control *, int );
	
	APICallback = (void (*)(const EI_Control *, int )) dlsym(RTLD_DEFAULT, "EI_SetControlDrawImmediate");
	if (APICallback)
		return (APICallback(ioControl, iDrawImmediateFlag));
	else
		return;
}

int EI_GetControlDrawImmediate(const EI_Control *iControl)
{
	int (*APICallback)(const EI_Control *);
		
	APICallback = (int (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_GetControlDrawImmediate");
	if (APICallback)
		return (APICallback(iControl));
	else
		return 0;
}

void EI_SetControlVisible(EI_Control *iControl, int iVisible)
{
	void (*APICallback)(const EI_Control *, int);
		
	APICallback = (void (*)(const EI_Control *, int)) dlsym(RTLD_DEFAULT, "EI_SetControlVisible");
	if (APICallback)
		return (APICallback(iControl, iVisible));
	else
		return;
}

int EI_GetControlVisible(const EI_Control *iControl)
{
	int (*APICallback)(const EI_Control *);
		
	APICallback = (int (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_GetControlVisible");
	if (APICallback)
		return (APICallback(iControl));
	else
		return 0;
}

int EI_GetControlEnable(const EI_Control *iControl)
{
	int (*APICallback)(const EI_Control *);
		
	APICallback = (int (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_GetControlEnable");
	if (APICallback)
		return (APICallback(iControl));
	else
		return 0;
}

void EI_SetControlEnable(EI_Control *iControl, int iEnabled)
{
	void (*APICallback)(const EI_Control *, int);
		
	APICallback = (void (*)(const EI_Control *, int)) dlsym(RTLD_DEFAULT, "EI_SetControlEnable");
	if (APICallback)
		return (APICallback(iControl, iEnabled));
	else
		return;
}

void EI_SetControlTitle(EI_Control *ioControl, const char *iTitleString)
{
	void (*APICallback)(const EI_Control *, const char*);
		
	APICallback = (void (*)(const EI_Control *, const char*)) dlsym(RTLD_DEFAULT, "EI_SetControlTitle");
	if (APICallback)
		return (APICallback(ioControl, iTitleString));
	else
		return;
}

int EI_GetControlTitleSize(const EI_Control *iControl)
{
	int (*APICallback)(const EI_Control *);
		
	APICallback = (int (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_GetControlTitleSize");
	if (APICallback)
		return (APICallback(iControl));
	else
		return 0;
}

void EI_GetControlTitle(const EI_Control *iControl, char *oText, int *ioNumChars)
{
	void (*APICallback)(const EI_Control *, char *, int *);
		
	APICallback = (void (*)(const EI_Control *, char *, int *)) dlsym(RTLD_DEFAULT, "EI_GetControlTitle");
	if (APICallback)
		return (APICallback(iControl, oText, ioNumChars));
	else
		return;
}

int EI_GetControlValue(const EI_Control *iControl)
{
	int (*APICallback)(const EI_Control *);
		
	APICallback = (int (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_GetControlValue");
	if (APICallback)
		return (APICallback(iControl));
	else
		return 0;
}

void EI_SetControlValue(EI_Control *ioControl, int iNewValue)
{
	void (*APICallback)(const EI_Control *, int);
		
	APICallback = (void (*)(const EI_Control *, int)) dlsym(RTLD_DEFAULT, "EI_SetControlValue");
	if (APICallback)
		return (APICallback(ioControl, iNewValue));
	else
		return;
}

int EI_GetControlMax(const EI_Control *iControl)
{
	int (*APICallback)(const EI_Control *);
		
	APICallback = (int (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_GetControlMax");
	if (APICallback)
		return (APICallback(iControl));
	else
		return 0;
}

void EI_SetControlMax(EI_Control *ioControl, int iNewMax)
{
	void (*APICallback)(const EI_Control *, int);
		
	APICallback = (void (*)(const EI_Control *, int)) dlsym(RTLD_DEFAULT, "EI_SetControlMax");
	if (APICallback)
		return (APICallback(ioControl, iNewMax));
	else
		return;
}

void *EI_GetControlExtraData(const EI_Control *iControl)
{
	void * (*APICallback)(const EI_Control *);
		
	APICallback = (void *(*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_GetControlExtraData");
	if (APICallback)
		return (APICallback(iControl));
	else
		return NULL;
}

void EI_SetControlExtraData(EI_Control *ioControl, void *iExtraData)
{
	void (*APICallback)(const EI_Control *, void *);
		
	APICallback = (void (*)(const EI_Control *, void *)) dlsym(RTLD_DEFAULT, "EI_SetControlExtraData");
	if (APICallback)
		return (APICallback(ioControl, iExtraData));
	else
		return;
}

void EI_SetKeyControl(EI_Control *iControl)
{
	void (*APICallback)(EI_Control *);
		
	APICallback = (void (*)(EI_Control *)) dlsym(RTLD_DEFAULT, "EI_SetKeyControl");
	if (APICallback)
		return (APICallback(iControl));
	else
		return;
}

EI_Control *EI_GetKeyControl(const EI_Dialog *iDialog)
{
	EI_Control* (*APICallback)(const EI_Dialog *);
		
	APICallback = (EI_Control* (*)(const EI_Dialog *)) dlsym(RTLD_DEFAULT, "EI_GetKeyControl");
	if (APICallback)
		return (APICallback(iDialog));
	else
		return NULL;
}

void EI_SetEditTextString(EI_Control *ioControl, const char *iEditString)
{
	void (*APICallback)(EI_Control *, const char *);
		
	APICallback = (void (*)(EI_Control *, const char *)) dlsym(RTLD_DEFAULT, "EI_SetEditTextString");
	if (APICallback)
		return (APICallback(ioControl, iEditString));
	else
		return;
}

int EI_GetEditTextStringSize(const EI_Control *iControl)
{
	int (*APICallback)(const EI_Control *);
		
	APICallback = (int (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_GetEditTextStringSize");
	if (APICallback)
		return (APICallback(iControl));
	else
		return 0;
}

void EI_GetEditTextString(const EI_Control *iControl, char *oText, int *ioNumChars)
{
	void (*APICallback)(const EI_Control *, char *, int *);
		
	APICallback = (void (*)(const EI_Control *, char *, int *)) dlsym(RTLD_DEFAULT, "EI_GetEditTextString");
	if (APICallback)
		return (APICallback(iControl, oText, ioNumChars));
	else
		return;
}

void EI_SetEditTextSelection(EI_Control *ioControl, int iSelectionStart, int iSelectionEnd)
{
	void (*APICallback)(EI_Control *, int , int );
		
	APICallback = (void (*)(EI_Control *, int , int )) dlsym(RTLD_DEFAULT, "EI_SetEditTextSelection");
	if (APICallback)
		return (APICallback(ioControl, iSelectionStart, iSelectionEnd));
	else
		return;
}

void EI_GetEditTextSelection(const EI_Control *ioControl, int *oSelectionStart, int *oSelectionEnd)
{
	void (*APICallback)(const EI_Control *, int *, int *);
		
	APICallback = (void (*)(const EI_Control *, int *, int *)) dlsym(RTLD_DEFAULT, "EI_GetEditTextSelection");
	if (APICallback)
		return (APICallback(ioControl, oSelectionStart, oSelectionEnd));
	else
		return;
}

void EI_SetEditTextFilter(EI_Control *ioControl, EI_EditFilterType iFilterType, 
						  EI_EditFilterFunction iFilterFunction)
{
	void (*APICallback)(EI_Control *, EI_EditFilterType, EI_EditFilterFunction);
		
	APICallback = (void (*)(EI_Control *, EI_EditFilterType, EI_EditFilterFunction)) dlsym(RTLD_DEFAULT, "EI_SetEditTextFilter");
	if (APICallback)
		return (APICallback(ioControl, iFilterType, iFilterFunction));
	else
		return;
}

void EI_GetColorButtonColor(const EI_Control *iControl, int *oRed, int *oGreen,	int *oBlue, int *oAlpha) 
{
	void (*APICallback)(const EI_Control *, int *, int *,	int *, int *);
		
	APICallback = (void (*)(const EI_Control *, int *, int *,	int *, int *)) dlsym(RTLD_DEFAULT, "EI_GetColorButtonColor");
	if (APICallback)
		return (APICallback(iControl, oRed, oGreen,oBlue, oAlpha));
	else
		return;
}

void EI_SetColorButtonColor(EI_Control *ioControl, int iRed, int iGreen, int iBlue, int iAlpha)
{
	void (*APICallback)(EI_Control *, int , int , int , int );
		
	APICallback = (void (*)(EI_Control *, int , int , int , int )) dlsym(RTLD_DEFAULT, "EI_SetColorButtonColor");
	if (APICallback)
		return (APICallback(ioControl, iRed, iGreen, iBlue, iAlpha));
	else
		return;
}

int EI_TrackMouseDown(const EI_Control* iControl, int* oMouseX, int* oMouseY)
{
	int (*APICallback)(const EI_Control *, int *, int *);
		
	APICallback = (int (*)(const EI_Control *, int *, int *)) dlsym(RTLD_DEFAULT, "EI_TrackMouseDown");
	if (APICallback)
		return (APICallback(iControl, oMouseX, oMouseY));
	else
		return 0;
}

void EI_SetListString(EI_Control *ioControl, int iWhichItem, const char *iItemString)
{
	void (*APICallback)(EI_Control *, int , const char *);
		
	APICallback = (void (*)(EI_Control *, int , const char *)) dlsym(RTLD_DEFAULT, "EI_SetListString");
	if (APICallback)
		return (APICallback(ioControl, iWhichItem, iItemString));
	else
		return;
}

int EI_GetListStringSize(const EI_Control *iControl, int iWhichItem)
{
	int (*APICallback)(const EI_Control *, int);
		
	APICallback = (int (*)(const EI_Control *, int)) dlsym(RTLD_DEFAULT, "EI_GetListStringSize");
	if (APICallback)
		return (APICallback(iControl, iWhichItem));
	else
		return 0;
}

void EI_GetListString(const EI_Control *iControl, int iWhichItem, char *oText, int *ioNumChars)
{
	void (*APICallback)(const EI_Control *, int, char *, int *);
		
	APICallback = (void (*)(const EI_Control *, int, char *, int *)) dlsym(RTLD_DEFAULT, "EI_GetListString");
	if (APICallback)
		return (APICallback(iControl, iWhichItem, oText, ioNumChars));
	else
		return;
}

void EI_SetListItemStyle(EI_Control *ioControl,
							int iWhichItem,
							int iStyleFlags)
{
	void (*APICallback)(EI_Control *, int, int);
		
	APICallback = (void (*)(EI_Control *, int, int)) dlsym(RTLD_DEFAULT, "EI_SetListItemStyle");
	if (APICallback)
		return (APICallback(ioControl, iWhichItem, iStyleFlags));
	else
		return;
}

int EI_GetListItemStyle(const EI_Control *iControl, int iWhichItem)
{
	int (*APICallback)(const EI_Control *, int);
		
	APICallback = (int (*)(const EI_Control *, int)) dlsym(RTLD_DEFAULT, "EI_GetListItemStyle");
	if (APICallback)
		return (APICallback(iControl, iWhichItem));
	else
		return 0;
}

int EI_GetListSelectedItem(const EI_Control *iControl)
{
	int (*APICallback)(const EI_Control *);
		
	APICallback = (int (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_GetListSelectedItem");
	if (APICallback)
		return (APICallback(iControl));
	else
		return 0;
}

void EI_SelectListItem(EI_Control *ioControl, int iWhichItem, int iSelect)
{
	void (*APICallback)(EI_Control *, int, int);
		
	APICallback = (void (*)(EI_Control *, int, int)) dlsym(RTLD_DEFAULT, "EI_SelectListItem");
	if (APICallback)
		return (APICallback(ioControl, iWhichItem, iSelect));
	else
		return;
}

int EI_IsListItemSelected(const EI_Control *iControl, int iWhichItem)
{
	int (*APICallback)(const EI_Control *, int);
		
	APICallback = (int (*)(const EI_Control *, int)) dlsym(RTLD_DEFAULT, "EI_IsListItemSelected");
	if (APICallback)
		return (APICallback(iControl, iWhichItem));
	else
		return 0;
}

int EI_CountListItems(const EI_Control *iControl)
{
	int (*APICallback)(const EI_Control *);
		
	APICallback = (int (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_CountListItems");
	if (APICallback)
		return (APICallback(iControl));
	else
		return 0;
}

void EI_InsertListItem(EI_Control *ioControl, int iBeforeWhichItem, const char *iItemString)
{
	void (*APICallback)(EI_Control *, int , const char *);
		
	APICallback = (void (*)(EI_Control *, int , const char *)) dlsym(RTLD_DEFAULT, "EI_InsertListItem");
	if (APICallback)
		return (APICallback(ioControl, iBeforeWhichItem, iItemString));
	else
		return;
}

void EI_RemoveListItem(EI_Control *ioControl, int iWhichItem)
{
	void (*APICallback)(EI_Control *, int );
		
	APICallback = (void (*)(EI_Control *, int )) dlsym(RTLD_DEFAULT, "EI_RemoveListItem");
	if (APICallback)
		return (APICallback(ioControl, iWhichItem));
	else
		return;
}

void EI_SetListFlags(EI_Control *ioControl, int iListFlags)
{
	void (*APICallback)(EI_Control *, int );
		
	APICallback = (void (*)(EI_Control *, int )) dlsym(RTLD_DEFAULT, "EI_SetListFlags");
	if (APICallback)
		return (APICallback(ioControl, iListFlags));
	else
		return;
}

int EI_GetListFlags(const EI_Control *iControl)
{
	int (*APICallback)(const EI_Control * );
		
	APICallback = (int (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_GetListFlags");
	if (APICallback)
		return (APICallback(iControl));
	else
		return 0;
}

void EI_SetListDoubleClickFunction(EI_Control *ioControl, EI_ListDoubleClickFunction iFunction)
{
	void (*APICallback)(EI_Control *, EI_ListDoubleClickFunction );
		
	APICallback = (void (*)(EI_Control *, EI_ListDoubleClickFunction )) dlsym(RTLD_DEFAULT, "EI_SetListDoubleClickFunction");
	if (APICallback)
		return (APICallback(ioControl, iFunction));
	else
		return;
}

void EI_SetListReorderFunction(EI_Control *ioControl, EI_ListReorderFunction iFunction)
{
	void (*APICallback)(EI_Control *, EI_ListReorderFunction );
		
	APICallback = (void (*)(EI_Control *, EI_ListReorderFunction )) dlsym(RTLD_DEFAULT, "EI_SetListReorderFunction");
	if (APICallback)
		return (APICallback(ioControl, iFunction));
	else
		return;
}

void EI_SetControlHitFunction(EI_Control *ioControl, EI_ControlHitFunction iFunction)
{
	void (*APICallback)(EI_Control *, EI_ControlHitFunction );
		
	APICallback = (void (*)(EI_Control *, EI_ControlHitFunction )) dlsym(RTLD_DEFAULT, "EI_SetControlHitFunction");
	if (APICallback)
		return (APICallback(ioControl, iFunction));
	else
		return;
}

void EI_SetUserControlDraw(EI_Control *ioControl, EI_UserDrawFunction iFunction)
{
	void (*APICallback)(EI_Control *, EI_UserDrawFunction );
		
	APICallback = (void (*)(EI_Control *, EI_UserDrawFunction )) dlsym(RTLD_DEFAULT, "EI_SetUserControlDraw");
	if (APICallback)
		return (APICallback(ioControl, iFunction));
	else
		return;
}

void EI_SetUserControlClick(EI_Control *ioControl, EI_UserClickFunction iFunction)
{
	void (*APICallback)(EI_Control *, EI_UserClickFunction );
		
	APICallback = (void (*)(EI_Control *, EI_UserClickFunction )) dlsym(RTLD_DEFAULT, "EI_SetUserControlClick");
	if (APICallback)
		return (APICallback(ioControl, iFunction));
	else
		return;
}

EI_Dialog *EI_MakeDialog(EI_ResourceFile *iResourceFile, int iDialogID)
{
	EI_Dialog* (*APICallback)(EI_ResourceFile *, int );
		
	APICallback = (EI_Dialog* (*)(EI_ResourceFile *, int )) dlsym(RTLD_DEFAULT, "EI_MakeDialog");
	if (APICallback)
		return (APICallback(iResourceFile, iDialogID));
	else
		return NULL;
}

void EI_DestroyDialog(EI_Dialog *iDialog)
{
	void (*APICallback)(EI_Dialog *);
		
	APICallback = (void (*)(EI_Dialog *)) dlsym(RTLD_DEFAULT, "EI_DestroyDialog");
	if (APICallback)
		return (APICallback(iDialog));
	else
		return;
}

void EI_SetDialogTitle(EI_Dialog *ioDialog, const char *iTitleString)
{
	void (*APICallback)(EI_Dialog *, const char *);
		
	APICallback = (void (*)(EI_Dialog *, const char *)) dlsym(RTLD_DEFAULT, "EI_SetDialogTitle");
	if (APICallback)
		return (APICallback(ioDialog, iTitleString));
	else
		return;
}

void EI_MoveDialog(EI_Dialog *iDialog, int iNewLeft, int iNewTop)
{
	void (*APICallback)(EI_Dialog *, int , int );
		
	APICallback = (void (*)(EI_Dialog *, int , int )) dlsym(RTLD_DEFAULT, "EI_MoveDialog");
	if (APICallback)
		return (APICallback(iDialog, iNewLeft, iNewTop));
	else
		return;
}

void EI_GetDialogPosition(const EI_Dialog *iDialog, int *oLeft, int *oTop)
{
	void (*APICallback)(const EI_Dialog *, int *, int *);
		
	APICallback = (void (*)(const EI_Dialog *, int *, int *)) dlsym(RTLD_DEFAULT, "EI_GetDialogPosition");
	if (APICallback)
		return (APICallback(iDialog, oLeft, oTop));
	else
		return;
}

void *EI_GetDialogExtraData(const EI_Dialog *iDialog)
{
	void * (*APICallback)(const EI_Dialog *);
		
	APICallback = (void *(*)(const EI_Dialog *)) dlsym(RTLD_DEFAULT, "EI_GetDialogExtraData");
	if (APICallback)
		return (APICallback(iDialog));
	else
		return NULL;
}

void EI_SetDialogExtraData(EI_Dialog *ioDialog, void *iExtraData)
{
	void (*APICallback)(EI_Dialog *, void *);
		
	APICallback = (void (*)(EI_Dialog *, void *)) dlsym(RTLD_DEFAULT, "EI_SetDialogExtraData");
	if (APICallback)
		return (APICallback(ioDialog, iExtraData));
	else
		return;
}

int EI_ExecuteDialog(EI_Dialog *iDialog)
{
	int (*APICallback)(EI_Dialog *);
		
	APICallback = (int (*)(EI_Dialog *)) dlsym(RTLD_DEFAULT, "EI_ExecuteDialog");
	if (APICallback)
		return (APICallback(iDialog));
	else
		return 0;
}

void EI_StopDialog(EI_Dialog *iDialog, int iResult)
{
	void (*APICallback)(EI_Dialog *, int );
		
	APICallback = (void (*)(EI_Dialog *, int )) dlsym(RTLD_DEFAULT, "EI_StopDialog");
	if (APICallback)
		return (APICallback(iDialog, iResult));
	else
		return;
}

void EI_SetDialogHitFunction(EI_Dialog *iDialog, EI_DialogHitFunction iFunction)
{
	void (*APICallback)(EI_Dialog *, EI_DialogHitFunction );
		
	APICallback = (void (*)(EI_Dialog *, EI_DialogHitFunction )) dlsym(RTLD_DEFAULT, "EI_SetDialogHitFunction");
	if (APICallback)
		return (APICallback(iDialog, iFunction));
	else
		return;
}

void EI_GetDialogFrameOffset(const EI_Dialog *iDialog, int *oOffsetX, int *oOffsetY)
{
	void (*APICallback)(const EI_Dialog *, int *, int *);
		
	APICallback = (void (*)(const EI_Dialog *, int *, int *)) dlsym(RTLD_DEFAULT, "EI_GetDialogFrameOffset");
	if (APICallback)
		return (APICallback(iDialog, oOffsetX, oOffsetY));
	else
		return;
}

EI_DiskImage *EI_MakeDiskImage(EI_ResourceFile *iResourceFile, unsigned long iType, int iID)
{
	EI_DiskImage* (*APICallback)(EI_ResourceFile *, unsigned long , int );
		
	APICallback = (EI_DiskImage* (*)(EI_ResourceFile *, unsigned long , int )) dlsym(RTLD_DEFAULT, "EI_MakeDiskImage");
	if (APICallback)
		return (APICallback(iResourceFile, iType, iID));
	else
		return NULL;
}

void EI_DestroyDiskImage(EI_DiskImage *iImage)
{
	void (*APICallback)(EI_DiskImage *);
		
	APICallback = (void (*)(EI_DiskImage *)) dlsym(RTLD_DEFAULT, "EI_DestroyDiskImage");
	if (APICallback)
		return (APICallback(iImage));
	else
		return;
}

const EI_ImageBuffer *EI_GetDiskImageBuffer(EI_DiskImage *iImage)
{
	const EI_ImageBuffer* (*APICallback)(EI_DiskImage *);
		
	APICallback = (const EI_ImageBuffer* (*)(EI_DiskImage *)) dlsym(RTLD_DEFAULT, "EI_GetDiskImageBuffer");
	if (APICallback)
		return (APICallback(iImage));
	else
		return NULL;
}

EI_DrawContext *EI_MakeContextForImageBuffer(EI_ImageBuffer *iImage)
{
	EI_DrawContext* (*APICallback)(EI_ImageBuffer *);
		
	APICallback = (EI_DrawContext* (*)(EI_ImageBuffer *)) dlsym(RTLD_DEFAULT, "EI_MakeContextForImageBuffer");
	if (APICallback)
		return (APICallback(iImage));
	else
		return NULL;
}

EI_DrawContext *EI_MakeContextForDialog(EI_Dialog *iDialog)
{
	EI_DrawContext* (*APICallback)(EI_Dialog *);
		
	APICallback = (EI_DrawContext* (*)(EI_Dialog *)) dlsym(RTLD_DEFAULT, "EI_MakeContextForDialog");
	if (APICallback)
		return (APICallback(iDialog));
	else
		return NULL;
}

void EI_DestroyContext(EI_DrawContext *iContext)
{
	void (*APICallback)(EI_DrawContext *);
		
	APICallback = (void (*)(EI_DrawContext *)) dlsym(RTLD_DEFAULT, "EI_DestroyContext");
	if (APICallback)
		return (APICallback(iContext));
	else
		return;
}

void EI_SetClip(EI_DrawContext *ioContext, const EI_Rect *iClipRect)
{
	void (*APICallback)(EI_DrawContext *, const EI_Rect *);
		
	APICallback = (void (*)(EI_DrawContext *, const EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_SetClip");
	if (APICallback)
		return (APICallback(ioContext, iClipRect));
	else
		return;
}

void EI_GetClip(const EI_DrawContext *iContext, EI_Rect *oClipRect)
{
	void (*APICallback)(const EI_DrawContext *, EI_Rect *);
		
	APICallback = (void (*)(const EI_DrawContext *, EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_GetClip");
	if (APICallback)
		return (APICallback(iContext, oClipRect));
	else
		return;
}
				
void EI_SetPenMode(EI_DrawContext *ioContext, EI_PenMode iPenMode)
{
	void (*APICallback)(EI_DrawContext *, EI_PenMode );
		
	APICallback = (void (*)(EI_DrawContext *, EI_PenMode )) dlsym(RTLD_DEFAULT, "EI_SetPenMode");
	if (APICallback)
		return (APICallback(ioContext, iPenMode));
	else
		return;
}

EI_PenMode EI_GetPenMode(const EI_DrawContext *iContext)
{
	EI_PenMode (*APICallback)(const EI_DrawContext *);
		
	APICallback = (EI_PenMode (*)(const EI_DrawContext *)) dlsym(RTLD_DEFAULT, "EI_GetPenMode");
	if (APICallback)
		return (APICallback(iContext));
	else
		return cEI_PenColor;
}

void EI_MoveTo(EI_DrawContext *ioContext, int iPenH, int iPenV)
{
	void (*APICallback)(EI_DrawContext *, int , int );
		
	APICallback = (void (*)(EI_DrawContext *, int , int )) dlsym(RTLD_DEFAULT, "EI_MoveTo");
	if (APICallback)
		return (APICallback(ioContext, iPenH, iPenV));
	else
		return;
}

void EI_LineTo(EI_DrawContext *ioContext, int iPenH, int iPenV)
{
	void (*APICallback)(EI_DrawContext *, int , int );
		
	APICallback = (void (*)(EI_DrawContext *, int , int )) dlsym(RTLD_DEFAULT, "EI_LineTo");
	if (APICallback)
		return (APICallback(ioContext, iPenH, iPenV));
	else
		return;
}

void EI_GetPenLocation(const EI_DrawContext *iContext, int *oPenH, int *oPenV)
{
	void (*APICallback)(const EI_DrawContext *, int *, int *);
		
	APICallback = (void (*)(const EI_DrawContext *, int *, int *)) dlsym(RTLD_DEFAULT, "EI_GetPenLocation");
	if (APICallback)
		return (APICallback(iContext, oPenH, oPenV));
	else
		return;
}

void EI_SetForeColor(EI_DrawContext *ioContext, const EI_Color *iForeColor)
{
	void (*APICallback)(EI_DrawContext *, const EI_Color *);
		
	APICallback = (void (*)(EI_DrawContext *, const EI_Color *)) dlsym(RTLD_DEFAULT, "EI_SetForeColor");
	if (APICallback)
		return (APICallback(ioContext, iForeColor));
	else
		return;
}

void EI_GetForeColor(const EI_DrawContext *iContext, EI_Color *oForeColor)
{
	void (*APICallback)(const EI_DrawContext *, EI_Color *);
		
	APICallback = (void (*)(const EI_DrawContext *, EI_Color *)) dlsym(RTLD_DEFAULT, "EI_GetForeColor");
	if (APICallback)
		return (APICallback(iContext, oForeColor));
	else
		return;
}

void EI_SetBackColor(EI_DrawContext *ioContext, const EI_Color *iBackColor)
{
	void (*APICallback)(EI_DrawContext *, const EI_Color *);
		
	APICallback = (void (*)(EI_DrawContext *, const EI_Color *)) dlsym(RTLD_DEFAULT, "EI_SetBackColor");
	if (APICallback)
		return (APICallback(ioContext, iBackColor));
	else
		return;
}

void EI_GetBackColor(const EI_DrawContext *iContext, EI_Color *oBackColor)
{
	void (*APICallback)(const EI_DrawContext *, EI_Color *);
		
	APICallback = (void (*)(const EI_DrawContext *, EI_Color *)) dlsym(RTLD_DEFAULT, "EI_GetBackColor");
	if (APICallback)
		return (APICallback(iContext, oBackColor));
	else
		return;
}

void EI_FrameRect(EI_DrawContext *ioContext, const EI_Rect *iFrame)
{
	void (*APICallback)(EI_DrawContext *, const EI_Rect *);
		
	APICallback = (void (*)(EI_DrawContext *, const EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_FrameRect");
	if (APICallback)
		return (APICallback(ioContext, iFrame));
	else
		return;
}

void EI_PaintRect(EI_DrawContext *ioContext, const EI_Rect *iFrame)
{
	void (*APICallback)(EI_DrawContext *, const EI_Rect *);
		
	APICallback = (void (*)(EI_DrawContext *, const EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_PaintRect");
	if (APICallback)
		return (APICallback(ioContext, iFrame));
	else
		return;
}

void EI_EraseRect(EI_DrawContext *ioContext, const EI_Rect *iFrame)
{
	void (*APICallback)(EI_DrawContext *, const EI_Rect *);
		
	APICallback = (void (*)(EI_DrawContext *, const EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_EraseRect");
	if (APICallback)
		return (APICallback(ioContext, iFrame));
	else
		return;
}

void EI_FrameOval(EI_DrawContext *ioContext, const EI_Rect *iFrame)
{
	void (*APICallback)(EI_DrawContext *, const EI_Rect *);
		
	APICallback = (void (*)(EI_DrawContext *, const EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_FrameOval");
	if (APICallback)
		return (APICallback(ioContext, iFrame));
	else
		return;
}

void EI_PaintOval(EI_DrawContext *ioContext, const EI_Rect *iFrame)
{
	void (*APICallback)(EI_DrawContext *, const EI_Rect *);
		
	APICallback = (void (*)(EI_DrawContext *, const EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_PaintOval");
	if (APICallback)
		return (APICallback(ioContext, iFrame));
	else
		return;
}

void EI_DrawImage(EI_DrawContext *ioContext, const EI_ImageBuffer *iImage,
					const EI_Rect *iSourceRect, const EI_Rect *iDestRect)
{
	void (*APICallback)(EI_DrawContext *, const EI_ImageBuffer *, const EI_Rect *, const EI_Rect *);
		
	APICallback = (void (*)(EI_DrawContext *, const EI_ImageBuffer *, const EI_Rect *, const EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_DrawImage");
	if (APICallback)
		return (APICallback(ioContext, iImage, iSourceRect, iDestRect));
	else
		return;
}

void EI_CaptureImage(const EI_DrawContext *iContext, EI_ImageBuffer *iImage,
						const EI_Rect *iSourceRect, const EI_Rect *iDestRect)
{
	void (*APICallback)(const EI_DrawContext *, EI_ImageBuffer *, const EI_Rect *, const EI_Rect *);
		
	APICallback = (void (*)(const EI_DrawContext *, EI_ImageBuffer *, const EI_Rect *, const EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_CaptureImage");
	if (APICallback)
		return (APICallback(iContext, iImage, iSourceRect, iDestRect));
	else
		return;
}

void EI_SetTextSize(EI_DrawContext *ioContext, int iSize)
{
	void (*APICallback)(EI_DrawContext *, int );
		
	APICallback = (void (*)(EI_DrawContext *, int )) dlsym(RTLD_DEFAULT, "EI_SetTextSize");
	if (APICallback)
		return (APICallback(ioContext, iSize));
	else
		return;
}

int EI_GetTextSize(const EI_DrawContext *iContext)
{
	int (*APICallback)(const EI_DrawContext *);
		
	APICallback = (int (*)(const EI_DrawContext *)) dlsym(RTLD_DEFAULT, "EI_GetTextSize");
	if (APICallback)
		return (APICallback(iContext));
	else
		return 0;
}

void EI_SetTextStyle(EI_DrawContext *ioContext, int iStyleFlags)
{
	void (*APICallback)(EI_DrawContext *, int );
		
	APICallback = (void (*)(EI_DrawContext *, int )) dlsym(RTLD_DEFAULT, "EI_SetTextStyle");
	if (APICallback)
		return (APICallback(ioContext, iStyleFlags));
	else
		return;
}

int EI_GetTextStyle(const EI_DrawContext *iContext)
{
	int (*APICallback)(const EI_DrawContext *);
		
	APICallback = (int (*)(const EI_DrawContext *)) dlsym(RTLD_DEFAULT, "EI_GetTextStyle");
	if (APICallback)
		return (APICallback(iContext));
	else
		return 0;
}

int EI_MeasureText(const EI_DrawContext *iContext, const char *iText, int iNumChars)
{
	int (*APICallback)(const EI_DrawContext *, const char *, int );
		
	APICallback = (int (*)(const EI_DrawContext *, const char *, int )) dlsym(RTLD_DEFAULT, "EI_MeasureText");
	if (APICallback)
		return (APICallback(iContext, iText, iNumChars));
	else
		return 0;
}

void EI_GetFontMetrics(const EI_DrawContext *iContext, int *oAscent, int *oDescent, int *oLeading)
{
	void (*APICallback)(const EI_DrawContext *, int *, int *, int *);
		
	APICallback = (void (*)(const EI_DrawContext *, int *, int *, int *)) dlsym(RTLD_DEFAULT, "EI_GetFontMetrics");
	if (APICallback)
		return (APICallback(iContext, oAscent, oDescent, oLeading));
	else
		return;
}

void EI_DrawText(EI_DrawContext *ioContext, const char *iText, int iNumChars)
{
	void (*APICallback)(EI_DrawContext *, const char *, int );
		
	APICallback = (void (*)(EI_DrawContext *, const char *, int )) dlsym(RTLD_DEFAULT, "EI_DrawText");
	if (APICallback)
		return (APICallback(ioContext, iText, iNumChars));
	else
		return;
}

void EI_DrawTextToFit(EI_DrawContext *ioContext, 
					  const char *iText, int iNumChars, int iMaxPixelWidth, int iAllowCondensed)
{
	void (*APICallback)(EI_DrawContext *, const char *, int , int , int );
		
	APICallback = (void (*)(EI_DrawContext *, const char *, int , int , int )) dlsym(RTLD_DEFAULT, "EI_DrawTextToFit");
	if (APICallback)
		return (APICallback(ioContext, iText, iNumChars, iMaxPixelWidth, iAllowCondensed));
	else
		return;
}

void EI_DrawTextInRect(EI_DrawContext *ioContext, 
					   const char *iText, int iNumChars, const EI_Rect *iBounds)
{
	void (*APICallback)(EI_DrawContext *, const char *, int , const EI_Rect *);
		
	APICallback = (void (*)(EI_DrawContext *, const char *, int , const EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_DrawTextInRect");
	if (APICallback)
		return (APICallback(ioContext, iText, iNumChars, iBounds));
	else
		return;
}

void EI_HiliteRect(EI_DrawContext *ioContext, 
				   const EI_Rect *iBounds, const EI_Color *iHiliteColor, const EI_Color *iBackColor)
{
	void (*APICallback)(EI_DrawContext *, const EI_Rect *, const EI_Color *, const EI_Color *);
		
	APICallback = (void (*)(EI_DrawContext *, const EI_Rect *, const EI_Color *, const EI_Color *)) dlsym(RTLD_DEFAULT, "EI_HiliteRect");
	if (APICallback)
		return (APICallback(ioContext, iBounds, iHiliteColor, iBackColor));
	else
		return;
}

EI_DirectoryRef *EI_MakeDirectoryRef(void)
{
	EI_DirectoryRef* (*APICallback)(void);
		
	APICallback = (EI_DirectoryRef* (*)(void)) dlsym(RTLD_DEFAULT, "EI_DirectoryRef");
	if (APICallback)
		return (APICallback());
	else
		return NULL;
}

void EI_DestroyDirectoryRef(EI_DirectoryRef *iDirectoryRef)
{
	void (*APICallback)(EI_DirectoryRef *);
		
	APICallback = (void (*)(EI_DirectoryRef *)) dlsym(RTLD_DEFAULT, "EI_DestroyDirectoryRef");
	if (APICallback)
		return (APICallback(iDirectoryRef));
	else
		return;
}

int EI_AppendDirectoryName(EI_DirectoryRef *ioDirectoryRef, const char *iDirectoryString)
{
	int (*APICallback)(EI_DirectoryRef *, const char *);
		
	APICallback = (int (*)(EI_DirectoryRef *, const char *)) dlsym(RTLD_DEFAULT, "EI_AppendDirectoryName");
	if (APICallback)
		return (APICallback(ioDirectoryRef, iDirectoryString));
	else
		return 0;
}

void EI_RemoveLastDirectoryName(EI_DirectoryRef *ioDirectoryRef)
{
	void (*APICallback)(EI_DirectoryRef *);
		
	APICallback = (void (*)(EI_DirectoryRef *)) dlsym(RTLD_DEFAULT, "EI_RemoveLastDirectoryName");
	if (APICallback)
		return (APICallback(ioDirectoryRef));
	else
		return;
}

int EI_GetNumDirectoryNames(const EI_DirectoryRef *iDirectoryRef)
{
	int (*APICallback)(const EI_DirectoryRef *);
		
	APICallback = (int (*)(const EI_DirectoryRef *)) dlsym(RTLD_DEFAULT, "EI_GetNumDirectoryNames");
	if (APICallback)
		return (APICallback(iDirectoryRef));
	else
		return 0;
}

int EI_GetDirectoryNameSize(const EI_DirectoryRef *iDirectoryRef, int iNameIndex)
{
	int (*APICallback)(const EI_DirectoryRef *, int);
		
	APICallback = (int (*)(const EI_DirectoryRef *, int)) dlsym(RTLD_DEFAULT, "EI_GetDirectoryNameSize");
	if (APICallback)
		return (APICallback(iDirectoryRef, iNameIndex));
	else
		return 0;
}

void EI_GetDirectoryName(const EI_DirectoryRef *iDirectoryRef,
							int iNameIndex, char *iText, int *ioNumChars)
{
	void (*APICallback)(const EI_DirectoryRef *, int, char*, int*);
		
	APICallback = (void (*)(const EI_DirectoryRef *, int, char*, int*)) dlsym(RTLD_DEFAULT, "EI_GetDirectoryName");
	if (APICallback)
		return (APICallback(iDirectoryRef, iNameIndex, iText, ioNumChars));
	else
		return;
}

int EI_GetDirectoryPathNameSize(const EI_DirectoryRef *iDirectoryRef)
{
	int (*APICallback)(const EI_DirectoryRef *);
		
	APICallback = (int (*)(const EI_DirectoryRef *)) dlsym(RTLD_DEFAULT, "EI_GetDirectoryPathNameSize");
	if (APICallback)
		return (APICallback(iDirectoryRef));
	else
		return 0;
}

void EI_GetDirectoryPathName(const EI_DirectoryRef *iDirectoryRef, char *iText, int *ioNumChars)
{
	void (*APICallback)(const EI_DirectoryRef *, char *, int *);
		
	APICallback = (void (*)(const EI_DirectoryRef *, char *, int *)) dlsym(RTLD_DEFAULT, "EI_GetDirectoryPathName");
	if (APICallback)
		return (APICallback(iDirectoryRef, iText, ioNumChars));
	else
		return;
}

EI_DirectoryRef *EI_GetHomeDirectory(void)
{
	EI_DirectoryRef* (*APICallback)(void);
		
	APICallback = (EI_DirectoryRef* (*)(void)) dlsym(RTLD_DEFAULT, "EI_GetHomeDirectory");
	if (APICallback)
		return (APICallback());
	else
		return NULL;
}

EI_DirectoryRef *EI_GetPreferencesDirectory(void)
{
	EI_DirectoryRef* (*APICallback)(void);
		
	APICallback = (EI_DirectoryRef* (*)(void)) dlsym(RTLD_DEFAULT, "EI_GetPreferencesDirectory");
	if (APICallback)
		return (APICallback());
	else
		return NULL;
}

EI_DirectoryRef *EI_GetTemporaryDirectory(void)
{
	EI_DirectoryRef* (*APICallback)(void);
		
	APICallback = (EI_DirectoryRef* (*)(void)) dlsym(RTLD_DEFAULT, "EI_GetTemporaryDirectory");
	if (APICallback)
		return (APICallback());
	else
		return NULL;
}

int EI_CreateDirectory(const EI_DirectoryRef *iDirectoryRef)
{
	int (*APICallback)(const EI_DirectoryRef *);
		
	APICallback = (int (*)(const EI_DirectoryRef *)) dlsym(RTLD_DEFAULT, "EI_CreateDirectory");
	if (APICallback)
		return (APICallback(iDirectoryRef));
	else
		return 0;
}

EI_FileRef *EI_MakeEmptyFileRef(void)
{
	EI_FileRef* (*APICallback)(void);
		
	APICallback = (EI_FileRef* (*)(void)) dlsym(RTLD_DEFAULT, "EI_MakeEmptyFileRef");
	if (APICallback)
		return (APICallback());
	else
		return NULL;
}

EI_FileRef *EI_MakeFileRef(const EI_DirectoryRef *iDirectoryRef,
							const char *iNameString, unsigned long iFileType, const char *iExtensionString)
{
	EI_FileRef* (*APICallback)(const EI_DirectoryRef *, const char *, unsigned long , const char *);
		
	APICallback = (EI_FileRef* (*)(const EI_DirectoryRef *, const char *, unsigned long , const char *)) dlsym(RTLD_DEFAULT, "EI_MakeFileRef");
	if (APICallback)
		return (APICallback(iDirectoryRef, iNameString, iFileType, iExtensionString));
	else
		return NULL;
}

void EI_DestroyFileRef(EI_FileRef *iFileRef)
{
	void (*APICallback)(EI_FileRef *);
		
	APICallback = (void (*)(EI_FileRef *)) dlsym(RTLD_DEFAULT, "EI_DestroyFileRef");
	if (APICallback)
		return (APICallback(iFileRef));
	else
		return;
}

EI_DirectoryRef *EI_GetFileDirectory(EI_FileRef *iFileRef)
{
	EI_DirectoryRef* (*APICallback)(EI_FileRef *);
		
	APICallback = (EI_DirectoryRef* (*)(EI_FileRef *)) dlsym(RTLD_DEFAULT, "EI_GetFileDirectory");
	if (APICallback)
		return (APICallback(iFileRef));
	else
		return NULL;
}

int EI_SetFileDirectory(EI_FileRef *iFileRef, const EI_DirectoryRef *iNewDirectoryRef)
{
	int (*APICallback)(EI_FileRef *, const EI_DirectoryRef *);
		
	APICallback = (int (*)(EI_FileRef *, const EI_DirectoryRef *)) dlsym(RTLD_DEFAULT, "EI_SetFileDirectory");
	if (APICallback)
		return (APICallback(iFileRef, iNewDirectoryRef));
	else
		return 0;
}

int EI_SetFileRefType(EI_FileRef *iFileRef, unsigned long iFileType)
{
	int (*APICallback)(EI_FileRef *, unsigned long);
		
	APICallback = (int (*)(EI_FileRef *, unsigned long)) dlsym(RTLD_DEFAULT, "EI_SetFileRefType");
	if (APICallback)
		return (APICallback(iFileRef, iFileType));
	else
		return 0;
}

unsigned long EI_GetFileRefType(const EI_FileRef *iFileRef)
{
	unsigned long (*APICallback)(const EI_FileRef *);
		
	APICallback = (unsigned long (*)(const EI_FileRef *)) dlsym(RTLD_DEFAULT, "EI_GetFileRefType");
	if (APICallback)
		return (APICallback(iFileRef));
	else
		return 0;
}

int EI_SetFileRefExtension(EI_FileRef *iFileRef, const char *iExtensionString)
{
	int (*APICallback)(EI_FileRef *, const char *);
		
	APICallback = (int (*)(EI_FileRef *, const char *)) dlsym(RTLD_DEFAULT, "EI_SetFileRefExtension");
	if (APICallback)
		return (APICallback(iFileRef, iExtensionString));
	else
		return 0;
}

int EI_GetFileRefExtensionSize(const EI_FileRef *iFileRef)
{
	int (*APICallback)(const EI_FileRef *);
		
	APICallback = (int (*)(const EI_FileRef *)) dlsym(RTLD_DEFAULT, "EI_GetFileRefExtensionSize");
	if (APICallback)
		return (APICallback(iFileRef));
	else
		return 0;
}

void EI_GetFileRefExtension(const EI_FileRef *iFileRef, char *iText, int *ioNumChars)
{
	void (*APICallback)(const EI_FileRef *, char *, int *);
		
	APICallback = (void (*)(const EI_FileRef *, char *, int *)) dlsym(RTLD_DEFAULT, "EI_GetFileRefExtension");
	if (APICallback)
		return (APICallback(iFileRef, iText, ioNumChars));
	else
		return;
}

int EI_SetFileName(EI_FileRef *iFileRef, const char *iNameString)
{
	int (*APICallback)(EI_FileRef *, const char *);
		
	APICallback = (int (*)(EI_FileRef *, const char *)) dlsym(RTLD_DEFAULT, "EI_SetFileName");
	if (APICallback)
		return (APICallback(iFileRef, iNameString));
	else
		return 0;
}

int EI_GetFileNameSize(const EI_FileRef *iFileRef)
{
	int (*APICallback)(const EI_FileRef *);
		
	APICallback = (int (*)(const EI_FileRef *)) dlsym(RTLD_DEFAULT, "EI_GetFileNameSize");
	if (APICallback)
		return (APICallback(iFileRef));
	else
		return 0;
}

void EI_GetFileName(const EI_FileRef *iFileRef, char *iText, int *ioNumChars)
{
	void (*APICallback)(const EI_FileRef *, char *, int *);
		
	APICallback = (void (*)(const EI_FileRef *, char *, int *)) dlsym(RTLD_DEFAULT, "EI_GetFileName");
	if (APICallback)
		return (APICallback(iFileRef, iText, ioNumChars));
	else
		return;
}

int EI_GetFilePathNameSize(const EI_FileRef *iFileRef)
{
	int (*APICallback)(const EI_FileRef *);
		
	APICallback = (int (*)(const EI_FileRef *)) dlsym(RTLD_DEFAULT, "EI_GetFilePathNameSize");
	if (APICallback)
		return (APICallback(iFileRef));
	else
		return 0;
}

void EI_GetFilePathName(const EI_FileRef *iFileRef, char *iText, int *ioNumChars)
{
	void (*APICallback)(const EI_FileRef *, char *, int *);
		
	APICallback = (void (*)(const EI_FileRef *, char *, int *)) dlsym(RTLD_DEFAULT, "EI_GetFilePathName");
	if (APICallback)
		return (APICallback(iFileRef, iText, ioNumChars));
	else
		return;
}

EI_FileRef *EI_AskUserForNewFile(const char *iPromptString)
{
	EI_FileRef* (*APICallback)(const char *);
		
	APICallback = (EI_FileRef* (*)(const char *)) dlsym(RTLD_DEFAULT, "EI_AskUserForNewFile");
	if (APICallback)
		return (APICallback(iPromptString));
	else
		return NULL;
}

EI_FileRef *EI_AskUserForExistingFile(const char *iPromptString)
{
	EI_FileRef* (*APICallback)(const char *);
		
	APICallback = (EI_FileRef* (*)(const char *)) dlsym(RTLD_DEFAULT, "EI_AskUserForExistingFile");
	if (APICallback)
		return (APICallback(iPromptString));
	else
		return NULL;
}

int EI_DeleteFile(const EI_FileRef *iFileRef)
{
	int (*APICallback)(const EI_FileRef *);
		
	APICallback = (int (*)(const EI_FileRef *)) dlsym(RTLD_DEFAULT, "EI_DeleteFile");
	if (APICallback)
		return (APICallback(iFileRef));
	else
		return 0;
}

EI_ImageBuffer *EI_MakeImageBuffer(const EI_Rect *iBounds)
{
	EI_ImageBuffer* (*APICallback)(const EI_Rect *);
		
	APICallback = (EI_ImageBuffer* (*)(const EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_MakeImageBuffer");
	if (APICallback)
		return (APICallback(iBounds));
	else
		return NULL;
}

void EI_DestroyImageBuffer(EI_ImageBuffer *iImage)
{
	void (*APICallback)(EI_ImageBuffer *);
		
	APICallback = (void (*)(EI_ImageBuffer *)) dlsym(RTLD_DEFAULT, "EI_DestroyImageBuffer");
	if (APICallback)
		return (APICallback(iImage));
	else
		return;
}

int EI_GetBufferRowBytes(const EI_ImageBuffer *iImage)
{
	int (*APICallback)(const EI_ImageBuffer *);
		
	APICallback = (int (*)(const EI_ImageBuffer *)) dlsym(RTLD_DEFAULT, "EI_GetBufferRowBytes");
	if (APICallback)
		return (APICallback(iImage));
	else
		return 0;
}

char *EI_GetBufferBaseAddress(EI_ImageBuffer *iImage)
{
	char* (*APICallback)(EI_ImageBuffer *);
		
	APICallback = (char* (*)(EI_ImageBuffer *)) dlsym(RTLD_DEFAULT, "EI_GetBufferBaseAddress");
	if (APICallback)
		return (APICallback(iImage));
	else
		return NULL;
}

void EI_GetBufferBounds(const EI_ImageBuffer *iImage, EI_Rect *oRect)
{
	void (*APICallback)(const EI_ImageBuffer *, EI_Rect *);
		
	APICallback = (void (*)(const EI_ImageBuffer *, EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_GetBufferBounds");
	if (APICallback)
		return (APICallback(iImage, oRect));
	else
		return;
}

int EI_GetRectWidth(const EI_Rect *iRect)
{
	int (*APICallback)(const EI_Rect *);
		
	APICallback = (int (*)(const EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_GetRectWidth");
	if (APICallback)
		return (APICallback(iRect));
	else
		return 0;
}

int EI_GetRectHeight(const EI_Rect *iRect)
{
	int (*APICallback)(const EI_Rect *);
		
	APICallback = (int (*)(const EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_GetRectHeight");
	if (APICallback)
		return (APICallback(iRect));
	else
		return 0;
}

void EI_SetRect(EI_Rect *oRect, int iLeft, int iTop, int iRight, int iBottom)
{
	void (*APICallback)(EI_Rect *, int , int , int , int );
		
	APICallback = (void (*)(EI_Rect *, int , int , int , int )) dlsym(RTLD_DEFAULT, "EI_SetRect");
	if (APICallback)
		return (APICallback(oRect, iLeft, iTop, iRight, iBottom));
	else
		return;
}

int EI_IsRectEmpty(const EI_Rect *iRect)
{
	int (*APICallback)(const EI_Rect *);
		
	APICallback = (int (*)(const EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_IsRectEmpty");
	if (APICallback)
		return (APICallback(iRect));
	else
		return 0;
}

int EI_IsPtInRect(const EI_Rect *iRect, int iX, int iY)
{
	int (*APICallback)(const EI_Rect *, int, int);
		
	APICallback = (int (*)(const EI_Rect *, int, int)) dlsym(RTLD_DEFAULT, "EI_IsPtInRect");
	if (APICallback)
		return (APICallback(iRect, iX, iY));
	else
		return 0;
}

void EI_UnionRects(const EI_Rect *iRect1, const EI_Rect *iRect2, EI_Rect *oResult)
{
	void (*APICallback)(const EI_Rect *, const EI_Rect *, EI_Rect *);
		
	APICallback = (void (*)(const EI_Rect *, const EI_Rect *, EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_UnionRects");
	if (APICallback)
		return (APICallback(iRect1, iRect2, oResult));
	else
		return;
}

int EI_IntersectRects(const EI_Rect *iRect1, const EI_Rect *iRect2, EI_Rect *oResult)
{
	int (*APICallback)(const EI_Rect *, const EI_Rect *, EI_Rect *);
		
	APICallback = (int (*)(const EI_Rect *, const EI_Rect *, EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_IntersectRects");
	if (APICallback)
		return (APICallback(iRect1, iRect2, oResult));
	else
		return 0;
}

void EI_InsetRect(EI_Rect *ioRect, int iInsetX, int iInsetY)
{
	void (*APICallback)(EI_Rect *, int, int);
		
	APICallback = (void (*)(EI_Rect *, int, int)) dlsym(RTLD_DEFAULT, "EI_InsetRect");
	if (APICallback)
		return (APICallback(ioRect, iInsetX, iInsetY));
	else
		return;
}

void EI_OffsetRect(EI_Rect *ioRect,	int iOffsetX, int iOffsetY)
{
	void (*APICallback)(const EI_Rect *, int, int);
		
	APICallback = (void (*)(const EI_Rect *, int, int)) dlsym(RTLD_DEFAULT, "EI_OffsetRect");
	if (APICallback)
		return (APICallback(ioRect, iOffsetX, iOffsetY));
	else
		return;
}

int EI_AreRectsEqual(const EI_Rect *iRect1, const EI_Rect *iRect2)
{
	int (*APICallback)(const EI_Rect *, const EI_Rect *);
		
	APICallback = (int (*)(const EI_Rect *, const EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_AreRectsEqual");
	if (APICallback)
		return (APICallback(iRect1, iRect2));
	else
		return 0;
}

EI_ResourceFile *EI_OpenResourceFile(const EI_FileRef *iFileRef)
{
	EI_ResourceFile* (*APICallback)(const EI_FileRef *);
		
	APICallback = (EI_ResourceFile* (*)(const EI_FileRef *)) dlsym(RTLD_DEFAULT, "EI_OpenResourceFile");
	if (APICallback)
		return (APICallback(iFileRef));
	else
		return NULL;
}

void EI_CloseResourceFile(EI_ResourceFile *iResourceFile)
{
	void (*APICallback)(EI_ResourceFile *);
		
	APICallback = (void (*)(EI_ResourceFile *)) dlsym(RTLD_DEFAULT, "EI_CloseResourceFile");
	if (APICallback)
		return (APICallback(iResourceFile));
	else
		return;
}

void *EI_GetResource(EI_ResourceFile *iResourceFile,
						unsigned long iResourceType,  int iResourceID, long *oResourceSize)
{
	void * (*APICallback)(EI_ResourceFile *, unsigned long ,  int , long *);
		
	APICallback = (void *(*)(EI_ResourceFile *, unsigned long ,  int , long *)) dlsym(RTLD_DEFAULT, "EI_GetResource");
	if (APICallback)
		return (APICallback(iResourceFile, iResourceType, iResourceID, oResourceSize));
	else
		return NULL;
}

void EI_ReleaseResource(void* iResource)
{
	void (*APICallback)(void*);
		
	APICallback = (void (*)(void*)) dlsym(RTLD_DEFAULT, "EI_ReleaseResource");
	if (APICallback)
		return (APICallback(iResource));
	else
		return;
}

EI_Stream *EI_OpenFileStream(const EI_FileRef *iFileRef, EI_StreamAccess iAccess)
{
	EI_Stream* (*APICallback)(const EI_FileRef *, EI_StreamAccess);
		
	APICallback = (EI_Stream* (*)(const EI_FileRef *, EI_StreamAccess)) dlsym(RTLD_DEFAULT, "EI_OpenFileStream");
	if (APICallback)
		return (APICallback(iFileRef, iAccess));
	else
		return NULL;
}

void EI_CloseStream(EI_Stream *iStream)
{
	void (*APICallback)(EI_Stream *);
		
	APICallback = (void (*)(EI_Stream *)) dlsym(RTLD_DEFAULT, "EI_CloseStream");
	if (APICallback)
		return (APICallback(iStream));
	else
		return;
}

int EI_SetStreamPosition(EI_Stream *iStream, long iPosition)
{
	int (*APICallback)(EI_Stream *, long );
		
	APICallback = (int (*)(EI_Stream *, long )) dlsym(RTLD_DEFAULT, "EI_SetStreamPosition");
	if (APICallback)
		return (APICallback(iStream, iPosition));
	else
		return 0;
}

long EI_GetStreamPosition(EI_Stream *iStream)
{
	long (*APICallback)(EI_Stream *);
		
	APICallback = (long (*)(EI_Stream *)) dlsym(RTLD_DEFAULT, "EI_GetStreamPosition");
	if (APICallback)
		return (APICallback(iStream));
	else
		return 0;
}

long EI_GetStreamSize(EI_Stream *iStream)
{
	long (*APICallback)(EI_Stream *);
		
	APICallback = (long (*)(EI_Stream *)) dlsym(RTLD_DEFAULT, "EI_GetStreamSize");
	if (APICallback)
		return (APICallback(iStream));
	else
		return 0;
}

int EI_ReadStream(EI_Stream *iStream, char *oBuffer, long *ioCount)
{
	int (*APICallback)(EI_Stream *, char *, long *);
		
	APICallback = (int (*)(EI_Stream *, char *, long *)) dlsym(RTLD_DEFAULT, "EI_ReadStream");
	if (APICallback)
		return (APICallback(iStream, oBuffer, ioCount));
	else
		return 0;
}

int EI_WriteStream(EI_Stream *iStream, char *iBuffer, long *ioCount)
{
	int (*APICallback)(EI_Stream *, char *, long *);
		
	APICallback = (int (*)(EI_Stream *, char *, long *)) dlsym(RTLD_DEFAULT, "EI_WriteStream");
	if (APICallback)
		return (APICallback(iStream, iBuffer, ioCount));
	else
		return 0;
}

int EI_WriteUInt8(EI_Stream *iStream, unsigned char iData)
{
	int (*APICallback)(EI_Stream *, unsigned char );
		
	APICallback = (int (*)(EI_Stream *, unsigned char )) dlsym(RTLD_DEFAULT, "EI_WriteUInt8");
	if (APICallback)
		return (APICallback(iStream, iData));
	else
		return 0;
}

int EI_WriteSInt16(EI_Stream *iStream, short iData)
{
	int (*APICallback)(EI_Stream *, short );
		
	APICallback = (int (*)(EI_Stream *, short )) dlsym(RTLD_DEFAULT, "EI_WriteSInt16");
	if (APICallback)
		return (APICallback(iStream, iData));
	else
		return 0;
}

int EI_WriteUInt16(EI_Stream *iStream, unsigned short iData)
{
	int (*APICallback)(EI_Stream *, unsigned short );
		
	APICallback = (int (*)(EI_Stream *, unsigned short )) dlsym(RTLD_DEFAULT, "EI_WriteUInt16");
	if (APICallback)
		return (APICallback(iStream, iData));
	else
		return 0;
}

int EI_WriteSInt32(EI_Stream *iStream, long iData)
{
	int (*APICallback)(EI_Stream *, long );
		
	APICallback = (int (*)(EI_Stream *, long )) dlsym(RTLD_DEFAULT, "EI_WriteSInt32");
	if (APICallback)
		return (APICallback(iStream, iData));
	else
		return 0;
}

int EI_WriteUInt32(EI_Stream *iStream, unsigned long iData)
{
	int (*APICallback)(EI_Stream *, unsigned long );
		
	APICallback = (int (*)(EI_Stream *, unsigned long )) dlsym(RTLD_DEFAULT, "EI_WriteUInt32");
	if (APICallback)
		return (APICallback(iStream, iData));
	else
		return 0;
}

int EI_WriteFloat32(EI_Stream *iStream, float iData)
{
	int (*APICallback)(EI_Stream *, float );
		
	APICallback = (int (*)(EI_Stream *, float )) dlsym(RTLD_DEFAULT, "EI_WriteFloat32");
	if (APICallback)
		return (APICallback(iStream, iData));
	else
		return 0;
}

int EI_WriteFloat64(EI_Stream *iStream, double iData)
{
	int (*APICallback)(EI_Stream *, double );
		
	APICallback = (int (*)(EI_Stream *, double )) dlsym(RTLD_DEFAULT, "EI_WriteFloat64");
	if (APICallback)
		return (APICallback(iStream, iData));
	else
		return 0;
}

int EI_ReadUInt8(EI_Stream *iStream, unsigned char *oData)
{
	int (*APICallback)(EI_Stream *, unsigned char *);
		
	APICallback = (int (*)(EI_Stream *, unsigned char *)) dlsym(RTLD_DEFAULT, "EI_ReadUInt8");
	if (APICallback)
		return (APICallback(iStream, oData));
	else
		return 0;
}

int EI_ReadSInt16(EI_Stream *iStream, short *oData)
{
	int (*APICallback)(EI_Stream *, short *);
		
	APICallback = (int (*)(EI_Stream *, short *)) dlsym(RTLD_DEFAULT, "EI_ReadSInt16");
	if (APICallback)
		return (APICallback(iStream, oData));
	else
		return 0;
}

int EI_ReadUInt16(EI_Stream *iStream, unsigned short *oData)
{
	int (*APICallback)(EI_Stream *, unsigned short *);
		
	APICallback = (int (*)(EI_Stream *, unsigned short *)) dlsym(RTLD_DEFAULT, "EI_ReadUInt16");
	if (APICallback)
		return (APICallback(iStream, oData));
	else
		return 0;
}

int EI_ReadSInt32(EI_Stream *iStream, long *oData)
{
	int (*APICallback)(EI_Stream *, long *);
		
	APICallback = (int (*)(EI_Stream *, long *)) dlsym(RTLD_DEFAULT, "EI_ReadSInt32");
	if (APICallback)
		return (APICallback(iStream, oData));
	else
		return 0;
}

int EI_ReadUInt32(EI_Stream *iStream, unsigned long *oData)
{
	int (*APICallback)(EI_Stream *, unsigned long *);
		
	APICallback = (int (*)(EI_Stream *, unsigned long *)) dlsym(RTLD_DEFAULT, "EI_ReadUInt32");
	if (APICallback)
		return (APICallback(iStream, oData));
	else
		return 0;
}

int EI_ReadFloat32(EI_Stream *iStream, float *oData)
{
	int (*APICallback)(EI_Stream *, float *);
		
	APICallback = (int (*)(EI_Stream *, float *)) dlsym(RTLD_DEFAULT, "EI_ReadFloat32");
	if (APICallback)
		return (APICallback(iStream, oData));
	else
		return 0;
}

int EI_ReadFloat64(EI_Stream *iStream, double *oData)
{
	int (*APICallback)(EI_Stream *, double *);
		
	APICallback = (int (*)(EI_Stream *, double *)) dlsym(RTLD_DEFAULT, "EI_ReadFloat64");
	if (APICallback)
		return (APICallback(iStream, oData));
	else
		return 0;
}

int EI_FileRefToFSSpec(EI_FileRef *iFileRef, FSSpec *oFSSpec)
{
	int (*APICallback)(EI_FileRef *, FSSpec *);
		
	APICallback = (int (*)(EI_FileRef *, FSSpec *)) dlsym(RTLD_DEFAULT, "EI_FileRefToFSSpec");
	if (APICallback)
		return (APICallback(iFileRef, oFSSpec));
	else
		return 0;
}

int EI_DialogToWindowPtr(EI_Dialog *iDialog, WindowPtr *oWindowPtr)
{
	int (*APICallback)(EI_Dialog *, WindowPtr *);
		
	APICallback = (int (*)(EI_Dialog *, WindowPtr *)) dlsym(RTLD_DEFAULT, "EI_DialogToWindowPtr");
	if (APICallback)
		return (APICallback(iDialog, oWindowPtr));
	else
		return 0;
}

int EI_ImageBufferToGWorld(EI_ImageBuffer *iImageBuffer, GWorldPtr *oGWorld)
{
	int (*APICallback)(EI_ImageBuffer *, GWorldPtr *);
		
	APICallback = (int (*)(EI_ImageBuffer *, GWorldPtr *)) dlsym(RTLD_DEFAULT, "EI_ImageBufferToGWorld");
	if (APICallback)
		return (APICallback(iImageBuffer, oGWorld));
	else
		return 0;
}

EI_Timer *EI_MakeTimer(void)
{
	EI_Timer* (*APICallback)(void);
		
	APICallback = (EI_Timer* (*)(void)) dlsym(RTLD_DEFAULT, "EI_MakeTimer");
	if (APICallback)
		return (APICallback());
	else
		return NULL;
}

void EI_DestroyTimer(EI_Timer *iTimer)
{
	void (*APICallback)(EI_Timer *);
		
	APICallback = (void (*)(EI_Timer *)) dlsym(RTLD_DEFAULT, "EI_DestroyTimer");
	if (APICallback)
		return (APICallback(iTimer));
	else
		return;
}

void EI_SetTimerFunction(EI_Timer *ioTimer, EI_TimerFunction iFunction)
{
	void (*APICallback)(EI_Timer *, EI_TimerFunction);
		
	APICallback = (void (*)(EI_Timer *, EI_TimerFunction)) dlsym(RTLD_DEFAULT, "EI_SetTimerFunction");
	if (APICallback)
		return (APICallback(ioTimer, iFunction));
	else
		return;
}

void EI_SetTimerExtraData(EI_Timer *ioTimer, void *iExtraData)
{
	void (*APICallback)(EI_Timer *, void *);
		
	APICallback = (void (*)(EI_Timer *, void *)) dlsym(RTLD_DEFAULT, "EI_SetTimerExtraData");
	if (APICallback)
		return (APICallback(ioTimer, iExtraData));
	else
		return;
}

void *EI_GetTimerExtraData(const EI_Timer *iTimer)
{
	void *(*APICallback)(const EI_Timer *);
		
	APICallback = (void *(*)(const EI_Timer *)) dlsym(RTLD_DEFAULT, "EI_GetTimerExtraData");
	if (APICallback)
		return (APICallback(iTimer));
	else
		return NULL;
}

int EI_StartPeriodicTimer(EI_Timer *iTimer, int iMilliseconds)
{
	int (*APICallback)(EI_Timer *, int );
		
	APICallback = (int (*)(EI_Timer *, int )) dlsym(RTLD_DEFAULT, "EI_StartPeriodicTimer");
	if (APICallback)
		return (APICallback(iTimer, iMilliseconds));
	else
		return 0;
}

int EI_StartOneShotTimer(EI_Timer *iTimer,	int iMilliseconds)
{
	int (*APICallback)(EI_Timer *, int );
		
	APICallback = (int (*)(EI_Timer *, int )) dlsym(RTLD_DEFAULT, "EI_StartOneShotTimer");
	if (APICallback)
		return (APICallback(iTimer, iMilliseconds));
	else
		return 0;
}

int EI_StopTimer(EI_Timer *iTimer)
{
	int (*APICallback)(EI_Timer *);
		
	APICallback = (int (*)(EI_Timer *)) dlsym(RTLD_DEFAULT, "EI_StopTimer");
	if (APICallback)
		return (APICallback(iTimer));
	else
		return 0;
}

void EI_SetGLDrawFunction(EI_Control *ioControl,
						  EI_GLDrawFunction iFunction)
{
	void (*APICallback)(EI_Control *, EI_GLDrawFunction );
		
	APICallback = (void (*)(EI_Control *, EI_GLDrawFunction )) dlsym(RTLD_DEFAULT, "EI_SetGLDrawFunction");
	if (APICallback)
		return (APICallback(ioControl, iFunction));
	else
		return;
}

void EI_SetGLClickFunction(EI_Control *ioControl,
						   EI_GLClickFunction iFunction)
{
	void (*APICallback)(EI_Control *, EI_GLClickFunction );
		
	APICallback = (void (*)(EI_Control *, EI_GLClickFunction )) dlsym(RTLD_DEFAULT, "EI_SetGLClickFunction");
	if (APICallback)
		return (APICallback(ioControl, iFunction));
	else
		return;
}

int EI_BeginGLDrawing(const EI_Control *iControl)
{
	int (*APICallback)(const EI_Control *);
		
	APICallback = (int (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_BeginGLDrawing");
	if (APICallback)
		return (APICallback(iControl));
	else
		return 0;
}

void EI_EndGLDrawing(const EI_Control *iControl)
{
	void (*APICallback)(const EI_Control *);
		
	APICallback = (void (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_EndGLDrawing");
	if (APICallback)
		return (APICallback(iControl));
	else
		return;
}

void EI_WindowToGL(const EI_Dialog *iDialog, int *ioX, int *ioY)
{
	void (*APICallback)(const EI_Dialog *, int *, int *);
		
	APICallback = (void (*)(const EI_Dialog *, int *, int *)) dlsym(RTLD_DEFAULT, "EI_WindowToGL");
	if (APICallback)
		return (APICallback(iDialog, ioX, ioY));
	else
		return;
}

void EI_GLToWindow(const EI_Dialog *iDialog, int *ioX, int *ioY)
{
	void (*APICallback)(const EI_Dialog *, int *, int *);
		
	APICallback = (void (*)(const EI_Dialog *, int *, int *)) dlsym(RTLD_DEFAULT, "EI_GLToWindow");
	if (APICallback)
		return (APICallback(iDialog, ioX, ioY));
	else
		return;
}

int EI_GetNumTabPanels(const EI_Control *iControl)
{
	int (*APICallback)(const EI_Control *);
		
	APICallback = (int (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_GetNumTabPanels");
	if (APICallback)
		return (APICallback(iControl));
	else
		return 0;
}

int EI_GetCurrentTabPanel(const EI_Control *iControl)
{
	int (*APICallback)(const EI_Control *);
		
	APICallback = (int (*)(const EI_Control *)) dlsym(RTLD_DEFAULT, "EI_GetCurrentTabPanel");
	if (APICallback)
		return (APICallback(iControl));
	else
		return 0;
}

void EI_SetCurrentTabPanel(EI_Control *ioControl, int iPanelNumber)
{
	void (*APICallback)(const EI_Control *, int);
		
	APICallback = (void (*)(const EI_Control *, int)) dlsym(RTLD_DEFAULT, "EI_SetCurrentTabPanel");
	if (APICallback)
		return (APICallback(ioControl, iPanelNumber));
	else
		return;
}

void EI_GetDialogSize(const EI_Dialog *iDialog,	int *oWidth, int *oHeight)
{
	void (*APICallback)(const EI_Dialog *, int *, int *);
		
	APICallback = (void (*)(const EI_Dialog *, int *, int *)) dlsym(RTLD_DEFAULT, "EI_GetDialogSize");
	if (APICallback)
		return (APICallback(iDialog, oWidth, oHeight));
	else
		return;
}

void EI_SetDialogSize(EI_Dialog *ioDialog, int iWidth, int iHeight)
{
	void (*APICallback)(EI_Dialog *, int, int);
		
	APICallback = (void (*)(EI_Dialog *, int, int)) dlsym(RTLD_DEFAULT, "EI_SetDialogSize");
	if (APICallback)
		return (APICallback(ioDialog, iWidth, iHeight));
	else
		return;
}

void EI_GetMonitorBounds(EI_Rect *oMonitorBounds)
{
	void (*APICallback)(EI_Rect *);
		
	APICallback = (void (*)(EI_Rect *)) dlsym(RTLD_DEFAULT, "EI_GetMonitorBounds");
	if (APICallback)
		return (APICallback(oMonitorBounds));
	else
		return;
}

void EI_SetDialogKeyFilterFunction(EI_Dialog* iDialog, EI_DialogKeyFilter iFunction)
{
	void (*APICallback)(EI_Dialog*, EI_DialogKeyFilter);
		
	APICallback = (void (*)(EI_Dialog*, EI_DialogKeyFilter)) dlsym(RTLD_DEFAULT, "EI_SetDialogKeyFilterFunction");
	if (APICallback)
		return (APICallback(iDialog, iFunction));
	else
		return;
}

int EI_GetControlImageID(const EI_Control *iControl)
{
	int (*APICallback)(const EI_Control*);
		
	APICallback = (int (*)(const EI_Control*)) dlsym(RTLD_DEFAULT, "EI_GetControlImageID");
	if (APICallback)
		return (APICallback(iControl));
	else
		return 0;
}

void EI_SetControlImageID(EI_Control *ioControl, int iNewImageID)
{
	void (*APICallback)(EI_Control*, int);
		
	APICallback = (void (*)(EI_Control*, int)) dlsym(RTLD_DEFAULT, "EI_SetControlImageID");
	if (APICallback)
		return (APICallback(ioControl, iNewImageID));
	else
		return;
}

EI_Cursor* EI_MakeCursor(int iCursorID)
{
	EI_Cursor* (*APICallback)(int);
	
	APICallback = (EI_Cursor* (*)(int)) dlsym(RTLD_DEFAULT, "EI_MakeCursor");
	if (APICallback)
		return (APICallback(iCursorID));
	else
		return NULL;
}

void EI_DestroyCursor(EI_Cursor *iCursor)
{
	void (*APICallback)(EI_Cursor*);
	
	APICallback = (void (*)(EI_Cursor*)) dlsym(RTLD_DEFAULT, "EI_DestroyCursor");
	if (APICallback)
		return (APICallback(iCursor));
	else
		return;
}

void EI_SetCursor(EI_Cursor *iCursor)
{
	void (*APICallback)(EI_Cursor*);
	
	APICallback = (void (*)(EI_Cursor*)) dlsym(RTLD_DEFAULT, "EI_SetCursor");
	if (APICallback)
		return (APICallback(iCursor));
	else
		return;
}

void EI_SetUserControlMouseMoved(EI_Control *ioControl, EI_UserMouseMovedFunction iFunction)
{
	void (*APICallback)(EI_Control*, EI_UserMouseMovedFunction);
		
	APICallback = (void (*)(EI_Control*, EI_UserMouseMovedFunction)) dlsym(RTLD_DEFAULT, "EI_SetUserControlMouseMoved");
	if (APICallback)
		return (APICallback(ioControl, iFunction));
	else
		return;
}

void EI_SetGLControlMouseMoved(EI_Control *ioControl, EI_GLMouseMovedFunction iFunction)
{
	void (*APICallback)(EI_Control*, EI_GLMouseMovedFunction);
		
	APICallback = (void (*)(EI_Control*, EI_GLMouseMovedFunction)) dlsym(RTLD_DEFAULT, "EI_SetGLControlMouseMoved");
	if (APICallback)
		return (APICallback(ioControl, iFunction));
	else
		return;
}

void EI_GetPushButtonLayout(const EI_Control *iControl, int *oLabelPosition,
							int *oIconPosition)
{
	void (*APICallback)(const EI_Control*, int*, int*);
		
	APICallback = (void (*)(const EI_Control*, int*, int*)) dlsym(RTLD_DEFAULT, "EI_GetPushButtonLayout");
	if (APICallback)
		return (APICallback(iControl, oLabelPosition, oIconPosition));
	else
		return;
}

void EI_SetPushButtonLayout(EI_Control *ioControl, int iLabelPosition, int iIconPosition)
{
	void (*APICallback)(EI_Control*, int, int);
		
	APICallback = (void (*)(EI_Control*, int, int)) dlsym(RTLD_DEFAULT, "EI_SetPushButtonLayout");
	if (APICallback)
		return (APICallback(ioControl, iLabelPosition, iIconPosition));
	else
		return;
}
