/*
Copyright 1995-1998 Electric Image, Inc. This source code may only be used by
registered owners of the ElectricImage Plug-In Kit. Owners of the Kit may
include the source code in their products (freeware, shareware, or commercial).
This copyright notice must be included in its original form in all such
products. This source code may not be offered for sale or distributed by any
entity without the written permission of Electric Image, Inc.
*/
#ifndef __Implicit_h__
#define __Implicit_h__

#ifndef __System_h__
#include "System.h"
#endif

#ifndef __FACTStuff_h__
#include "FACTStuff.h"
#endif

typedef struct Vertex {		/* surface vertex */
	Coordinate position;	/* vertex position */
	Coordinate normal;		/* vertex normal */
	Coordinate blur;		/* vertex motion blur vector */
	Coordinate texture;		/* vertex texture position */
	ARGB color;				/* vertex color (in HSV) */
	long index;			/* vertex index number (from AddVertex) */
} Vertex, *VertexPtr;

typedef struct BlobRec {
	Coordinate center;		/* blob center */
	Coordinate blur;		/* blob motion blur vector */
	ARGB color;				/* blob color */
	float radius;			/* blob radius */
	float n1, n2, n3, n4;	/* blob precomputed radius colors */
	struct BlobRec *next;	/* blob link */
} BlobRec, *BlobPtr;

/*	 double FunctionRoutine (x, y, z)
 *		 double x, y, z (an arbitrary 3D point)
 *	     the implicit surface function
 *	     return positive for inside, negative for outside */
typedef double (*FunctionRoutine)(double x, double y, double z);
typedef double (*ColorFunctionRoutine)(CoordPtr position, CoordPtr blur, CoordPtr texture, ARGBPtr theColor);

/* These routines are defined in MrBlobby.c */

boolean StatusTime(void);
void UpdateStatus(float thePos, long theValue);
void CheckStatus(void);
void SetupStatus(PStrPtr theUnit, PStrPtr theInfo, long min, long max, float thePos, long theValue);
char* AllocateMem(long nitems, long nbytes);
VertexPtr NewVertex(VertexPtr v);
void NewFacet(VertexPtr v1, VertexPtr v2, VertexPtr v3, VertexPtr v4);

/* This routine is defined in Implicit.c */

void Polygonize(double theSize, double x, double y, double z, boolean tetMode);

#endif
