/*
 *		Copyright (c)1991-1998 Electric Image, Inc.
 *		This code is confidential and is a trade secret of Electric Image, Inc.
 */

#include "EI_Control.h"
#include "EI_Dialog.h"
#include "EI_FileRef.h"
#include "EI_ResourceFile.h"
#include "MemoryUtils.h"
#include "PluginFPU.h"
#include "PluginServer.h"
#include "PluginUtils.h"
#include "PString.h"
#include "FPU.h"
#include <stdio.h>
#include <string.h>

#define pluginType			QUADBYTE('S', 'H', 'A', 'P')	/* The owner plugin's unique type */
#define pluginVersion		0x00000001	/* The owner plugin's version number 0:0:version:revision */
#define pluginName			PS("ElectricImage Standard Shape")
#define pluginCopyright		PS("Copyright  1991-1998, Electric Image, Inc.")
#define pluginAuthor		PS("Markus Houy|Electric Image, Inc.|117 East Colorado Boulevard, Suite 300|Pasadena, Ca 91105|(818) 577-1627")
#define pluginDescription	PS("This plugin creates five standard solids.|They are plane, cube, cylinder, cone, and sphere.")
#define pluginPermission	BIT(pluginSensitivityFlag)

enum {
	interfaceDialogID = 16001
};

enum {
	cShapeRB = 4,
	cPolygonColorCB = 5,
	cPolygonColorST = 7,
	cOrientationST = 8,
	cOrientationRB = 9,
	cET1 = 10,
	cET2 = 11,
	cET3 = 12,
	cET4 = 13,
	cTotalPolygonsST = 16
};

enum {
	kStandardModelNone,
	kStandardModelPlane,
	kStandardModelCube,
	kStandardModelCylinder,
	kStandardModelCone,
	kStandardModelSphere,
	kStandardModelTeapot
};

enum {
	kPlaneYZ = 1,
	kPlaneXZ,
	kPlaneXY
};

//	"Backwards compatibility":
enum {
	oldControlSize = 136,
	
	oldStandardModelType = 0,
	oldStandardModelFlag = 2,
	oldStandardModelColor = 6,
	
	oldRunTimeTiming = 10,
	oldRunTimeTotalPolygons = 14,
	oldRunTimeCurrentPolygons = 18,
	
	oldPlaneWidth = 22,
	oldPlaneLength = 30,
	oldPlaneDivision = 38,
	oldPlaneControl = 42,

	oldCubeWidth = 44,
	oldCubeLength = 52,
	oldCubeHeight = 60,
	oldCubeDivision = 68,

	oldCylinderRadius = 72,
	oldCylinderHeight = 80,
	oldCylinderLatitude = 88,
	oldCylinderLongitude = 92,

	oldConeRadius = 96,
	oldConeHeight = 104,
	oldConeLatitude = 112,
	oldConeLongitude = 116,

	oldSphereRadius = 120,
	oldSphereLatitude = 128,
	oldSphereLongitude = 132
};

#if 0
#define kStandardModelNone		0
#define kStandardModelPlane		1
#define kStandardModelCube		2
#define kStandardModelCylinder	3
#define kStandardModelCone		4
#define kStandardModelSphere	5
#define kStandardModelTeapot	6

#define kPlaneYZ		1
#define kPlaneXZ		2
#define kPlaneXY		3

#define oldControlSize				136

#define oldStandardModelType		0
#define oldStandardModelFlag		2
#define oldStandardModelColor		6

#define oldRunTimeTiming			10
#define oldRunTimeTotalPolygons		14
#define oldRunTimeCurrentPolygons	18

#define oldPlaneWidth				22
#define oldPlaneLength				30
#define oldPlaneDivision			38
#define oldPlaneControl				42

#define oldCubeWidth				44
#define oldCubeLength				52
#define oldCubeHeight				60
#define oldCubeDivision				68

#define oldCylinderRadius			72
#define oldCylinderHeight			80
#define oldCylinderLatitude			88
#define oldCylinderLongitude		92

#define oldConeRadius				96
#define oldConeHeight				104
#define oldConeLatitude				112
#define oldConeLongitude			116

#define oldSphereRadius				120
#define oldSphereLatitude			128
#define oldSphereLongitude			132

#define aboutDialogID		16000
#define interfaceDialogID	16001

#define guiPlaneID			5
#define guiCubeID			6
#define guiCylinderID		7
#define guiConeID			8
#define guiSphereID			9

#define guiPlaneWidthText			10
#define guiPlaneLengthText			11
#define guiPlaneDivisionText		12
#define guiPlaneXYButton			13
#define guiPlaneYZButton			14
#define guiPlaneXZButton			15
#define guiPlaneWidthTitle			16
#define guiPlaneLengthTitle			17
#define guiPlaneDivisionTitle		18
#define guiPlaneOrientationTitle	19

#define guiCubeWidthText			20
#define guiCubeLengthText			21
#define guiCubeHeightText			22
#define guiCubeDivisionText			23
#define guiCubeWidthTitle			24
#define guiCubeLengthTitle			25
#define guiCubeHeightTitle			26
#define guiCubeDivisionTitle		27

#define guiCylinderRadiusText		28
#define guiCylinderHeightText		29
#define guiCylinderLatitudeText		30
#define guiCylinderLongitudeText	31
#define guiCylinderRadiusTitle		32
#define guiCylinderHeightTitle		33
#define guiCylinderLatitudeTitle	34
#define guiCylinderLongitudeTitle	35

#define guiSphereRadiusText			36
#define guiSphereLatitudeText		37
#define guiSphereLongitudeText		38
#define guiSphereRadiusTitle		39
#define guiSphereLatitudeTitle		40
#define guiSphereLongitudeTitle		41

#define guiConeRadiusText			42
#define guiConeHeightText			43
#define guiConeLatitudeText			44
#define guiConeLongitudeText		45
#define guiConeRadiusTitle			46
#define guiConeHeightTitle			47
#define guiConeLatitudeTitle		48
#define guiConeLongitudeTitle		49

#define guiTotalPolygonText			50

#define guiColorButton				52
#endif

typedef struct StandardModelControlRec {
	unsigned long ownerType;			/* The owner plugin's unique type */
	long ownerVersion;   	/* The owner plugin's version number */

	double planeWidth;
	double planeLength;
	double cubeWidth;
	double cubeLength;
	double cubeHeight;
	double cylinderRadius;
	double cylinderHeight;
	double coneRadius;
	double coneHeight;
	double sphereRadius;

	long StandardModelType;
	long StandardModelFlag;
	ARGB StandardModelColor;
	long runTimeTiming;
	long runTimeTotalPolygons;
	long runTimeCurrentPolygons;
	long planeDivision;
	long planeControl;
	long cubeDivision;
	long cylinderLatitude;
	long cylinderLongitude;
	long coneLatitude;
	long coneLongitude;
	long sphereLatitude;
	long sphereLongitude;
	long filler;
} StandardModelControlRec, *StandardModelControlPtr;

typedef struct StandardModelDataRec {
	Matrix4 theMatrix;
	Matrix4 theRotMatrix;
	Matrix4 theBlurMatrix;
} StandardModelDataRec, *StandardModelDataPtr;

typedef struct VertexTableRecord {
	XCoordinate vertexArray[4];
	XCoordinate vertexNormal[4];
	long vertexIndex[4];
} VertexTableRecord, *VertexTablePtr;

typedef double CosTableArray[2701];

/* --------------------------------------------------------------- Internal Normal routines */

static void Normalize(XCoordinate *theNormal)
{
	double theNormalLength;

	theNormalLength = SQR(theNormal->x) + SQR(theNormal->y) + SQR(theNormal->z);
	if (theNormalLength > 0.0) {
		theNormalLength = 1.0 / SQRT(theNormalLength);
		theNormal->x *= theNormalLength;
		theNormal->y *= theNormalLength;
		theNormal->z *= theNormalLength;
	}
}

static void TransformPoint(double *x, double *y, double *z, Matrix4 *theMatrix)
{
	double xNew, yNew;

	xNew = *x * theMatrix->m.m00 + *y * theMatrix->m.m10 + *z * theMatrix->m.m20 + theMatrix->m.m30;
	yNew = *x * theMatrix->m.m01 + *y * theMatrix->m.m11 + *z * theMatrix->m.m21 + theMatrix->m.m31;
	*z = *x * theMatrix->m.m02 + *y * theMatrix->m.m12 + *z * theMatrix->m.m22 + theMatrix->m.m32;
	*x = xNew;
	*y = yNew;
}  /* TransformPoint */

/* --------------------------------------------------------------- Internal Creation routines */
static long CalculatePolygonTotal(StandardModelControlPtr theStandardModelControl)
{
	switch (theStandardModelControl->StandardModelType) {

	  case kStandardModelPlane:
		if (theStandardModelControl->planeDivision > 0)
			theStandardModelControl->runTimeTotalPolygons = theStandardModelControl->planeDivision * theStandardModelControl->planeDivision;
		else
			theStandardModelControl->runTimeTotalPolygons = 1;
		break;

	  case kStandardModelCube:
		if (theStandardModelControl->cubeDivision > 0)
			theStandardModelControl->runTimeTotalPolygons = theStandardModelControl->cubeDivision * theStandardModelControl->cubeDivision * 6;
		else
			theStandardModelControl->runTimeTotalPolygons = 6;
		break;

	  case kStandardModelCylinder:
		theStandardModelControl->runTimeTotalPolygons = ((theStandardModelControl->cylinderLongitude - 1) * theStandardModelControl->cylinderLongitude / 2 + theStandardModelControl->cylinderLongitude) * 8 + theStandardModelControl->cylinderLongitude * (theStandardModelControl->cylinderLatitude + 1) * 4;
		break;

	  case kStandardModelCone:
		theStandardModelControl->runTimeTotalPolygons = ((theStandardModelControl->coneLongitude - 1) * theStandardModelControl->coneLongitude / 2 + theStandardModelControl->coneLongitude) * 4 + theStandardModelControl->coneLongitude * theStandardModelControl->coneLatitude * 4;
		break;

	  case kStandardModelSphere:
		theStandardModelControl->runTimeTotalPolygons = theStandardModelControl->sphereLongitude * theStandardModelControl->sphereLatitude * 8;
		break;

	  case kStandardModelTeapot:
		theStandardModelControl->runTimeTotalPolygons = 0;
		break;

	  default:
		theStandardModelControl->runTimeTotalPolygons = 0;
		break;
	}
	theStandardModelControl->runTimeCurrentPolygons = 0;
	return theStandardModelControl->runTimeTotalPolygons;
}

static long AddGroup(PluginInfoPtr thePluginInfo, char* groupReference, long groupVertexFlags)
{
	HostModelAddGroupRec theAddGroup;

	theAddGroup.theReference = thePluginInfo->theHostReference;
	theAddGroup.theGroup = NULL;
	theAddGroup.theGroupTemplate = groupReference;
	theAddGroup.theGroupVertexFlags = groupVertexFlags;
	return thePluginInfo->theHostProc(PLUGCOMMAND(hostModelAddGroup), PARAMETERSIZE(sizeof(HostModelAddGroupRec)), &theAddGroup);
}  /* AddGroup */

static boolean CreateStart(StandardModelControlPtr theStandardModelControl, PluginInfoPtr thePluginInfo)
{
	long theError;
	HostStatusRec theHostStatus;

	theHostStatus.theReference = thePluginInfo->theHostReference;
	theHostStatus.theStatusPos = 0.0;
	PStrCpy(theHostStatus.theStatusUnit, PS("Polygons"));
	switch (theStandardModelControl->StandardModelType) {

	  case kStandardModelPlane:
		PStrCpy(theHostStatus.theStatusInfo, PS("Creating Plane"));
		break;

	  case kStandardModelCube:
		PStrCpy(theHostStatus.theStatusInfo, PS("Creating Cube"));
		break;

	  case kStandardModelCylinder:
		PStrCpy(theHostStatus.theStatusInfo, PS("Creating Cylinder"));
		break;

	  case kStandardModelCone:
		PStrCpy(theHostStatus.theStatusInfo, PS("Creating Cone"));
		break;

	  case kStandardModelSphere:
		PStrCpy(theHostStatus.theStatusInfo, PS("Creating Sphere"));
		break;

	  case kStandardModelTeapot:
		PStrCpy(theHostStatus.theStatusInfo, PS("Creating Teapot"));
		break;

	  default:
		PStrCpy(theHostStatus.theStatusInfo, PS("Creating Unknown"));
		break;
	}
	theHostStatus.theStatusMin = 1;
	theHostStatus.theStatusMax = CalculatePolygonTotal(theStandardModelControl);
	theHostStatus.theStatusValue = 1;
	theHostStatus.theStatusTick = 0;

	theError = thePluginInfo->theHostProc(PLUGCOMMAND(hostStatus), PARAMETERSIZE(sizeof(HostStatusRec)), &theHostStatus);

	if (theError == 0)
		theStandardModelControl->runTimeTiming = theHostStatus.theStatusTick;

	return theError == 0;
}  /* CreateStart */

static boolean CreateUpdate(StandardModelControlPtr theStandardModelControl, PluginInfoPtr thePluginInfo)
{
	long theError;
	boolean callStatus;
	HostStatusRec theHostStatus;

	theError = 0;
	callStatus = 0;
	if (theStandardModelControl->runTimeTiming < TICKCOUNT() || theStandardModelControl->runTimeCurrentPolygons == theStandardModelControl->runTimeTotalPolygons) {
		callStatus = 1;
		theHostStatus.theReference = thePluginInfo->theHostReference;
		theHostStatus.theStatusPos = (double)theStandardModelControl->runTimeCurrentPolygons / theStandardModelControl->runTimeTotalPolygons;
		theHostStatus.theStatusUnit[0] = '\0';
		theHostStatus.theStatusInfo[0] = '\0';
		theHostStatus.theStatusMin = 1;
		theHostStatus.theStatusMax = theStandardModelControl->runTimeTotalPolygons;
		theHostStatus.theStatusValue = theStandardModelControl->runTimeCurrentPolygons;
		theHostStatus.theStatusTick = theStandardModelControl->runTimeTiming;
	}

	if (callStatus) {
		theError = thePluginInfo->theHostProc(PLUGCOMMAND(hostStatus), PARAMETERSIZE(sizeof(HostStatusRec)), &theHostStatus);
	
		if (theError == 0)
			theStandardModelControl->runTimeTiming = theHostStatus.theStatusTick;
	}
	return theError == 0;
}  /* CreateUpdate */

static long CreateVertex(XCoordPtr vertexNormal, XCoordPtr vertexPosition, StandardModelControlPtr theStandardModelControl, PluginInfoPtr thePluginInfo, long *theVertex)
{
	StandardModelDataPtr theData = (StandardModelDataPtr)thePluginInfo->thePluginData;
	HostModelAddVertexRec theHostModelAddVertex;
	double xp, yp, zp;
	long theError;

	theHostModelAddVertex.theReference = thePluginInfo->theHostReference;
	theHostModelAddVertex.theColor = theStandardModelControl->StandardModelColor;

	xp = vertexNormal->x;
	yp = vertexNormal->y;
	zp = vertexNormal->z;
	TransformPoint(&xp, &yp, &zp, &theData->theRotMatrix);
	theHostModelAddVertex.theNormal.x = xp;
	theHostModelAddVertex.theNormal.y = yp;
	theHostModelAddVertex.theNormal.z = zp;

	xp = vertexPosition->x;
	yp = vertexPosition->y;
	zp = vertexPosition->z;
	TransformPoint(&xp, &yp, &zp, &theData->theMatrix);
	theHostModelAddVertex.thePosition.x = xp;
	theHostModelAddVertex.thePosition.y = yp;
	theHostModelAddVertex.thePosition.z = zp;

	xp = vertexPosition->x;
	yp = vertexPosition->y;
	zp = vertexPosition->z;
	TransformPoint(&xp, &yp, &zp, &theData->theBlurMatrix);
	theHostModelAddVertex.theBlurPosition.x = xp;
	theHostModelAddVertex.theBlurPosition.y = yp;
	theHostModelAddVertex.theBlurPosition.z = zp;

	theHostModelAddVertex.theTexturePosition.x = vertexPosition->x;
	theHostModelAddVertex.theTexturePosition.y = vertexPosition->y;
	theHostModelAddVertex.theTexturePosition.z = vertexPosition->z;

	theError = thePluginInfo->theHostProc(PLUGCOMMAND(hostModelAddVertex), PARAMETERSIZE(sizeof(HostModelAddVertexRec)), &theHostModelAddVertex);
	if (theError == 0)
		*theVertex = theHostModelAddVertex.theIndex;
	return theError;
}  /* CreateVertex */

static boolean CreateFacet(long theVertexTotal, VertexTablePtr theVertexTable, StandardModelControlPtr theStandardModelControl, PluginInfoPtr thePluginInfo)
{
	long index, theError;
	boolean interrupted;
	HostModelAddFacetRec theHostModelAddFacet;
	StandardModelDataPtr theData;

	interrupted = 0;

	theHostModelAddFacet.theReference = thePluginInfo->theHostReference;
	theHostModelAddFacet.theColor = theStandardModelControl->StandardModelColor;
	theHostModelAddFacet.theNumVertices = theVertexTotal;
	theHostModelAddFacet.theVertices[0] = 0;
	theHostModelAddFacet.theVertices[1] = 0;
	theHostModelAddFacet.theVertices[2] = 0;
	theHostModelAddFacet.theVertices[3] = 0;

	index = 0;
	theError = 0;
	theData = (StandardModelDataPtr)thePluginInfo->thePluginData;

	while (index < theVertexTotal && theError == 0) {
		theError = CreateVertex(&theVertexTable->vertexNormal[index], &theVertexTable->vertexArray[index], theStandardModelControl, thePluginInfo, &theHostModelAddFacet.theVertices[index]);
		index++;
	}
	if (theError == 0)
		theError = thePluginInfo->theHostProc(PLUGCOMMAND(hostModelAddFacet), PARAMETERSIZE(sizeof(HostModelAddFacetRec)), &theHostModelAddFacet);

	if (theError == 0) {
		theStandardModelControl->runTimeCurrentPolygons++;
		interrupted = !CreateUpdate(theStandardModelControl, thePluginInfo);
	}

	return (theError == 0) && (!interrupted);
}  /* CreateFacet */

static boolean CreateConeCap(double yTop, double yBottom, double radius, XCoordinate theNormal, long theCosTableTotal, double *theCosTable, StandardModelControlPtr theStandardModelControl, PluginInfoPtr thePluginInfo)
{
	long total, index;
	double theSin, theCos;
	boolean interrupted;
	long sinTableIndex, cosTableIndex, modTableIndex, quadTableIndex;
	VertexTableRecord theVertexTable;

	interrupted = 0;
	total = theCosTableTotal * 4;

	theVertexTable.vertexArray[0].x = 0.0;
	theVertexTable.vertexArray[0].y = yTop;
	theVertexTable.vertexArray[0].z = 0.0;

	theVertexTable.vertexArray[1].x = radius;
	theVertexTable.vertexArray[1].y = yBottom;
	theVertexTable.vertexArray[1].z = 0.0;

	theVertexTable.vertexArray[2].x = radius * theCosTable[1];
	theVertexTable.vertexArray[2].y = yBottom;
	theVertexTable.vertexArray[2].z = radius * theCosTable[theCosTableTotal - 1];

	theVertexTable.vertexNormal[0] = theNormal;
	theVertexTable.vertexNormal[1] = theNormal;
	theVertexTable.vertexNormal[2] = theNormal;

	theVertexTable.vertexNormal[2].x = theNormal.x * theCosTable[1] + theNormal.z * theCosTable[theCosTableTotal - 1];
	theVertexTable.vertexNormal[2].z = theNormal.z * theCosTable[1] + theNormal.x * theCosTable[theCosTableTotal - 1];

	interrupted = !CreateFacet(3, &theVertexTable, theStandardModelControl, thePluginInfo);
	index = 2;
	while ((index <= total) && (!interrupted)) {
		modTableIndex = index % theCosTableTotal;
		quadTableIndex = (index / theCosTableTotal) & 3;
		switch (quadTableIndex) {

		  case 0:
			cosTableIndex = modTableIndex;
			sinTableIndex = theCosTableTotal - modTableIndex;
			theSin = theCosTable[sinTableIndex];
			theCos = theCosTable[cosTableIndex];
			break;

		  case 1:
			sinTableIndex = modTableIndex;
			cosTableIndex = theCosTableTotal - modTableIndex;
			theSin = theCosTable[sinTableIndex];
			theCos = -theCosTable[cosTableIndex];
			break;

		  case 2:
			cosTableIndex = modTableIndex;
			sinTableIndex = theCosTableTotal - modTableIndex;
			theSin = -theCosTable[sinTableIndex];
			theCos = -theCosTable[cosTableIndex];
			break;

		  case 3:
			sinTableIndex = modTableIndex;
			cosTableIndex = theCosTableTotal - modTableIndex;
			theSin = -theCosTable[sinTableIndex];
			theCos = theCosTable[cosTableIndex];
			break;
		}
		theVertexTable.vertexArray[1] = theVertexTable.vertexArray[2];
		theVertexTable.vertexArray[2].x = radius * theCos;
		theVertexTable.vertexArray[2].z = radius * theSin;
		theVertexTable.vertexNormal[0] = theVertexTable.vertexNormal[2];
		theVertexTable.vertexNormal[1] = theVertexTable.vertexNormal[2];
		theVertexTable.vertexNormal[2].x = theNormal.x * theCos + theNormal.z * theSin;
		theVertexTable.vertexNormal[2].z = theNormal.z * theCos + theNormal.x * theSin;
		interrupted = !CreateFacet(3, &theVertexTable, theStandardModelControl, thePluginInfo);
		index++;
	}
	return !interrupted;
}

static boolean CreateSpericalCap(double yTop, double yBottom, double radius, long theCosTableTotal, double *theCosTable, StandardModelControlPtr theStandardModelControl, PluginInfoPtr thePluginInfo)
{
	long total, index;
	double theSin, theCos;
	boolean interrupted;
	long sinTableIndex, cosTableIndex, modTableIndex, quadTableIndex;
	VertexTableRecord theVertexTable, theVertexTableAux;

	interrupted = 0;
	total = theCosTableTotal * 4;

	theVertexTable.vertexArray[0].x = 0.0;
	theVertexTable.vertexArray[0].y = yTop;
	theVertexTable.vertexArray[0].z = 0.0;

	theVertexTable.vertexArray[1].x = radius;
	theVertexTable.vertexArray[1].y = yBottom;
	theVertexTable.vertexArray[1].z = 0.0;

	theVertexTable.vertexArray[2].x = radius * theCosTable[1];
	theVertexTable.vertexArray[2].y = yBottom;
	theVertexTable.vertexArray[2].z = radius * theCosTable[theCosTableTotal - 1];

	theVertexTable.vertexNormal[0].x = 0.0;
	theVertexTable.vertexNormal[0].y = yTop / FABS(yTop);
	theVertexTable.vertexNormal[0].z = 0.0;

	theVertexTable.vertexNormal[1] = theVertexTable.vertexArray[1];
	Normalize(&theVertexTable.vertexNormal[1]);
	theVertexTable.vertexNormal[2] = theVertexTable.vertexArray[2];
	Normalize(&theVertexTable.vertexNormal[2]);

	theVertexTableAux = theVertexTable;
	theVertexTableAux.vertexArray[1] = theVertexTable.vertexArray[2];
	theVertexTableAux.vertexArray[2] = theVertexTable.vertexArray[1];

	interrupted = !CreateFacet(3, &theVertexTableAux, theStandardModelControl, thePluginInfo);
	index = 2;
	while (index <= total && !interrupted) {
		modTableIndex = index % theCosTableTotal;
		quadTableIndex = (index / theCosTableTotal) & 3;
		switch (quadTableIndex) {

		  case 0:
			cosTableIndex = modTableIndex;
			sinTableIndex = theCosTableTotal - modTableIndex;
			theSin = theCosTable[sinTableIndex];
			theCos = theCosTable[cosTableIndex];
			break;

		  case 1:
			sinTableIndex = modTableIndex;
			cosTableIndex = theCosTableTotal - modTableIndex;
			theSin = theCosTable[sinTableIndex];
			theCos = -theCosTable[cosTableIndex];
			break;

		  case 2:
			cosTableIndex = modTableIndex;
			sinTableIndex = theCosTableTotal - modTableIndex;
			theSin = -theCosTable[sinTableIndex];
			theCos = -theCosTable[cosTableIndex];
			break;

		  case 3:
			sinTableIndex = modTableIndex;
			cosTableIndex = theCosTableTotal - modTableIndex;
			theSin = -theCosTable[sinTableIndex];
			theCos = theCosTable[cosTableIndex];
			break;
		}
		theVertexTable.vertexArray[1] = theVertexTable.vertexArray[2];
		theVertexTable.vertexNormal[1] = theVertexTable.vertexNormal[2];

		theVertexTable.vertexArray[2].x = radius * theCos;
		theVertexTable.vertexArray[2].z = radius * theSin;

		theVertexTable.vertexNormal[1] = theVertexTable.vertexNormal[2];
		theVertexTable.vertexNormal[2] = theVertexTable.vertexArray[2];
		Normalize(&theVertexTable.vertexNormal[2]);

		theVertexTableAux = theVertexTable;
		theVertexTableAux.vertexArray[1] = theVertexTable.vertexArray[2];
		theVertexTableAux.vertexArray[2] = theVertexTable.vertexArray[1];

		interrupted = !CreateFacet(3, &theVertexTableAux, theStandardModelControl, thePluginInfo);

		index++;
	}
	return !interrupted;
}

static boolean CreateRectangularCap(double top, double radius, double theNormal, long theCosTableTotal, double *theCosTable, int invert, StandardModelControlPtr theStandardModelControl, PluginInfoPtr thePluginInfo)
{
	long index, sIndex, qindex;
	double theSin, theCos;
	boolean interrupted;
	long sinTableIndex, cosTableIndex, modTableIndex, quadTableIndex;
	VertexTableRecord theVertexTable, theVertexTableAux;

	interrupted = 0;

	theVertexTable.vertexArray[0].y = top;
	theVertexTable.vertexArray[1].y = top;
	theVertexTable.vertexArray[2].y = top;
	theVertexTable.vertexArray[3].y = top;

	theVertexTable.vertexNormal[0].x = 0.0;
	theVertexTable.vertexNormal[0].y = theNormal;
	theVertexTable.vertexNormal[0].z = 0.0;
	theVertexTable.vertexNormal[1] = theVertexTable.vertexNormal[0];
	theVertexTable.vertexNormal[2] = theVertexTable.vertexNormal[0];
	theVertexTable.vertexNormal[3] = theVertexTable.vertexNormal[0];

	qindex = 1;
	while ((qindex <= 4) && (!interrupted)) {
		switch (qindex) {

		  case 1:
			theVertexTable.vertexArray[0].x = radius;
			theVertexTable.vertexArray[0].z = 0.0;
			theVertexTable.vertexArray[1].x = radius * theCosTable[1];
			theVertexTable.vertexArray[1].z = 0.0;
			theVertexTable.vertexArray[2].x = theVertexTable.vertexArray[1].x;
			theVertexTable.vertexArray[2].z = radius * theCosTable[theCosTableTotal - 1];
			break;

		  case 2:
			theVertexTable.vertexArray[0].x = 0.0;
			theVertexTable.vertexArray[0].z = radius;
			theVertexTable.vertexArray[1].x = 0.0;
			theVertexTable.vertexArray[1].z = radius * theCosTable[1];
			theVertexTable.vertexArray[2].x = -radius * theCosTable[theCosTableTotal - 1];
			theVertexTable.vertexArray[2].z = theVertexTable.vertexArray[1].z;
			break;

		  case 3:
			theVertexTable.vertexArray[0].x = -radius;
			theVertexTable.vertexArray[0].z = 0.0;
			theVertexTable.vertexArray[1].x = -radius * theCosTable[1];
			theVertexTable.vertexArray[1].z = 0.0;
			theVertexTable.vertexArray[2].x = theVertexTable.vertexArray[1].x;
			theVertexTable.vertexArray[2].z = -radius * theCosTable[theCosTableTotal - 1];
			break;

		  case 4:
			theVertexTable.vertexArray[0].x = 0.0;
			theVertexTable.vertexArray[0].z = -radius;
			theVertexTable.vertexArray[1].x = 0.0;
			theVertexTable.vertexArray[1].z = -radius * theCosTable[1];
			theVertexTable.vertexArray[2].x = radius * theCosTable[theCosTableTotal - 1];
			theVertexTable.vertexArray[2].z = theVertexTable.vertexArray[1].z;
			break;
		}

		theVertexTableAux = theVertexTable;
		if (invert != 0)
		{
			theVertexTableAux.vertexArray[1] = theVertexTable.vertexArray[2];
			theVertexTableAux.vertexArray[2] = theVertexTable.vertexArray[1];
		}
		interrupted = !CreateFacet(3, &theVertexTableAux, theStandardModelControl, thePluginInfo);

		index = 2;
		while ((index <= theCosTableTotal) && (!interrupted)) {
			modTableIndex = index % theCosTableTotal;
			quadTableIndex = (index / theCosTableTotal) & 3;
			switch (quadTableIndex) {

			  case 0:
				cosTableIndex = modTableIndex;
				sinTableIndex = theCosTableTotal - modTableIndex;
				theSin = theCosTable[sinTableIndex];
				theCos = theCosTable[cosTableIndex];
				break;

			  case 1:
				sinTableIndex = modTableIndex;
				cosTableIndex = theCosTableTotal - modTableIndex;
				theSin = theCosTable[sinTableIndex];
				theCos = -theCosTable[cosTableIndex];
				break;

			  case 2:
				cosTableIndex = modTableIndex;
				sinTableIndex = theCosTableTotal - modTableIndex;
				theSin = -theCosTable[sinTableIndex];
				theCos = -theCosTable[cosTableIndex];
				break;

			  case 3:
				sinTableIndex = modTableIndex;
				cosTableIndex = theCosTableTotal - modTableIndex;
				theSin = -theCosTable[sinTableIndex];
				theCos = theCosTable[cosTableIndex];
				break;
			}
			theVertexTable.vertexArray[0].x = theVertexTable.vertexArray[2].x;
			theVertexTable.vertexArray[0].z = theVertexTable.vertexArray[2].z;
			switch (qindex) {

			  case 1:
				theVertexTable.vertexArray[1].x = radius * theCos;
				theVertexTable.vertexArray[1].z = theVertexTable.vertexArray[0].z;
				theVertexTable.vertexArray[2].x = theVertexTable.vertexArray[1].x;
				theVertexTable.vertexArray[2].z = radius * theSin;
				break;

			  case 2:
				theVertexTable.vertexArray[1].x = theVertexTable.vertexArray[0].x;
				theVertexTable.vertexArray[1].z = radius * theCos;
				theVertexTable.vertexArray[2].x = -radius * theSin;
				theVertexTable.vertexArray[2].z = theVertexTable.vertexArray[1].z;
				break;

			  case 3:
				theVertexTable.vertexArray[1].x = -radius * theCos;
				theVertexTable.vertexArray[1].z = theVertexTable.vertexArray[0].z;
				theVertexTable.vertexArray[2].x = theVertexTable.vertexArray[1].x;
				theVertexTable.vertexArray[2].z = -radius * theSin;
				break;

			  case 4:
				theVertexTable.vertexArray[1].x = theVertexTable.vertexArray[0].x;
				theVertexTable.vertexArray[1].z = -radius * theCos;
				theVertexTable.vertexArray[2].x = radius * theSin;
				theVertexTable.vertexArray[2].z = theVertexTable.vertexArray[1].z;
				break;
			}
			theVertexTableAux = theVertexTable;
			if (invert != 0)
			{
				theVertexTableAux.vertexArray[1] = theVertexTable.vertexArray[2];
				theVertexTableAux.vertexArray[2] = theVertexTable.vertexArray[1];
			}
			interrupted = !CreateFacet(3, &theVertexTableAux, theStandardModelControl, thePluginInfo);

			index++;
		}
		index = 1;
		while ((index < theCosTableTotal) && (!interrupted)) {
			switch (qindex) {

			  case 1:
				theVertexTable.vertexArray[0].x = radius * theCosTable[index];
				theVertexTable.vertexArray[1].x = radius * theCosTable[index + 1];
				theVertexTable.vertexArray[2].z = radius * theCosTable[theCosTableTotal - 1];
				theVertexTable.vertexArray[2].x = theVertexTable.vertexArray[1].x;
				theVertexTable.vertexArray[3].x = theVertexTable.vertexArray[0].x;
				theVertexTable.vertexArray[0].z = 0.0;
				theVertexTable.vertexArray[1].z = 0.0;
				theVertexTable.vertexArray[3].z = theVertexTable.vertexArray[2].z;
				theVertexTableAux = theVertexTable;
				if (invert != 0)
				{
					theVertexTableAux.vertexArray[1] = theVertexTable.vertexArray[3];
					theVertexTableAux.vertexArray[3] = theVertexTable.vertexArray[1];
				}
				break;

			  case 2:
				theVertexTable.vertexArray[0].x = -radius * theCosTable[index];
				theVertexTable.vertexArray[1].x = -radius * theCosTable[index + 1];
				theVertexTable.vertexArray[2].z = radius * theCosTable[theCosTableTotal - 1];
				theVertexTable.vertexArray[2].x = theVertexTable.vertexArray[1].x;
				theVertexTable.vertexArray[3].x = theVertexTable.vertexArray[0].x;
				theVertexTable.vertexArray[0].z = 0.0;
				theVertexTable.vertexArray[1].z = 0.0;
				theVertexTable.vertexArray[3].z = theVertexTable.vertexArray[2].z;
				theVertexTableAux = theVertexTable;
				if (invert == 0)
				{
					theVertexTableAux.vertexArray[1] = theVertexTable.vertexArray[3];
					theVertexTableAux.vertexArray[3] = theVertexTable.vertexArray[1];
				}
				break;

			  case 3:
				theVertexTable.vertexArray[0].x = -radius * theCosTable[index];
				theVertexTable.vertexArray[1].x = -radius * theCosTable[index + 1];
				theVertexTable.vertexArray[2].z = -radius * theCosTable[theCosTableTotal - 1];
				theVertexTable.vertexArray[2].x = theVertexTable.vertexArray[1].x;
				theVertexTable.vertexArray[3].x = theVertexTable.vertexArray[0].x;
				theVertexTable.vertexArray[0].z = 0.0;
				theVertexTable.vertexArray[1].z = 0.0;
				theVertexTable.vertexArray[3].z = theVertexTable.vertexArray[2].z;
				theVertexTableAux = theVertexTable;
				if (invert != 0)
				{
					theVertexTableAux.vertexArray[1] = theVertexTable.vertexArray[3];
					theVertexTableAux.vertexArray[3] = theVertexTable.vertexArray[1];
				}
				break;

			  case 4:
				theVertexTable.vertexArray[0].x = radius * theCosTable[index];
				theVertexTable.vertexArray[1].x = radius * theCosTable[index + 1];
				theVertexTable.vertexArray[2].z = -radius * theCosTable[theCosTableTotal - 1];
				theVertexTable.vertexArray[2].x = theVertexTable.vertexArray[1].x;
				theVertexTable.vertexArray[3].x = theVertexTable.vertexArray[0].x;
				theVertexTable.vertexArray[0].z = 0.0;
				theVertexTable.vertexArray[1].z = 0.0;
				theVertexTable.vertexArray[3].z = theVertexTable.vertexArray[2].z;
				theVertexTableAux = theVertexTable;
				if (invert == 0)
				{
					theVertexTableAux.vertexArray[1] = theVertexTable.vertexArray[3];
					theVertexTableAux.vertexArray[3] = theVertexTable.vertexArray[1];
				}
				break;
			}
			interrupted = !CreateFacet(4, &theVertexTableAux, theStandardModelControl, thePluginInfo);
			sIndex = 2;
			while (sIndex <= index && !interrupted) {
				theVertexTable.vertexArray[0].z = theVertexTable.vertexArray[3].z;
				theVertexTable.vertexArray[1].z = theVertexTable.vertexArray[2].z;
				switch (qindex) {

				  case 1:
					theVertexTable.vertexArray[2].z = radius * theCosTable[theCosTableTotal - sIndex];
					theVertexTable.vertexArray[3].z = theVertexTable.vertexArray[2].z;
					theVertexTableAux = theVertexTable;
					if (invert != 0)
					{
						theVertexTableAux.vertexArray[1] = theVertexTable.vertexArray[3];
						theVertexTableAux.vertexArray[3] = theVertexTable.vertexArray[1];
					}
					break;
				  case 2:
					theVertexTable.vertexArray[2].z = radius * theCosTable[theCosTableTotal - sIndex];
					theVertexTable.vertexArray[3].z = theVertexTable.vertexArray[2].z;
					theVertexTableAux = theVertexTable;
					if (invert == 0)
					{
						theVertexTableAux.vertexArray[1] = theVertexTable.vertexArray[3];
						theVertexTableAux.vertexArray[3] = theVertexTable.vertexArray[1];
					}
					break;
				  case 3:
					theVertexTable.vertexArray[2].z = -radius * theCosTable[theCosTableTotal - sIndex];
					theVertexTable.vertexArray[3].z = theVertexTable.vertexArray[2].z;
					theVertexTableAux = theVertexTable;
					if (invert != 0)
					{
						theVertexTableAux.vertexArray[1] = theVertexTable.vertexArray[3];
						theVertexTableAux.vertexArray[3] = theVertexTable.vertexArray[1];
					}
					break;
				  case 4:
					theVertexTable.vertexArray[2].z = -radius * theCosTable[theCosTableTotal - sIndex];
					theVertexTable.vertexArray[3].z = theVertexTable.vertexArray[2].z;
					theVertexTableAux = theVertexTable;
					if (invert == 0)
					{
						theVertexTableAux.vertexArray[1] = theVertexTable.vertexArray[3];
						theVertexTableAux.vertexArray[3] = theVertexTable.vertexArray[1];
					}
					break;
				}
				interrupted = !CreateFacet(4, &theVertexTableAux, theStandardModelControl, thePluginInfo);
				sIndex++;
			}
			index++;
		}
		qindex++;
	}
	return !interrupted;
}

static boolean CreateConeMantel(double yTop, double yBottom, double rTop, double rBottom, XCoordinate theNormal, long theCosTableTotal, double *theCosTable, StandardModelControlPtr theStandardModelControl, PluginInfoPtr thePluginInfo)
{
	long total, index;
	double theSin, theCos;
	boolean interrupted;
	long sinTableIndex, cosTableIndex, modTableIndex, quadTableIndex;
	VertexTableRecord theVertexTable;

	interrupted = 0;
	total = theCosTableTotal * 4;

	theVertexTable.vertexArray[0].x = rTop * theCosTable[1];
	theVertexTable.vertexArray[0].y = yTop;
	theVertexTable.vertexArray[0].z = rTop * theCosTable[theCosTableTotal - 1];

	theVertexTable.vertexArray[1].x = rBottom * theCosTable[1];
	theVertexTable.vertexArray[1].y = yBottom;
	theVertexTable.vertexArray[1].z = rBottom * theCosTable[theCosTableTotal - 1];

	theVertexTable.vertexArray[2].x = rBottom;
	theVertexTable.vertexArray[2].y = yBottom;
	theVertexTable.vertexArray[2].z = 0.0;

	theVertexTable.vertexArray[3].x = rTop;
	theVertexTable.vertexArray[3].y = yTop;
	theVertexTable.vertexArray[3].z = 0.0;

	theVertexTable.vertexNormal[0] = theNormal;
	theVertexTable.vertexNormal[1] = theNormal;
	theVertexTable.vertexNormal[2] = theNormal;
	theVertexTable.vertexNormal[3] = theNormal;

	theVertexTable.vertexNormal[0].x = theVertexTable.vertexNormal[1].x = theNormal.x * theCosTable[1] + theNormal.z * theCosTable[theCosTableTotal - 1];
	theVertexTable.vertexNormal[0].z = theVertexTable.vertexNormal[1].z = theNormal.z * theCosTable[1] + theNormal.x * theCosTable[theCosTableTotal - 1];

	interrupted = !CreateFacet(4, &theVertexTable, theStandardModelControl, thePluginInfo);

	index = 2;
	while ((index <= total) && (!interrupted)) {
		modTableIndex = index % theCosTableTotal;
		quadTableIndex = (index / theCosTableTotal) & 3;
		switch (quadTableIndex) {

		  case 0:
			cosTableIndex = modTableIndex;
			sinTableIndex = theCosTableTotal - modTableIndex;
			theSin = theCosTable[sinTableIndex];
			theCos = theCosTable[cosTableIndex];
			break;

		  case 1:
			sinTableIndex = modTableIndex;
			cosTableIndex = theCosTableTotal - modTableIndex;
			theSin = theCosTable[sinTableIndex];
			theCos = -theCosTable[cosTableIndex];
			break;

		  case 2:
			cosTableIndex = modTableIndex;
			sinTableIndex = theCosTableTotal - modTableIndex;
			theSin = -theCosTable[sinTableIndex];
			theCos = -theCosTable[cosTableIndex];
			break;

		  case 3:
			sinTableIndex = modTableIndex;
			cosTableIndex = theCosTableTotal - modTableIndex;
			theSin = -theCosTable[sinTableIndex];
			theCos = theCosTable[cosTableIndex];
			break;
		}
		theVertexTable.vertexArray[3] = theVertexTable.vertexArray[0];
		theVertexTable.vertexArray[2] = theVertexTable.vertexArray[1];
		theVertexTable.vertexArray[0].x = rTop * theCos;
		theVertexTable.vertexArray[0].z = rTop * theSin;
		theVertexTable.vertexArray[1].x = rBottom * theCos;
		theVertexTable.vertexArray[1].z = rBottom * theSin;

		theVertexTable.vertexNormal[3] = theVertexTable.vertexNormal[0];
		theVertexTable.vertexNormal[2] = theVertexTable.vertexNormal[1];
		theVertexTable.vertexNormal[0].x = theVertexTable.vertexNormal[1].x = theNormal.x * theCos + theNormal.z * theSin;
		theVertexTable.vertexNormal[0].z = theVertexTable.vertexNormal[1].z = theNormal.z * theCos + theNormal.x * theSin;

		interrupted = !CreateFacet(4, &theVertexTable, theStandardModelControl, thePluginInfo);
		index++;
	}
	return !interrupted;
}

static boolean CreateCylinderMantel(double yTop, double yBottom, double radius, long theCosTableTotal, double *theCosTable, StandardModelControlPtr theStandardModelControl, PluginInfoPtr thePluginInfo)
{
	long total, index;
	double theSin, theCos;
	boolean interrupted;
	long sinTableIndex, cosTableIndex, modTableIndex, quadTableIndex;
	XCoordinate theNormal;
	VertexTableRecord theVertexTable;

	interrupted = 0;
	total = theCosTableTotal * 4;

	theVertexTable.vertexArray[0].x = radius;
	theVertexTable.vertexArray[0].y = yTop;
	theVertexTable.vertexArray[0].z = 0.0;

	theVertexTable.vertexArray[1].x = radius * theCosTable[1];
	theVertexTable.vertexArray[1].y = yTop;
	theVertexTable.vertexArray[1].z = radius * theCosTable[theCosTableTotal - 1];

	theVertexTable.vertexArray[2].x = radius * theCosTable[1];
	theVertexTable.vertexArray[2].y = yBottom;
	theVertexTable.vertexArray[2].z = radius * theCosTable[theCosTableTotal - 1];

	theVertexTable.vertexArray[3].x = radius;
	theVertexTable.vertexArray[3].y = yBottom;
	theVertexTable.vertexArray[3].z = 0.0;

	theNormal.x = 1.0;
	theNormal.y = 0.0;
	theNormal.z = 0.0;

	theVertexTable.vertexNormal[0] = theNormal;
	theVertexTable.vertexNormal[1] = theNormal;
	theVertexTable.vertexNormal[2] = theNormal;
	theVertexTable.vertexNormal[3] = theNormal;

	theVertexTable.vertexNormal[1].x = theNormal.x * theCosTable[1] + theNormal.z * theCosTable[theCosTableTotal - 1];
	theVertexTable.vertexNormal[1].z = theNormal.z * theCosTable[1] + theNormal.x * theCosTable[theCosTableTotal - 1];
	theVertexTable.vertexNormal[2] = theVertexTable.vertexNormal[1];

	interrupted = !CreateFacet(4, &theVertexTable, theStandardModelControl, thePluginInfo);

	index = 2;
	while ((index <= total) && (!interrupted)) {
		modTableIndex = index % theCosTableTotal;
		quadTableIndex = (index / theCosTableTotal) & 3;
		switch (quadTableIndex) {

		  case 0:
			cosTableIndex = modTableIndex;
			sinTableIndex = theCosTableTotal - modTableIndex;
			theSin = theCosTable[sinTableIndex];
			theCos = theCosTable[cosTableIndex];
			break;

		  case 1:
			sinTableIndex = modTableIndex;
			cosTableIndex = theCosTableTotal - modTableIndex;
			theSin = theCosTable[sinTableIndex];
			theCos = -theCosTable[cosTableIndex];
			break;

		  case 2:
			cosTableIndex = modTableIndex;
			sinTableIndex = theCosTableTotal - modTableIndex;
			theSin = -theCosTable[sinTableIndex];
			theCos = -theCosTable[cosTableIndex];
			break;

		  case 3:
			sinTableIndex = modTableIndex;
			cosTableIndex = theCosTableTotal - modTableIndex;
			theSin = -theCosTable[sinTableIndex];
			theCos = theCosTable[cosTableIndex];
			break;
		}
		theVertexTable.vertexArray[0] = theVertexTable.vertexArray[1];
		theVertexTable.vertexArray[3] = theVertexTable.vertexArray[2];
		theVertexTable.vertexArray[1].x = radius * theCos;
		theVertexTable.vertexArray[1].z = radius * theSin;
		theVertexTable.vertexArray[2].x = radius * theCos;
		theVertexTable.vertexArray[2].z = radius * theSin;

		theVertexTable.vertexNormal[0] = theVertexTable.vertexNormal[1];
		theVertexTable.vertexNormal[3] = theVertexTable.vertexNormal[2];
		theVertexTable.vertexNormal[1].x = theNormal.x * theCos + theNormal.z * theSin;
		theVertexTable.vertexNormal[1].z = theNormal.z * theCos + theNormal.x * theSin;
		theVertexTable.vertexNormal[2] = theVertexTable.vertexNormal[1];

		interrupted = !CreateFacet(4, &theVertexTable, theStandardModelControl, thePluginInfo);
		index++;
	}
	return !interrupted;
}

static boolean CreateSphereMantel(double yTop, double yBottom, double rTop, double rBottom, long theCosTableTotal, double *theCosTable, StandardModelControlPtr theStandardModelControl, PluginInfoPtr thePluginInfo)
{
	long total, index;
	double theSin, theCos;
	boolean interrupted;
	long sinTableIndex, cosTableIndex, modTableIndex, quadTableIndex;
	VertexTableRecord theVertexTable;

	interrupted = 0;
	total = theCosTableTotal * 4;

	theVertexTable.vertexArray[0].x = rTop * theCosTable[1];
	theVertexTable.vertexArray[0].y = yTop;
	theVertexTable.vertexArray[0].z = rTop * theCosTable[theCosTableTotal - 1];

	theVertexTable.vertexArray[1].x = rBottom * theCosTable[1];
	theVertexTable.vertexArray[1].y = yBottom;
	theVertexTable.vertexArray[1].z = rBottom * theCosTable[theCosTableTotal - 1];

	theVertexTable.vertexArray[2].x = rBottom;
	theVertexTable.vertexArray[2].y = yBottom;
	theVertexTable.vertexArray[2].z = 0.0;

	theVertexTable.vertexArray[3].x = rTop;
	theVertexTable.vertexArray[3].y = yTop;
	theVertexTable.vertexArray[3].z = 0.0;

	theVertexTable.vertexNormal[0] = theVertexTable.vertexArray[0];
	Normalize(&theVertexTable.vertexNormal[0]);
	theVertexTable.vertexNormal[1] = theVertexTable.vertexArray[1];
	Normalize(&theVertexTable.vertexNormal[1]);
	theVertexTable.vertexNormal[2] = theVertexTable.vertexArray[2];
	Normalize(&theVertexTable.vertexNormal[2]);
	theVertexTable.vertexNormal[3] = theVertexTable.vertexArray[3];
	Normalize(&theVertexTable.vertexNormal[3]);

	interrupted = !CreateFacet(4, &theVertexTable, theStandardModelControl, thePluginInfo);

	index = 2;
	while ((index <= total) && (!interrupted)) {
		modTableIndex = index % theCosTableTotal;
		quadTableIndex = (index / theCosTableTotal) & 3;
		switch (quadTableIndex) {

		  case 0:
			cosTableIndex = modTableIndex;
			sinTableIndex = theCosTableTotal - modTableIndex;
			theSin = theCosTable[sinTableIndex];
			theCos = theCosTable[cosTableIndex];
			break;

		  case 1:
			sinTableIndex = modTableIndex;
			cosTableIndex = theCosTableTotal - modTableIndex;
			theSin = theCosTable[sinTableIndex];
			theCos = -theCosTable[cosTableIndex];
			break;

		  case 2:
			cosTableIndex = modTableIndex;
			sinTableIndex = theCosTableTotal - modTableIndex;
			theSin = -theCosTable[sinTableIndex];
			theCos = -theCosTable[cosTableIndex];
			break;

		  case 3:
			sinTableIndex = modTableIndex;
			cosTableIndex = theCosTableTotal - modTableIndex;
			theSin = -theCosTable[sinTableIndex];
			theCos = theCosTable[cosTableIndex];
			break;
		}
		theVertexTable.vertexArray[3] = theVertexTable.vertexArray[0];
		theVertexTable.vertexArray[2] = theVertexTable.vertexArray[1];
		theVertexTable.vertexArray[0].x = rTop * theCos;
		theVertexTable.vertexArray[0].z = rTop * theSin;
		theVertexTable.vertexArray[1].x = rBottom * theCos;
		theVertexTable.vertexArray[1].z = rBottom * theSin;
		theVertexTable.vertexNormal[3] = theVertexTable.vertexNormal[0];
		theVertexTable.vertexNormal[2] = theVertexTable.vertexNormal[1];
		theVertexTable.vertexNormal[0] = theVertexTable.vertexArray[0];
		Normalize(&theVertexTable.vertexNormal[0]);
		theVertexTable.vertexNormal[1] = theVertexTable.vertexArray[1];
		Normalize(&theVertexTable.vertexNormal[1]);
		interrupted = !CreateFacet(4, &theVertexTable, theStandardModelControl, thePluginInfo);
		index++;
	}
	return !interrupted;
}

static boolean CreatePlane(double left, double right, double top, double bottom, double depth, short depthControl, long theBreakup, double theNormal, StandardModelControlPtr theStandardModelControl, PluginInfoPtr thePluginInfo)
{
	long xIndex, yIndex;
	boolean interrupted;
	double deltaVertical, deltaHorizontal;
	VertexTableRecord theVertexTable;

	interrupted = 0;
	deltaVertical = (bottom - top) / theBreakup;
	deltaHorizontal = (right - left) / theBreakup;
	switch (depthControl) {

	  case kPlaneYZ:
		theVertexTable.vertexNormal[0].x = theNormal;
		theVertexTable.vertexNormal[0].y = 0.0;
		theVertexTable.vertexNormal[0].z = 0.0;
		break;

	  case kPlaneXZ:
		theVertexTable.vertexNormal[0].x = 0.0;
		theVertexTable.vertexNormal[0].y = theNormal;
		theVertexTable.vertexNormal[0].z = 0.0;
		break;

	  case kPlaneXY:
		theVertexTable.vertexNormal[0].x = 0.0;
		theVertexTable.vertexNormal[0].y = 0.0;
		theVertexTable.vertexNormal[0].z = theNormal;
		break;
	}
	theVertexTable.vertexNormal[1] = theVertexTable.vertexNormal[0];
	theVertexTable.vertexNormal[2] = theVertexTable.vertexNormal[0];
	theVertexTable.vertexNormal[3] = theVertexTable.vertexNormal[0];
	switch (depthControl) {

	  case kPlaneYZ:
		theVertexTable.vertexArray[0].x = depth;
		theVertexTable.vertexArray[0].y = top;
		theVertexTable.vertexArray[1].x = depth;
		theVertexTable.vertexArray[1].y = top;
		theVertexTable.vertexArray[2].x = depth;
		theVertexTable.vertexArray[2].y = top + deltaVertical;
		theVertexTable.vertexArray[3].x = depth;
		theVertexTable.vertexArray[3].y = top + deltaVertical;
		xIndex = 1;
		while ((xIndex <= theBreakup) && (!interrupted)) {
			theVertexTable.vertexArray[0].z = left;
			theVertexTable.vertexArray[1].z = left + deltaHorizontal;
			theVertexTable.vertexArray[2].z = theVertexTable.vertexArray[1].z;
			theVertexTable.vertexArray[3].z = theVertexTable.vertexArray[0].z;
			interrupted = !CreateFacet(4, &theVertexTable, theStandardModelControl, thePluginInfo);
			yIndex = 2;
			while ((yIndex <= theBreakup) && (!interrupted)) {
				theVertexTable.vertexArray[0].z = theVertexTable.vertexArray[1].z;
				theVertexTable.vertexArray[3].z = theVertexTable.vertexArray[2].z;
				theVertexTable.vertexArray[1].z += deltaHorizontal;
				theVertexTable.vertexArray[2].z = theVertexTable.vertexArray[1].z;
				interrupted = !CreateFacet(4, &theVertexTable, theStandardModelControl, thePluginInfo);
				yIndex++;
			}
			theVertexTable.vertexArray[0].y = theVertexTable.vertexArray[3].y;
			theVertexTable.vertexArray[1].y = theVertexTable.vertexArray[2].y;
			theVertexTable.vertexArray[2].y += deltaVertical;
			theVertexTable.vertexArray[3].y = theVertexTable.vertexArray[2].y;
			xIndex++;
		}
		break;

	  case kPlaneXZ:
		theVertexTable.vertexArray[0].y = depth;
		theVertexTable.vertexArray[0].z = top;
		theVertexTable.vertexArray[1].y = depth;
		theVertexTable.vertexArray[1].z = top;
		theVertexTable.vertexArray[2].y = depth;
		theVertexTable.vertexArray[2].z = top + deltaVertical;
		theVertexTable.vertexArray[3].y = depth;
		theVertexTable.vertexArray[3].z = top + deltaVertical;
		xIndex = 1;
		while ((xIndex <= theBreakup) && (!interrupted)) {
			theVertexTable.vertexArray[0].x = left;
			theVertexTable.vertexArray[1].x = left + deltaHorizontal;
			theVertexTable.vertexArray[2].x = theVertexTable.vertexArray[1].x;
			theVertexTable.vertexArray[3].x = theVertexTable.vertexArray[0].x;
			interrupted = !CreateFacet(4, &theVertexTable, theStandardModelControl, thePluginInfo);
			yIndex = 2;
			while ((yIndex <= theBreakup) && (!interrupted)) {
				theVertexTable.vertexArray[0].x = theVertexTable.vertexArray[1].x;
				theVertexTable.vertexArray[3].x = theVertexTable.vertexArray[2].x;
				theVertexTable.vertexArray[1].x += deltaHorizontal;
				theVertexTable.vertexArray[2].x = theVertexTable.vertexArray[1].x;
				interrupted = !CreateFacet(4, &theVertexTable, theStandardModelControl, thePluginInfo);
				yIndex++;
			}
			theVertexTable.vertexArray[0].z = theVertexTable.vertexArray[3].z;
			theVertexTable.vertexArray[1].z = theVertexTable.vertexArray[2].z;
			theVertexTable.vertexArray[2].z += deltaVertical;
			theVertexTable.vertexArray[3].z = theVertexTable.vertexArray[2].z;
			xIndex++;
		}
		break;

	  case kPlaneXY:
		theVertexTable.vertexArray[0].z = depth;
		theVertexTable.vertexArray[0].y = top;
		theVertexTable.vertexArray[1].z = depth;
		theVertexTable.vertexArray[1].y = top;
		theVertexTable.vertexArray[2].z = depth;
		theVertexTable.vertexArray[2].y = top + deltaVertical;
		theVertexTable.vertexArray[3].z = depth;
		theVertexTable.vertexArray[3].y = top + deltaVertical;
		xIndex = 1;
		while ((xIndex <= theBreakup) && (!interrupted)) {
			theVertexTable.vertexArray[0].x = left;
			theVertexTable.vertexArray[1].x = left + deltaHorizontal;
			theVertexTable.vertexArray[2].x = theVertexTable.vertexArray[1].x;
			theVertexTable.vertexArray[3].x = theVertexTable.vertexArray[0].x;
			interrupted = !CreateFacet(4, &theVertexTable, theStandardModelControl, thePluginInfo);
			yIndex = 2;
			while ((yIndex <= theBreakup) && (!interrupted)) {
				theVertexTable.vertexArray[0].x = theVertexTable.vertexArray[1].x;
				theVertexTable.vertexArray[3].x = theVertexTable.vertexArray[2].x;
				theVertexTable.vertexArray[1].x += deltaHorizontal;
				theVertexTable.vertexArray[2].x = theVertexTable.vertexArray[1].x;
				interrupted = !CreateFacet(4, &theVertexTable, theStandardModelControl, thePluginInfo);
				yIndex++;
			}
			theVertexTable.vertexArray[0].y = theVertexTable.vertexArray[3].y;
			theVertexTable.vertexArray[1].y = theVertexTable.vertexArray[2].y;
			theVertexTable.vertexArray[2].y += deltaVertical;
			theVertexTable.vertexArray[3].y = theVertexTable.vertexArray[2].y;
			xIndex++;
		}
		break;
	}
	return !interrupted;
}

/* --------------------------------------------------------------- Creation routines */
static boolean GeneratePlaneData(StandardModelControlPtr theStandardModelControl, PluginInfoPtr thePluginInfo)
{
	boolean done;

	if (CreateStart(theStandardModelControl, thePluginInfo)) {
		done = CreatePlane(theStandardModelControl->planeLength / 2.0, theStandardModelControl->planeLength / -2.0, theStandardModelControl->planeWidth / -2.0, theStandardModelControl->planeWidth / 2.0, 0.0, theStandardModelControl->planeControl, theStandardModelControl->planeDivision, 1.0, theStandardModelControl, thePluginInfo);
	} else
		done = 0;
	return done;
}

static boolean GenerateCubeData(StandardModelControlPtr theStandardModelControl, PluginInfoPtr thePluginInfo)
{
	boolean done;

	if (!CreateStart(theStandardModelControl, thePluginInfo)) {
		done = 0;
		return done;
	}
	done = 1;
	if (done)
		done = CreatePlane(theStandardModelControl->cubeLength / 2.0, theStandardModelControl->cubeLength / -2.0, theStandardModelControl->cubeWidth / -2.0, theStandardModelControl->cubeWidth / 2.0, theStandardModelControl->cubeHeight / 2, kPlaneXZ, theStandardModelControl->cubeDivision, 1.0, theStandardModelControl, thePluginInfo);
	if (done)
		done = CreatePlane(theStandardModelControl->cubeLength / -2.0, theStandardModelControl->cubeLength / 2.0, theStandardModelControl->cubeWidth / -2.0, theStandardModelControl->cubeWidth / 2.0, theStandardModelControl->cubeHeight / -2, kPlaneXZ, theStandardModelControl->cubeDivision, -1.0, theStandardModelControl, thePluginInfo);
	if (done)
		done = CreatePlane(theStandardModelControl->cubeLength / -2.0, theStandardModelControl->cubeLength / 2.0, theStandardModelControl->cubeHeight / -2.0, theStandardModelControl->cubeHeight / 2.0, theStandardModelControl->cubeWidth / 2.0, kPlaneXY, theStandardModelControl->cubeDivision, 1.0, theStandardModelControl, thePluginInfo);
	if (done)
		done = CreatePlane(theStandardModelControl->cubeLength / 2.0, theStandardModelControl->cubeLength / -2.0, theStandardModelControl->cubeHeight / -2.0, theStandardModelControl->cubeHeight / 2.0, theStandardModelControl->cubeWidth / -2.0, kPlaneXY, theStandardModelControl->cubeDivision, -1.0, theStandardModelControl, thePluginInfo);
	if (done)
		done = CreatePlane(theStandardModelControl->cubeWidth / 2.0, theStandardModelControl->cubeWidth / -2.0, theStandardModelControl->cubeHeight / -2.0, theStandardModelControl->cubeHeight / 2.0, theStandardModelControl->cubeLength / 2.0, kPlaneYZ, theStandardModelControl->cubeDivision, 1.0, theStandardModelControl, thePluginInfo);
	if (done)
		done = CreatePlane(theStandardModelControl->cubeWidth / -2.0, theStandardModelControl->cubeWidth / 2.0, theStandardModelControl->cubeHeight / -2.0, theStandardModelControl->cubeHeight / 2.0, theStandardModelControl->cubeLength / -2.0, kPlaneYZ, theStandardModelControl->cubeDivision, -1.0, theStandardModelControl, thePluginInfo);
	return done;
}

static boolean GenerateCylinderData(StandardModelControlPtr theStandardModelControl, PluginInfoPtr thePluginInfo, char* theTemporaryData)
{
	boolean done;
	long index;
	double yTop, dBottom, yBottom, longitudeAngle;
	double *theLongitudeTable;

	if (!CreateStart(theStandardModelControl, thePluginInfo)) {
		done = 0;
		return done;
	}
	done = 1;
	/* Create cosine table for latitude */
	longitudeAngle = 90.0 / theStandardModelControl->cylinderLongitude * 0.017453292;
	theLongitudeTable = (double *)(theTemporaryData + sizeof(StandardModelDataRec));
	theLongitudeTable[0] = 1.0;
	theLongitudeTable[theStandardModelControl->cylinderLongitude] = 0.0;
	for (index = 1; index < theStandardModelControl->cylinderLongitude; index++)
		theLongitudeTable[index] = COS(longitudeAngle * index);
	/* Create Data */
	if (theStandardModelControl->cylinderLatitude <= 0) {
		done = CreateCylinderMantel(theStandardModelControl->cylinderHeight, 0.0, theStandardModelControl->cylinderRadius, theStandardModelControl->cylinderLongitude, theLongitudeTable, theStandardModelControl, thePluginInfo);
		if (done)
			done = CreateRectangularCap(theStandardModelControl->cylinderHeight, theStandardModelControl->cylinderRadius, 1.0, theStandardModelControl->cylinderLongitude, theLongitudeTable, 0, theStandardModelControl, thePluginInfo);
		if (done)
			done = CreateRectangularCap(0.0, theStandardModelControl->cylinderRadius, -1.0, theStandardModelControl->cylinderLongitude, theLongitudeTable, 1, theStandardModelControl, thePluginInfo);
		return done;
	}
	dBottom = theStandardModelControl->cylinderHeight / theStandardModelControl->cylinderLatitude;
	yTop = theStandardModelControl->cylinderHeight;
	yBottom = theStandardModelControl->cylinderHeight - dBottom;
	done = CreateCylinderMantel(yTop, yBottom, theStandardModelControl->cylinderRadius, theStandardModelControl->cylinderLongitude, theLongitudeTable, theStandardModelControl, thePluginInfo);
	index = 2;
	while ((index <= theStandardModelControl->cylinderLatitude) && done) {
		yTop = yBottom;
		yBottom = theStandardModelControl->cylinderHeight - dBottom * index;
		done = CreateCylinderMantel(yTop, yBottom, theStandardModelControl->cylinderRadius, theStandardModelControl->cylinderLongitude, theLongitudeTable, theStandardModelControl, thePluginInfo);
		index++;
	}
	if (done)
		done = CreateRectangularCap(theStandardModelControl->cylinderHeight, theStandardModelControl->cylinderRadius, 1.0, theStandardModelControl->cylinderLongitude, theLongitudeTable, 0, theStandardModelControl, thePluginInfo);
	if (done)
		done = CreateRectangularCap(0.0, theStandardModelControl->cylinderRadius, -1.0, theStandardModelControl->cylinderLongitude, theLongitudeTable, 1, theStandardModelControl, thePluginInfo);
	return done;
}

static boolean GenerateConeData(StandardModelControlPtr theStandardModelControl, PluginInfoPtr thePluginInfo, char* theTemporaryData)
{
	boolean done;
	long index;
	double rTop, yTop, dBottom, rBottom, yBottom, theAngle;
	XCoordinate theNormal;
	double *theLongitudeTable;

	if (!CreateStart(theStandardModelControl, thePluginInfo)) {
		done = 0;
		return done;
	}
	done = 1;
	/* Create cosine table for latitude */
	theAngle = 90.0 / theStandardModelControl->coneLongitude * 0.017453292;
	theLongitudeTable = (double *)((long)theTemporaryData + sizeof(StandardModelDataRec));
	theLongitudeTable[0] = 1.0;
	theLongitudeTable[theStandardModelControl->coneLongitude] = 0.0;
	for (index = 1; index < theStandardModelControl->coneLongitude; index++)
		theLongitudeTable[index] = COS(theAngle * index);
	/* Create Normal Data */
	theAngle = ATAN(theStandardModelControl->coneRadius / theStandardModelControl->coneHeight);
	theNormal.x = COS(theAngle);
	theNormal.y = SIN(theAngle);
	theNormal.z = 0.0;
	/* Create Data */
	dBottom = theStandardModelControl->coneHeight / theStandardModelControl->coneLatitude;
	yTop = theStandardModelControl->coneHeight;
	yBottom = theStandardModelControl->coneHeight - dBottom;
	rTop = 0.0;
	rBottom = theStandardModelControl->coneRadius * (1.0 / theStandardModelControl->coneLatitude);
	index = 1;
	while ((index <= theStandardModelControl->coneLatitude) && done) {
		done = CreateConeMantel(yTop, yBottom, rTop, rBottom, theNormal, theStandardModelControl->coneLongitude, theLongitudeTable, theStandardModelControl, thePluginInfo);
		rTop = rBottom;
		yTop = yBottom;
		index++;
		yBottom = theStandardModelControl->coneHeight - dBottom * index;
		rBottom = theStandardModelControl->coneRadius * ((double)index / theStandardModelControl->coneLatitude);
	}
	if (done)
		done = CreateRectangularCap(0.0, theStandardModelControl->coneRadius, -1.0, theStandardModelControl->coneLongitude, theLongitudeTable, 1, theStandardModelControl, thePluginInfo);
	return done;
}

static boolean GenerateSphereData(StandardModelControlPtr theStandardModelControl, PluginInfoPtr thePluginInfo, char* theTemporaryData)
{
	boolean done;
	long index;
	double rTop, yTop, rBottom, yBottom;
	long latitudeIndex;
	double latitudeAngle, longitudeAngle;
	double *theLatitudeTable, *theLongitudeTable;

	if (!CreateStart(theStandardModelControl, thePluginInfo))
		done = 0;
	else {
		done = 1;
		/* Create cosine table for latitude */
		latitudeAngle = 90.0 / theStandardModelControl->sphereLatitude * 0.017453292;
		theLatitudeTable = (double *)(theTemporaryData + sizeof(StandardModelDataRec));
		theLatitudeTable[0] = 1.0;
		theLatitudeTable[theStandardModelControl->sphereLatitude] = 0.0;
		for (index = 1; index < theStandardModelControl->sphereLatitude; index++)
			theLatitudeTable[index] = COS(latitudeAngle * index);
		/* Create cosine table for longitude */
		longitudeAngle = 90.0 / theStandardModelControl->sphereLongitude * 0.017453292;
		theLongitudeTable = (double *)((void *)(theTemporaryData + sizeof(StandardModelDataRec) + (theStandardModelControl->sphereLatitude + 1) * sizeof(double)));
		theLongitudeTable[0] = 1.0;
		theLongitudeTable[theStandardModelControl->sphereLongitude] = 0.0;
		for (index = 1; index < theStandardModelControl->sphereLongitude; index++)
			theLongitudeTable[index] = COS(longitudeAngle * index);
		/* Create Top/Bottom Cap */
		rTop = 0.0;
		yTop = theStandardModelControl->sphereRadius;
		yBottom = theStandardModelControl->sphereRadius * theLatitudeTable[1];
		rBottom = theStandardModelControl->sphereRadius * theLatitudeTable[theStandardModelControl->sphereLatitude - 1];
		if (done)
			done = CreateSpericalCap(yTop, yBottom, rBottom, theStandardModelControl->sphereLongitude, theLongitudeTable, theStandardModelControl, thePluginInfo);
		if (done)
			done = CreateSpericalCap(-yTop, -yBottom, rBottom, theStandardModelControl->sphereLongitude, theLongitudeTable, theStandardModelControl, thePluginInfo);
		/* Create Body */
		latitudeIndex = 2;
		while (latitudeIndex <= theStandardModelControl->sphereLatitude && done) {
			rTop = rBottom;
			yTop = yBottom;
			yBottom = theStandardModelControl->sphereRadius * theLatitudeTable[latitudeIndex];
			rBottom = theStandardModelControl->sphereRadius * theLatitudeTable[theStandardModelControl->sphereLatitude - latitudeIndex];
			if (done)
				done = CreateSphereMantel(yTop, yBottom, rTop, rBottom, theStandardModelControl->sphereLongitude, theLongitudeTable, theStandardModelControl, thePluginInfo);
			if (done)
				done = CreateSphereMantel(-yBottom, -yTop, rBottom, rTop, theStandardModelControl->sphereLongitude, theLongitudeTable, theStandardModelControl, thePluginInfo);
			latitudeIndex++;
		}
	}
	return done;
}

/* --------------------------------------------------------------- Control/Plugin Data Routines */
static void FixControl(StandardModelControlPtr theStandardModelControl)
{
	theStandardModelControl->ownerType = pluginType;
	theStandardModelControl->ownerVersion = pluginVersion;

	FixField(theStandardModelControl->StandardModelType);
	FixField(theStandardModelControl->StandardModelFlag);
	
	FixField(theStandardModelControl->runTimeTiming);
	FixField(theStandardModelControl->runTimeTotalPolygons);
	FixField(theStandardModelControl->runTimeCurrentPolygons);

	FixField(theStandardModelControl->planeWidth);
	FixField(theStandardModelControl->planeLength);
	FixField(theStandardModelControl->planeDivision);
	FixField(theStandardModelControl->planeControl);

	FixField(theStandardModelControl->cubeWidth);
	FixField(theStandardModelControl->cubeLength);
	FixField(theStandardModelControl->cubeHeight);
	FixField(theStandardModelControl->cubeDivision);

	FixField(theStandardModelControl->cylinderRadius);
	FixField(theStandardModelControl->cylinderHeight);
	FixField(theStandardModelControl->cylinderLatitude);
	FixField(theStandardModelControl->cylinderLongitude);

	FixField(theStandardModelControl->coneRadius);
	FixField(theStandardModelControl->coneHeight);
	FixField(theStandardModelControl->coneLatitude);
	FixField(theStandardModelControl->coneLongitude);

	FixField(theStandardModelControl->sphereRadius);
	FixField(theStandardModelControl->sphereLatitude);
	FixField(theStandardModelControl->sphereLongitude);
}  /* FixControl */

static void UpdateControl(unsigned char* oldControl, StandardModelControlPtr newControl)
{
	short temp;
	
	newControl->ownerType = pluginType;
	newControl->ownerVersion = pluginVersion;

	CopyField(oldControl + oldStandardModelType, temp);
	newControl->StandardModelType = temp;
	CopyField(oldControl + oldStandardModelFlag, newControl->StandardModelFlag);
	CopyField(oldControl + oldStandardModelColor, newControl->StandardModelColor);

	CopyField(oldControl + oldRunTimeTiming, newControl->runTimeTiming);
	CopyField(oldControl + oldRunTimeTotalPolygons, newControl->runTimeTotalPolygons);
	CopyField(oldControl + oldRunTimeCurrentPolygons, newControl->runTimeCurrentPolygons);
	
	CopyField(oldControl + oldPlaneWidth, newControl->planeWidth);
	CopyField(oldControl + oldPlaneLength, newControl->planeLength);
	CopyField(oldControl + oldPlaneDivision, newControl->planeDivision);
	CopyField(oldControl + oldPlaneControl, temp);
	newControl->planeControl = temp;

	CopyField(oldControl + oldCubeWidth, newControl->cubeWidth);
	CopyField(oldControl + oldCubeLength, newControl->cubeLength);
	CopyField(oldControl + oldCubeHeight, newControl->cubeHeight);
	CopyField(oldControl + oldCubeDivision, newControl->cubeDivision);

	CopyField(oldControl + oldCylinderRadius, newControl->cylinderRadius);
	CopyField(oldControl + oldCylinderHeight, newControl->cylinderHeight);
	CopyField(oldControl + oldCylinderLatitude, newControl->cylinderLatitude);
	CopyField(oldControl + oldCylinderLongitude, newControl->cylinderLongitude);

	CopyField(oldControl + oldConeRadius, newControl->coneRadius);
	CopyField(oldControl + oldConeHeight, newControl->coneHeight);
	CopyField(oldControl + oldConeLatitude, newControl->coneLatitude);
	CopyField(oldControl + oldConeLongitude, newControl->coneLongitude);

	CopyField(oldControl + oldSphereRadius, newControl->sphereRadius);
	CopyField(oldControl + oldSphereLatitude, newControl->sphereLatitude);
	CopyField(oldControl + oldSphereLongitude, newControl->sphereLongitude);
}  /* UpdateControl */

static void IntializeControlData(StandardModelControlPtr theStandardModelControl)
{
	theStandardModelControl->ownerType = pluginType;
	theStandardModelControl->ownerVersion = pluginVersion;
	
	theStandardModelControl->StandardModelType = 1;
	theStandardModelControl->StandardModelFlag = 0;
	SetARGB(&theStandardModelControl->StandardModelColor, 255, 255, 255, 255);

	theStandardModelControl->runTimeTiming = 0;
	theStandardModelControl->runTimeTotalPolygons = 0;
	theStandardModelControl->runTimeCurrentPolygons = 0;

	theStandardModelControl->planeWidth = 200.0;
	theStandardModelControl->planeLength = 200.0;
	theStandardModelControl->planeDivision = 10;
	theStandardModelControl->planeControl = 2;

	theStandardModelControl->cubeWidth = 200.0;
	theStandardModelControl->cubeLength = 200.0;
	theStandardModelControl->cubeHeight = 200.0;
	theStandardModelControl->cubeDivision = 10;

	theStandardModelControl->cylinderRadius = 200.0;
	theStandardModelControl->cylinderHeight = 200.0;
	theStandardModelControl->cylinderLatitude = 10;
	theStandardModelControl->cylinderLongitude = 10;

	theStandardModelControl->coneRadius = 100.0;
	theStandardModelControl->coneHeight = 200.0;
	theStandardModelControl->coneLatitude = 10;
	theStandardModelControl->coneLongitude = 10;

	theStandardModelControl->sphereRadius = 200.0;
	theStandardModelControl->sphereLatitude = 18;
	theStandardModelControl->sphereLongitude = 18;
}

static void IntializePluginData(StandardModelDataPtr theStandardModelData)
{
	InitMatrix4(&theStandardModelData->theMatrix);
	InitMatrix4(&theStandardModelData->theRotMatrix);
	InitMatrix4(&theStandardModelData->theBlurMatrix);
}  /* InitializePluginData */

static long AllocateControlData(PluginInfoPtr thePluginInfo)
{
	short theError;
	long theTotal;
	HostAllocateRec theHostAllocate;
	StandardModelControlPtr theControl;
	StandardModelControlRec theControlData;

	theError = 0;
	theTotal = sizeof(StandardModelControlRec);

	if ((thePluginInfo->theControlDataSize < theTotal) || (thePluginInfo->theControlDataSize > FixAlignment(theTotal))) {
		if (thePluginInfo->theControlDataSize == oldControlSize)
			UpdateControl((unsigned char*)thePluginInfo->theControlData, &theControlData);
		else
			IntializeControlData(&theControlData);
		
		theHostAllocate.theReference = thePluginInfo->theHostReference;
		theHostAllocate.theControlData = thePluginInfo->theControlData;
		theHostAllocate.theControlDataSize = theTotal;
		theHostAllocate.thePluginData = thePluginInfo->thePluginData;
		theHostAllocate.thePluginDataSize = thePluginInfo->thePluginDataSize;
	
		theError = thePluginInfo->theHostProc(PLUGCOMMAND(hostAllocate), PARAMETERSIZE(sizeof(HostAllocateRec)), &theHostAllocate);
	
		if (theError == 0) {
			thePluginInfo->theControlData = theHostAllocate.theControlData;
			thePluginInfo->theControlDataSize = theHostAllocate.theControlDataSize;
			*(StandardModelControlPtr)thePluginInfo->theControlData = theControlData;
		}
	} else {
		theControl = (StandardModelControlPtr)thePluginInfo->theControlData;
		if (((theControl->ownerType != pluginType) && (theControl->ownerType != REVERSELONG(pluginType))) || ((theControl->ownerVersion != pluginVersion) && (theControl->ownerVersion != REVERSELONG(pluginVersion))))
			IntializeControlData(theControl);
		else if (theControl->ownerVersion == REVERSELONG(pluginVersion))
			FixControl(theControl);
	}

	return theError;
}

static long AllocatePluginData(PluginInfoPtr thePluginInfo)
{
	short theError;
	long theTotal;
	HostAllocateRec theHostAllocate;
	StandardModelControlPtr theControl;

	theError = 0;

	theTotal = sizeof(StandardModelDataRec);
	theControl = (StandardModelControlPtr)thePluginInfo->theControlData;
	switch (theControl->StandardModelType) {

	  case kStandardModelNone:
	  case kStandardModelPlane:
	  case kStandardModelCube:
		/*  blank case  */
		break;

	  case kStandardModelCylinder:
		theTotal += (theControl->cylinderLongitude + 1) * sizeof(double);
		break;

	  case kStandardModelCone:
		theTotal += (theControl->coneLongitude + 1) * sizeof(double);
		break;

	  case kStandardModelSphere:
		theTotal += (theControl->sphereLatitude + theControl->sphereLongitude + 2) * sizeof(double);
		break;

	  case kStandardModelTeapot:
		/*  blank case  */
		break;
	}
	if ((thePluginInfo->thePluginDataSize < theTotal) || (thePluginInfo->thePluginDataSize > FixAlignment(theTotal))) {
		theHostAllocate.theReference = thePluginInfo->theHostReference;
		theHostAllocate.theControlData = thePluginInfo->theControlData;
		theHostAllocate.theControlDataSize = thePluginInfo->theControlDataSize;
		theHostAllocate.thePluginData = thePluginInfo->thePluginData;
		theHostAllocate.thePluginDataSize = theTotal;
	
		theError = thePluginInfo->theHostProc(PLUGCOMMAND(hostAllocate), PARAMETERSIZE(sizeof(HostAllocateRec)), &theHostAllocate);
	
		if (theError == 0) {
			thePluginInfo->thePluginData = theHostAllocate.thePluginData;
			thePluginInfo->thePluginDataSize = theHostAllocate.thePluginDataSize;
			IntializePluginData((StandardModelDataPtr)thePluginInfo->thePluginData);
		}
	}
	return theError;
}

/* --------------------------------------------------------------- Dialog Routines */

static double StringToDouble(PStrPtr iStr)

{
	return FixDouble(Str2Num(iStr));
}

static void SetupPlaneControl(EI_Dialog *theDialog, StandardModelControlPtr theStandardModelControlRecord)
{
	switch (theStandardModelControlRecord->planeControl) {

	  case kPlaneYZ:
	  	SetButton(theDialog, cOrientationRB, 1);
		break;

	  case kPlaneXZ:
	  	SetButton(theDialog, cOrientationRB, 2);
		break;

	  case kPlaneXY:
	  	SetButton(theDialog, cOrientationRB, 0);
		break;
	}
}

static void UpdatePolygonTotal(EI_Dialog* iDialog, StandardModelControlPtr controlPtr)

{
	char		str[100];
	
	sprintf(str, "%d", CalculatePolygonTotal(controlPtr));
	SetControlIDTitle(iDialog, cTotalPolygonsST, str);
}

static void EditTextHitFunction(EI_Control *iControl, EI_DrawContext *iContext)

{
	EI_Dialog*					dialog;
	StandardModelControlPtr		controlPtr;
	
//	Unused argument:
	iContext;
	
	dialog = EI_GetControlDialog(iControl);
	controlPtr = (StandardModelControlPtr) EI_GetDialogExtraData(dialog);
	if (controlPtr != NULL) {
		unsigned char		str[256];
		int					size;
		
		size = 256;
		EI_GetEditTextString(iControl, (char*) str, &size);
		PStrC2P((char*) str);
		switch (EI_GetControlID(iControl)) {
			case cET1:
				switch (controlPtr->StandardModelType) {
					case kStandardModelPlane:
						controlPtr->planeWidth = StringToDouble(str);
						break;
					case kStandardModelCube:
						controlPtr->cubeWidth = StringToDouble(str);
						break;
					case kStandardModelCylinder:
						controlPtr->cylinderRadius = StringToDouble(str);
						break;
					case kStandardModelCone:
						controlPtr->coneRadius = StringToDouble(str);
						break;
					case kStandardModelSphere:
						controlPtr->sphereRadius = StringToDouble(str);
						break;
				}
				break;
			case cET2:
				switch (controlPtr->StandardModelType) {
					case kStandardModelPlane:
						controlPtr->planeLength = StringToDouble(str);
						break;
					case kStandardModelCube:
						controlPtr->cubeLength = StringToDouble(str);
						break;
					case kStandardModelCylinder:
						controlPtr->cylinderHeight = StringToDouble(str);
						break;
					case kStandardModelCone:
						controlPtr->coneHeight = StringToDouble(str);
						break;
					case kStandardModelSphere:
						controlPtr->sphereLatitude = Str2Int(str);
						UpdatePolygonTotal(dialog, controlPtr);
						break;
				}
				break;
			case cET3:
				switch (controlPtr->StandardModelType) {
					case kStandardModelPlane:
						controlPtr->planeDivision = Str2Int(str);
						UpdatePolygonTotal(dialog, controlPtr);
						break;
					case kStandardModelCube:
						controlPtr->cubeHeight = StringToDouble(str);
						break;
					case kStandardModelCylinder:
						controlPtr->cylinderLatitude = Str2Int(str);
						UpdatePolygonTotal(dialog, controlPtr);
						break;
					case kStandardModelCone:
						controlPtr->coneLatitude = Str2Int(str);
						UpdatePolygonTotal(dialog, controlPtr);
						break;
					case kStandardModelSphere:
						controlPtr->sphereLongitude = Str2Int(str);
						UpdatePolygonTotal(dialog, controlPtr);
						break;
				}
				break;
			case cET4:
				switch (controlPtr->StandardModelType) {
					case kStandardModelPlane:
						/*	Nothing.	*/
						break;
					case kStandardModelCube:
						controlPtr->cubeDivision = Str2Int(str);
						UpdatePolygonTotal(dialog, controlPtr);
						break;
					case kStandardModelCylinder:
						controlPtr->cylinderLongitude = Str2Int(str);
						UpdatePolygonTotal(dialog, controlPtr);
						break;
					case kStandardModelCone:
						controlPtr->coneLongitude = Str2Int(str);
						UpdatePolygonTotal(dialog, controlPtr);
						break;
					case kStandardModelSphere:
						/*	Nothing.	*/
						break;
				}
				break;
		}
	}
}

static void OrientationHitFunction(EI_Control *iControl, EI_DrawContext *iContext)

{
	EI_Dialog*					dialog;
	StandardModelControlPtr		controlPtr;
	
//	Unused argument:
	iContext;
	
	dialog = EI_GetControlDialog(iControl);
	controlPtr = (StandardModelControlPtr) EI_GetDialogExtraData(dialog);
	if (controlPtr != NULL) {
		switch (EI_GetControlValue(iControl)) {
			case 0:
				controlPtr->planeControl = kPlaneXY;
				break;
			case 1:
				controlPtr->planeControl = kPlaneYZ;
				break;
			case 2:
				controlPtr->planeControl = kPlaneXZ;
				break;
		}
		SetupPlaneControl(dialog, controlPtr);
	}
}

static void ColorHitFunction(EI_Control *iControl, EI_DrawContext *iContext)

{
	EI_Dialog*					dialog;
	StandardModelControlPtr		controlPtr;
	int							red, green, blue;
	
//	Unused argument:
	iContext;
	
	dialog = EI_GetControlDialog(iControl);
	controlPtr = (StandardModelControlPtr) EI_GetDialogExtraData(dialog);
	if (controlPtr != NULL) {
		EI_GetColorButtonColor(iControl, &red, &green, &blue, NULL);
		controlPtr->StandardModelColor.argb.r = (unsigned char) red;
		controlPtr->StandardModelColor.argb.g = (unsigned char) green;
		controlPtr->StandardModelColor.argb.b = (unsigned char) blue;
	}
}

static void SetUpDialog(EI_Dialog *theDialog, StandardModelControlPtr theStandardModelControlRecord)
{
	PStr255		str;
	
	switch (theStandardModelControlRecord->StandardModelType) {
		
		case kStandardModelPlane:
			SetControlIDVisible(theDialog, cOrientationST, 1);
			SetControlIDVisible(theDialog, cOrientationRB, 1);
			SetControlIDVisible(theDialog, cET4, 0);
			
			SetControlIDTitle(theDialog, cET1, "Width:");
			SetControlIDTitle(theDialog, cET2, "Length:");
			SetControlIDTitle(theDialog, cET3, "Division:");
			
			Num2Str(4, theStandardModelControlRecord->planeWidth, str);
			SetEditTextIDString(theDialog, cET1, PStrP2C(str));
			Num2Str(4, theStandardModelControlRecord->planeLength, str);
			SetEditTextIDString(theDialog, cET2, PStrP2C(str));
			Num2Str(4, theStandardModelControlRecord->planeDivision, str);
			SetEditTextIDString(theDialog, cET3, PStrP2C(str));
			
			SetupPlaneControl(theDialog, theStandardModelControlRecord);
			break;

		case kStandardModelCube:
			SetControlIDVisible(theDialog, cOrientationST, 0);
			SetControlIDVisible(theDialog, cOrientationRB, 0);
			SetControlIDVisible(theDialog, cET4, 1);
			
			SetControlIDTitle(theDialog, cET1, "Width:");
			SetControlIDTitle(theDialog, cET2, "Length:");
			SetControlIDTitle(theDialog, cET3, "Height:");
			SetControlIDTitle(theDialog, cET4, "Division:");
			
			Num2Str(4, theStandardModelControlRecord->cubeWidth, str);
			SetEditTextIDString(theDialog, cET1, PStrP2C(str));
			Num2Str(4, theStandardModelControlRecord->cubeLength, str);
			SetEditTextIDString(theDialog, cET2, PStrP2C(str));
			Num2Str(4, theStandardModelControlRecord->cubeHeight, str);
			SetEditTextIDString(theDialog, cET3, PStrP2C(str));
			Num2Str(4, theStandardModelControlRecord->cubeDivision, str);
			SetEditTextIDString(theDialog, cET4, PStrP2C(str));
		break;

		case kStandardModelCylinder:
			SetControlIDVisible(theDialog, cOrientationST, 0);
			SetControlIDVisible(theDialog, cOrientationRB, 0);
			SetControlIDVisible(theDialog, cET4, 1);
			
			SetControlIDTitle(theDialog, cET1, "Radius:");
			SetControlIDTitle(theDialog, cET2, "Height:");
			SetControlIDTitle(theDialog, cET3, "Latitude Division:");
			SetControlIDTitle(theDialog, cET4, "Longitude Division:");
			
			Num2Str(4, theStandardModelControlRecord->cylinderRadius, str);
			SetEditTextIDString(theDialog, cET1, PStrP2C(str));
			Num2Str(4, theStandardModelControlRecord->cylinderHeight, str);
			SetEditTextIDString(theDialog, cET2, PStrP2C(str));
			Num2Str(4, theStandardModelControlRecord->cylinderLatitude, str);
			SetEditTextIDString(theDialog, cET3, PStrP2C(str));
			Num2Str(4, theStandardModelControlRecord->cylinderLongitude, str);
			SetEditTextIDString(theDialog, cET4, PStrP2C(str));
		break;

		case kStandardModelCone:
			SetControlIDVisible(theDialog, cOrientationST, 0);
			SetControlIDVisible(theDialog, cOrientationRB, 0);
			SetControlIDVisible(theDialog, cET4, 1);
			
			SetControlIDTitle(theDialog, cET1, "Radius:");
			SetControlIDTitle(theDialog, cET2, "Height:");
			SetControlIDTitle(theDialog, cET3, "Latitude Division:");
			SetControlIDTitle(theDialog, cET4, "Longitude Division:");
			
			Num2Str(4, theStandardModelControlRecord->coneRadius, str);
			SetEditTextIDString(theDialog, cET1, PStrP2C(str));
			Num2Str(4, theStandardModelControlRecord->coneHeight, str);
			SetEditTextIDString(theDialog, cET2, PStrP2C(str));
			Num2Str(4, theStandardModelControlRecord->coneLatitude, str);
			SetEditTextIDString(theDialog, cET3, PStrP2C(str));
			Num2Str(4, theStandardModelControlRecord->coneLongitude, str);
			SetEditTextIDString(theDialog, cET4, PStrP2C(str));
		break;

		case kStandardModelSphere:
			SetControlIDVisible(theDialog, cOrientationST, 0);
			SetControlIDVisible(theDialog, cOrientationRB, 0);
			SetControlIDVisible(theDialog, cET4, 0);
			
			SetControlIDTitle(theDialog, cET1, "Radius:");
			SetControlIDTitle(theDialog, cET2, "Latitude Division:");
			SetControlIDTitle(theDialog, cET3, "Longitude Division:");
			
			Num2Str(4, theStandardModelControlRecord->sphereRadius, str);
			SetEditTextIDString(theDialog, cET1, PStrP2C(str));
			Num2Str(4, theStandardModelControlRecord->sphereLatitude, str);
			SetEditTextIDString(theDialog, cET2, PStrP2C(str));
			Num2Str(4, theStandardModelControlRecord->sphereLongitude, str);
			SetEditTextIDString(theDialog, cET3, PStrP2C(str));
		break;

		case kStandardModelTeapot:
			break;
	}
	SetColorButtonIDARGB(theDialog, cPolygonColorCB, &theStandardModelControlRecord->StandardModelColor);
}

static void SetUpStandardModel(EI_Dialog *theDialog, StandardModelControlPtr theStandardModelControlRecord)
{
	switch (theStandardModelControlRecord->StandardModelType) {

		case kStandardModelPlane:
			SetButton(theDialog, cShapeRB, 0);
			break;

		case kStandardModelCube:
			SetButton(theDialog, cShapeRB, 1);
			break;

		case kStandardModelCylinder:
			SetButton(theDialog, cShapeRB, 2);
			break;

		case kStandardModelCone:
			SetButton(theDialog, cShapeRB, 3);
			break;

		case kStandardModelSphere:
			SetButton(theDialog, cShapeRB, 4);
			break;

		case kStandardModelTeapot:
			/*  blank case  */
			break;
	}
	
	UpdatePolygonTotal(theDialog, theStandardModelControlRecord);
}

static void RadioButtonHitFunction(EI_Control *iControl, EI_DrawContext *iContext)

{
	EI_Dialog*					dialog;
	StandardModelControlPtr		controlPtr;
	
//	Unused argument:
	iContext;
	
	dialog = EI_GetControlDialog(iControl);
	controlPtr = (StandardModelControlPtr) EI_GetDialogExtraData(dialog);
	if (controlPtr != NULL) {
		switch (EI_GetControlValue(iControl)) {
			case 0:
				controlPtr->StandardModelType = kStandardModelPlane;
				break;
			case 1:
				controlPtr->StandardModelType = kStandardModelCube;
				break;
			case 2:
				controlPtr->StandardModelType = kStandardModelCylinder;
				break;
			case 3:
				controlPtr->StandardModelType = kStandardModelCone;
				break;
			case 4:
				controlPtr->StandardModelType = kStandardModelSphere;
				break;
		}
		SetUpDialog(dialog, controlPtr);
		SetUpStandardModel(dialog, controlPtr);
	}
}

static long InterfaceDialog(StandardModelControlPtr theStandardModelControl, PluginInfoPtr thePluginInfo)
{
	EI_DirectoryRef				*homeDir;
	EI_FileRef					*resourceFileRef;
	EI_ResourceFile				*resFile;
	EI_Dialog					*dialog;
	StandardModelControlRec		theStandardModelControlRecord;
	long						theResult;
	
//	Unused:
	thePluginInfo;
	
	theResult = pluginNoErr;
	
	homeDir = EI_GetHomeDirectory();
	if (homeDir == NULL)
		theResult = pluginMemoryErr;
	resourceFileRef = EI_MakeFileRef(homeDir, "Standard Shape.rsc",
										EI_MAKE_FILE_TYPE('R', 'S', 'R', 'C'), "rsc");
	if (resourceFileRef == NULL)
		theResult = pluginMemoryErr;
	if (theResult == pluginNoErr) {
		resFile = EI_OpenResourceFile(resourceFileRef);
		if (resFile == NULL)
			theResult = pluginResourceErr;
	}
	
	EI_DestroyFileRef(resourceFileRef);
	EI_DestroyDirectoryRef(homeDir);
	
	if (theResult == pluginNoErr) {
		dialog = EI_MakeDialog(resFile, interfaceDialogID);
		if (dialog == NULL)
			theResult = pluginResourceErr;
	}
	
	theStandardModelControlRecord = *theStandardModelControl;
	if (theResult == pluginNoErr) {
		SetUpDialog(dialog, &theStandardModelControlRecord);
		SetUpStandardModel(dialog, &theStandardModelControlRecord);
		
	//	Attach callbacks to items here...
		
		EI_SetDialogExtraData(dialog, &theStandardModelControlRecord);
		
		SetControlIDHitFunction(dialog, cShapeRB, RadioButtonHitFunction);
		SetControlIDHitFunction(dialog, cET1, EditTextHitFunction);
		SetControlIDHitFunction(dialog, cET2, EditTextHitFunction);
		SetControlIDHitFunction(dialog, cET3, EditTextHitFunction);
		SetControlIDHitFunction(dialog, cET4, EditTextHitFunction);
		SetControlIDHitFunction(dialog, cOrientationRB, OrientationHitFunction);
		SetControlIDHitFunction(dialog, cPolygonColorCB, ColorHitFunction);
		if (EI_ExecuteDialog(dialog))
			*theStandardModelControl = theStandardModelControlRecord;
		else
			theResult = pluginCancelErr;
	}
	
/*	Dispose the dialog:	*/
	EI_DestroyDialog(dialog);
	
/*	Close the resource file:	*/
	EI_CloseResourceFile(resFile);
	
	return theResult;
}

/* --------------------------------------------------------------- Dialog Routines */
static long CommandModelReset(PluginInfoPtr thePluginInfo)
{
	short theError;
	HostModelResetRec theHostModelReset;

	theHostModelReset.theReference = thePluginInfo->theHostReference;

	theError = thePluginInfo->theHostProc(PLUGCOMMAND(hostModelReset), PARAMETERSIZE(sizeof(HostModelResetRec)), &theHostModelReset);

	return theError;
}

static long CommandInitializePlugin(long theSize, void* theParameter)
{
	long theError;
	PluginInfoRec thePluginInfo;
	PluginInitializePtr theInitialize;

	theError = 0;
	if (PLUGPARAMETERFAIL(theSize, sizeof(PluginInitializeRec)))
		theError = pluginParameterErr;
	else {
		theInitialize = (PluginInitializePtr)theParameter;
		thePluginInfo = theInitialize->theInfo;
		theError = AllocateControlData(&thePluginInfo);
		if (theError == 0)
			theInitialize->theInfo = thePluginInfo;
		if (theError == 0)
			theError = CommandModelReset(&thePluginInfo);
	}
	return theError;
}

static long CommandFinishPlugin(long theSize, void* theParameter)
{
	long theError;
	PluginFinishPtr theFinish;

	theError = 0;
	if (PLUGPARAMETERFAIL(theSize, sizeof(PluginFinishRec)))
		theError = pluginParameterErr;
	else
		theFinish = (PluginFinishPtr)theParameter;
	return theError;
}

static long CommandCheckPlugin(long theSize, void* theParameter)
{
	PluginCheckPtr theParameters;

	if (PLUGPARAMETERFAIL(theSize, sizeof(PluginCheckRec)))
		return pluginParameterErr;

	theParameters = (PluginCheckPtr)theParameter;
	switch (COMMANDOCHECK(theParameters->theCommand)) {
		case pluginInitialize:
		case pluginFinish:
		case pluginInformation:
		case pluginCheck:
		case pluginDescribe:
		case pluginInterface:
		case pluginModelSetup:
		case pluginModelGINF:
		case pluginModelGATR:
		case pluginModelGBLR:
		case pluginModelTMAP:
		case pluginModelRMAP:
		case pluginModelBMAP:
		case pluginModelGenerate:
			return pluginNoErr;
			break;
	}
	return pluginCommandErr;
}

static long CommandInformation(long theSize, void* theParameter)
{
	long theError;
	PluginInformationPtr theInformation;

	theError = 0;
	if (PLUGPARAMETERFAIL(theSize, sizeof(PluginInformationRec)))
		theError = pluginParameterErr;
	else {
		theInformation = (PluginInformationPtr)theParameter;
		theInformation->thePluginType = pluginType;
		theInformation->thePluginVersion = pluginVersion;
		PStrCpy(theInformation->thePluginName, pluginName);
		PStrCpy(theInformation->thePluginCopyright, pluginCopyright);
		PStrCpy(theInformation->thePluginAuthor, pluginAuthor);
		PStrCpy(theInformation->thePluginDescription, pluginDescription);
		theInformation->thePluginPermission = pluginPermission;
	}
	return theError;
}

static long CommandDescribe(long theSize, void* theParameter)
{
	long theError;
	PluginDescribePtr theDescribe;

	theError = 0;
	if (PLUGPARAMETERFAIL(theSize, sizeof(PluginDescribeRec)))
		theError = pluginParameterErr;
	else {
		theDescribe = (PluginDescribePtr)theParameter;
		DescribePluginError(theDescribe->theResult, theDescribe->theDescription);
	}
	return theError;
}

static long CommandInterface(long theSize, void* theParameter)
{
	PluginInfoRec thePluginInfo;
	long theError;
	PluginInterfacePtr theInterface;

	theError = 0;
	theInterface = (PluginInterfacePtr)theParameter;
	if ((PLUGPARAMETERFAIL(theSize, sizeof(PluginInterfaceRec))) || (theInterface->theInfo.theControlData == NULL) || (theInterface->theInfo.theControlDataSize < sizeof(StandardModelControlRec)))
		theError = pluginParameterErr;
	if (theError == 0) {
		thePluginInfo = theInterface->theInfo;
		theError = InterfaceDialog((StandardModelControlPtr)thePluginInfo.theControlData, &thePluginInfo);
		if (theError == 0)
			theError = CommandModelReset(&thePluginInfo);
	}
	return theError;
}

static long CommandModelSetup(long theSize, void* theParameter)
{
	long theError;
	PluginModelSetupPtr theSetup;
	StandardModelDataPtr theData;

	theError = 0;
	if (PLUGPARAMETERFAIL(theSize, sizeof(PluginModelSetupRec)))
		theError = pluginParameterErr;
	else {
		theSetup = (PluginModelSetupPtr)theParameter;
		theError = AllocatePluginData(&theSetup->theInfo);
		if (theError == 0) {
			theData = (StandardModelDataPtr)theSetup->theInfo.thePluginData;
			theData->theMatrix = theSetup->theGroupMatrix;
			theData->theRotMatrix = theSetup->theGroupRotMatrix;
			theData->theBlurMatrix = theSetup->theGroupBlurMatrix;
		}
	}
	return theError;
}

/* Setup the group GINF record */

static long CommandModelGINF(long theCommandRecSize, void* theCommandRec)
{
	PluginModelGINFPtr theParameters;
	long theResult;

	theResult = pluginNoErr;
	theParameters = (PluginModelGINFPtr)theCommandRec;
	if (PLUGPARAMETERFAIL(theCommandRecSize, sizeof(PluginModelGINFRec)))
		theResult = pluginParameterErr;
	else
		theParameters->theChangeFlag = 0;
	return theResult;
}  /* CommandModelGINF */

/* Setup the group GATR record */

static long CommandModelGATR(long theCommandRecSize, void* theCommandRec)
{
	PluginModelGATRPtr theParameters;
	long theResult;

	theResult = pluginNoErr;
	theParameters = (PluginModelGATRPtr)theCommandRec;
	if (PLUGPARAMETERFAIL(theCommandRecSize, sizeof(PluginModelGATRRec)))
		theResult = pluginParameterErr;
	else
		theParameters->theChangeFlag = 0;
	return theResult;
}  /* CommandModelGATR */

/* Setup the group GBLR record */

static long CommandModelGBLR(long theCommandRecSize, void* theCommandRec)
{
	PluginModelGBLRPtr theParameters;
	long theResult;

	theResult = pluginNoErr;
	theParameters = (PluginModelGBLRPtr)theCommandRec;
	if (PLUGPARAMETERFAIL(theCommandRecSize, sizeof(PluginModelGBLRRec)))
		theResult = pluginParameterErr;
	else
		theParameters->theChangeFlag = 0;
	return theResult;
}  /* DoGBLR */

/* Setup the group TMAP record */

static long CommandModelTMAP(long theCommandRecSize, void* theCommandRec)
{
	PluginModelTMAPPtr theParameters;
	long theResult;

	theResult = pluginNoErr;
	theParameters = (PluginModelTMAPPtr)theCommandRec;
	if (PLUGPARAMETERFAIL(theCommandRecSize, sizeof(PluginModelTMAPRec)))
		theResult = pluginParameterErr;
	else
		theParameters->theChangeFlag = 0;
	return theResult;
}  /* CommandModelTMAP */

/* Setup the group BMAP record */

static long CommandModelBMAP(long theCommandRecSize, void* theCommandRec)
{
	PluginModelBMAPPtr theParameters;
	long theResult;

	theResult = pluginNoErr;
	theParameters = (PluginModelBMAPPtr)theCommandRec;
	if (PLUGPARAMETERFAIL(theCommandRecSize, sizeof(PluginModelBMAPRec)))
		theResult = pluginParameterErr;
	else
		theParameters->theChangeFlag = 0;
	return theResult;
}  /* CommandModelBMAP */

/* Setup the group RMAP record */

static long CommandModelRMAP(long theCommandRecSize, void* theCommandRec)
{
	PluginModelRMAPPtr theParameters;
	long theResult;

	theResult = pluginNoErr;
	theParameters = (PluginModelRMAPPtr)theCommandRec;
	if (PLUGPARAMETERFAIL(theCommandRecSize, sizeof(PluginModelRMAPRec)))
		theResult = pluginParameterErr;
	else
		theParameters->theChangeFlag = 0;
	return theResult;
}  /* CommandModelRMAP */

static long CommandModelGenerate(long theSize, void* theParameter)
{
	boolean done;
	long theError;
	PluginInfoRec thePluginInfo;
	PluginModelGeneratePtr theParameters = (PluginModelGeneratePtr)theParameter;

	theError = 0;
	if ((PLUGPARAMETERFAIL(theSize, sizeof(PluginModelGenerateRec))) || (theParameters->theInfo.theControlData == NULL))
		theError = pluginParameterErr;
	else {
		StandardModelControlPtr theControl;
		
		done = 1;
		thePluginInfo = theParameters->theInfo;
		theControl = (StandardModelControlPtr)thePluginInfo.theControlData;
		theError = AllocatePluginData(&thePluginInfo);
		if (theError == 0) {
			long theVertexFlags = theParameters->theVertexFlags;

			BCLR(theVertexFlags, pluginVertexTextureFlag);
			BCLR(theVertexFlags, pluginVertexColorFlag);
			theError = AddGroup(&thePluginInfo, NULL, theVertexFlags);
		}
		if (theError == 0) {
			switch (theControl->StandardModelType) {
			  case kStandardModelNone:
				break;
	
			  case kStandardModelPlane:
				done = GeneratePlaneData(theControl, &thePluginInfo);
				break;
	
			  case kStandardModelCube:
				done = GenerateCubeData(theControl, &thePluginInfo);
				break;
	
			  case kStandardModelCylinder:
				done = GenerateCylinderData(theControl, &thePluginInfo, thePluginInfo.thePluginData);
				break;
	
			  case kStandardModelCone:
				done = GenerateConeData(theControl, &thePluginInfo, thePluginInfo.thePluginData);
				break;
	
			  case kStandardModelSphere:
				done = GenerateSphereData(theControl, &thePluginInfo, thePluginInfo.thePluginData);
				break;
	
			  case kStandardModelTeapot:
				break;
			}
		}
		if (!done)
			theError = pluginCancelErr;
	}
	return theError;
}  /* CommandModelGenerate */

/* --------------------------------------------------------------- Main Routine */
EXPDLL long Main(unsigned long theCommand, long theCommandRecSize, void* theCommandRec)
{
	long theError;

	switch (HOSTCOMMAND(theCommand)) {
		case pluginInitialize:		theError = CommandInitializePlugin(theCommandRecSize, theCommandRec);	break;
		case pluginFinish:			theError = CommandFinishPlugin(theCommandRecSize, theCommandRec);		break;
		case pluginCheck:			theError = CommandCheckPlugin(theCommandRecSize, theCommandRec);		break;
		case pluginInformation:		theError = CommandInformation(theCommandRecSize, theCommandRec);		break;
		case pluginDescribe:		theError = CommandDescribe(theCommandRecSize, theCommandRec);			break;
		case pluginInterface:		theError = CommandInterface(theCommandRecSize, theCommandRec);			break;
		case pluginModelSetup:		theError = CommandModelSetup(theCommandRecSize, theCommandRec);			break;
		case pluginModelGINF:		theError = CommandModelGINF(theCommandRecSize, theCommandRec);			break;
		case pluginModelGATR:		theError = CommandModelGATR(theCommandRecSize, theCommandRec);			break;
		case pluginModelGBLR:		theError = CommandModelGBLR(theCommandRecSize, theCommandRec);			break;
		case pluginModelTMAP:		theError = CommandModelTMAP(theCommandRecSize, theCommandRec);			break;
		case pluginModelBMAP:		theError = CommandModelBMAP(theCommandRecSize, theCommandRec);			break;
		case pluginModelRMAP:		theError = CommandModelRMAP(theCommandRecSize, theCommandRec);			break;
		case pluginModelGenerate:	theError = CommandModelGenerate(theCommandRecSize, theCommandRec);		break;
		default:					theError = pluginCommandErr;											break;
	}
	return theError;
}
