/*
 *		Copyright (c) 1987-1999 Play, Inc.
 */

#ifndef __RoundUtils_h__
#define __RoundUtils_h__

#ifdef __cplusplus
extern "C" {
#endif

#define LIMIT01(n)		((n) <= 0.0f ? 0.0f : (n) >= 1.0f ? 1.0f : (n))
#define INTEGERLIMIT(a) ((a) < -32767 ? -32767 : ((a) > 32767 ? 32767 : (a)))

//extern double RU_Magic;

#ifdef __LITTLE_ENDIAN__
#define LONGTOFLOAT(d, n) {\
	unsigned long _temp[2];\
	_temp[0] = (unsigned long)(n);\
	_temp[1] = 0x43380000 - (_temp[0] >> 31);\
	(d) = *((double*)_temp) - 6755399441055744.0;\
}  /* LONGTOFLOAT */

#define ULONGTOFLOAT(d, n) {\
	unsigned long _temp[2];\
	_temp[0] = (unsigned long)(n);\
	_temp[1] = 0x43380000;\
	(d) = *((double*)_temp) - 6755399441055744.0;\
}  /* ULONGTOFLOAT */
#else
#define LONGTOFLOAT(d, n) {\
	unsigned long _temp[2];\
	_temp[1] = (unsigned long)(n);\
	_temp[0] = 0x43380000 - (_temp[1] >> 31);\
	(d) = *((double*)_temp) - 6755399441055744.0;\
}  /* LONGTOFLOAT */

#define ULONGTOFLOAT(d, n) {\
	unsigned long _temp[2];\
	_temp[1] = (unsigned long)(n);\
	_temp[0] = 0x43380000;\
	(d) = *((double*)_temp) - 6755399441055744.0;\
}  /* ULONGTOFLOAT */
#endif

#ifdef __ROUNDTOFLOOR__

/* Automaticly rounding towards negative infinity */

long Floor(double n);
long Floor2(double* n);
#define FLOOR(n) ((long)(n))

long Ceil(double n);
long Ceil2(double* n);
#define CEIL(d, n) (-(long)(-(n)))

long CalcFactor(double n);
long CalcFactor2(double* n);
#define CALCFACTOR(n) ((long)((n) * 1024.0) & 1023)
	
long CalcLFactor(double n);
long CalcLFactor2(double* n);
#define CALCLFACTOR(n) ((long)((n) * 1024.0) & 16383)

double CalcRFactor(double n);
double CalcRFactor2(double* n);
void CalcRFactor3(double* n);
#define CALCRFACTOR(d, n) ((n) - (long)(n))

long SplitFactor(double* n);
#define SPLITFACTOR(d, n) {\
	d = n;\
	n -= d;\
}

long ARound(double n);
long ARound2(double* n);
#define AROUND(n) ((long)((n) + 0.5))

long ARoundI(double n);
long ARoundI2(double* n);
#define AROUNDI(d, n) {\
	long _TEMP_ = (long)((n) + 0.5);\
	d = INTEGERLIMIT(_TEMP_);\
}

long LRound(double n);
long LRound2(double* n);
#define LROUND(n) ((long)((n) * 65536.0))

long IRound(double n);
long IRound2(double* n);
#define IROUND(d, n) {\
	long _TEMP_ = (n) * 16384.0;\
	d = INTEGERLIMIT(_TEMP_);\
}

long IRoundR(float n);
long IRoundR2(float* n);
#define IROUNDR(d, n) {\
	long _TEMP_ = (float)(n) * 16384.0;\
	d = INTEGERLIMIT(_TEMP_);\
}
	
long ITrunc(double n);
long ITrunc2(double* n);
#define ITRUNC(d, n) {\
	long _TEMP_ = (n);\
	d = INTEGERLIMIT(d);\
}

long FTrunc(double n);
long FTrunc2(double* n);
#define FTRUNC(d, n) (d = (long)((n) * 1024.0))

long TruncLo(double n);
long TruncLo2(double* n);
#define TRUNCLO(d, n) {double _n_ = (n) + 1.0; d = FLOOR(_n_)}

long TruncHi(double n);
long TruncHi2(double* n);
#define TRUNCHI FLOORF

#else

long Floor(double n);
long Floor2(double* n);
#define FLOOR(n) ((n) >= 0.0 ? (long)(n) : (long)((double)(n) - 0.999999999))

long Ceil(double n);
long Ceil2(double* n);
#define CEIL(n) ((n) < 0.0 ? (long)(n) : (long)((double)(n) + 0.999999999));

long CalcFactor(double n);
long CalcFactor2(double* n);
#define CALCFACTOR(n) ((long)((n) * 1024.0) & 1023)
	
long CalcLFactor(double n);
long CalcLFactor2(double* n);
#define CALCLFACTOR(n) ((long)((n) * 16384.0) & 16383)

double CalcRFactor(double n);
double CalcRFactor2(double* n);
void CalcRFactor3(double* n);
#define CALCRFACTOR(n) ((n) - FLOOR(n))

long SplitFactor(double* n);
#define SPLITFACTOR(d, n) {\
	d = FLOOR(n);\
	n -= d;\
}

/* ARoundI(n) == (long)floor(n + 0.5) */

long ARound(double n);
long ARound2(double* n);
#define AROUND(n) ((n) < 0.0 ? (long)((n) - 0.5) : (long)((n) + 0.5))

/* ARoundI(n) == (short)floor(n + 0.5) */

long ARoundI(double n);
long ARoundI2(double* n);
#define AROUNDI(d, n) {\
	if (n < 0.0) {\
		long _TEMP_ = n - 0.5;\
		if (_TEMP_ <= -32767)\
			d = -32767;\
		else\
			d = _TEMP_;\
	} else {\
		long _TEMP_ = n + 0.5;\
		if (_TEMP_ >= 32767)\
			d = 32767;\
		else\
			d = _TEMP_;\
	} \
}

/* IRound(n) == (long)(n * 65536.0) */

long LRound(double n);
long LRound2(double* n);
#define LROUND(n) ((long)((n) * 65536.0))

/* IRound(n) == (short)(n * 16384.0) */

long IRound(double n);
long IRound2(double* n);
#define IROUND(d, n) {\
	long _TEMP_ = (n) * 16384.0;\
	d = INTEGERLIMIT(_TEMP_);\
}

/* IRoundR(n) == (short)((float)n * 16384.0) */

long IRoundR(float n);
long IRoundR2(float* n);
#define IROUNDR(d, n) {\
	long _TEMP_ = (float)(n) * 16384.0;\
	d = INTEGERLIMIT(_TEMP_);\
}

/* ITrunc(n) == (short)floor(n) */

long ITrunc(double n);
long ITrunc2(double* n);
#define ITRUNC(d, n) {\
	if (n <= -32767.0)\
		d = -32767.0;\
	else if (n >= 32767.0)\
		d = 32767.0;\
	else\
		d = FLOOR(n);\
}

/* FTrunc(n) == (long)floor(n * 1024.0) */

long FTrunc(double n);
long FTrunc2(double* n);
#define FTRUNC(d, n) {\
	double _T_ = (n) * 1024.0;\
	d = FLOOR(_T_);\
}

/* TruncLo(n) == (long)floor(n + 1.0) */

long TruncLo(double n);
long TruncLo2(double* n);
#define TRUNCLO(d, n) {long _n_ = (long)(n); d = (_n_ != MAXINT) && (_n_ <= (n)) ? _n_ + 1 : _n_;}

/* TruncHi(n) == (long)floor(n) */

long TruncHi(double n);
long TruncHi2(double* n);
#define TRUNCHI(d, n) {long _n_ = (long)(n); d = (_n_ != MININT) && (_n_ > (n)) ? _n_ - 1 : _n_;}

#endif

#ifdef __cplusplus
}
#endif

#endif
