/*
Copyright 1995-1998 Electric Image, Inc. This source code may only be used by
registered owners of the ElectricImage Plug-In Kit. Owners of the Kit may
include the source code in their products (freeware, shareware, or commercial).
This copyright notice must be included in its original form in all such
products. This source code may not be offered for sale or distributed by any
entity without the written permission of Electric Image, Inc.
*/

#include "PluginFPU.h"
#include "PString.h"

/* 8, 10, 12 Byte Floating Point Conversions */

void ConvertFloat12To10(void* the12, void* the10)
{
	*(short*)the10 = *(short*)the12;
	*(long*)((char *)the10 + 2) = *((long*)the12 + 1);
	*(long*)((char *)the10 + 6) = *((long*)the12 + 2);
}  /* ConvertFloat12To10 */

void ConvertFloat12To8(void* the12, void* the8)
{
	uchar the10[10];
	
	ConvertFloat12To10(the12, the10);
	ConvertFloat10To8(the10, the8);
}  /* ConvertFloat12To8 */

void ConvertFloat10To12(void* the10, void* the12)
{
	*(short*)the12 = *(short*)the10;
	*((short*)the12 + 1) = 0;
	*((long*)the12 + 1) = *(long*)((char *)the10 + 2);
	*((long*)the12 + 2) = *(long*)((char *)the10 + 6);
}  /* ConvertFloat10To12 */

void ConvertFloat10To8(void* the10, void* the8)
{
	ushort exponent, low4;
	long i;
	union {
		ulong l[2];
		ushort i[4];
		uchar c[8];
	} fraction;
	boolean signSet, iSet, zeroFrac;
	
	signSet = (*(char *)the10 & 0x80) != 0;
	exponent = *(ushort*)the10 & 0x7FFF;
	fraction.l[0] = *(long*)((char *)the10 + 2);
	fraction.l[1] = *(long*)((char *)the10 + 6);
	iSet = (fraction.c[0] & 0x80) != 0;
	fraction.l[0] <<= 1;
	fraction.l[0] |= (fraction.l[1] & 0x80000000) ? 1 : 0;
	fraction.l[1] <<= 1;
	zeroFrac = (fraction.l[0] == 0) && (fraction.l[1] == 0);
	low4 = fraction.c[0] >> 4;
	for (i = 0; i < 6; i++)
		fraction.c[i] = BOR((fraction.c[i] << 4), (fraction.c[i + 1] >> 4));
	if (exponent != 32767)
		if (((short)exponent - 16383) > 1023) {
			exponent = 32767;			/* overflow to Infinity */
			zeroFrac = 1;
		} else if (((short)exponent - 16383) < -1022) {
			iSet = 0;				/* underflow to Zero */
			zeroFrac = 1;
		}
	if (exponent == 32767)
		if (zeroFrac) {					/* Infinity */
			exponent = 2047 << 4;
			if (signSet)
				exponent |= 0x8000;
			*(ushort*)the8 = exponent;
			*((ushort*)the8 + 1) = 0;
			*((ulong*)the8 + 1) = 0;
		} else {						/* NaN */
			exponent = (2047 << 4) | low4;
			if (signSet)
				exponent |= 0x8000;
			*(ushort*)the8 = exponent;
			*((ushort*)the8 + 1) = fraction.i[0];
			*((ushort*)the8 + 2) = fraction.i[1];
			*((ushort*)the8 + 3) = fraction.i[2];
		}
	else if (!iSet)
		if (zeroFrac) {					/* Zero */
			*(ulong*)the8 = signSet ? 0x80000000 : 0;
			*((ulong*)the8 + 1) = 0;
		} else {						/* Denormalized */
			exponent = ((exponent - 16383 + 1022) << 4) | low4;
			if (signSet)
				exponent |= 0x8000;
			else
				exponent &= 0x7FFF;
			*(ushort*)the8 = exponent;
			*((ushort*)the8 + 1) = fraction.i[0];
			*((ushort*)the8 + 2) = fraction.i[1];
			*((ushort*)the8 + 3) = fraction.i[2];
		}
	else {								/* Normalized */
		exponent = ((exponent - 16383 + 1023) << 4) | low4;
		if (signSet)
			exponent |= 0x8000;
		else
			exponent &= 0x7FFF;
		*(ushort*)the8 = exponent;
		*((ushort*)the8 + 1) = fraction.i[0];
		*((ushort*)the8 + 2) = fraction.i[1];
		*((ushort*)the8 + 3) = fraction.i[2];
	}
}  /* ConvertFloat10To8 */

void ConvertFloat8To10(void* the8, void* the10)
{
	ushort exponent;
	long i;
	union {
		ulong l[2];
		uchar c[8];
	} fraction;
	boolean signSet, zeroFrac;
	ulong bitSet;
	
	signSet = (*(uchar*)the8 & 0x80) != 0;
	exponent = (*(ushort*)the8 & 0x7FFF) >> 4;
	for (i = 0, zeroFrac = 1; i < 6; i++) {
		fraction.c[i] = *((char *)the8 + i + 1) << 4 | *((char *)the8 + i + 2) >> 4;
		zeroFrac = zeroFrac && fraction.c[i] == 0;
	}
	fraction.c[6] = *((char *)the8 + 7) << 4;
	fraction.c[7] = 0;
	zeroFrac = zeroFrac && fraction.c[6] == 0;
	bitSet = fraction.l[0] & 0x1;
	fraction.l[0] >>= 1;
	fraction.l[1] = BOR((fraction.l[1] >> 1), (bitSet << 31));
	if (exponent == 0)
		if (zeroFrac) {					/* Zero */
			*(ushort*)the10 = signSet ? 0x8000 : 0;
			*(long*)((char *)the10 + 2) = 0;
			*(long*)((char *)the10 + 6) = 0;
		} else {						/* Denormalized */
			exponent += 16383 - 1022;
			*(ushort*)the10 = signSet ? exponent | 0x8000 : exponent;
			*(long*)((char *)the10 + 2) = fraction.l[0];
			*(long*)((char *)the10 + 6) = fraction.l[1];
		}
	else if (exponent == 2047)
		if (zeroFrac) {					/* Infinity */
			*(ushort*)the10 = signSet ? 0xFFFF : 0x7FFF;
			*(long*)((char *)the10 + 2) = 0;
			*(long*)((char *)the10 + 6) = 0;
		} else {						/* NaN */
			*(ushort*)the10 = signSet ? 0xFFFF : 0x7FFF;
			*(long*)((char *)the10 + 2) = fraction.l[0];
			*(long*)((char *)the10 + 6) = fraction.l[1];
		}
	else {								/* Normalized */
		exponent += 16383 - 1023;
		*(ushort*)the10 = signSet ? exponent | 0x8000 : exponent;
		*(long*)((char *)the10 + 2) = fraction.l[0] | 0x80000000;
		*(long*)((char *)the10 + 6) = fraction.l[1];
	}
}  /* ConvertFloat8To10 */

void ConvertFloat8To12(void* the8, void* the12)
{
	uchar the10[10];
	
	ConvertFloat8To10(the8, the10);
	ConvertFloat10To12(the10, the12);
}  /* ConvertFloat8To12 */

