/*
 *		Copyright (c) 1987-1999 Play, Inc.
 *		This code is confidential and is a trade secret of Electric Image, Inc.
 */

#include "RoundUtils.h"

#define MAXINT 0x7FFFFFFF
#define MININT 0x80000000

// double RU_Magic = (double)((long long)((long long)1 << 51) + (long long)((long long)1 << 52));

#ifdef __ROUNDTOFLOOR__

long CalcFactor(double n)
{
	return (n - (long)n) * 1024.0;
}  /* CalcFactor */

long CalcFactor2(double* n)
{
	return (*n - (long)*n) * 1024.0;
}  /* CalcFactor2 */

long CalcLFactor(double n)
{
	return (n - (long)n) * 16384.0;
}  /* CalcLFactor */

long CalcLFactor2(double* n)
{
	return (*n - (long)*n) * 16384.0;
}  /* CalcLFactor2 */

double CalcRFactor(double n)
{
	return n - (long)n;
}  /* CalcRFactor */

double CalcRFactor2(double* n)
{
	return *n - (long)*n;
}  /* CalcRFactor2 */

void CalcRFactor3(double* n)
{
	*n -= (long)*n;
}  /* CalcRFactor3 */

long SplitFactor(double* n)
{
	long temp = *n;
	*n -= temp;
	return temp;
}  /* SplitFactor */

long ARound(double n)
{
	if (n < 0.0)
		return n - 0.5;
	return n + 0.5;
}  /* ARound */

long ARound2(double* n)
{
	if (*n < 0.0)
		return *n - 0.5;
	return *n + 0.5;
}  /* ARound2 */

long ARoundI(double n)
{
	long temp = n;
	return INTEGERLIMIT(temp);
}  /* ARoundI */

long ARoundI2(double* n)
{
	long temp = *n;
	return INTEGERLIMIT(temp);
}  /* ARoundI2 */

long LRound(double n)
{
	return n * 65536.0;
}  /* LRound */

long LRound2(double* n)
{
	return *n * 65536.0;
}  /* LRound2 */

long IRound(double n)
{
	long temp = n * 16384.0;
	return INTEGERLIMIT(temp);
}  /* IRound */

long IRound2(double* n)
{
	long temp = *n * 16384.0;
	return INTEGERLIMIT(temp);
}  /* IRound2 */

long IRoundR(float n)
{
	long temp = n * 16384.0;
	return INTEGERLIMIT(temp);
}  /* IRoundR */

long IRoundR2(float* n)
{
	long temp = *n * 16384.0;
	return INTEGERLIMIT(temp);
}  /* IRoundR2 */

long ITrunc(double n)
{
	long temp = n;
	return INTEGERLIMIT(temp);
}  /* ITrunc */

long ITrunc2(double* n)
{
	long temp = *n;
	return INTEGERLIMIT(temp);
}  /* ITrunc2 */

long FTrunc(double n)
{
	return n * 1024.0;
}  /* FTrunc */

long FTrunc2(double* n)
{
	return *n * 1024.0;
}  /* FTrunc2 */

long TruncLo(double n)
{
	return n + 1.0;
}  /* TruncLo */

long TruncLo2(double* n)
{
	return *n + 1.0;
}  /* TruncLo2 */

long TruncHi(double n)
{
	return n;
}  /* TruncHi */

long TruncHi2(double* n)
{
	return *n;
}  /* TruncHi2 */

long Floor(double n)
{
	return n;
}  /* Floor */

long Floor2(double* n)
{
	return *n;
}  /* Floor2 */

long Ceil(double n)
{
	return -(long)(-n);
}  /* Ceil */

long Ceil2(double* n)
{
	return -(long)(-*n);
}  /* Ceil2 */

#else

long CalcFactor(double n)
{
	long temp = (long)n;
	if (temp > n)
		temp--;
	return (long)((n - temp) * 1024.0);
}  /* CalcFactor */

long CalcFactor2(double* n)
{
	long temp = (long)*n;
	if (temp > *n)
		temp--;
	return (long)((*n - temp) * 1024.0);
}  /* CalcFactor2 */

long CalcLFactor(double n)
{
	long temp = (long)n;
	if (temp > n)
		temp--;
	return (long)((n - temp) * 16384.0);
}  /* CalcLFactor */

long CalcLFactor2(double* n)
{
	long temp = (long)*n;
	if (temp > *n)
		temp--;
	return (long)((*n - temp) * 16384.0);
}  /* CalcLFactor2 */

double CalcRFactor(double n)
{
	long temp = (long)n;
	if (temp > n)
		temp--;
	return n - temp;
}  /* CalcRFactor */

double CalcRFactor2(double* n)
{
	long temp = (long)*n;
	if (temp > *n)
		temp--;
	return *n - temp;
}  /* CalcRFactor2 */

void CalcRFactor3(double* n)
{
	long temp = (long)*n;
	if (temp > *n)
		temp--;
	*n -= temp;
}  /* CalcRFactor3 */

long SplitFactor(double* n)
{
	long temp = (long)*n;

	*n -= temp;
	if ((temp != MININT) && (*n < 0.0)) {
		*n += 1.0;
		return temp - 1;
	}
	return temp;
}  /* SplitFactor */

long ARound(double n)
{
	if (n < 0.0)
		return (long)(n - 0.5);
	else
		return (long)(n + 0.5);
}  /* ARound */

long ARound2(double* n)
{
	if (*n < 0.0)
		return (long)(*n - 0.5);
	else
		return (long)(*n + 0.5);
}  /* ARound2 */

long ARoundI(double n)
{
	if (n < 0.0) {
		long temp = (long)(n - 0.5);
		if (temp <= -32767)
			return -32767;
		else
			return temp;
	} else {
		long temp = (long)(n + 0.5);
		if (temp >= 32767)
			return 32767;
		else
			return temp;
	}
}  /* ARoundI */

long ARoundI2(double* n)
{
	if (*n < 0.0) {
		long temp = (long)(*n - 0.5);
		if (temp <= -32767)
			return -32767;
		else
			return temp;
	} else {
		long temp = (long)(*n + 0.5);
		if (temp >= 32767)
			return 32767;
		else
			return temp;
	}
}  /* ARoundI2 */

long LRound(double n)
{
	return (long)(n * 65536.0);
}  /* LRound */

long LRound2(double* n)
{
	return (long)(*n * 65536.0);
}  /* LRound2 */

long IRound(double n)
{
	long temp = (long)(n * 16384.0);
	return INTEGERLIMIT(temp);
}  /* IRound */

long IRound2(double* n)
{
	long temp = (long)(*n * 16384.0);
	return INTEGERLIMIT(temp);
}  /* IRound2 */

long IRoundR(float n)
{
	long temp = (long)(n * 16384.0);
	return INTEGERLIMIT(temp);
}  /* IRoundR */

long IRoundR2(float* n)
{
	long temp = (long)(*n * 16384.0);
	return INTEGERLIMIT(temp);
}  /* IRoundR2 */

long ITrunc(double n)
{
	long temp = (long)n;
	if (temp <= -32767)
		return -32767;
	else if (temp >= 32767)
		return 32767;
	else if (temp > n)
		return temp - 1;
	else
		return temp;
}  /* ITrunc */

long ITrunc2(double* n)
{
	long temp = (long)*n;
	if (temp <= -32767)
		return -32767;
	else if (temp >= 32767)
		return 32767;
	else if (temp > *n)
		return temp - 1;
	else
		return temp;
}  /* ITrunc2 */

long FTrunc(double n)
{
	long temp;

	n *= 1024.0;
	temp = (long)n;
	if ((temp != MININT) && (temp > n))
		return temp - 1;
	else
		return temp;
}  /* FTrunc */

long FTrunc2(double* n)
{
	double t = *n * 1024.0;
	long temp = (long)t;
	
	if ((temp != MININT) && (temp > t))
		return temp - 1;
	else
		return temp;
}  /* FTrunc2 */

long TruncLo(double n)
{
	long temp = (long)n;
	
	if ((temp != MAXINT) && (temp <= n))
		return temp + 1;
	else
		return temp;
}  /* TruncLo */

long TruncLo2(double* n)
{
	long temp = (long)*n;
	if ((temp != MAXINT) && (temp <= *n))
		return temp + 1;
	else
		return temp;
}  /* TruncLo2 */

long TruncHi(double n)
{
	long temp = (long)n;
	if ((temp != MININT) && (temp > n))
		return temp - 1;
	else
		return temp;
}  /* TruncHi */

long TruncHi2(double* n)
{
	long temp = (long)*n;
	if ((temp != MININT) && (temp > *n))
		return temp - 1;
	else
		return temp;
}  /* TruncHi2 */

long Floor(double n)
{
	long temp = (long)n;
	if ((temp != MININT) && (temp > n))
		return temp - 1;
	else
		return temp;
}  /* Floor */

long Floor2(double* n)
{
	long temp = (long)*n;
	if ((temp != MININT) && (temp > *n))
		return temp - 1;
	else
		return temp;
}  /* Floor2 */

long Ceil(double n)
{
	long temp = (long)n;
	if ((temp != MAXINT) && (temp < n))
		return temp + 1;
	else
		return temp;
}  /* Ceil */

long Ceil2(double* n)
{
	long temp = (long)*n;
	if ((temp != MAXINT) && (temp < *n))
		return temp + 1;
	else
		return temp;
}  /* Ceil2 */

#endif
