/*
 *		Copyright (c)1991-1997 Electric Image, Inc.
 *		This code is confidential and is a trade secret of Electric Image, Inc.
 */

#include "EIShader.h"
#include "FPU.h"
#include "Matrix.h"
#include "Noise.h"
#include "PString.h"
#include <stddef.h>
#include "System.h"

#define MIX(a, b, f) ((a) + (f) * ((b) - (a)))

#define shaderType				'NORM'
#define shaderVersion			0x00000001
#define shaderRevVersion		0x01000000

typedef struct {
	unsigned long shaderOwnerType;
	unsigned long shaderVersionNum;
} ShaderInterfaceRec, *ShaderInterfacePtr;

typedef struct {
	long dummy;
} ShaderDataRec, *ShaderDataPtr;

/* Static functions */

static void CalcNormals(RMatrix3Ptr theMatrix, float *a, float *b, float *c)
{
	float an = *a * theMatrix->m.m00 + *b * theMatrix->m.m01 + *c * theMatrix->m.m02;
	float bn = *a * theMatrix->m.m10 + *b * theMatrix->m.m11 + *c * theMatrix->m.m12;
	float cn = *a * theMatrix->m.m20 + *b * theMatrix->m.m21 + *c * theMatrix->m.m22;

	float distance = 1.0 / SQRT(SQR(an) + SQR(bn) + SQR(cn));
	
	*a = an * distance;
	*b = bn * distance;
	*c = cn * distance;
}  /* CalcNormals */

static void DefaultShaderInterface(ShaderInterfacePtr theShaderInterface)
{
	theShaderInterface->shaderOwnerType = shaderType;
	theShaderInterface->shaderVersionNum = shaderVersion;
}  /* DefaultShaderInterface */

static void FixShaderInterface(ShaderInterfacePtr theShaderInterface)
{
	theShaderInterface;
	
	FIXFIELD(theShaderInterface->shaderOwnerType);
	FIXFIELD(theShaderInterface->shaderVersionNum);
}  /* FixShaderInterface */

static void GetShaderInterface(void* theInterface, long theSize, ShaderInterfacePtr theShaderInterface)
{
	/* Copy the shader interface data */
	
	if (theSize == sizeof(ShaderInterfaceRec))
		*theShaderInterface = *(ShaderInterfacePtr)theInterface;
	else {
		theShaderInterface->shaderOwnerType = 0;
		theShaderInterface->shaderVersionNum = 0;
	}
	
	/* Validate the shader interface data */
	
	FixShaderInterface(theShaderInterface);
	if (theShaderInterface->shaderOwnerType != shaderType ||
		theShaderInterface->shaderVersionNum != shaderVersion)
		DefaultShaderInterface(theShaderInterface);
}  /* GetShaderInterface */

/* Export functions */

long EIShaderInformation(EIShaderInformationRec *theInformationPtr)
{
	/* Setup the shader feature flags */
	
	BSET(theInformationPtr->shaderFeatureFlags, shaderColorFlag);
	return shaderNoError;
}  /* EIShaderInformation */

long EIShaderInitialize(EIShaderInitializeRec *theInitializePtr)
{
	ShaderDataPtr theShaderData;
	
	/* Allocate the shader's private data record */
	
	theShaderData = (ShaderDataPtr)theInitializePtr->HostMalloc(sizeof(ShaderDataRec));
	if (theShaderData) {
		ShaderInterfaceRec theShaderInterface;
		
		/* Get the shader interface parameters */
		
		GetShaderInterface(theInitializePtr->shaderInterface, theInitializePtr->shaderInterfaceSize, &theShaderInterface);
		
		/* Return the pointer to the shader's private data to the host */
		
		theInitializePtr->shaderData = theShaderData;
		return shaderNoError;
	} else
		return shaderMemoryError;
}  /* EIShaderInitialize */

long EIShaderFinish(EIShaderFinishRec *theFinishPtr)
{
	/* Get rid of the shader's private data */
	
	theFinishPtr->HostFree(theFinishPtr->shaderData);
	return shaderNoError;
}  /* EIShaderFinish */

long EIShaderShade(EIShaderShadeRec *theShadePtr)
{
	if (BTST(theShadePtr->shaderFeatureFlags, shaderColorFlag)) {
		ShaderDataPtr theShaderData = (ShaderDataPtr)theShadePtr->shaderData;
		EIShaderMapPtr theMap = theShadePtr->shaderMap;
		EIShaderPixelPtr thePixel = theShadePtr->shaderPixel;
		float a = thePixel->shadeNormalA0;
		float b = thePixel->shadeNormalB0;
		float c = thePixel->shadeNormalC0;

		CalcNormals(&theMap->mapBumpMatrix, &a, &b, &c);
	
		theShadePtr->shaderColor.a = 1.0;
		theShadePtr->shaderColor.r = 0.5 * (1.0 + a);
		theShadePtr->shaderColor.g = 0.5 * (1.0 + b);
		theShadePtr->shaderColor.b = 0.5 * (1.0 + c);
	}
	return shaderNoError;
}  /* EIShaderShade */
