/*
 *		Copyright (c) 1987-1999 Play, Inc.
 *		This code is confidential and is a trade secret of Electric Image, Inc.
 */

/* This code is designed to run on both big-endian and little-endian CPUs. */

/* ReadSmallBlock, ReadFileSmallBlock, WriteSmallBlock and UFWriteEndian will read and write big-endian */
/* data to and from a data file. */

/* ReadSmallBlockForward, UFRead, WriteSmallBlockForward and UFWrite will read and write raw */
/* data to and from a data file. */

#include "FACTIO2.h"
#include "BufferedIO3.h"
#include "FPU.h"
#include "MemoryManager.h"
#include "UniversalIO.h"
#include "UniversalMemory.h"

typedef uchar Extended12[12];		/* So we can blindly read block off disk */

/* Generic file reading macro */

#define RD(readFunction, readSize) {\
	if (theFlag && (theSize >= (readSize))) {\
		readFunction;\
		theSize -= (readSize);\
	} else\
		theFlag = 0;\
}

/* Data packing and unpacking routines */

void PackBlockForward(char **thePtr, void *theData, ulong theSize)
{
	char *srcPtr = (char*)theData;
	char *dstPtr = *thePtr;
	char *endPtr = dstPtr + theSize;
	
	while (dstPtr != endPtr)
		*(dstPtr++) = *(srcPtr++);
	*thePtr = dstPtr;
}  /* PackBlockForward */

void UnpackBlockForward(char **thePtr, void *theData, ulong theSize)
{
	char *srcPtr = *thePtr;
	char *dstPtr = (char*)theData;
	char *endPtr = srcPtr + theSize;
	
	while (srcPtr != endPtr)
		*(dstPtr++) = *(srcPtr++);
	*thePtr = srcPtr;
}  /* UnpackBlockForward */

void PackBlockBackward(char **thePtr, void *theData, ulong theSize)
{
	char *srcPtr = (char*)theData + theSize;
	char *dstPtr = (char*)thePtr;
	char *endPtr = dstPtr + theSize;
	
	while (dstPtr != endPtr)
		*(dstPtr++) = *(--srcPtr);
	*thePtr = dstPtr;
}  /* PackBlockBackward */

void UnpackBlockBackward(char **thePtr, void *theData, ulong theSize)
{
	char *srcPtr = *thePtr;
	char *dstPtr = (char*)theData + theSize;
	char *endPtr = srcPtr + theSize;
	
	while (srcPtr != endPtr)
		*(--dstPtr) = *(srcPtr++);
	*thePtr = srcPtr;
}  /* UnpackBlockBackward */

void PackIFF(char **thePtr, ulong theSize, IFFType theType)
{
	PackBlockForward(thePtr, &theType, Type_size);
	PackBlock(thePtr, &theSize, LongInt_size);
}  /* PackIFF */

void UnpackIFF(char **thePtr, long *theSize, IFFType *theType)
{
	UnpackBlockForward(thePtr, theType, Type_size);
	UnpackBlock(thePtr, theSize, LongInt_size);
}  /* UnpackIFF */

void PackIFFBlock(char **thePtr, void *theData, ulong theSize, IFFType theType)
{
	PackIFF(thePtr, theSize, theType);
	PackBlockForward(thePtr, theData, theSize);
}  /* PackIFFBlock */

void PackFORM(char **thePtr, ulong theSize, IFFType theType)
{
	PackIFF(thePtr, theSize + Type_size, QUADWORD('FORM'));
	PackBlockForward(thePtr, &theType, Type_size);
}  /* PackFORM */

void UnpackFORM(char **thePtr, IFFType *theType)
{
	UnpackBlockForward(thePtr, theType, Type_size);
}  /* PackFORM */

/* These routines are designed to facilitate reading and writing of missaligned data */
/* records and records containing imbedded 12 byte long double floating point numbers */

void ReadARGB(UFFile theFile, ARGB* theARGB)
{
	ReadSmallBlockForward(theFile, theARGB, ARGB_size);
}  /* ReadARGB */

void ReadFileARGB(UFFile theFile, ARGB* theARGB)
{
	UFRead(theFile, theARGB, ARGB_size);
}  /* ReadFileARGB */

void WriteARGB(UFFile theFile, ARGB* theARGB)
{
	WriteSmallBlockForward(theFile, theARGB, ARGB_size);
}  /* WriteARGB */

void WriteFileARGB(UFFile theFile, ARGB* theARGB)
{
	UFWrite(theFile, theARGB, ARGB_size);
}  /* WriteFileARGB */

void PackARGB(char **thePtr, ARGB* theARGB)
{
	PackBlockForward(thePtr, theARGB, ARGB_size);
}  /* PackARGB */

void UnpackARGB(char **thePtr, ARGB* theARGB)
{
	UnpackBlockForward(thePtr, theARGB, ARGB_size);
}  /* UnpackARGB */

void ReadSRect(UFFile theFile, SRect* theRect)
{
#ifdef __LITTLE_ENDIAN__
	ReadSmallBlock(theFile, &theRect->top, Integer_size);
	ReadSmallBlock(theFile, &theRect->left, Integer_size);
	ReadSmallBlock(theFile, &theRect->bottom, Integer_size);
	ReadSmallBlock(theFile, &theRect->right, Integer_size);
#else
	ReadSmallBlock(theFile, theRect, Rect_size);
#endif
}  /* ReadSRect */

void ReadFileSRect(UFFile theFile, SRect* theRect)
{
#ifdef __LITTLE_ENDIAN__
	UFReadEndian(theFile, &theRect->top, Integer_size);
	UFReadEndian(theFile, &theRect->left, Integer_size);
	UFReadEndian(theFile, &theRect->bottom, Integer_size);
	UFReadEndian(theFile, &theRect->right, Integer_size);
#else
	UFReadEndian(theFile, theRect, Rect_size);
#endif
}  /* ReadFileSRect */

void WriteSRect(UFFile theFile, SRect* theRect)
{
#ifdef __LITTLE_ENDIAN__
	WriteSmallBlock(theFile, &theRect->top, Integer_size);
	WriteSmallBlock(theFile, &theRect->left, Integer_size);
	WriteSmallBlock(theFile, &theRect->bottom, Integer_size);
	WriteSmallBlock(theFile, &theRect->right, Integer_size);
#else
	WriteSmallBlock(theFile, theRect, Rect_size);
#endif
}  /* WriteSRect */

void WriteFileSRect(UFFile theFile, SRect* theRect)
{
#ifdef __LITTLE_ENDIAN__
	UFWriteEndian(theFile, &theRect->top, Integer_size);
	UFWriteEndian(theFile, &theRect->left, Integer_size);
	UFWriteEndian(theFile, &theRect->bottom, Integer_size);
	UFWriteEndian(theFile, &theRect->right, Integer_size);
#else
	UFWriteEndian(theFile, theRect, Rect_size);
#endif
}  /* WriteFileSRect */

void PackSRect(char **thePtr, SRect* theRect)
{
#ifdef __LITTLE_ENDIAN__
	PackBlock(thePtr, &theRect->top, Integer_size);
	PackBlock(thePtr, &theRect->left, Integer_size);
	PackBlock(thePtr, &theRect->bottom, Integer_size);
	PackBlock(thePtr, &theRect->right, Integer_size);
#else
	PackBlock(thePtr, theRect, Rect_size);
#endif
}  /* PackSRect */

void UnpackSRect(char **thePtr, SRect* theRect)
{
#ifdef __LITTLE_ENDIAN__
	UnpackBlock(thePtr, &theRect->top, Integer_size);
	UnpackBlock(thePtr, &theRect->left, Integer_size);
	UnpackBlock(thePtr, &theRect->bottom, Integer_size);
	UnpackBlock(thePtr, &theRect->right, Integer_size);
#else
	UnpackBlock(thePtr, theRect, Rect_size);
#endif
}  /* UnpackSRect */

void ReadImageHeader(UFFile theFile, ImageHeaderPtr theImageHeader)
{
	ReadSmallBlock(theFile, &theImageHeader->imageVersion, Integer_size);
	ReadSmallBlock(theFile, &theImageHeader->imageFrames, LongInt_size);
}  /* ReadImageHeader */

void ReadFileImageHeader(UFFile theFile, ImageHeaderPtr theImageHeader)
{
	UFReadEndian(theFile, &theImageHeader->imageVersion, Integer_size);
	UFReadEndian(theFile, &theImageHeader->imageFrames, LongInt_size);
}  /* ReadFileImageHeader */

void WriteImageHeader(UFFile theFile, ImageHeaderPtr theImageHeader)
{
	WriteSmallBlock(theFile, &theImageHeader->imageVersion, Integer_size);
	WriteSmallBlock(theFile, &theImageHeader->imageFrames, LongInt_size);
}  /* WriteImageHeader */

void WriteFileImageHeader(UFFile theFile, ImageHeaderPtr theImageHeader)
{
	UFWriteEndian(theFile, &theImageHeader->imageVersion, Integer_size);
	UFWriteEndian(theFile, &theImageHeader->imageFrames, LongInt_size);
}  /* WriteFileImageHeader */

void PackImageHeader(char **thePtr, ImageHeaderPtr theImageHeader)
{
	PackBlock(thePtr, &theImageHeader->imageVersion, Integer_size);
	PackBlock(thePtr, &theImageHeader->imageFrames, LongInt_size);
}  /* PackImageHeader */

void UnpackImageHeader(char **thePtr, ImageHeaderPtr theImageHeader)
{
	UnpackBlock(thePtr, &theImageHeader->imageVersion, Integer_size);
	UnpackBlock(thePtr, &theImageHeader->imageFrames, LongInt_size);
}  /* UnpackImageHeader */

void ReadImageFrame(UFFile theFile, ImageFramePtr theImageFrame)
{
	ReadSmallBlock(theFile, &theImageFrame->frameTime, Real_size);
	ReadSRect(theFile, &theImageFrame->frameRect);
	ReadSmallBlock(theFile, &theImageFrame->frameBitDepth, Byte_size);
	ReadSmallBlock(theFile, &theImageFrame->frameType, Byte_size);
	ReadSRect(theFile, &theImageFrame->framePackRect);
	ReadSmallBlock(theFile, &theImageFrame->framePacking, Byte_size);
	ReadSmallBlock(theFile, &theImageFrame->frameAlpha, Byte_size);
	ReadSmallBlock(theFile, &theImageFrame->frameSize, LongInt_size);
	ReadSmallBlock(theFile, &theImageFrame->framePalettes, Integer_size);
	ReadSmallBlock(theFile, &theImageFrame->frameBackground, Integer_size);
}  /* ReadImageFrame */

void ReadFileImageFrame(UFFile theFile, ImageFramePtr theImageFrame)
{
	UFReadEndian(theFile, &theImageFrame->frameTime, Real_size);
	ReadFileSRect(theFile, &theImageFrame->frameRect);
	UFReadEndian(theFile, &theImageFrame->frameBitDepth, Byte_size);
	UFReadEndian(theFile, &theImageFrame->frameType, Byte_size);
	ReadFileSRect(theFile, &theImageFrame->framePackRect);
	UFReadEndian(theFile, &theImageFrame->framePacking, Byte_size);
	UFReadEndian(theFile, &theImageFrame->frameAlpha,  Byte_size);
	UFReadEndian(theFile, &theImageFrame->frameSize, LongInt_size);
	UFReadEndian(theFile, &theImageFrame->framePalettes, Integer_size);
	UFReadEndian(theFile, &theImageFrame->frameBackground, Integer_size);
}  /* ReadFileImageFrame */

void WriteImageFrame(UFFile theFile, ImageFramePtr theImageFrame)
{
	WriteSmallBlock(theFile, &theImageFrame->frameTime, Real_size);
	WriteSRect(theFile, &theImageFrame->frameRect);
	WriteSmallBlock(theFile, &theImageFrame->frameBitDepth, Byte_size);
	WriteSmallBlock(theFile, &theImageFrame->frameType, Byte_size);
	WriteSRect(theFile, &theImageFrame->framePackRect);
	WriteSmallBlock(theFile, &theImageFrame->framePacking, Byte_size);
	WriteSmallBlock(theFile, &theImageFrame->frameAlpha, Byte_size);
	WriteSmallBlock(theFile, &theImageFrame->frameSize, LongInt_size);
	WriteSmallBlock(theFile, &theImageFrame->framePalettes, Integer_size);
	WriteSmallBlock(theFile, &theImageFrame->frameBackground, Integer_size);
}  /* WriteImageFrame */

void WriteFileImageFrame(UFFile theFile, ImageFramePtr theImageFrame)
{
	UFWriteEndian(theFile, &theImageFrame->frameTime, Real_size);
	WriteFileSRect(theFile, &theImageFrame->frameRect);
	UFWriteEndian(theFile, &theImageFrame->frameBitDepth, Byte_size);
	UFWriteEndian(theFile, &theImageFrame->frameType, Byte_size);
	WriteFileSRect(theFile, &theImageFrame->framePackRect);
	UFWriteEndian(theFile, &theImageFrame->framePacking, Byte_size);
	UFWriteEndian(theFile, &theImageFrame->frameAlpha,  Byte_size);
	UFWriteEndian(theFile, &theImageFrame->frameSize, LongInt_size);
	UFWriteEndian(theFile, &theImageFrame->framePalettes, Integer_size);
	UFWriteEndian(theFile, &theImageFrame->frameBackground, Integer_size);
}  /* WriteFileImageFrame */

void PackImageFrame(char **thePtr, ImageFramePtr theImageFrame)
{
	PackBlock(thePtr, &theImageFrame->frameTime, Real_size);
	PackSRect(thePtr, &theImageFrame->frameRect);
	PackBlock(thePtr, &theImageFrame->frameBitDepth, Byte_size);
	PackBlock(thePtr, &theImageFrame->frameType, Byte_size);
	PackSRect(thePtr, &theImageFrame->framePackRect);
	PackBlock(thePtr, &theImageFrame->framePacking, Byte_size);
	PackBlock(thePtr, &theImageFrame->frameAlpha, Byte_size);
	PackBlock(thePtr, &theImageFrame->frameSize, LongInt_size);
	PackBlock(thePtr, &theImageFrame->framePalettes, Integer_size);
	PackBlock(thePtr, &theImageFrame->frameBackground, Integer_size);
}  /* PackImageFrame */

void UnpackImageFrame(char **thePtr, ImageFramePtr theImageFrame)
{
	UnpackBlock(thePtr, &theImageFrame->frameTime, Real_size);
	UnpackSRect(thePtr, &theImageFrame->frameRect);
	UnpackBlock(thePtr, &theImageFrame->frameBitDepth, Byte_size);
	UnpackBlock(thePtr, &theImageFrame->frameType, Byte_size);
	UnpackSRect(thePtr, &theImageFrame->framePackRect);
	UnpackBlock(thePtr, &theImageFrame->framePacking, Byte_size);
	UnpackBlock(thePtr, &theImageFrame->frameAlpha, Byte_size);
	UnpackBlock(thePtr, &theImageFrame->frameSize, LongInt_size);
	UnpackBlock(thePtr, &theImageFrame->framePalettes, Integer_size);
	UnpackBlock(thePtr, &theImageFrame->frameBackground, Integer_size);
}  /* UnpackImageFrame */

void ReadMinMax(UFFile theFile, MinMax *theMinMax)
{
#ifdef __LITTLE_ENDIAN__
	ReadSmallBlock(theFile, &theMinMax->min, Real_size);
	ReadSmallBlock(theFile, &theMinMax->max, Real_size);
#else
	ReadSmallBlock(theFile, theMinMax, MinMax_size);
#endif
}  /* ReadMinMax */

void ReadFileMinMax(UFFile theFile, MinMax *theMinMax)
{
#ifdef __LITTLE_ENDIAN__
	UFReadEndian(theFile, &theMinMax->min, Real_size);
	UFReadEndian(theFile, &theMinMax->max, Real_size);
#else
	UFReadEndian(theFile, theMinMax, MinMax_size);
#endif
}  /* ReadFileMinMax */

void WriteMinMax(UFFile theFile, MinMax *theMinMax)
{
#ifdef __LITTLE_ENDIAN__
	WriteSmallBlock(theFile, &theMinMax->min, Real_size);
	WriteSmallBlock(theFile, &theMinMax->max, Real_size);
#else
	WriteSmallBlock(theFile, theMinMax, MinMax_size);
#endif
}  /* WriteMinMax */

void WriteFileMinMax(UFFile theFile, MinMax *theMinMax)
{
#ifdef __LITTLE_ENDIAN__
	UFWriteEndian(theFile, &theMinMax->min, Real_size);
	UFWriteEndian(theFile, &theMinMax->max, Real_size);
#else
	UFWriteEndian(theFile, theMinMax, MinMax_size);
#endif
}  /* WriteFileMinMax */

void PackMinMax(char **thePtr, MinMax *theMinMax)
{
#ifdef __LITTLE_ENDIAN__
	PackBlock(thePtr, &theMinMax->min, Real_size);
	PackBlock(thePtr, &theMinMax->max, Real_size);
#else
	PackBlock(thePtr, theMinMax, MinMax_size);
#endif
}  /* PackMinMax */

void UnpackMinMax(char **thePtr, MinMax *theMinMax)
{
#ifdef __LITTLE_ENDIAN__
	UnpackBlock(thePtr, &theMinMax->min, Real_size);
	UnpackBlock(thePtr, &theMinMax->max, Real_size);
#else
	UnpackBlock(thePtr, theMinMax, MinMax_size);
#endif
}  /* UnpackMinMax */

void ReadCoordinate(UFFile theFile, CoordPtr theCoord)
{
#ifdef __LITTLE_ENDIAN__
	ReadSmallBlock(theFile, &theCoord->x, Real_size);
	ReadSmallBlock(theFile, &theCoord->y, Real_size);
	ReadSmallBlock(theFile, &theCoord->z, Real_size);
#else
	ReadSmallBlock(theFile, theCoord, Coordinate_size);
#endif
}  /* ReadCoordinate */

void ReadFileCoordinate(UFFile theFile, CoordPtr theCoord)
{
#ifdef __LITTLE_ENDIAN__
	UFReadEndian(theFile, &theCoord->x, Real_size);
	UFReadEndian(theFile, &theCoord->y, Real_size);
	UFReadEndian(theFile, &theCoord->z, Real_size);
#else
	UFReadEndian(theFile, theCoord, Coordinate_size);
#endif
}  /* ReadFileCoordinate */

void WriteCoordinate(UFFile theFile, CoordPtr theCoord)
{
#ifdef __LITTLE_ENDIAN__
	WriteSmallBlock(theFile, &theCoord->x, Real_size);
	WriteSmallBlock(theFile, &theCoord->y, Real_size);
	WriteSmallBlock(theFile, &theCoord->z, Real_size);
#else
	WriteSmallBlock(theFile, theCoord, Coordinate_size);
#endif
}  /* WriteCoordinate */

void WriteFileCoordinate(UFFile theFile, CoordPtr theCoord)
{
#ifdef __LITTLE_ENDIAN__
	UFWriteEndian(theFile, &theCoord->x, Real_size);
	UFWriteEndian(theFile, &theCoord->y, Real_size);
	UFWriteEndian(theFile, &theCoord->z, Real_size);
#else
	UFWriteEndian(theFile, theCoord, Coordinate_size);
#endif
}  /* WriteFileCoordinate */

void PackCoordinate(char **thePtr, CoordPtr theCoord)
{
#ifdef __LITTLE_ENDIAN__
	PackBlock(thePtr, &theCoord->x, Real_size);
	PackBlock(thePtr, &theCoord->y, Real_size);
	PackBlock(thePtr, &theCoord->z, Real_size);
#else
	PackBlock(thePtr, theCoord, Coordinate_size);
#endif
}  /* PackCoordinate */

void UnpackCoordinate(char **thePtr, CoordPtr theCoord)
{
#ifdef __LITTLE_ENDIAN__
	UnpackBlock(thePtr, &theCoord->x, Real_size);
	UnpackBlock(thePtr, &theCoord->y, Real_size);
	UnpackBlock(thePtr, &theCoord->z, Real_size);
#else
	UnpackBlock(thePtr, theCoord, Coordinate_size);
#endif
}  /* UnpackCoordinate */

void ReadDCoordinate(UFFile theFile, DCoordPtr theCoord)
{
#ifdef __LITTLE_ENDIAN__
	ReadSmallBlock(theFile, &theCoord->x, Double_size);
	ReadSmallBlock(theFile, &theCoord->y, Double_size);
	ReadSmallBlock(theFile, &theCoord->z, Double_size);
#else
	ReadSmallBlock(theFile, theCoord, DCoordinate_size);
#endif
}  /* ReadCoordinate */

void ReadFileDCoordinate(UFFile theFile, DCoordPtr theCoord)
{
#ifdef __LITTLE_ENDIAN__
	UFReadEndian(theFile, &theCoord->x, Double_size);
	UFReadEndian(theFile, &theCoord->y, Double_size);
	UFReadEndian(theFile, &theCoord->z, Double_size);
#else
	UFReadEndian(theFile, theCoord, DCoordinate_size);
#endif
}  /* ReadFileCoordinate */

void WriteDCoordinate(UFFile theFile, DCoordPtr theCoord)
{
#ifdef __LITTLE_ENDIAN__
	WriteSmallBlock(theFile, &theCoord->x, Double_size);
	WriteSmallBlock(theFile, &theCoord->y, Double_size);
	WriteSmallBlock(theFile, &theCoord->z, Double_size);
#else
	WriteSmallBlock(theFile, theCoord, DCoordinate_size);
#endif
}  /* WriteDCoordinate */

void WriteFileDCoordinate(UFFile theFile, DCoordPtr theCoord)
{
#ifdef __LITTLE_ENDIAN__
	UFWriteEndian(theFile, &theCoord->x, Double_size);
	UFWriteEndian(theFile, &theCoord->y, Double_size);
	UFWriteEndian(theFile, &theCoord->z, Double_size);
#else
	UFWriteEndian(theFile, theCoord, DCoordinate_size);
#endif
}  /* WriteFileCoordinate */

void PackDCoordinate(char **thePtr, DCoordPtr theCoord)
{
#ifdef __LITTLE_ENDIAN__
	PackBlock(thePtr, &theCoord->x, Double_size);
	PackBlock(thePtr, &theCoord->y, Double_size);
	PackBlock(thePtr, &theCoord->z, Double_size);
#else
	PackBlock(thePtr, theCoord, DCoordinate_size);
#endif
}  /* PackDCoordinate */

void UnpackDCoordinate(char **thePtr, DCoordPtr theCoord)
{
#ifdef __LITTLE_ENDIAN__
	UnpackBlock(thePtr, &theCoord->x, Double_size);
	UnpackBlock(thePtr, &theCoord->y, Double_size);
	UnpackBlock(thePtr, &theCoord->z, Double_size);
#else
	UnpackBlock(thePtr, theCoord, DCoordinate_size);
#endif
}  /* UnpackDCoordinate */

void ReadExtent(UFFile theFile, ExtentPtr theExtent)
{
#ifdef __LITTLE_ENDIAN__
	ReadCoordinate(theFile, &theExtent->minCoord);
	ReadCoordinate(theFile, &theExtent->maxCoord);
#else
	ReadSmallBlock(theFile, theExtent, Extent_size);
#endif
}  /* ReadExtent */

void ReadFileExtent(UFFile theFile, ExtentPtr theExtent)
{
#ifdef __LITTLE_ENDIAN__
	ReadFileCoordinate(theFile, &theExtent->minCoord);
	ReadFileCoordinate(theFile, &theExtent->maxCoord);
#else
	UFReadEndian(theFile, theExtent, Extent_size);
#endif
}  /* ReadFileExtent */

void WriteExtent(UFFile theFile, ExtentPtr theExtent)
{
#ifdef __LITTLE_ENDIAN__
	WriteCoordinate(theFile, &theExtent->minCoord);
	WriteCoordinate(theFile, &theExtent->maxCoord);
#else
	WriteSmallBlock(theFile, theExtent, Extent_size);
#endif
}  /* WriteExtent */

void WriteFileExtent(UFFile theFile, ExtentPtr theExtent)
{
#ifdef __LITTLE_ENDIAN__
	WriteFileCoordinate(theFile, &theExtent->minCoord);
	WriteFileCoordinate(theFile, &theExtent->maxCoord);
#else
	UFWriteEndian(theFile, theExtent, Extent_size);
#endif
}  /* WriteFileExtent */

void PackExtent(char **thePtr, ExtentPtr theExtent)
{
#ifdef __LITTLE_ENDIAN__
	PackCoordinate(thePtr, &theExtent->minCoord);
	PackCoordinate(thePtr, &theExtent->maxCoord);
#else
	PackBlock(thePtr, theExtent, Extent_size);
#endif
}  /* PackExtent */

void UnpackExtent(char **thePtr, ExtentPtr theExtent)
{
#ifdef __LITTLE_ENDIAN__
	UnpackCoordinate(thePtr, &theExtent->minCoord);
	UnpackCoordinate(thePtr, &theExtent->maxCoord);
#else
	UnpackBlock(thePtr, theExtent, Extent_size);
#endif
}  /* PackExtent */

void ReadRMatrix3(UFFile theFile, RMatrix3Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	ReadSmallBlock(theFile, &theMatrix->m.m00, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m01, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m02, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m10, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m11, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m12, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m20, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m21, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m22, Real_size);
#else
	ReadSmallBlock(theFile, theMatrix, RMatrix3_size);
#endif
}  /* ReadRMatrix3 */

void ReadFileRMatrix3(UFFile theFile, RMatrix3Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	UFReadEndian(theFile, &theMatrix->m.m00, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m01, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m02, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m10, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m11, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m12, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m20, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m21, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m22, Real_size);
#else
	UFReadEndian(theFile, theMatrix, RMatrix3_size);
#endif
}  /* ReadFileRMatrix3 */

void WriteRMatrix3(UFFile theFile, RMatrix3Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	WriteSmallBlock(theFile, &theMatrix->m.m00, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m01, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m02, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m10, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m11, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m12, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m20, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m21, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m22, Real_size);
#else
	WriteSmallBlock(theFile, theMatrix, RMatrix3_size);
#endif
}  /* WriteRMatrix3 */

void WriteFileRMatrix3(UFFile theFile, RMatrix3Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	UFWriteEndian(theFile, &theMatrix->m.m00, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m01, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m02, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m10, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m11, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m12, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m20, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m21, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m22, Real_size);
#else
	UFWriteEndian(theFile, theMatrix, RMatrix3_size);
#endif
}  /* WriteFileRMatrix3 */

void PackRMatrix3(char **thePtr, RMatrix3Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	PackBlock(thePtr, &theMatrix->m.m00, Real_size);
	PackBlock(thePtr, &theMatrix->m.m01, Real_size);
	PackBlock(thePtr, &theMatrix->m.m02, Real_size);
	PackBlock(thePtr, &theMatrix->m.m10, Real_size);
	PackBlock(thePtr, &theMatrix->m.m11, Real_size);
	PackBlock(thePtr, &theMatrix->m.m12, Real_size);
	PackBlock(thePtr, &theMatrix->m.m20, Real_size);
	PackBlock(thePtr, &theMatrix->m.m21, Real_size);
	PackBlock(thePtr, &theMatrix->m.m22, Real_size);
#else
	PackBlock(thePtr, theMatrix, RMatrix3_size);
#endif
}  /* PackRMatrix3 */

void UnpackRMatrix3(char **thePtr, RMatrix3Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	UnpackBlock(thePtr, &theMatrix->m.m00, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m01, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m02, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m10, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m11, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m12, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m20, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m21, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m22, Real_size);
#else
	UnpackBlock(thePtr, theMatrix, RMatrix3_size);
#endif
}  /* PackRMatrix3 */

void ReadRMatrix4(UFFile theFile, RMatrix4Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	ReadSmallBlock(theFile, &theMatrix->m.m00, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m01, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m02, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m03, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m10, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m11, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m12, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m13, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m20, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m21, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m22, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m23, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m30, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m31, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m32, Real_size);
	ReadSmallBlock(theFile, &theMatrix->m.m33, Real_size);
#else
	ReadSmallBlock(theFile, theMatrix, RMatrix4_size);
#endif
}  /* ReadRMatrix4 */

void ReadFileRMatrix4(UFFile theFile, RMatrix4Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	UFReadEndian(theFile, &theMatrix->m.m00, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m01, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m02, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m03, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m10, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m11, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m12, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m13, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m20, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m21, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m22, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m23, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m30, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m31, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m32, Real_size);
	UFReadEndian(theFile, &theMatrix->m.m33, Real_size);
#else
	UFReadEndian(theFile, theMatrix, RMatrix4_size);
#endif
}  /* ReadFileRMatrix4 */

void WriteRMatrix4(UFFile theFile, RMatrix4Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	WriteSmallBlock(theFile, &theMatrix->m.m00, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m01, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m02, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m03, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m10, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m11, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m12, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m13, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m20, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m21, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m22, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m23, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m30, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m31, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m32, Real_size);
	WriteSmallBlock(theFile, &theMatrix->m.m33, Real_size);
#else
	WriteSmallBlock(theFile, theMatrix, RMatrix4_size);
#endif
}  /* WriteRMatrix4 */

void WriteFileRMatrix4(UFFile theFile, RMatrix4Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	UFWriteEndian(theFile, &theMatrix->m.m00, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m01, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m02, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m03, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m10, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m11, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m12, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m13, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m20, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m21, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m22, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m23, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m30, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m31, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m32, Real_size);
	UFWriteEndian(theFile, &theMatrix->m.m33, Real_size);
#else
	UFWriteEndian(theFile, theMatrix, RMatrix4_size);
#endif
}  /* WriteFileRMatrix4 */

void PackRMatrix4(char **thePtr, RMatrix4Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	PackBlock(thePtr, &theMatrix->m.m00, Real_size);
	PackBlock(thePtr, &theMatrix->m.m01, Real_size);
	PackBlock(thePtr, &theMatrix->m.m02, Real_size);
	PackBlock(thePtr, &theMatrix->m.m03, Real_size);
	PackBlock(thePtr, &theMatrix->m.m10, Real_size);
	PackBlock(thePtr, &theMatrix->m.m11, Real_size);
	PackBlock(thePtr, &theMatrix->m.m12, Real_size);
	PackBlock(thePtr, &theMatrix->m.m13, Real_size);
	PackBlock(thePtr, &theMatrix->m.m20, Real_size);
	PackBlock(thePtr, &theMatrix->m.m21, Real_size);
	PackBlock(thePtr, &theMatrix->m.m22, Real_size);
	PackBlock(thePtr, &theMatrix->m.m23, Real_size);
	PackBlock(thePtr, &theMatrix->m.m30, Real_size);
	PackBlock(thePtr, &theMatrix->m.m31, Real_size);
	PackBlock(thePtr, &theMatrix->m.m32, Real_size);
	PackBlock(thePtr, &theMatrix->m.m33, Real_size);
#else
	PackBlock(thePtr, theMatrix, RMatrix4_size);
#endif
}  /* PackRMatrix4 */

void UnpackRMatrix4(char **thePtr, RMatrix4Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	UnpackBlock(thePtr, &theMatrix->m.m00, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m01, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m02, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m03, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m10, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m11, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m12, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m13, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m20, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m21, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m22, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m23, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m30, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m31, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m32, Real_size);
	UnpackBlock(thePtr, &theMatrix->m.m33, Real_size);
#else
	UnpackBlock(thePtr, theMatrix, RMatrix4_size);
#endif
}  /* UnpackRMatrix4 */

void ReadMatrix3(UFFile theFile, Matrix3Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	ReadSmallBlock(theFile, &theMatrix->m.m00, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m01, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m02, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m10, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m11, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m12, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m20, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m21, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m22, Double_size);
#else
	ReadSmallBlock(theFile, theMatrix, Matrix3_size);
#endif
}  /* ReadMatrix3 */

void ReadFileMatrix3(UFFile theFile, Matrix3Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	UFReadEndian(theFile, &theMatrix->m.m00, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m01, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m02, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m10, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m11, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m12, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m20, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m21, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m22, Double_size);
#else
	UFReadEndian(theFile, theMatrix, Matrix3_size);
#endif
}  /* ReadFileMatrix3 */

void WriteMatrix3(UFFile theFile, Matrix3Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	WriteSmallBlock(theFile, &theMatrix->m.m00, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m01, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m02, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m10, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m11, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m12, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m20, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m21, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m22, Double_size);
#else
	WriteSmallBlock(theFile, theMatrix, Matrix3_size);
#endif
}  /* WriteMatrix3 */

void WriteFileMatrix3(UFFile theFile, Matrix3Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	UFWriteEndian(theFile, &theMatrix->m.m00, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m01, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m02, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m10, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m11, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m12, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m20, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m21, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m22, Double_size);
#else
	UFWriteEndian(theFile, theMatrix, Matrix3_size);
#endif
}  /* WriteFileMatrix3 */

void PackMatrix3(char **thePtr, Matrix3Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	PackBlock(thePtr, &theMatrix->m.m00, Double_size);
	PackBlock(thePtr, &theMatrix->m.m01, Double_size);
	PackBlock(thePtr, &theMatrix->m.m02, Double_size);
	PackBlock(thePtr, &theMatrix->m.m10, Double_size);
	PackBlock(thePtr, &theMatrix->m.m11, Double_size);
	PackBlock(thePtr, &theMatrix->m.m12, Double_size);
	PackBlock(thePtr, &theMatrix->m.m20, Double_size);
	PackBlock(thePtr, &theMatrix->m.m21, Double_size);
	PackBlock(thePtr, &theMatrix->m.m22, Double_size);
#else
	PackBlock(thePtr, theMatrix, Matrix3_size);
#endif
}  /* PackMatrix3 */

void UnpackMatrix3(char **thePtr, Matrix3Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	UnpackBlock(thePtr, &theMatrix->m.m00, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m01, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m02, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m10, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m11, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m12, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m20, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m21, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m22, Double_size);
#else
	UnpackBlock(thePtr, theMatrix, Matrix3_size);
#endif
}  /* PackMatrix3 */

void ReadMatrix4(UFFile theFile, Matrix4Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	ReadSmallBlock(theFile, &theMatrix->m.m00, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m01, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m02, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m03, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m10, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m11, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m12, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m13, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m20, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m21, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m22, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m23, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m30, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m31, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m32, Double_size);
	ReadSmallBlock(theFile, &theMatrix->m.m33, Double_size);
#else
	ReadSmallBlock(theFile, theMatrix, Matrix4_size);
#endif
}  /* ReadMatrix4 */

void ReadFileMatrix4(UFFile theFile, Matrix4Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	UFReadEndian(theFile, &theMatrix->m.m00, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m01, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m02, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m03, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m10, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m11, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m12, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m13, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m20, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m21, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m22, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m23, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m30, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m31, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m32, Double_size);
	UFReadEndian(theFile, &theMatrix->m.m33, Double_size);
#else
	UFReadEndian(theFile, theMatrix, Matrix4_size);
#endif
}  /* ReadFileMatrix4 */

void WriteMatrix4(UFFile theFile, Matrix4Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	WriteSmallBlock(theFile, &theMatrix->m.m00, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m01, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m02, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m03, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m10, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m11, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m12, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m13, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m20, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m21, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m22, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m23, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m30, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m31, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m32, Double_size);
	WriteSmallBlock(theFile, &theMatrix->m.m33, Double_size);
#else
	WriteSmallBlock(theFile, theMatrix, Matrix4_size);
#endif
}  /* WriteMatrix4 */

void WriteFileMatrix4(UFFile theFile, Matrix4Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	UFWriteEndian(theFile, &theMatrix->m.m00, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m01, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m02, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m03, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m10, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m11, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m12, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m13, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m20, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m21, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m22, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m23, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m30, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m31, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m32, Double_size);
	UFWriteEndian(theFile, &theMatrix->m.m33, Double_size);
#else
	UFWriteEndian(theFile, theMatrix, Matrix4_size);
#endif
}  /* WriteFileMatrix4 */

void PackMatrix4(char **thePtr, Matrix4Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	PackBlock(thePtr, &theMatrix->m.m00, Double_size);
	PackBlock(thePtr, &theMatrix->m.m01, Double_size);
	PackBlock(thePtr, &theMatrix->m.m02, Double_size);
	PackBlock(thePtr, &theMatrix->m.m03, Double_size);
	PackBlock(thePtr, &theMatrix->m.m10, Double_size);
	PackBlock(thePtr, &theMatrix->m.m11, Double_size);
	PackBlock(thePtr, &theMatrix->m.m12, Double_size);
	PackBlock(thePtr, &theMatrix->m.m13, Double_size);
	PackBlock(thePtr, &theMatrix->m.m20, Double_size);
	PackBlock(thePtr, &theMatrix->m.m21, Double_size);
	PackBlock(thePtr, &theMatrix->m.m22, Double_size);
	PackBlock(thePtr, &theMatrix->m.m23, Double_size);
	PackBlock(thePtr, &theMatrix->m.m30, Double_size);
	PackBlock(thePtr, &theMatrix->m.m31, Double_size);
	PackBlock(thePtr, &theMatrix->m.m32, Double_size);
	PackBlock(thePtr, &theMatrix->m.m33, Double_size);
#else
	PackBlock(thePtr, theMatrix, Matrix4_size);
#endif
}  /* PackMatrix4 */

void UnpackMatrix4(char **thePtr, Matrix4Ptr theMatrix)
{
#ifdef __LITTLE_ENDIAN__
	UnpackBlock(thePtr, &theMatrix->m.m00, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m01, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m02, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m03, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m10, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m11, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m12, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m13, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m20, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m21, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m22, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m23, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m30, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m31, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m32, Double_size);
	UnpackBlock(thePtr, &theMatrix->m.m33, Double_size);
#else
	UnpackBlock(thePtr, theMatrix, Matrix4_size);
#endif
}  /* UnpackMatrix4 */

void ReadExtended12(UFFile theFile, double* theExtended)
{
	Extended12 buffer;

	ReadSmallBlock(theFile, buffer, Extended_size);
	ConvertFloat12To8(buffer, theExtended);
}  /* ReadExtended12 */

void ReadFileExtended12(UFFile theFile, double* theExtended)
{
	Extended12 buffer;

	UFReadEndian(theFile, buffer, Extended_size);
	ConvertFloat12To8(buffer, theExtended);
}  /* ReadFileExtended12 */

void WriteExtended12(UFFile theFile, double* theExtended)
{
	Extended12 buffer;

	ConvertFloat8To12(theExtended, buffer);
	WriteSmallBlock(theFile, buffer, Extended_size);
}  /* WriteExtended12 */

void WriteFileExtended12(UFFile theFile, double* theExtended)
{
	Extended12 buffer;

	ConvertFloat8To12(theExtended, buffer);
	UFWriteEndian(theFile, buffer, Extended_size);
}  /* WriteFileExtended12 */

void PackExtended12(char **thePtr, double* theExtended)
{
	Extended12 buffer;

	ConvertFloat8To12(theExtended, buffer);
	PackBlock(thePtr, buffer, Extended_size);
}  /* PackExtended12 */

void UnpackExtended12(char **thePtr, double* theExtended)
{
	Extended12 buffer;

	UnpackBlock(thePtr, buffer, Extended_size);
	ConvertFloat12To8(buffer, theExtended);
}  /* UnpackExtended12 */

void ReadXMatrix3(UFFile theFile, XMatrix3Ptr theMatrix)
{
	READEXTENDED12(theFile, &theMatrix->m.m00);
	READEXTENDED12(theFile, &theMatrix->m.m01);
	READEXTENDED12(theFile, &theMatrix->m.m02);
	READEXTENDED12(theFile, &theMatrix->m.m10);
	READEXTENDED12(theFile, &theMatrix->m.m11);
	READEXTENDED12(theFile, &theMatrix->m.m12);
	READEXTENDED12(theFile, &theMatrix->m.m20);
	READEXTENDED12(theFile, &theMatrix->m.m21);
	READEXTENDED12(theFile, &theMatrix->m.m22);
}  /* ReadXMatrix3 */

void ReadFileXMatrix3(UFFile theFile, XMatrix3Ptr theMatrix)
{
	READFILEEXTENDED12(theFile, &theMatrix->m.m00);
	READFILEEXTENDED12(theFile, &theMatrix->m.m01);
	READFILEEXTENDED12(theFile, &theMatrix->m.m02);
	READFILEEXTENDED12(theFile, &theMatrix->m.m10);
	READFILEEXTENDED12(theFile, &theMatrix->m.m11);
	READFILEEXTENDED12(theFile, &theMatrix->m.m12);
	READFILEEXTENDED12(theFile, &theMatrix->m.m20);
	READFILEEXTENDED12(theFile, &theMatrix->m.m21);
	READFILEEXTENDED12(theFile, &theMatrix->m.m22);
}  /* ReadFileXMatrix3 */

void WriteXMatrix3(UFFile theFile, XMatrix3Ptr theMatrix)
{
	WRITEEXTENDED12(theFile, &theMatrix->m.m00);
	WRITEEXTENDED12(theFile, &theMatrix->m.m01);
	WRITEEXTENDED12(theFile, &theMatrix->m.m02);
	WRITEEXTENDED12(theFile, &theMatrix->m.m10);
	WRITEEXTENDED12(theFile, &theMatrix->m.m11);
	WRITEEXTENDED12(theFile, &theMatrix->m.m12);
	WRITEEXTENDED12(theFile, &theMatrix->m.m20);
	WRITEEXTENDED12(theFile, &theMatrix->m.m21);
	WRITEEXTENDED12(theFile, &theMatrix->m.m22);
}  /* WriteXMatrix3 */

void WriteFileXMatrix3(UFFile theFile, XMatrix3Ptr theMatrix)
{
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m00);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m01);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m02);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m10);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m11);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m12);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m20);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m21);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m22);
}  /* WriteXFileMatrix3 */

void PackXMatrix3(char **thePtr, XMatrix3Ptr theMatrix)
{
	PACKEXTENDED12(thePtr, &theMatrix->m.m00);
	PACKEXTENDED12(thePtr, &theMatrix->m.m01);
	PACKEXTENDED12(thePtr, &theMatrix->m.m02);
	PACKEXTENDED12(thePtr, &theMatrix->m.m10);
	PACKEXTENDED12(thePtr, &theMatrix->m.m11);
	PACKEXTENDED12(thePtr, &theMatrix->m.m12);
	PACKEXTENDED12(thePtr, &theMatrix->m.m20);
	PACKEXTENDED12(thePtr, &theMatrix->m.m21);
	PACKEXTENDED12(thePtr, &theMatrix->m.m22);
}  /* PackXMatrix3 */

void UnpackXMatrix3(char **thePtr, XMatrix3Ptr theMatrix)
{
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m00);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m01);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m02);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m10);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m11);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m12);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m20);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m21);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m22);
}  /* UnpackXMatrix3 */

void ReadXMatrix4(UFFile theFile, XMatrix4Ptr theMatrix)
{
	READEXTENDED12(theFile, &theMatrix->m.m00);
	READEXTENDED12(theFile, &theMatrix->m.m01);
	READEXTENDED12(theFile, &theMatrix->m.m02);
	READEXTENDED12(theFile, &theMatrix->m.m03);
	READEXTENDED12(theFile, &theMatrix->m.m10);
	READEXTENDED12(theFile, &theMatrix->m.m11);
	READEXTENDED12(theFile, &theMatrix->m.m12);
	READEXTENDED12(theFile, &theMatrix->m.m13);
	READEXTENDED12(theFile, &theMatrix->m.m20);
	READEXTENDED12(theFile, &theMatrix->m.m21);
	READEXTENDED12(theFile, &theMatrix->m.m22);
	READEXTENDED12(theFile, &theMatrix->m.m23);
	READEXTENDED12(theFile, &theMatrix->m.m30);
	READEXTENDED12(theFile, &theMatrix->m.m31);
	READEXTENDED12(theFile, &theMatrix->m.m32);
	READEXTENDED12(theFile, &theMatrix->m.m33);
}  /* ReadXMatrix4 */

void ReadFileXMatrix4(UFFile theFile, XMatrix4Ptr theMatrix)
{
	READFILEEXTENDED12(theFile, &theMatrix->m.m00);
	READFILEEXTENDED12(theFile, &theMatrix->m.m01);
	READFILEEXTENDED12(theFile, &theMatrix->m.m02);
	READFILEEXTENDED12(theFile, &theMatrix->m.m03);
	READFILEEXTENDED12(theFile, &theMatrix->m.m10);
	READFILEEXTENDED12(theFile, &theMatrix->m.m11);
	READFILEEXTENDED12(theFile, &theMatrix->m.m12);
	READFILEEXTENDED12(theFile, &theMatrix->m.m13);
	READFILEEXTENDED12(theFile, &theMatrix->m.m20);
	READFILEEXTENDED12(theFile, &theMatrix->m.m21);
	READFILEEXTENDED12(theFile, &theMatrix->m.m22);
	READFILEEXTENDED12(theFile, &theMatrix->m.m23);
	READFILEEXTENDED12(theFile, &theMatrix->m.m30);
	READFILEEXTENDED12(theFile, &theMatrix->m.m31);
	READFILEEXTENDED12(theFile, &theMatrix->m.m32);
	READFILEEXTENDED12(theFile, &theMatrix->m.m33);
}  /* ReadFileXMatrix4 */

void WriteXMatrix4(UFFile theFile, XMatrix4Ptr theMatrix)
{
	WRITEEXTENDED12(theFile, &theMatrix->m.m00);
	WRITEEXTENDED12(theFile, &theMatrix->m.m01);
	WRITEEXTENDED12(theFile, &theMatrix->m.m02);
	WRITEEXTENDED12(theFile, &theMatrix->m.m03);
	WRITEEXTENDED12(theFile, &theMatrix->m.m10);
	WRITEEXTENDED12(theFile, &theMatrix->m.m11);
	WRITEEXTENDED12(theFile, &theMatrix->m.m12);
	WRITEEXTENDED12(theFile, &theMatrix->m.m13);
	WRITEEXTENDED12(theFile, &theMatrix->m.m20);
	WRITEEXTENDED12(theFile, &theMatrix->m.m21);
	WRITEEXTENDED12(theFile, &theMatrix->m.m22);
	WRITEEXTENDED12(theFile, &theMatrix->m.m23);
	WRITEEXTENDED12(theFile, &theMatrix->m.m30);
	WRITEEXTENDED12(theFile, &theMatrix->m.m31);
	WRITEEXTENDED12(theFile, &theMatrix->m.m32);
	WRITEEXTENDED12(theFile, &theMatrix->m.m33);
}  /* WriteXMatrix4 */

void WriteFileXMatrix4(UFFile theFile, XMatrix4Ptr theMatrix)
{
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m00);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m01);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m02);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m03);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m10);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m11);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m12);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m13);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m20);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m21);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m22);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m23);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m30);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m31);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m32);
	WRITEFILEEXTENDED12(theFile, &theMatrix->m.m33);
}  /* WriteFileXMatrix4 */

void PackXMatrix4(char **thePtr, XMatrix4Ptr theMatrix)
{
	PACKEXTENDED12(thePtr, &theMatrix->m.m00);
	PACKEXTENDED12(thePtr, &theMatrix->m.m01);
	PACKEXTENDED12(thePtr, &theMatrix->m.m02);
	PACKEXTENDED12(thePtr, &theMatrix->m.m03);
	PACKEXTENDED12(thePtr, &theMatrix->m.m10);
	PACKEXTENDED12(thePtr, &theMatrix->m.m11);
	PACKEXTENDED12(thePtr, &theMatrix->m.m12);
	PACKEXTENDED12(thePtr, &theMatrix->m.m13);
	PACKEXTENDED12(thePtr, &theMatrix->m.m20);
	PACKEXTENDED12(thePtr, &theMatrix->m.m21);
	PACKEXTENDED12(thePtr, &theMatrix->m.m22);
	PACKEXTENDED12(thePtr, &theMatrix->m.m23);
	PACKEXTENDED12(thePtr, &theMatrix->m.m30);
	PACKEXTENDED12(thePtr, &theMatrix->m.m31);
	PACKEXTENDED12(thePtr, &theMatrix->m.m32);
	PACKEXTENDED12(thePtr, &theMatrix->m.m33);
}  /* ReadXMatrix4 */

void UnpackXMatrix4(char **thePtr, XMatrix4Ptr theMatrix)
{
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m00);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m01);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m02);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m03);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m10);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m11);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m12);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m13);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m20);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m21);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m22);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m23);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m30);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m31);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m32);
	UNPACKEXTENDED12(thePtr, &theMatrix->m.m33);
}  /* UnpackXMatrix4 */

void ReadXCoordinate(UFFile theFile, XCoordPtr theCoord)
{
	READEXTENDED12(theFile, &theCoord->x);
	READEXTENDED12(theFile, &theCoord->y);
	READEXTENDED12(theFile, &theCoord->z);
}  /* ReadXCoordinate */

void ReadFileXCoordinate(UFFile theFile, XCoordPtr theCoord)
{
	READFILEEXTENDED12(theFile, &theCoord->x);
	READFILEEXTENDED12(theFile, &theCoord->y);
	READFILEEXTENDED12(theFile, &theCoord->z);
}  /* ReadFileXCoordinate */

void WriteXCoordinate(UFFile theFile, XCoordPtr theCoord)
{
	WRITEEXTENDED12(theFile, &theCoord->x);
	WRITEEXTENDED12(theFile, &theCoord->y);
	WRITEEXTENDED12(theFile, &theCoord->z);
}  /* WriteXCoordinate */

void WriteFileXCoordinate(UFFile theFile, XCoordPtr theCoord)
{
	WRITEFILEEXTENDED12(theFile, &theCoord->x);
	WRITEFILEEXTENDED12(theFile, &theCoord->y);
	WRITEFILEEXTENDED12(theFile, &theCoord->z);
}  /* WriteFileXCoordinate */

void PackXCoordinate(char **thePtr, XCoordPtr theCoord)
{
	PACKEXTENDED12(thePtr, &theCoord->x);
	PACKEXTENDED12(thePtr, &theCoord->y);
	PACKEXTENDED12(thePtr, &theCoord->z);
}  /* PackXCoordinate */

void UnpackXCoordinate(char **thePtr, XCoordPtr theCoord)
{
	UNPACKEXTENDED12(thePtr, &theCoord->x);
	UNPACKEXTENDED12(thePtr, &theCoord->y);
	UNPACKEXTENDED12(thePtr, &theCoord->z);
}  /* UnpackXCoordinate */

void ReadGINF(UFFile theFile, void* theData, UFSize theSize)
{
	GINFPtr theGINF = (GINFPtr)theData;
	boolean theFlag = 1;
	
	RD(ReadSmallBlock(theFile, &theGINF->totalCoordinates, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theGINF->totalPolygons, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theGINF->groupTemp, LongInt_size), LongInt_size);
	RD(ReadExtent(theFile, &theGINF->extents), Extent_size);
	RD(ReadSmallBlock(theFile, &theGINF->flags, LongInt_size), LongInt_size);
	RD(ReadSmallBlockForward(theFile, &theGINF->name, Str31_size), Str31_size);
	RD(ReadSmallBlock(theFile, &theGINF->groupDate, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theGINF->groupID, Integer_size), Integer_size);
	RD(ReadXMatrix4(theFile, &theGINF->coordinateAbsMatrix), XMatrix4_size);
	RD(ReadXMatrix4(theFile, &theGINF->normalAbsMatrix), XMatrix4_size);
	RD(ReadXMatrix4(theFile, &theGINF->coordinateRelMatrix), XMatrix4_size);
	RD(ReadXMatrix4(theFile, &theGINF->normalRelMatrix), XMatrix4_size);
	RD(ReadSmallBlock(theFile, &theGINF->childCount, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theGINF->childTotal, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theGINF->cycleCount, LongInt_size), LongInt_size);
	if (theSize > 0)
		SkipBytes(theFile, theSize);
}  /* ReadGINF */

void ReadFileGINF(UFFile theFile, void* theData, UFSize theSize)
{
	GINFPtr theGINF = (GINFPtr)theData;
	boolean theFlag = 1;
	
	RD(UFReadEndian(theFile, &theGINF->totalCoordinates, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theGINF->totalPolygons, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theGINF->groupTemp, LongInt_size), LongInt_size);
	RD(ReadFileExtent(theFile, &theGINF->extents), Extent_size);
	RD(UFReadEndian(theFile, &theGINF->flags, LongInt_size), LongInt_size);
	RD(UFRead(theFile, &theGINF->name, Str31_size), Str31_size);
	RD(UFReadEndian(theFile, &theGINF->groupDate, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theGINF->groupID, Integer_size), Integer_size);
	RD(ReadFileXMatrix4(theFile, &theGINF->coordinateAbsMatrix), XMatrix4_size);
	RD(ReadFileXMatrix4(theFile, &theGINF->normalAbsMatrix), XMatrix4_size);
	RD(ReadFileXMatrix4(theFile, &theGINF->coordinateRelMatrix), XMatrix4_size);
	RD(ReadFileXMatrix4(theFile, &theGINF->normalRelMatrix), XMatrix4_size);
	RD(UFReadEndian(theFile, &theGINF->childCount, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theGINF->childTotal, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theGINF->cycleCount, LongInt_size), LongInt_size);
	if (theSize > 0)
		UFSetPos(theFile, UFGetPos(theFile) + theSize);
}  /* ReadFileGINF */

void WriteGINF(UFFile theFile, GINFPtr theGINF)
{
	ulong theSize = GINF_size;
	
	WriteSmallBlockForward(theFile, (void*)"GINF", Type_size);
	WriteSmallBlock(theFile, &theSize, LongInt_size);
	WriteSmallBlock(theFile, &theGINF->totalCoordinates, LongInt_size);
	WriteSmallBlock(theFile, &theGINF->totalPolygons, LongInt_size);
	WriteSmallBlock(theFile,  &theGINF->groupTemp, LongInt_size);
	WriteExtent(theFile, &theGINF->extents);
	WriteSmallBlock(theFile, &theGINF->flags, LongInt_size);
	WriteSmallBlockForward(theFile, &theGINF->name, Str31_size);
	WriteSmallBlock(theFile, &theGINF->groupDate, LongInt_size);
	WriteSmallBlock(theFile, &theGINF->groupID, Integer_size);
	WriteXMatrix4(theFile, &theGINF->coordinateAbsMatrix);
	WriteXMatrix4(theFile, &theGINF->normalAbsMatrix);
	WriteXMatrix4(theFile, &theGINF->coordinateRelMatrix);
	WriteXMatrix4(theFile, &theGINF->normalRelMatrix);
	WriteSmallBlock(theFile, &theGINF->childCount, LongInt_size);
	WriteSmallBlock(theFile, &theGINF->childTotal, LongInt_size);
	WriteSmallBlock(theFile, &theGINF->cycleCount, LongInt_size);
}  /* WriteGINF */

void WriteFileGINF(UFFile theFile, GINFPtr theGINF)
{
	ulong theSize = GINF_size;
	
	UFWrite(theFile, (void*)"GINF", Type_size);
	UFWriteEndian(theFile, &theSize, LongInt_size);
	UFWriteEndian(theFile, &theGINF->totalCoordinates, LongInt_size);
	UFWriteEndian(theFile, &theGINF->totalPolygons, LongInt_size);
	UFWriteEndian(theFile, &theGINF->groupTemp, LongInt_size);
	WriteFileExtent(theFile, &theGINF->extents);
	UFWriteEndian(theFile, &theGINF->flags, LongInt_size);
	UFWrite(theFile, &theGINF->name, Str31_size);
	UFWriteEndian(theFile, &theGINF->groupDate, LongInt_size);
	UFWriteEndian(theFile, &theGINF->groupID, Integer_size);
	WriteFileXMatrix4(theFile, &theGINF->coordinateAbsMatrix);
	WriteFileXMatrix4(theFile, &theGINF->normalAbsMatrix);
	WriteFileXMatrix4(theFile, &theGINF->coordinateRelMatrix);
	WriteFileXMatrix4(theFile, &theGINF->normalRelMatrix);
	UFWriteEndian(theFile, &theGINF->childCount, LongInt_size);
	UFWriteEndian(theFile, &theGINF->childTotal, LongInt_size);
	UFWriteEndian(theFile, &theGINF->cycleCount, LongInt_size);
}  /* WriteFileGINF */

void PackGINF(char **thePtr, GINFPtr theGINF)
{
	ulong theSize = GINF_size;
	
	PackBlockForward(thePtr, (void*)"GINF", Type_size);
	PackBlock(thePtr, &theSize, LongInt_size);
	PackBlock(thePtr, &theGINF->totalCoordinates, LongInt_size);
	PackBlock(thePtr, &theGINF->totalPolygons, LongInt_size);
	PackBlock(thePtr, &theGINF->groupTemp, LongInt_size);
	PackExtent(thePtr, &theGINF->extents);
	PackBlock(thePtr, &theGINF->flags, LongInt_size);
	PackBlockForward(thePtr, &theGINF->name, Str31_size);
	PackBlock(thePtr, &theGINF->groupDate, LongInt_size);
	PackBlock(thePtr, &theGINF->groupID, Integer_size);
	PackXMatrix4(thePtr, &theGINF->coordinateAbsMatrix);
	PackXMatrix4(thePtr, &theGINF->normalAbsMatrix);
	PackXMatrix4(thePtr, &theGINF->coordinateRelMatrix);
	PackXMatrix4(thePtr, &theGINF->normalRelMatrix);
	PackBlock(thePtr, &theGINF->childCount, LongInt_size);
	PackBlock(thePtr, &theGINF->childTotal, LongInt_size);
	PackBlock(thePtr, &theGINF->cycleCount, LongInt_size);
}  /* PackGINF */

void UnpackGINF(char **thePtr, void* theData, ulong theSize)
{
	GINFPtr theGINF = (GINFPtr)theData;
	boolean theFlag = 1;

	RD(UnpackBlock(thePtr, &theGINF->totalCoordinates, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theGINF->totalPolygons, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theGINF->groupTemp, LongInt_size), LongInt_size);
	RD(UnpackExtent(thePtr, &theGINF->extents), Extent_size);
	RD(UnpackBlock(thePtr, &theGINF->flags, LongInt_size), LongInt_size);
	RD(UnpackBlockForward(thePtr, &theGINF->name, Str31_size), Str31_size);
	RD(UnpackBlock(thePtr, &theGINF->groupDate, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theGINF->groupID, Integer_size), Integer_size);
	RD(UnpackXMatrix4(thePtr, &theGINF->coordinateAbsMatrix), XMatrix4_size);
	RD(UnpackXMatrix4(thePtr, &theGINF->normalAbsMatrix), XMatrix4_size);
	RD(UnpackXMatrix4(thePtr, &theGINF->coordinateRelMatrix), XMatrix4_size);
	RD(UnpackXMatrix4(thePtr, &theGINF->normalRelMatrix), XMatrix4_size);
	RD(UnpackBlock(thePtr, &theGINF->childCount, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theGINF->childTotal, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theGINF->cycleCount, LongInt_size), LongInt_size);
	*thePtr += theSize;
}  /* UnpackGINF */

void ReadGLNK(UFFile theFile, void* theData, ulong theSize)
{
	GLNKPtr theGLNK = (GLNKPtr)theData;
	boolean theFlag = 1;
	
	RD(ReadSmallBlock(theFile, &theGLNK->lType, Integer_size), Integer_size);
	RD(ReadXCoordinate(theFile, &theGLNK->positionMinLimit), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theGLNK->positionMaxLimit), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theGLNK->rotationMinLimit), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theGLNK->rotationMaxLimit), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theGLNK->localOffset), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theGLNK->parentOffset), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theGLNK->parentRotation), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theGLNK->localRotation), XCoordinate_size);
	RD(ReadXMatrix4(theFile, &theGLNK->preRotation), XMatrix4_size);
	RD(ReadSmallBlock(theFile, &theGLNK->lFlag, LongInt_size), LongInt_size);
	RD(ReadXCoordinate(theFile, &theGLNK->preScale), XCoordinate_size);
	if (theSize > 0)
		SkipBytes(theFile, theSize);
}  /* ReadGLNK */

void ReadFileGLNK(UFFile theFile, void* theData, ulong theSize)
{
	GLNKPtr theGLNK = (GLNKPtr)theData;
	boolean theFlag = 1;
	
	RD(UFReadEndian(theFile, &theGLNK->lType, Integer_size), Integer_size);
	RD(ReadFileXCoordinate(theFile, &theGLNK->positionMinLimit), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theGLNK->positionMaxLimit), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theGLNK->rotationMinLimit), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theGLNK->rotationMaxLimit), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theGLNK->localOffset), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theGLNK->parentOffset), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theGLNK->parentRotation), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theGLNK->localRotation), XCoordinate_size);
	RD(ReadFileXMatrix4(theFile, &theGLNK->preRotation), XMatrix4_size);
	RD(UFReadEndian(theFile, &theGLNK->lFlag, LongInt_size), LongInt_size);
	RD(ReadFileXCoordinate(theFile, &theGLNK->preScale), XCoordinate_size);
	if (theSize > 0)
		UFSetPos(theFile, UFGetPos(theFile) + theSize);
}  /* ReadFileGLNK */

void WriteGLNK(UFFile theFile, GLNKPtr theGLNK)
{
	ulong theSize = GLNK_size;
	
	WriteSmallBlockForward(theFile, (void*)"GLNK", Type_size);
	WriteSmallBlock(theFile, &theSize, LongInt_size);
	WriteSmallBlock(theFile, &theGLNK->lType, Integer_size);
	WriteXCoordinate(theFile, &theGLNK->positionMinLimit);
	WriteXCoordinate(theFile, &theGLNK->positionMaxLimit);
	WriteXCoordinate(theFile, &theGLNK->rotationMinLimit);
	WriteXCoordinate(theFile, &theGLNK->rotationMaxLimit);
	WriteXCoordinate(theFile, &theGLNK->localOffset);
	WriteXCoordinate(theFile, &theGLNK->parentOffset);
	WriteXCoordinate(theFile, &theGLNK->parentRotation);
	WriteXCoordinate(theFile, &theGLNK->localRotation);
	WriteXMatrix4(theFile, &theGLNK->preRotation);
	WriteSmallBlock(theFile, &theGLNK->lFlag, LongInt_size);
	WriteXCoordinate(theFile, &theGLNK->preScale);
}  /* WriteGLNK */

void WriteFileGLNK(UFFile theFile, GLNKPtr theGLNK)
{
	ulong theSize = GLNK_size;
	
	UFWriteEndian(theFile, (void*)"GLNK", Type_size);
	UFWriteEndian(theFile, &theSize, LongInt_size);
	UFWriteEndian(theFile, &theGLNK->lType, Integer_size);
	WriteFileXCoordinate(theFile, &theGLNK->positionMinLimit);
	WriteFileXCoordinate(theFile, &theGLNK->positionMaxLimit);
	WriteFileXCoordinate(theFile, &theGLNK->rotationMinLimit);
	WriteFileXCoordinate(theFile, &theGLNK->rotationMaxLimit);
	WriteFileXCoordinate(theFile, &theGLNK->localOffset);
	WriteFileXCoordinate(theFile, &theGLNK->parentOffset);
	WriteFileXCoordinate(theFile, &theGLNK->parentRotation);
	WriteFileXCoordinate(theFile, &theGLNK->localRotation);
	WriteFileXMatrix4(theFile, &theGLNK->preRotation);
	UFWriteEndian(theFile, &theGLNK->lFlag, LongInt_size);
	WriteFileXCoordinate(theFile, &theGLNK->preScale);
}  /* WriteFileGLNK */

void PackGLNK(char **thePtr, GLNKPtr theGLNK)
{
	ulong theSize = GLNK_size;
	
	PackBlockForward(thePtr, (void*)"GLNK", Type_size);
	PackBlock(thePtr, &theSize, LongInt_size);
	PackBlock(thePtr, &theGLNK->lType, Integer_size);
	PackXCoordinate(thePtr, &theGLNK->positionMinLimit);
	PackXCoordinate(thePtr, &theGLNK->positionMaxLimit);
	PackXCoordinate(thePtr, &theGLNK->rotationMinLimit);
	PackXCoordinate(thePtr, &theGLNK->rotationMaxLimit);
	PackXCoordinate(thePtr, &theGLNK->localOffset);
	PackXCoordinate(thePtr, &theGLNK->parentOffset);
	PackXCoordinate(thePtr, &theGLNK->parentRotation);
	PackXCoordinate(thePtr, &theGLNK->localRotation);
	PackXMatrix4(thePtr, &theGLNK->preRotation);
	PackBlock(thePtr, &theGLNK->lFlag, LongInt_size);
	PackXCoordinate(thePtr, &theGLNK->preScale);
}  /* PackGLNK */

void UnpackGLNK(char **thePtr, void *theData, ulong theSize)
{
	GLNKPtr theGLNK = (GLNKPtr)theData;
	boolean theFlag = 1;
	
	RD(UnpackBlock(thePtr, &theGLNK->lType, Integer_size), Integer_size);
	RD(UnpackXCoordinate(thePtr, &theGLNK->positionMinLimit), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theGLNK->positionMaxLimit), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theGLNK->rotationMinLimit), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theGLNK->rotationMaxLimit), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theGLNK->localOffset), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theGLNK->parentOffset), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theGLNK->parentRotation), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theGLNK->localRotation), XCoordinate_size);
	RD(UnpackXMatrix4(thePtr, &theGLNK->preRotation), XMatrix4_size);
	RD(UnpackBlock(thePtr, &theGLNK->lFlag, LongInt_size), LongInt_size);
	RD(UnpackXCoordinate(thePtr, &theGLNK->preScale), XCoordinate_size);
	*thePtr += theSize;
}  /* UnpackGLNK */

void ReadGSTR(UFFile theFile, void* theData, ulong theSize)
{
	GSTRPtr theGSTR = (GSTRPtr)theData;
	boolean theFlag = 1;
	
	RD(ReadXCoordinate(theFile, &theGSTR->offset), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theGSTR->center), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theGSTR->rotation), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theGSTR->scale), XCoordinate_size);
	if (theSize > 0)
		SkipBytes(theFile, theSize);
}  /* ReadGSTR */

void ReadFileGSTR(UFFile theFile, void* theData, ulong theSize)
{
	GSTRPtr theGSTR = (GSTRPtr)theData;
	boolean theFlag = 1;
	
	RD(ReadFileXCoordinate(theFile, &theGSTR->offset), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theGSTR->center), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theGSTR->rotation), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theGSTR->scale), XCoordinate_size);
	if (theSize > 0)
		UFSetPos(theFile, UFGetPos(theFile) + theSize);
}  /* ReadFileGSTR */

void WriteGSTR(UFFile theFile, GSTRPtr theGSTR)
{
	ulong theSize = GSTR_size;
	
	WriteSmallBlockForward(theFile, (void*)"GSTR", Type_size);
	WriteSmallBlock(theFile, &theSize, LongInt_size);
	WriteXCoordinate(theFile, &theGSTR->offset);
	WriteXCoordinate(theFile, &theGSTR->center);
	WriteXCoordinate(theFile, &theGSTR->rotation);
	WriteXCoordinate(theFile, &theGSTR->scale);
}  /* WriteGSTR */

void WriteFileGSTR(UFFile theFile, GSTRPtr theGSTR)
{
	ulong theSize = GSTR_size;
	
	UFWrite(theFile, (void*)"GSTR", Type_size);
	UFWriteEndian(theFile, &theSize, LongInt_size);
	WriteFileXCoordinate(theFile, &theGSTR->offset);
	WriteFileXCoordinate(theFile, &theGSTR->center);
	WriteFileXCoordinate(theFile, &theGSTR->rotation);
	WriteFileXCoordinate(theFile, &theGSTR->scale);
}  /* WriteFileGSTR */

void PackGSTR(char **thePtr, GSTRPtr theGSTR)
{
	ulong theSize = GSTR_size;
	
	PackBlockForward(thePtr, (void*)"GSTR", Type_size);
	PackBlock(thePtr, &theSize, LongInt_size);
	PackXCoordinate(thePtr, &theGSTR->offset);
	PackXCoordinate(thePtr, &theGSTR->center);
	PackXCoordinate(thePtr, &theGSTR->rotation);
	PackXCoordinate(thePtr, &theGSTR->scale);
}  /* PackGSTR */

void UnpackGSTR(char **thePtr, void *theData, ulong theSize)
{
	GSTRPtr theGSTR = (GSTRPtr)theData;
	boolean theFlag = 1;
	
	RD(UnpackXCoordinate(thePtr, &theGSTR->offset), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theGSTR->center), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theGSTR->rotation), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theGSTR->scale), XCoordinate_size);
	*thePtr += theSize;
}  /* UnpackGSTR */

void ReadGBLR(UFFile theFile, void* theData, UFSize theSize)
{
	GBLRPtr theGBLR = (GBLRPtr)theData;
	boolean theFlag = 1;
	
	RD(ReadSmallBlock(theFile, &theGBLR->blurFlags, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theGBLR->blurIntensity, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGBLR->blurLayer, LongInt_size), LongInt_size);
	RD(ReadXMatrix4(theFile, &theGBLR->blurMatrix), XMatrix4_size);
	if (theSize > 0)
		SkipBytes(theFile, theSize);
}  /* ReadGBLR */

void ReadFileGBLR(UFFile theFile, void* theData, UFSize theSize)
{
	GBLRPtr theGBLR = (GBLRPtr)theData;
	boolean theFlag = 1;
	
	RD(UFReadEndian(theFile, &theGBLR->blurFlags, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theGBLR->blurIntensity, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGBLR->blurLayer, LongInt_size), LongInt_size);
	RD(ReadFileXMatrix4(theFile, &theGBLR->blurMatrix), XMatrix4_size);
	if (theSize > 0)
		UFSetPos(theFile, UFGetPos(theFile) + theSize);
}  /* ReadFileGBLR */

void WriteGBLR(UFFile theFile, GBLRPtr theGBLR)
{
	ulong theSize = GBLR_size;
	
	WriteSmallBlockForward(theFile, (void*)"GBLR", Type_size);
	WriteSmallBlock(theFile, &theSize, LongInt_size);
	WriteSmallBlock(theFile, &theGBLR->blurFlags, LongInt_size);
	WriteSmallBlock(theFile, &theGBLR->blurIntensity, Real_size);
	WriteSmallBlock(theFile, &theGBLR->blurLayer, LongInt_size);
	WriteXMatrix4(theFile, &theGBLR->blurMatrix);
}  /* WriteGBLR */

void WriteFileGBLR(UFFile theFile, GBLRPtr theGBLR)
{
	ulong theSize = GBLR_size;
	
	UFWrite(theFile, (void*)"GBLR", Type_size);
	UFWriteEndian(theFile, &theSize, LongInt_size);
	UFWriteEndian(theFile, &theGBLR->blurFlags, LongInt_size);
	UFWriteEndian(theFile, &theGBLR->blurIntensity, Real_size);
	UFWriteEndian(theFile, &theGBLR->blurLayer, LongInt_size);
	WriteFileXMatrix4(theFile, &theGBLR->blurMatrix);
}  /* WriteFileGBLR */

void PackGBLR(char **thePtr, GBLRPtr theGBLR)
{
	ulong theSize = GBLR_size;
	
	PackBlockForward(thePtr, (void*)"GBLR", Type_size);
	PackBlock(thePtr, &theSize, LongInt_size);
	PackBlock(thePtr, &theGBLR->blurFlags, LongInt_size);
	PackBlock(thePtr, &theGBLR->blurIntensity, Real_size);
	PackBlock(thePtr, &theGBLR->blurLayer, LongInt_size);
	PackXMatrix4(thePtr, &theGBLR->blurMatrix);
}  /* PackGBLR */

void UnpackGBLR(char **thePtr, void *theData, ulong theSize)
{
	GBLRPtr theGBLR = (GBLRPtr)theData;
	boolean theFlag = 1;
	
	RD(UnpackBlock(thePtr, &theGBLR->blurFlags, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theGBLR->blurIntensity, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGBLR->blurLayer, LongInt_size), LongInt_size);
	RD(UnpackXMatrix4(thePtr, &theGBLR->blurMatrix), XMatrix4_size);
	*thePtr += theSize;
}  /* UnpackGBLR */

void ReadGOUT(UFFile theFile, void* theData, UFSize theSize)
{
	GOUTPtr theGOUT = (GOUTPtr)theData;
	boolean theFlag = 1;
	
	RD(ReadSmallBlock(theFile, &theGOUT->outlineFlags, LongInt_size), LongInt_size);
	RD(ReadSmallBlockForward(theFile, &theGOUT->outlinePolygonColor, ARGB_size), ARGB_size);
	RD(ReadSmallBlock(theFile, &theGOUT->outlinePolygonThickness, Real_size), Real_size);
	RD(ReadSmallBlockForward(theFile, &theGOUT->outlineEdgeColor, ARGB_size), ARGB_size);
	RD(ReadSmallBlock(theFile, &theGOUT->outlineEdgeThickness, Real_size), Real_size);
	RD(ReadSmallBlockForward(theFile, &theGOUT->outlineSilhouetteColor, ARGB_size), ARGB_size);
	RD(ReadSmallBlock(theFile, &theGOUT->outlineSilhouetteThickness, Real_size), Real_size);
	if (theSize > 0)
		SkipBytes(theFile, theSize);
}  /* ReadGOUT */

void ReadFileGOUT(UFFile theFile, void* theData, UFSize theSize)
{
	GOUTPtr theGOUT = (GOUTPtr)theData;
	boolean theFlag = 1;
	
	RD(UFReadEndian(theFile, &theGOUT->outlineFlags, LongInt_size), LongInt_size);
	RD(UFRead(theFile, &theGOUT->outlinePolygonColor, ARGB_size), ARGB_size);
	RD(UFReadEndian(theFile, &theGOUT->outlinePolygonThickness, Real_size), Real_size);
	RD(UFRead(theFile, &theGOUT->outlineEdgeColor, ARGB_size), ARGB_size);
	RD(UFReadEndian(theFile, &theGOUT->outlineEdgeThickness, Real_size), Real_size);
	RD(UFRead(theFile, &theGOUT->outlineSilhouetteColor, ARGB_size), ARGB_size);
	RD(UFReadEndian(theFile, &theGOUT->outlineSilhouetteThickness, Real_size), Real_size);
	if (theSize > 0)
		SkipBytes(theFile, theSize);
}  /* ReadFileGOUT */

void WriteGOUT(UFFile theFile, GOUTPtr theGOUT)
{
	ulong theSize = GOUT_size;
	
	WriteSmallBlockForward(theFile, (void*)"GOUT", Type_size);
	WriteSmallBlock(theFile, &theSize, LongInt_size);
	WriteSmallBlock(theFile, &theGOUT->outlineFlags, LongInt_size);
	WriteSmallBlockForward(theFile, &theGOUT->outlinePolygonColor, ARGB_size);
	WriteSmallBlock(theFile, &theGOUT->outlinePolygonThickness, Real_size);
	WriteSmallBlockForward(theFile, &theGOUT->outlineEdgeColor, ARGB_size);
	WriteSmallBlock(theFile, &theGOUT->outlineEdgeThickness, Real_size);
	WriteSmallBlockForward(theFile, &theGOUT->outlineSilhouetteColor, ARGB_size);
	WriteSmallBlock(theFile, &theGOUT->outlineSilhouetteThickness, Real_size);
}  /* WriteGOUT */

void WriteFileGOUT(UFFile theFile, GOUTPtr theGOUT)
{
	ulong theSize = GOUT_size;
	
	UFWrite(theFile, (void*)"GOUT", Type_size);
	UFWriteEndian(theFile, &theSize, LongInt_size);
	UFWriteEndian(theFile, &theGOUT->outlineFlags, LongInt_size);
	UFWrite(theFile, &theGOUT->outlinePolygonColor, ARGB_size);
	UFWriteEndian(theFile, &theGOUT->outlinePolygonThickness, Real_size);
	UFWrite(theFile, &theGOUT->outlineEdgeColor, ARGB_size);
	UFWriteEndian(theFile, &theGOUT->outlineEdgeThickness, Real_size);
	UFWrite(theFile, &theGOUT->outlineSilhouetteColor, ARGB_size);
	UFWriteEndian(theFile, &theGOUT->outlineSilhouetteThickness, Real_size);
}  /* WriteFileGOUT */

void PackGOUT(char **thePtr, GOUTPtr theGOUT)
{
	ulong theSize = GOUT_size;

	PackBlockForward(thePtr, (void*)"GOUT", Type_size);
	PackBlock(thePtr, &theSize, LongInt_size);
	PackBlock(thePtr, &theGOUT->outlineFlags, LongInt_size);
	PackBlockForward(thePtr, &theGOUT->outlinePolygonColor, ARGB_size);
	PackBlock(thePtr, &theGOUT->outlinePolygonThickness, Real_size);
	PackBlockForward(thePtr, &theGOUT->outlineEdgeColor, ARGB_size);
	PackBlock(thePtr, &theGOUT->outlineEdgeThickness, Real_size);
	PackBlockForward(thePtr, &theGOUT->outlineSilhouetteColor, ARGB_size);
	PackBlock(thePtr, &theGOUT->outlineSilhouetteThickness, Real_size);
}  /* PackGOUT */

void UnpackGOUT(char **thePtr, void *theData, ulong theSize)
{
	GOUTPtr theGOUT = (GOUTPtr)theData;
	boolean theFlag = 1;
	
	RD(UnpackBlock(thePtr, &theGOUT->outlineFlags, LongInt_size), LongInt_size);
	RD(UnpackBlockForward(thePtr, &theGOUT->outlinePolygonColor, ARGB_size), ARGB_size);
	RD(UnpackBlock(thePtr, &theGOUT->outlinePolygonThickness, Real_size), Real_size);
	RD(UnpackBlockForward(thePtr, &theGOUT->outlineEdgeColor, ARGB_size), ARGB_size);
	RD(UnpackBlock(thePtr, &theGOUT->outlineEdgeThickness, Real_size), Real_size);
	RD(UnpackBlockForward(thePtr, &theGOUT->outlineSilhouetteColor, ARGB_size), ARGB_size);
	RD(UnpackBlock(thePtr, &theGOUT->outlineSilhouetteThickness, Real_size), Real_size);
	*thePtr += theSize;
}  /* UnpacGOUT */

void ReadGATR(UFFile theFile, void* theData, UFSize theSize)
{
	GATRPtr theGATR = (GATRPtr)theData;
	boolean theFlag = 1;
	
	RD(ReadSmallBlock(theFile, &theGATR->shadeFlags, Integer_size), Integer_size);
	RD(ReadSmallBlockForward(theFile, &theGATR->reference, ARGB_size), ARGB_size);
	RD(ReadSmallBlockForward(theFile, &theGATR->ambient, ARGB_size), ARGB_size);
	RD(ReadSmallBlockForward(theFile, &theGATR->specular, ARGB_size), ARGB_size);
	RD(ReadSmallBlock(theFile, &theGATR->spread, Real_size), Real_size);
	RD(ReadSmallBlockForward(theFile, &theGATR->transparency, ARGB_size), ARGB_size);
	RD(ReadSmallBlock(theFile, &theGATR->refraction, Real_size), Real_size);
	RD(ReadSmallBlockForward(theFile, &theGATR->reflection, ARGB_size), ARGB_size);
	RD(ReadSmallBlockForward(theFile, &theGATR->luminance, ARGB_size), ARGB_size);
	RD(ReadSmallBlock(theFile, &theGATR->lineWeight, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->edgeTransparencyDropoff, Real_size), Real_size);
	RD(ReadSmallBlockForward(theFile, &theGATR->edgeTransparency, ARGB_size), ARGB_size);
	RD(ReadSmallBlockForward(theFile, &theGATR->diffuse, ARGB_size), ARGB_size);
	RD(ReadSmallBlock(theFile, &theGATR->terminator, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->highlight, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->edgeDiffuseValue, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->edgeSpecularValue, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->edgeReflectionValue, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->edgeAmbientValue, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->edgeLuminanceValue, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->shadeFlags2, Integer_size), Integer_size);
	RD(ReadSmallBlockForward(theFile, &theGATR->transmission, ARGB_size), ARGB_size);
	RD(ReadSmallBlock(theFile, &theGATR->edgeTransmissionValue, Real_size), Real_size);
	RD(ReadSmallBlockForward(theFile, &theGATR->glow, ARGB_size), ARGB_size);
	RD(ReadSmallBlock(theFile, &theGATR->edgeDiffuseDropoff, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->edgeSpecularDropoff, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->edgeReflectionDropoff, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->edgeAmbientDropoff, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->edgeLuminanceDropoff, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->edgeTransmissionDropoff, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->edgeTransparencyValue, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->reflectionBlur, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->refractionBlur, Real_size), Real_size);
	
	RD(ReadSmallBlock(theFile, &theGATR->reflectionBlurType, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theGATR->reflectionBlurSamples, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theGATR->reflectionBlurAngle, Real_size), Real_size);

	RD(ReadSmallBlock(theFile, &theGATR->refractionBlurType, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theGATR->refractionBlurSamples, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theGATR->refractionBlurAngle, Real_size), Real_size);

	RD(ReadSmallBlock(theFile, &theGATR->refrCausticsAmount, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->refrCausticsFactor, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->refrCausticsDropoff, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->refrCausticsReceive, Real_size), Real_size);

	RD(ReadSmallBlock(theFile, &theGATR->reflCausticsAmount, Real_size), Real_size);
	RD(ReadSmallBlockForward(theFile, &theGATR->reflCausticsColor, ARGB_size), ARGB_size);
	RD(ReadSmallBlock(theFile, &theGATR->reflCausticsFactor, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->reflCausticsDropoff, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->reflCausticsReceive, Real_size), Real_size);

	RD(ReadSmallBlock(theFile, &theGATR->translucentAmount, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->translucentThickness, Real_size), Real_size);
	RD(ReadSmallBlockForward(theFile, &theGATR->translucentColor, ARGB_size), ARGB_size);
	RD(ReadSmallBlock(theFile, &theGATR->translucentScatter, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGATR->translucentFocus, Real_size), Real_size);

	RD(ReadSmallBlock(theFile, &theGATR->transparentThickness, Real_size), Real_size);
	
	if (theSize > 0)
		SkipBytes(theFile, theSize);
}  /* ReadGATR */

void ReadFileGATR(UFFile theFile, void* theData, UFSize theSize)
{
	GATRPtr theGATR = (GATRPtr)theData;
	boolean theFlag = 1;
	
	RD(UFReadEndian(theFile, &theGATR->shadeFlags, Integer_size), Integer_size);
	RD(UFRead(theFile, &theGATR->reference, ARGB_size), ARGB_size);
	RD(UFRead(theFile, &theGATR->ambient, ARGB_size), ARGB_size);
	RD(UFRead(theFile, &theGATR->specular, ARGB_size), ARGB_size);
	RD(UFReadEndian(theFile, &theGATR->spread, Real_size), Real_size);
	RD(UFRead(theFile, &theGATR->transparency, ARGB_size), ARGB_size);
	RD(UFReadEndian(theFile, &theGATR->refraction, Real_size), Real_size);
	RD(UFRead(theFile, &theGATR->reflection, ARGB_size), ARGB_size);
	RD(UFRead(theFile, &theGATR->luminance, ARGB_size), ARGB_size);
	RD(UFReadEndian(theFile, &theGATR->lineWeight, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->edgeTransparencyDropoff, Real_size), Real_size);
	RD(UFRead(theFile, &theGATR->edgeTransparency, ARGB_size), ARGB_size);
	RD(UFRead(theFile, &theGATR->diffuse, ARGB_size), ARGB_size);
	RD(UFReadEndian(theFile, &theGATR->terminator, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->highlight, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->edgeDiffuseValue, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->edgeSpecularValue, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->edgeReflectionValue, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->edgeAmbientValue, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->edgeLuminanceValue, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->shadeFlags2, Integer_size), Integer_size);
	RD(UFRead(theFile, &theGATR->transmission, ARGB_size), ARGB_size);
	RD(UFReadEndian(theFile, &theGATR->edgeTransmissionValue, Real_size), Real_size);
	RD(UFRead(theFile, &theGATR->glow, ARGB_size), ARGB_size);
	RD(UFReadEndian(theFile, &theGATR->edgeDiffuseDropoff, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->edgeSpecularDropoff, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->edgeReflectionDropoff, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->edgeAmbientDropoff, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->edgeLuminanceDropoff, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->edgeTransmissionDropoff, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->edgeTransparencyValue, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->reflectionBlur, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->refractionBlur, Real_size), Real_size);

	RD(UFReadEndian(theFile, &theGATR->reflectionBlurType, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theGATR->reflectionBlurSamples, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theGATR->reflectionBlurAngle, Real_size), Real_size);

	RD(UFReadEndian(theFile, &theGATR->refractionBlurType, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theGATR->refractionBlurSamples, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theGATR->refractionBlurAngle, Real_size), Real_size);

	RD(UFReadEndian(theFile, &theGATR->refrCausticsAmount, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->refrCausticsFactor, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->refrCausticsDropoff, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->refrCausticsReceive, Real_size), Real_size);

	RD(UFReadEndian(theFile, &theGATR->reflCausticsAmount, Real_size), Real_size);
	RD(UFRead(theFile, &theGATR->reflCausticsColor, ARGB_size), ARGB_size);
	RD(UFReadEndian(theFile, &theGATR->reflCausticsFactor, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->reflCausticsDropoff, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->reflCausticsReceive, Real_size), Real_size);
	
	RD(UFReadEndian(theFile, &theGATR->translucentAmount, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->translucentThickness, Real_size), Real_size);
	RD(UFRead(theFile, &theGATR->translucentColor, ARGB_size), ARGB_size);
	RD(UFReadEndian(theFile, &theGATR->translucentScatter, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGATR->translucentFocus, Real_size), Real_size);
	
	RD(UFReadEndian(theFile, &theGATR->transparentThickness, Real_size), Real_size);

	if (theSize > 0)
		UFSetPos(theFile, UFGetPos(theFile) + theSize);
}  /* ReadFileGATR */

void WriteGATR(UFFile theFile, GATRPtr theGATR, IFFType theType)
{
	ulong theSize = GATR_size;
	
	WriteSmallBlockForward(theFile, &theType, Type_size);
	WriteSmallBlock(theFile, &theSize, LongInt_size);
	WriteSmallBlock(theFile, &theGATR->shadeFlags, Integer_size);
	WriteSmallBlockForward(theFile, &theGATR->reference, ARGB_size);
	WriteSmallBlockForward(theFile, &theGATR->ambient, ARGB_size);
	WriteSmallBlockForward(theFile, &theGATR->specular, ARGB_size);
	WriteSmallBlock(theFile, &theGATR->spread, Real_size);
	WriteSmallBlockForward(theFile, &theGATR->transparency, ARGB_size);
	WriteSmallBlock(theFile, &theGATR->refraction, Real_size);
	WriteSmallBlockForward(theFile, &theGATR->reflection, ARGB_size);
	WriteSmallBlockForward(theFile, &theGATR->luminance, ARGB_size);
	WriteSmallBlock(theFile, &theGATR->lineWeight, Real_size);
	WriteSmallBlock(theFile, &theGATR->edgeTransparencyDropoff, Real_size);
	WriteSmallBlockForward(theFile, &theGATR->edgeTransparency, ARGB_size);
	WriteSmallBlockForward(theFile, &theGATR->diffuse, ARGB_size);
	WriteSmallBlock(theFile, &theGATR->terminator, Real_size);
	WriteSmallBlock(theFile, &theGATR->highlight, Real_size);
	WriteSmallBlock(theFile, &theGATR->edgeDiffuseValue, Real_size);
	WriteSmallBlock(theFile, &theGATR->edgeSpecularValue, Real_size);
	WriteSmallBlock(theFile, &theGATR->edgeReflectionValue, Real_size);
	WriteSmallBlock(theFile, &theGATR->edgeAmbientValue, Real_size);
	WriteSmallBlock(theFile, &theGATR->edgeLuminanceValue, Real_size);
	WriteSmallBlock(theFile, &theGATR->shadeFlags2, Integer_size);
	WriteSmallBlockForward(theFile, &theGATR->transmission, ARGB_size);
	WriteSmallBlock(theFile, &theGATR->edgeTransmissionValue, Real_size);
	WriteSmallBlockForward(theFile, &theGATR->glow, ARGB_size);
	WriteSmallBlock(theFile, &theGATR->edgeDiffuseDropoff, Real_size);
	WriteSmallBlock(theFile, &theGATR->edgeSpecularDropoff, Real_size);
	WriteSmallBlock(theFile, &theGATR->edgeReflectionDropoff, Real_size);
	WriteSmallBlock(theFile, &theGATR->edgeAmbientDropoff, Real_size);
	WriteSmallBlock(theFile, &theGATR->edgeLuminanceDropoff, Real_size);
	WriteSmallBlock(theFile, &theGATR->edgeTransmissionDropoff, Real_size);
	WriteSmallBlock(theFile, &theGATR->edgeTransparencyValue, Real_size);
	WriteSmallBlock(theFile, &theGATR->reflectionBlur, Real_size);
	WriteSmallBlock(theFile, &theGATR->refractionBlur, Real_size);

	WriteSmallBlock(theFile, &theGATR->reflectionBlurType, LongInt_size);
	WriteSmallBlock(theFile, &theGATR->reflectionBlurSamples, LongInt_size);
	WriteSmallBlock(theFile, &theGATR->reflectionBlurAngle, Real_size);

	WriteSmallBlock(theFile, &theGATR->refractionBlurType, LongInt_size);
	WriteSmallBlock(theFile, &theGATR->refractionBlurSamples, LongInt_size);
	WriteSmallBlock(theFile, &theGATR->refractionBlurAngle, Real_size);

	WriteSmallBlock(theFile, &theGATR->refrCausticsAmount, Real_size);
	WriteSmallBlock(theFile, &theGATR->refrCausticsFactor, Real_size);
	WriteSmallBlock(theFile, &theGATR->refrCausticsDropoff, Real_size);
	WriteSmallBlock(theFile, &theGATR->refrCausticsReceive, Real_size);

	WriteSmallBlock(theFile, &theGATR->reflCausticsAmount, Real_size);
	WriteSmallBlockForward(theFile, &theGATR->reflCausticsColor, ARGB_size);
	WriteSmallBlock(theFile, &theGATR->reflCausticsFactor, Real_size);
	WriteSmallBlock(theFile, &theGATR->reflCausticsDropoff, Real_size);
	WriteSmallBlock(theFile, &theGATR->reflCausticsReceive, Real_size);
	
	WriteSmallBlock(theFile, &theGATR->translucentAmount, Real_size);
	WriteSmallBlock(theFile, &theGATR->translucentThickness, Real_size);
	WriteSmallBlockForward(theFile, &theGATR->translucentColor, ARGB_size);
	WriteSmallBlock(theFile, &theGATR->translucentScatter, Real_size);
	WriteSmallBlock(theFile, &theGATR->translucentFocus, Real_size);
	
	WriteSmallBlock(theFile, &theGATR->transparentThickness, Real_size);
}  /* WriteGATR */

void WriteFileGATR(UFFile theFile, GATRPtr theGATR, IFFType theType)
{
	ulong theSize = GATR_size;
	
	UFWrite(theFile, &theType, Type_size);
	UFWriteEndian(theFile, &theSize, LongInt_size);
	UFWriteEndian(theFile, &theGATR->shadeFlags, Integer_size);
	UFWrite(theFile, &theGATR->reference, ARGB_size);
	UFWrite(theFile, &theGATR->ambient, ARGB_size);
	UFWrite(theFile, &theGATR->specular, ARGB_size);
	UFWriteEndian(theFile, &theGATR->spread, Real_size);
	UFWrite(theFile, &theGATR->transparency, ARGB_size);
	UFWriteEndian(theFile, &theGATR->refraction, Real_size);
	UFWrite(theFile, &theGATR->reflection, ARGB_size);
	UFWrite(theFile, &theGATR->luminance, ARGB_size);
	UFWriteEndian(theFile, &theGATR->lineWeight, Real_size);
	UFWriteEndian(theFile, &theGATR->edgeTransparencyDropoff, Real_size);
	UFWrite(theFile, &theGATR->edgeTransparency, ARGB_size);
	UFWrite(theFile, &theGATR->diffuse, ARGB_size);
	UFWriteEndian(theFile, &theGATR->terminator, Real_size);
	UFWriteEndian(theFile, &theGATR->highlight, Real_size);
	UFWriteEndian(theFile, &theGATR->edgeDiffuseValue, Real_size);
	UFWriteEndian(theFile, &theGATR->edgeSpecularValue, Real_size);
	UFWriteEndian(theFile, &theGATR->edgeReflectionValue, Real_size);
	UFWriteEndian(theFile, &theGATR->edgeAmbientValue, Real_size);
	UFWriteEndian(theFile, &theGATR->edgeLuminanceValue, Real_size);
	UFWriteEndian(theFile, &theGATR->shadeFlags2, Integer_size);
	UFWrite(theFile, &theGATR->transmission, ARGB_size);
	UFWriteEndian(theFile, &theGATR->edgeTransmissionValue, Real_size);
	UFWrite(theFile, &theGATR->glow, ARGB_size);
	UFWriteEndian(theFile, &theGATR->edgeDiffuseDropoff, Real_size);
	UFWriteEndian(theFile, &theGATR->edgeSpecularDropoff, Real_size);
	UFWriteEndian(theFile, &theGATR->edgeReflectionDropoff, Real_size);
	UFWriteEndian(theFile, &theGATR->edgeAmbientDropoff, Real_size);
	UFWriteEndian(theFile, &theGATR->edgeLuminanceDropoff, Real_size);
	UFWriteEndian(theFile, &theGATR->edgeTransmissionDropoff, Real_size);
	UFWriteEndian(theFile, &theGATR->edgeTransparencyValue, Real_size);
	UFWriteEndian(theFile, &theGATR->reflectionBlur, Real_size);
	UFWriteEndian(theFile, &theGATR->refractionBlur, Real_size);
	
	UFWriteEndian(theFile, &theGATR->reflectionBlurType, LongInt_size);
	UFWriteEndian(theFile, &theGATR->reflectionBlurSamples, LongInt_size);
	UFWriteEndian(theFile, &theGATR->reflectionBlurAngle, Real_size);

	UFWriteEndian(theFile, &theGATR->refractionBlurType, LongInt_size);
	UFWriteEndian(theFile, &theGATR->refractionBlurSamples, LongInt_size);
	UFWriteEndian(theFile, &theGATR->refractionBlurAngle, Real_size);
	
	UFWriteEndian(theFile, &theGATR->refrCausticsAmount, Real_size);
	UFWriteEndian(theFile, &theGATR->refrCausticsFactor, Real_size);
	UFWriteEndian(theFile, &theGATR->refrCausticsDropoff, Real_size);
	UFWriteEndian(theFile, &theGATR->refrCausticsReceive, Real_size);

	UFWriteEndian(theFile, &theGATR->reflCausticsAmount, Real_size);
	UFWrite(theFile, &theGATR->reflCausticsColor, ARGB_size);
	UFWriteEndian(theFile, &theGATR->reflCausticsFactor, Real_size);
	UFWriteEndian(theFile, &theGATR->reflCausticsDropoff, Real_size);
	UFWriteEndian(theFile, &theGATR->reflCausticsReceive, Real_size);
	
	UFWriteEndian(theFile, &theGATR->translucentAmount, Real_size);
	UFWriteEndian(theFile, &theGATR->translucentThickness, Real_size);
	UFWrite(theFile, &theGATR->translucentColor, ARGB_size);
	UFWriteEndian(theFile, &theGATR->translucentScatter, Real_size);
	UFWriteEndian(theFile, &theGATR->translucentFocus, Real_size);

	UFWriteEndian(theFile, &theGATR->transparentThickness, Real_size);
}  /* WriteFileGATR */

void PackGATR(char **thePtr, GATRPtr theGATR, IFFType theType)
{
	ulong theSize = GATR_size;
	
	PackBlockForward(thePtr, &theType, Type_size);
	PackBlock(thePtr, &theSize, LongInt_size);
	PackBlock(thePtr, &theGATR->shadeFlags, Integer_size);
	PackBlockForward(thePtr, &theGATR->reference, ARGB_size);
	PackBlockForward(thePtr, &theGATR->ambient, ARGB_size);
	PackBlockForward(thePtr, &theGATR->specular, ARGB_size);
	PackBlock(thePtr, &theGATR->spread, Real_size);
	PackBlockForward(thePtr, &theGATR->transparency, ARGB_size);
	PackBlock(thePtr, &theGATR->refraction, Real_size);
	PackBlockForward(thePtr, &theGATR->reflection, ARGB_size);
	PackBlockForward(thePtr, &theGATR->luminance, ARGB_size);
	PackBlock(thePtr, &theGATR->lineWeight, Real_size);
	PackBlock(thePtr, &theGATR->edgeTransparencyDropoff, Real_size);
	PackBlockForward(thePtr, &theGATR->edgeTransparency, ARGB_size);
	PackBlockForward(thePtr, &theGATR->diffuse, ARGB_size);
	PackBlock(thePtr, &theGATR->terminator, Real_size);
	PackBlock(thePtr, &theGATR->highlight, Real_size);
	PackBlock(thePtr, &theGATR->edgeDiffuseValue, Real_size);
	PackBlock(thePtr, &theGATR->edgeSpecularValue, Real_size);
	PackBlock(thePtr, &theGATR->edgeReflectionValue, Real_size);
	PackBlock(thePtr, &theGATR->edgeAmbientValue, Real_size);
	PackBlock(thePtr, &theGATR->edgeLuminanceValue, Real_size);
	PackBlock(thePtr, &theGATR->shadeFlags2, Integer_size);
	PackBlockForward(thePtr, &theGATR->transmission, ARGB_size);
	PackBlock(thePtr, &theGATR->edgeTransmissionValue, Real_size);
	PackBlockForward(thePtr, &theGATR->glow, ARGB_size);
	PackBlock(thePtr, &theGATR->edgeDiffuseDropoff, Real_size);
	PackBlock(thePtr, &theGATR->edgeSpecularDropoff, Real_size);
	PackBlock(thePtr, &theGATR->edgeReflectionDropoff, Real_size);
	PackBlock(thePtr, &theGATR->edgeAmbientDropoff, Real_size);
	PackBlock(thePtr, &theGATR->edgeLuminanceDropoff, Real_size);
	PackBlock(thePtr, &theGATR->edgeTransmissionDropoff, Real_size);
	PackBlock(thePtr, &theGATR->edgeTransparencyValue, Real_size);
	PackBlock(thePtr, &theGATR->reflectionBlur, Real_size);
	PackBlock(thePtr, &theGATR->refractionBlur, Real_size);
	
	PackBlock(thePtr, &theGATR->reflectionBlurType, LongInt_size);
	PackBlock(thePtr, &theGATR->reflectionBlurSamples, LongInt_size);
	PackBlock(thePtr, &theGATR->reflectionBlurAngle, Real_size);

	PackBlock(thePtr, &theGATR->refractionBlurType, LongInt_size);
	PackBlock(thePtr, &theGATR->refractionBlurSamples, LongInt_size);
	PackBlock(thePtr, &theGATR->refractionBlurAngle, Real_size);

	PackBlock(thePtr, &theGATR->refrCausticsAmount, Real_size);
	PackBlock(thePtr, &theGATR->refrCausticsFactor, Real_size);
	PackBlock(thePtr, &theGATR->refrCausticsDropoff, Real_size);
	PackBlock(thePtr, &theGATR->refrCausticsReceive, Real_size);
	
	PackBlock(thePtr, &theGATR->reflCausticsAmount, Real_size);
	PackBlockForward(thePtr, &theGATR->reflCausticsColor, ARGB_size);
	PackBlock(thePtr, &theGATR->reflCausticsFactor, Real_size);
	PackBlock(thePtr, &theGATR->reflCausticsDropoff, Real_size);
	PackBlock(thePtr, &theGATR->reflCausticsReceive, Real_size);
	
	PackBlock(thePtr, &theGATR->translucentAmount, Real_size);
	PackBlock(thePtr, &theGATR->translucentThickness, Real_size);
	PackBlockForward(thePtr, &theGATR->translucentColor, ARGB_size);
	PackBlock(thePtr, &theGATR->translucentScatter, Real_size);
	PackBlock(thePtr, &theGATR->translucentFocus, Real_size);

	PackBlock(thePtr, &theGATR->transparentThickness, Real_size);
}  /* PackGATR */

void UnpackGATR(char **thePtr, void* theData, ulong theSize)
{
	GATRPtr theGATR = (GATRPtr)theData;
	boolean theFlag = 1;
	
	RD(UnpackBlock(thePtr, &theGATR->shadeFlags, Integer_size), Integer_size);
	RD(UnpackBlockForward(thePtr, &theGATR->reference, ARGB_size), ARGB_size);
	RD(UnpackBlockForward(thePtr, &theGATR->ambient, ARGB_size), ARGB_size);
	RD(UnpackBlockForward(thePtr, &theGATR->specular, ARGB_size), ARGB_size);
	RD(UnpackBlock(thePtr, &theGATR->spread, Real_size), Real_size);
	RD(UnpackBlockForward(thePtr, &theGATR->transparency, ARGB_size), ARGB_size);
	RD(UnpackBlock(thePtr, &theGATR->refraction, Real_size), Real_size);
	RD(UnpackBlockForward(thePtr, &theGATR->reflection, ARGB_size), ARGB_size);
	RD(UnpackBlockForward(thePtr, &theGATR->luminance, ARGB_size), ARGB_size);
	RD(UnpackBlock(thePtr, &theGATR->lineWeight, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->edgeTransparencyDropoff, Real_size), Real_size);
	RD(UnpackBlockForward(thePtr, &theGATR->edgeTransparency, ARGB_size), ARGB_size);
	RD(UnpackBlockForward(thePtr, &theGATR->diffuse, ARGB_size), ARGB_size);
	RD(UnpackBlock(thePtr, &theGATR->terminator, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->highlight, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->edgeDiffuseValue, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->edgeSpecularValue, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->edgeReflectionValue, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->edgeAmbientValue, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->edgeLuminanceValue, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->shadeFlags2, Integer_size), Integer_size);
	RD(UnpackBlockForward(thePtr, &theGATR->transmission, ARGB_size), ARGB_size);
	RD(UnpackBlock(thePtr, &theGATR->edgeTransmissionValue, Real_size), Real_size);
	RD(UnpackBlockForward(thePtr, &theGATR->glow, ARGB_size), ARGB_size);
	RD(UnpackBlock(thePtr, &theGATR->edgeDiffuseDropoff, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->edgeSpecularDropoff, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->edgeReflectionDropoff, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->edgeAmbientDropoff, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->edgeLuminanceDropoff, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->edgeTransmissionDropoff, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->edgeTransparencyValue, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->reflectionBlur, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->refractionBlur, Real_size), Real_size);

	RD(UnpackBlock(thePtr, &theGATR->reflectionBlurType, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theGATR->reflectionBlurSamples, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theGATR->reflectionBlurAngle, Real_size), Real_size);

	RD(UnpackBlock(thePtr, &theGATR->refractionBlurType, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theGATR->refractionBlurSamples, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theGATR->refractionBlurAngle, Real_size), Real_size);

	RD(UnpackBlock(thePtr, &theGATR->refrCausticsAmount, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->refrCausticsFactor, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->refrCausticsDropoff, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->refrCausticsReceive, Real_size), Real_size);

	RD(UnpackBlock(thePtr, &theGATR->reflCausticsAmount, Real_size), Real_size);
	RD(UnpackBlockForward(thePtr, &theGATR->reflCausticsColor, ARGB_size), ARGB_size);
	RD(UnpackBlock(thePtr, &theGATR->reflCausticsFactor, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->reflCausticsDropoff, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->reflCausticsReceive, Real_size), Real_size);
	
	RD(UnpackBlock(thePtr, &theGATR->translucentAmount, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->translucentThickness, Real_size), Real_size);
	RD(UnpackBlockForward(thePtr, &theGATR->translucentColor, ARGB_size), ARGB_size);
	RD(UnpackBlock(thePtr, &theGATR->translucentScatter, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGATR->translucentFocus, Real_size), Real_size);
	
	RD(UnpackBlock(thePtr, &theGATR->transparentThickness, Real_size), Real_size);

	*thePtr += theSize;
}  /* UnpackGATR */

void ReadTMAP(UFFile theFile, void* theData, UFSize theSize)
{
	TMAPPtr theTMAP = (TMAPPtr)theData;
	boolean theFlag = 1;
	
	RD(ReadSmallBlock(theFile, &theTMAP->textureFlags, Integer_size), Integer_size);
	RD(ReadSmallBlock(theFile, &theTMAP->textureType, Integer_size), Integer_size);
	RD(ReadSmallBlockForward(theFile, &theTMAP->textureTypeName, Str31_size), Str31_size);
	RD(ReadSmallBlock(theFile, &theTMAP->mappingType, Integer_size), Integer_size);
	RD(ReadSmallBlockForward(theFile, &theTMAP->mappingTypeName, Str31_size), Str31_size);
	RD(ReadXMatrix4(theFile, &theTMAP->tMatrix), XMatrix4_size);
	RD(ReadSmallBlockForward(theFile, &theTMAP->textureName, Str31_size), Str31_size);
	RD(ReadSRect(theFile, &theTMAP->textureRect), Rect_size);
	RD(ReadXCoordinate(theFile, &theTMAP->offset), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theTMAP->scale), XCoordinate_size);
	RD(ReadSmallBlock(theFile, &theTMAP->textureReference, LongInt_size), LongInt_size);
	RD(ReadSmallBlockForward(theFile, &theTMAP->textureVolName, Str31_size), Str31_size);
	RD(READEXTENDED12(theFile, &theTMAP->bumpScale), Extended_size);
	RD(ReadXMatrix4(theFile, &theTMAP->bumpMatrix), XMatrix4_size);
	RD(ReadSmallBlock(theFile, &theTMAP->textureFrame, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theTMAP->textureTime, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theTMAP->textureDensity, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theTMAP->textureCutoff, Integer_size), Integer_size);
	RD(ReadXCoordinate(theFile, &theTMAP->backOffset), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theTMAP->backScale), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theTMAP->leftOffset), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theTMAP->leftScale), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theTMAP->rightOffset), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theTMAP->rightScale), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theTMAP->topOffset), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theTMAP->topScale), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theTMAP->bottomOffset), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theTMAP->bottomScale), XCoordinate_size);
	if (theSize > 0)
		SkipBytes(theFile, theSize);
}  /* ReadTMAP */

void ReadFileTMAP(UFFile theFile, void* theData, UFSize theSize)
{
	TMAPPtr theTMAP = (TMAPPtr)theData;
	boolean theFlag = 1;
	
	RD(UFReadEndian(theFile, &theTMAP->textureFlags, Integer_size), Integer_size);
	RD(UFReadEndian(theFile, &theTMAP->textureType, Integer_size), Integer_size);
	RD(UFRead(theFile, &theTMAP->textureTypeName, Str31_size), Str31_size);
	RD(UFReadEndian(theFile, &theTMAP->mappingType, Integer_size), Integer_size);
	RD(UFRead(theFile, &theTMAP->mappingTypeName, Str31_size), Str31_size);
	RD(ReadFileXMatrix4(theFile, &theTMAP->tMatrix), XMatrix4_size);
	RD(UFRead(theFile, &theTMAP->textureName, Str31_size), Str31_size);
	RD(ReadFileSRect(theFile, &theTMAP->textureRect), Rect_size);
	RD(ReadFileXCoordinate(theFile, &theTMAP->offset), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theTMAP->scale), XCoordinate_size);
	RD(UFReadEndian(theFile, &theTMAP->textureReference, LongInt_size), LongInt_size);
	RD(UFRead(theFile, &theTMAP->textureVolName, Str31_size), Str31_size);
	RD(READFILEEXTENDED12(theFile, &theTMAP->bumpScale), Extended_size);
	RD(ReadFileXMatrix4(theFile, &theTMAP->bumpMatrix), XMatrix4_size);
	RD(UFReadEndian(theFile, &theTMAP->textureFrame, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theTMAP->textureTime, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theTMAP->textureDensity, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theTMAP->textureCutoff, Integer_size), Integer_size);
	RD(ReadFileXCoordinate(theFile, &theTMAP->backOffset), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theTMAP->backScale), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theTMAP->leftOffset), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theTMAP->leftScale), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theTMAP->rightOffset), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theTMAP->rightScale), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theTMAP->topOffset), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theTMAP->topScale), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theTMAP->bottomOffset), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theTMAP->bottomScale), XCoordinate_size);
	if (theSize > 0)
		UFSetPos(theFile, UFGetPos(theFile) + theSize);
}  /* ReadFileTMAP */

void WriteTMAP(UFFile theFile, TMAPPtr theTMAP, IFFType theType)
{
	ulong theSize = TMAP_size;
	
	WriteSmallBlockForward(theFile, &theType, Type_size);
	WriteSmallBlock(theFile, &theSize, LongInt_size);
	WriteSmallBlock(theFile, &theTMAP->textureFlags, Integer_size);
	WriteSmallBlock(theFile, &theTMAP->textureType, Integer_size);
	WriteSmallBlockForward(theFile, &theTMAP->textureTypeName, Str31_size);
	WriteSmallBlock(theFile, &theTMAP->mappingType, Integer_size);
	WriteSmallBlockForward(theFile, &theTMAP->mappingTypeName, Str31_size);
	WriteXMatrix4(theFile, &theTMAP->tMatrix);
	WriteSmallBlockForward(theFile, &theTMAP->textureName, Str31_size);
	WriteSRect(theFile, &theTMAP->textureRect);
	WriteXCoordinate(theFile, &theTMAP->offset);
	WriteXCoordinate(theFile, &theTMAP->scale);
	WriteSmallBlock(theFile, &theTMAP->textureReference, LongInt_size);
	WriteSmallBlockForward(theFile, &theTMAP->textureVolName, Str31_size);
	WRITEEXTENDED12(theFile, &theTMAP->bumpScale);
	WriteXMatrix4(theFile, &theTMAP->bumpMatrix);
	WriteSmallBlock(theFile, &theTMAP->textureFrame, LongInt_size);
	WriteSmallBlock(theFile, &theTMAP->textureTime, Real_size);
	WriteSmallBlock(theFile, &theTMAP->textureDensity, Real_size);
	WriteSmallBlock(theFile, &theTMAP->textureCutoff, Integer_size);
	WriteXCoordinate(theFile, &theTMAP->backOffset);
	WriteXCoordinate(theFile, &theTMAP->backScale);
	WriteXCoordinate(theFile, &theTMAP->leftOffset);
	WriteXCoordinate(theFile, &theTMAP->leftScale);
	WriteXCoordinate(theFile, &theTMAP->rightOffset);
	WriteXCoordinate(theFile, &theTMAP->rightScale);
	WriteXCoordinate(theFile, &theTMAP->topOffset);
	WriteXCoordinate(theFile, &theTMAP->topScale);
	WriteXCoordinate(theFile, &theTMAP->bottomOffset);
	WriteXCoordinate(theFile, &theTMAP->bottomScale);
}  /* WriteTMAP */

void WriteFileTMAP(UFFile theFile, TMAPPtr theTMAP, IFFType theType)
{
	ulong theSize = TMAP_size;
	
	UFWrite(theFile, &theType, Type_size);
	UFWriteEndian(theFile, &theSize, LongInt_size);
	UFWriteEndian(theFile, &theTMAP->textureFlags, Integer_size);
	UFWriteEndian(theFile, &theTMAP->textureType, Integer_size);
	UFWrite(theFile, &theTMAP->textureTypeName, Str31_size);
	UFWriteEndian(theFile, &theTMAP->mappingType, Integer_size);
	UFWrite(theFile, &theTMAP->mappingTypeName, Str31_size);
	WriteFileXMatrix4(theFile, &theTMAP->tMatrix);
	UFWrite(theFile, &theTMAP->textureName, Str31_size);
	WriteFileSRect(theFile, &theTMAP->textureRect);
	WriteFileXCoordinate(theFile, &theTMAP->offset);
	WriteFileXCoordinate(theFile, &theTMAP->scale);
	UFWriteEndian(theFile, &theTMAP->textureReference, LongInt_size);
	UFWrite(theFile, &theTMAP->textureVolName, Str31_size);
	WRITEFILEEXTENDED12(theFile, &theTMAP->bumpScale);
	WriteFileXMatrix4(theFile, &theTMAP->bumpMatrix);
	UFWriteEndian(theFile, &theTMAP->textureFrame, LongInt_size);
	UFWriteEndian(theFile, &theTMAP->textureTime, Real_size);
	UFWriteEndian(theFile, &theTMAP->textureDensity, Real_size);
	UFWriteEndian(theFile, &theTMAP->textureCutoff, Integer_size);
	WriteFileXCoordinate(theFile, &theTMAP->backOffset);
	WriteFileXCoordinate(theFile, &theTMAP->backScale);
	WriteFileXCoordinate(theFile, &theTMAP->leftOffset);
	WriteFileXCoordinate(theFile, &theTMAP->leftScale);
	WriteFileXCoordinate(theFile, &theTMAP->rightOffset);
	WriteFileXCoordinate(theFile, &theTMAP->rightScale);
	WriteFileXCoordinate(theFile, &theTMAP->topOffset);
	WriteFileXCoordinate(theFile, &theTMAP->topScale);
	WriteFileXCoordinate(theFile, &theTMAP->bottomOffset);
	WriteFileXCoordinate(theFile, &theTMAP->bottomScale);
}  /* WriteFileTMAP */

void PackTMAP(char **thePtr, TMAPPtr theTMAP, IFFType theType)
{
	ulong theSize = TMAP_size;
	
	PackBlockForward(thePtr, &theType, Type_size);
	PackBlock(thePtr, &theSize, LongInt_size);
	PackBlock(thePtr, &theTMAP->textureFlags, Integer_size);
	PackBlock(thePtr, &theTMAP->textureType, Integer_size);
	PackBlockForward(thePtr, &theTMAP->textureTypeName, Str31_size);
	PackBlock(thePtr, &theTMAP->mappingType, Integer_size);
	PackBlockForward(thePtr, &theTMAP->mappingTypeName, Str31_size);
	PackXMatrix4(thePtr, &theTMAP->tMatrix);
	PackBlockForward(thePtr, &theTMAP->textureName, Str31_size);
	PackSRect(thePtr, &theTMAP->textureRect);
	PackXCoordinate(thePtr, &theTMAP->offset);
	PackXCoordinate(thePtr, &theTMAP->scale);
	PackBlock(thePtr, &theTMAP->textureReference, LongInt_size);
	PackBlockForward(thePtr, &theTMAP->textureVolName, Str31_size);
	PACKEXTENDED12(thePtr, &theTMAP->bumpScale);
	PackXMatrix4(thePtr, &theTMAP->bumpMatrix);
	PackBlock(thePtr, &theTMAP->textureFrame, LongInt_size);
	PackBlock(thePtr, &theTMAP->textureTime, Real_size);
	PackBlock(thePtr, &theTMAP->textureDensity, Real_size);
	PackBlock(thePtr, &theTMAP->textureCutoff, Integer_size);
	PackXCoordinate(thePtr, &theTMAP->backOffset);
	PackXCoordinate(thePtr, &theTMAP->backScale);
	PackXCoordinate(thePtr, &theTMAP->leftOffset);
	PackXCoordinate(thePtr, &theTMAP->leftScale);
	PackXCoordinate(thePtr, &theTMAP->rightOffset);
	PackXCoordinate(thePtr, &theTMAP->rightScale);
	PackXCoordinate(thePtr, &theTMAP->topOffset);
	PackXCoordinate(thePtr, &theTMAP->topScale);
	PackXCoordinate(thePtr, &theTMAP->bottomOffset);
	PackXCoordinate(thePtr, &theTMAP->bottomScale);
}  /* PackTMAP */

void UnpackTMAP(char **thePtr, void *theData, ulong theSize)
{
	TMAPPtr theTMAP = (TMAPPtr)theData;
	boolean theFlag = 1;
	
	RD(UnpackBlock(thePtr, &theTMAP->textureFlags, Integer_size), Integer_size);
	RD(UnpackBlock(thePtr, &theTMAP->textureType, Integer_size), Integer_size);
	RD(UnpackBlockForward(thePtr, &theTMAP->textureTypeName, Str31_size), Str31_size);
	RD(UnpackBlock(thePtr, &theTMAP->mappingType, Integer_size), Integer_size);
	RD(UnpackBlockForward(thePtr, &theTMAP->mappingTypeName, Str31_size), Str31_size);
	RD(UnpackXMatrix4(thePtr, &theTMAP->tMatrix), XMatrix4_size);
	RD(UnpackBlockForward(thePtr, &theTMAP->textureName, Str31_size), Str31_size);
	RD(UnpackSRect(thePtr, &theTMAP->textureRect), Rect_size);
	RD(UnpackXCoordinate(thePtr, &theTMAP->offset), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theTMAP->scale), XCoordinate_size);
	RD(UnpackBlock(thePtr, &theTMAP->textureReference, LongInt_size), LongInt_size);
	RD(UnpackBlockForward(thePtr, &theTMAP->textureVolName, Str31_size), Str31_size);
	RD(UNPACKEXTENDED12(thePtr, &theTMAP->bumpScale), Extended_size);
	RD(UnpackXMatrix4(thePtr, &theTMAP->bumpMatrix), XMatrix4_size);
	RD(UnpackBlock(thePtr, &theTMAP->textureFrame, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theTMAP->textureTime, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theTMAP->textureDensity, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theTMAP->textureCutoff, Integer_size), Integer_size);
	RD(UnpackXCoordinate(thePtr, &theTMAP->backOffset), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theTMAP->backScale), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theTMAP->leftOffset), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theTMAP->leftScale), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theTMAP->rightOffset), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theTMAP->rightScale), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theTMAP->topOffset), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theTMAP->topScale), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theTMAP->bottomOffset), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theTMAP->bottomScale), XCoordinate_size);
	*thePtr += theSize;
}  /* UnpackTMAP */

void ReadTMEI(UFFile theFile, void* theData, ulong theSize)
{
	TMEIPtr theTMEI = (TMEIPtr)theData;
	boolean theFlag = 1;
	
	RD(ReadSmallBlock(theFile, &theTMEI->texFlags, LongInt_size), LongInt_size);
	RD(ReadSRect(theFile, &theTMEI->fullRect), Rect_size);
	RD(ReadXCoordinate(theFile, &theTMEI->position), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theTMEI->offset), XCoordinate_size);
	RD(ReadXCoordinate(theFile, &theTMEI->rotation), XCoordinate_size);
	RD(ReadSmallBlock(theFile, &theTMEI->textureVolumeIndex, Integer_size), Integer_size);
	RD(ReadSmallBlock(theFile, &theTMEI->textureDirID, LongInt_size), LongInt_size);
	RD(READEXTENDED12(theFile, &theTMEI->u.Spherical.sphereRadius), Extended_size);
	RD(READEXTENDED12(theFile, &theTMEI->u.Spherical.sphereWrapAngle), Extended_size);
	RD(READEXTENDED12(theFile, &theTMEI->u.Spherical.sphereBandAngle), Extended_size);
	RD(READEXTENDED12(theFile, &theTMEI->u.Spherical.sphereBandAngleStart), Extended_size);
	if (theSize > 0)
		SkipBytes(theFile, theSize);
}  /* ReadTMEI */

void ReadFileTMEI(UFFile theFile, void* theData, ulong theSize)
{
	TMEIPtr theTMEI = (TMEIPtr)theData;
	boolean theFlag = 1;
	
	RD(UFReadEndian(theFile, &theTMEI->texFlags, LongInt_size), LongInt_size);
	RD(ReadFileSRect(theFile, &theTMEI->fullRect), Rect_size);
	RD(ReadFileXCoordinate(theFile, &theTMEI->position), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theTMEI->offset), XCoordinate_size);
	RD(ReadFileXCoordinate(theFile, &theTMEI->rotation), XCoordinate_size);
	RD(UFReadEndian(theFile, &theTMEI->textureVolumeIndex, Integer_size), Integer_size);
	RD(UFReadEndian(theFile, &theTMEI->textureDirID, LongInt_size), LongInt_size);
	RD(READFILEEXTENDED12(theFile, &theTMEI->u.Spherical.sphereRadius), Extended_size);
	RD(READFILEEXTENDED12(theFile, &theTMEI->u.Spherical.sphereWrapAngle), Extended_size);
	RD(READFILEEXTENDED12(theFile, &theTMEI->u.Spherical.sphereBandAngle), Extended_size);
	RD(READFILEEXTENDED12(theFile, &theTMEI->u.Spherical.sphereBandAngleStart), Extended_size);
	if (theSize > 0)
		UFSetPos(theFile, UFGetPos(theFile) + theSize);
}  /* ReadFileTMEI */

void WriteTMEI(UFFile theFile, TMEIPtr theTMEI, IFFType theType)
{
	ulong theSize = TMEI_size;
	
	WriteSmallBlockForward(theFile, &theType, Type_size);
	WriteSmallBlock(theFile, &theSize, LongInt_size);
	WriteSmallBlock(theFile, &theTMEI->texFlags, LongInt_size);
	WriteSRect(theFile, &theTMEI->fullRect);
	WriteXCoordinate(theFile, &theTMEI->position);
	WriteXCoordinate(theFile, &theTMEI->offset);
	WriteXCoordinate(theFile, &theTMEI->rotation);
	WriteSmallBlock(theFile, &theTMEI->textureVolumeIndex, Integer_size);
	WriteSmallBlock(theFile, &theTMEI->textureDirID, LongInt_size);
	WRITEEXTENDED12(theFile, &theTMEI->u.Spherical.sphereRadius);
	WRITEEXTENDED12(theFile, &theTMEI->u.Spherical.sphereWrapAngle);
	WRITEEXTENDED12(theFile, &theTMEI->u.Spherical.sphereBandAngle);
	WRITEEXTENDED12(theFile, &theTMEI->u.Spherical.sphereBandAngleStart);
}  /* WriteTMEI */

void WriteFileTMEI(UFFile theFile, TMEIPtr theTMEI, IFFType theType)
{
	ulong theSize = TMEI_size;
	
	UFWrite(theFile, &theType, Type_size);
	UFWriteEndian(theFile, &theSize, LongInt_size);
	UFWriteEndian(theFile, &theTMEI->texFlags, LongInt_size);
	WriteFileSRect(theFile, &theTMEI->fullRect);
	WriteFileXCoordinate(theFile, &theTMEI->position);
	WriteFileXCoordinate(theFile, &theTMEI->offset);
	WriteFileXCoordinate(theFile, &theTMEI->rotation);
	UFWriteEndian(theFile, &theTMEI->textureVolumeIndex, Integer_size);
	UFWriteEndian(theFile, &theTMEI->textureDirID, LongInt_size);
	WRITEFILEEXTENDED12(theFile, &theTMEI->u.Spherical.sphereRadius);
	WRITEFILEEXTENDED12(theFile, &theTMEI->u.Spherical.sphereWrapAngle);
	WRITEFILEEXTENDED12(theFile, &theTMEI->u.Spherical.sphereBandAngle);
	WRITEFILEEXTENDED12(theFile, &theTMEI->u.Spherical.sphereBandAngleStart);
}  /* WriteFileTMEI */

void PackTMEI(char **thePtr, TMEIPtr theTMEI, IFFType theType)
{
	ulong theSize = TMEI_size;
	
	PackBlockForward(thePtr, &theType, Type_size);
	PackBlock(thePtr, &theSize, LongInt_size);
	PackBlock(thePtr, &theTMEI->texFlags, LongInt_size);
	PackSRect(thePtr, &theTMEI->fullRect);
	PackXCoordinate(thePtr, &theTMEI->position);
	PackXCoordinate(thePtr, &theTMEI->offset);
	PackXCoordinate(thePtr, &theTMEI->rotation);
	PackBlock(thePtr, &theTMEI->textureVolumeIndex, Integer_size);
	PackBlock(thePtr, &theTMEI->textureDirID, LongInt_size);
	PACKEXTENDED12(thePtr, &theTMEI->u.Spherical.sphereRadius);
	PACKEXTENDED12(thePtr, &theTMEI->u.Spherical.sphereWrapAngle);
	PACKEXTENDED12(thePtr, &theTMEI->u.Spherical.sphereBandAngle);
	PACKEXTENDED12(thePtr, &theTMEI->u.Spherical.sphereBandAngleStart);
}  /* PackTMEI */

void UnpackTMEI(char **thePtr, void* theData, ulong theSize)
{
	TMEIPtr theTMEI = (TMEIPtr)theData;
	boolean theFlag = 1;
	
	RD(UnpackBlock(thePtr, &theTMEI->texFlags, LongInt_size), LongInt_size);
	RD(UnpackSRect(thePtr, &theTMEI->fullRect), Rect_size);
	RD(UnpackXCoordinate(thePtr, &theTMEI->position), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theTMEI->offset), XCoordinate_size);
	RD(UnpackXCoordinate(thePtr, &theTMEI->rotation), XCoordinate_size);
	RD(UnpackBlock(thePtr, &theTMEI->textureVolumeIndex, Integer_size), Integer_size);
	RD(UnpackBlock(thePtr, &theTMEI->textureDirID, LongInt_size), LongInt_size);
	RD(UNPACKEXTENDED12(thePtr, &theTMEI->u.Spherical.sphereRadius), Extended_size);
	RD(UNPACKEXTENDED12(thePtr, &theTMEI->u.Spherical.sphereWrapAngle), Extended_size);
	RD(UNPACKEXTENDED12(thePtr, &theTMEI->u.Spherical.sphereBandAngle), Extended_size);
	RD(UNPACKEXTENDED12(thePtr, &theTMEI->u.Spherical.sphereBandAngleStart), Extended_size);
	*thePtr += theSize;
}  /* UnpackTMEI */

void ReadGIMN(UFFile theFile, void* theData, ulong theSize)
{
	GIMNPtr theGIMN = (GIMNPtr)theData;
	boolean theFlag = 1;
	
	RD(ReadSmallBlock(theFile, &theGIMN->minFlags, LongInt_size), LongInt_size);
	RD(ReadMatrix4(theFile, &theGIMN->minCoordinateAbsMatrix), Matrix4_size);
	RD(ReadMatrix3(theFile, &theGIMN->minNormalAbsMatrix), Matrix3_size);
	RD(ReadSmallBlock(theFile, &theGIMN->minChildCount, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theGIMN->minChildTotal, LongInt_size), LongInt_size);
	RD(ReadMatrix4(theFile, &theGIMN->minCoordinateRelMatrix), Matrix4_size);
	RD(ReadMatrix4(theFile, &theGIMN->minNormalRelMatrix), Matrix4_size);
	if (theSize > 0)
		SkipBytes(theFile, theSize);
}  /* ReadGIMN */

void ReadFileGIMN(UFFile theFile, void* theData, ulong theSize)
{
	GIMNPtr theGIMN = (GIMNPtr)theData;
	boolean theFlag = 1;
	
	RD(UFReadEndian(theFile, &theGIMN->minFlags, LongInt_size), LongInt_size);
	RD(ReadFileMatrix4(theFile, &theGIMN->minCoordinateAbsMatrix), Matrix4_size);
	RD(ReadFileMatrix3(theFile, &theGIMN->minNormalAbsMatrix), Matrix3_size);
	RD(UFReadEndian(theFile, &theGIMN->minChildCount, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theGIMN->minChildTotal, LongInt_size), LongInt_size);
	RD(ReadFileMatrix4(theFile, &theGIMN->minCoordinateRelMatrix), Matrix4_size);
	RD(ReadFileMatrix4(theFile, &theGIMN->minNormalRelMatrix), Matrix4_size);
	if (theSize > 0)
		UFSetPos(theFile, UFGetPos(theFile) + theSize);
}  /* ReadFileGIMN */

void UnpackGIMN(char **thePtr, void* theData, ulong theSize)
{
	GIMNPtr theGIMN = (GIMNPtr)theData;
	boolean theFlag = 1;
	
	RD(UnpackBlock(thePtr, &theGIMN->minFlags, LongInt_size), LongInt_size);
	RD(UnpackMatrix4(thePtr, &theGIMN->minCoordinateAbsMatrix), Matrix4_size);
	RD(UnpackMatrix3(thePtr, &theGIMN->minNormalAbsMatrix), Matrix3_size);
	RD(UnpackBlock(thePtr, &theGIMN->minChildCount, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theGIMN->minChildTotal, LongInt_size), LongInt_size);
	RD(UnpackMatrix4(thePtr, &theGIMN->minCoordinateRelMatrix), Matrix4_size);
	RD(UnpackMatrix4(thePtr, &theGIMN->minNormalRelMatrix), Matrix4_size);
	*thePtr += theSize;
}  /* UnpackGIMN */

void ReadGMN2(UFFile theFile, void* theData, UFSize theSize)
{
	GMN2Ptr theGMN2 = (GMN2Ptr)theData;
	boolean theFlag = 1;
	
	RD(ReadMatrix4(theFile, &theGMN2->coordinateAbsMatrix), Matrix4_size);
	RD(ReadExtent(theFile, &theGMN2->extents), Extent_size);
	RD(ReadSmallBlock(theFile, &theGMN2->flags, LongInt_size), LongInt_size);
	RD(ReadSmallBlockForward(theFile, &theGMN2->name, Str31_size), Str31_size);
	RD(ReadSmallBlock(theFile, &theGMN2->childCount, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theGMN2->childTotal, LongInt_size), LongInt_size);
	if (theSize > 0)
		SkipBytes(theFile, theSize);
}  /* ReadGMN2 */

void ReadFileGMN2(UFFile theFile, void* theData, UFSize theSize)
{
	GMN2Ptr theGMN2 = (GMN2Ptr)theData;
	boolean theFlag = 1;
	
	RD(ReadFileMatrix4(theFile, &theGMN2->coordinateAbsMatrix), Matrix4_size);
	RD(ReadFileExtent(theFile, &theGMN2->extents), Extent_size);
	RD(UFReadEndian(theFile, &theGMN2->flags, LongInt_size), LongInt_size);
	RD(UFRead(theFile, &theGMN2->name, Str31_size), Str31_size);
	RD(UFReadEndian(theFile, &theGMN2->childCount, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theGMN2->childTotal, LongInt_size), LongInt_size);
	if (theSize > 0)
		UFSetPos(theFile, UFGetPos(theFile) + theSize);
}  /* ReadFileGMN2 */

void UnpackGMN2(char **thePtr, void* theData, ulong theSize)
{
	GMN2Ptr theGMN2 = (GMN2Ptr)theData;
	boolean theFlag = 1;
	
	RD(UnpackMatrix4(thePtr, &theGMN2->coordinateAbsMatrix), Matrix4_size);
	RD(UnpackExtent(thePtr, &theGMN2->extents), Extent_size);
	RD(UnpackBlock(thePtr, &theGMN2->flags, LongInt_size), LongInt_size);
	RD(UnpackBlockForward(thePtr, &theGMN2->name, Str31_size), Str31_size);
	RD(UnpackBlock(thePtr, &theGMN2->childCount, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theGMN2->childTotal, LongInt_size), LongInt_size);
	*thePtr += theSize;
}  /* UnpackGMN2 */

void ReadGBMN(UFFile theFile, void* theData, UFSize theSize)
{
	GBMNPtr theGBMN = (GBMNPtr)theData;
	boolean theFlag = 1;
	
	RD(ReadSmallBlock(theFile, &theGBMN->minBlurFlags, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theGBMN->minBlurIntensity, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theGBMN->minBlurLayer, LongInt_size), LongInt_size);
	RD(ReadMatrix4(theFile, &theGBMN->minBlurMatrix), Matrix4_size);
	if (theSize > 0)
		SkipBytes(theFile, theSize);
}  /* ReadGBMN */

void ReadFileGBMN(UFFile theFile, void* theData, UFSize theSize)
{
	GBMNPtr theGBMN = (GBMNPtr)theData;
	boolean theFlag = 1;
	
	RD(UFReadEndian(theFile, &theGBMN->minBlurFlags, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theGBMN->minBlurIntensity, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theGBMN->minBlurLayer, LongInt_size), LongInt_size);
	RD(ReadFileMatrix4(theFile, &theGBMN->minBlurMatrix), Matrix4_size);
	if (theSize > 0)
		UFSetPos(theFile, UFGetPos(theFile) + theSize);
}  /* ReadFileGBMN */

void UnpackGBMN(char **thePtr, void* theData, ulong theSize)
{
	GBMNPtr theGBMN = (GBMNPtr)theData;
	boolean theFlag = 1;
	
	RD(UnpackBlock(thePtr, &theGBMN->minBlurFlags, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theGBMN->minBlurIntensity, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theGBMN->minBlurLayer, LongInt_size), LongInt_size);
	RD(UnpackMatrix4(thePtr, &theGBMN->minBlurMatrix), Matrix4_size);
	*thePtr += theSize;
}  /* UnpackGBMN */

void ReadTMMN(UFFile theFile, void* theData, UFSize theSize)
{
	TMMNPtr theTMMN = (TMMNPtr)theData;
	boolean theFlag = 1;
	
	RD(ReadSmallBlock(theFile, &theTMMN->minTextureFlags, Integer_size), Integer_size);
	RD(ReadSmallBlock(theFile, &theTMMN->minTextureType, Integer_size), Integer_size);
	RD(ReadSmallBlock(theFile, &theTMMN->minMappingType, Integer_size), Integer_size);
	RD(ReadMatrix4(theFile, &theTMMN->minTMatrix), Matrix4_size);
	RD(ReadSRect(theFile, &theTMMN->minTextureRect), Rect_size);
	RD(ReadCoordinate(theFile, &theTMMN->minOffset), Coordinate_size);
	RD(ReadCoordinate(theFile, &theTMMN->minScale), Coordinate_size);
	RD(ReadSmallBlock(theFile, &theTMMN->minTextureReference, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theTMMN->minBumpScale, Real_size), Real_size);
	RD(ReadMatrix3(theFile, &theTMMN->minBumpMatrix), Matrix3_size);
	RD(ReadSmallBlock(theFile, &theTMMN->minTextureFrame, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theTMMN->minTextureDensity, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theTMMN->minTextureCutoff, Integer_size), Integer_size);
	if (theSize > 0)
		SkipBytes(theFile, theSize);
}  /* ReadTMMN */

void ReadFileTMMN(UFFile theFile, void* theData, UFSize theSize)
{
	TMMNPtr theTMMN = (TMMNPtr)theData;
	boolean theFlag = 1;
	
	RD(UFReadEndian(theFile, &theTMMN->minTextureFlags, Integer_size), Integer_size);
	RD(UFReadEndian(theFile, &theTMMN->minTextureType, Integer_size), Integer_size);
	RD(UFReadEndian(theFile, &theTMMN->minMappingType, Integer_size), Integer_size);
	RD(ReadFileMatrix4(theFile, &theTMMN->minTMatrix), Matrix4_size);
	RD(ReadFileSRect(theFile, &theTMMN->minTextureRect), Rect_size);
	RD(ReadFileCoordinate(theFile, &theTMMN->minOffset), Coordinate_size);
	RD(ReadFileCoordinate(theFile, &theTMMN->minScale), Coordinate_size);
	RD(UFReadEndian(theFile, &theTMMN->minTextureReference, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theTMMN->minBumpScale, Real_size), Real_size);
	RD(ReadFileMatrix3(theFile, &theTMMN->minBumpMatrix), Matrix3_size);
	RD(UFReadEndian(theFile, &theTMMN->minTextureFrame, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theTMMN->minTextureDensity, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theTMMN->minTextureCutoff, Integer_size), Integer_size);
	if (theSize > 0)
		UFSetPos(theFile, UFGetPos(theFile) + theSize);
}  /* ReadFileTMMN */

void UnpackTMMN(char **thePtr, void* theData, ulong theSize)
{
	TMMNPtr theTMMN = (TMMNPtr)theData;
	boolean theFlag = 1;
	
	RD(UnpackBlock(thePtr, &theTMMN->minTextureFlags, Integer_size), Integer_size);
	RD(UnpackBlock(thePtr, &theTMMN->minTextureType, Integer_size), Integer_size);
	RD(UnpackBlock(thePtr, &theTMMN->minMappingType, Integer_size), Integer_size);
	RD(UnpackMatrix4(thePtr, &theTMMN->minTMatrix), Matrix4_size);
	RD(UnpackSRect(thePtr, &theTMMN->minTextureRect), Rect_size);
	RD(UnpackCoordinate(thePtr, &theTMMN->minOffset), Coordinate_size);
	RD(UnpackCoordinate(thePtr, &theTMMN->minScale), Coordinate_size);
	RD(UnpackBlock(thePtr, &theTMMN->minTextureReference, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theTMMN->minBumpScale, Real_size), Real_size);
	RD(UnpackMatrix3(thePtr, &theTMMN->minBumpMatrix), Matrix3_size);
	RD(UnpackBlock(thePtr, &theTMMN->minTextureFrame, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theTMMN->minTextureDensity, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theTMMN->minTextureCutoff, Integer_size), Integer_size);
	*thePtr += theSize;
}  /* UnpackTMMN */

void ReadSHAD(UFFile theFile, void* theData, UFSize theSize)
{
	SHADPtr theSHAD = (SHADPtr)theData;
	boolean theFlag = 1;
	
	RD(ReadRMatrix4(theFile, &theSHAD->shaderMatrix), RMatrix4_size);
	RD(ReadSmallBlock(theFile, &theSHAD->shaderOpacity, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theSHAD->shaderBumpScale, Real_size), Real_size);
	RD(ReadSmallBlock(theFile, &theSHAD->shaderFlags, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theSHAD->shaderReference, LongInt_size), LongInt_size);
	RD(ReadSmallBlock(theFile, &theSHAD->shaderInterfaceSize, LongInt_size), LongInt_size);
	theSHAD->shaderInterface = (char*)AllocateMemoryBlock(theSHAD->shaderInterfaceSize);
	if (theSHAD->shaderInterface && theSHAD->shaderInterfaceSize && (theSHAD->shaderInterfaceSize <= theSize))
		RD(ReadSmallBlockForward(theFile, theSHAD->shaderInterface, theSHAD->shaderInterfaceSize), theSHAD->shaderInterfaceSize);
	if (theSize > 0)
		SkipBytes(theFile, theSize);
}  /* ReadSHAD */

void ReadFileSHAD(UFFile theFile, void* theData, UFSize theSize)
{
	SHADPtr theSHAD = (SHADPtr)theData;
	boolean theFlag = 1;
	
	RD(ReadFileRMatrix4(theFile, &theSHAD->shaderMatrix), RMatrix4_size);
	RD(UFReadEndian(theFile, &theSHAD->shaderOpacity, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theSHAD->shaderBumpScale, Real_size), Real_size);
	RD(UFReadEndian(theFile, &theSHAD->shaderFlags, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theSHAD->shaderReference, LongInt_size), LongInt_size);
	RD(UFReadEndian(theFile, &theSHAD->shaderInterfaceSize, LongInt_size), LongInt_size);
	theSHAD->shaderInterface = (char*)AllocateMemoryBlock(theSHAD->shaderInterfaceSize);
	if (theSHAD->shaderInterface && theSHAD->shaderInterfaceSize && (theSHAD->shaderInterfaceSize <= theSize))
		RD(UFRead(theFile, theSHAD->shaderInterface, theSHAD->shaderInterfaceSize), theSHAD->shaderInterfaceSize);
	if (theSize > 0)
		UFSetPos(theFile, UFGetPos(theFile) + theSize);
}  /* ReadFileSHAD */

void WriteSHAD(UFFile theFile, SHADPtr theSHAD)
{
	ulong theSize = SHAD_size + theSHAD->shaderInterfaceSize;

	WriteSmallBlockForward(theFile, (void*)"SHAD", Type_size);
	WriteSmallBlock(theFile, &theSize, LongInt_size);
	WriteRMatrix4(theFile, &theSHAD->shaderMatrix);
	WriteSmallBlock(theFile, &theSHAD->shaderOpacity, Real_size);
	WriteSmallBlock(theFile, &theSHAD->shaderBumpScale, Real_size);
	WriteSmallBlock(theFile, &theSHAD->shaderFlags, LongInt_size);
	WriteSmallBlock(theFile, &theSHAD->shaderReference, LongInt_size);
	WriteSmallBlock(theFile, &theSHAD->shaderInterfaceSize, LongInt_size);
	if (theSHAD->shaderInterfaceSize)
		WriteSmallBlockForward(theFile, theSHAD->shaderInterface, theSHAD->shaderInterfaceSize);
}  /* WriteSHAD */

void WriteFileSHAD(UFFile theFile, SHADPtr theSHAD)
{
	ulong theSize = SHAD_size + theSHAD->shaderInterfaceSize;
	
	UFWrite(theFile, (void*)"SHAD", Type_size);
	UFWriteEndian(theFile, &theSize, LongInt_size);
	WriteFileRMatrix4(theFile, &theSHAD->shaderMatrix);
	UFWriteEndian(theFile, &theSHAD->shaderOpacity, Real_size);
	UFWriteEndian(theFile, &theSHAD->shaderBumpScale, Real_size);
	UFWriteEndian(theFile, &theSHAD->shaderFlags, LongInt_size);
	UFWriteEndian(theFile, &theSHAD->shaderReference, LongInt_size);
	UFWriteEndian(theFile, &theSHAD->shaderInterfaceSize, LongInt_size);
	if (theSHAD->shaderInterfaceSize)
		UFWrite(theFile, theSHAD->shaderInterface, theSHAD->shaderInterfaceSize);
}  /* WriteFileSHAD */

void PackSHAD(char **thePtr, SHADPtr theSHAD)
{
	ulong theSize = SHAD_size + theSHAD->shaderInterfaceSize;
	
	PackBlockForward(thePtr, (void*)"SHAD", Type_size);
	PackBlock(thePtr, &theSize, LongInt_size);
	PackRMatrix4(thePtr, &theSHAD->shaderMatrix);
	PackBlock(thePtr, &theSHAD->shaderOpacity, Real_size);
	PackBlock(thePtr, &theSHAD->shaderBumpScale, Real_size);
	PackBlock(thePtr, &theSHAD->shaderFlags, LongInt_size);
	PackBlock(thePtr, &theSHAD->shaderReference, LongInt_size);
	PackBlock(thePtr, &theSHAD->shaderInterfaceSize, LongInt_size);
	if (theSHAD->shaderInterfaceSize)
		PackBlockForward(thePtr, theSHAD->shaderInterface, theSHAD->shaderInterfaceSize);
}  /* PackSHAD */

void UnpackSHAD(char **thePtr, void* theData, ulong theSize)
{
	SHADPtr theSHAD = (SHADPtr)theData;
	boolean theFlag = 1;
	
	RD(UnpackRMatrix4(thePtr, &theSHAD->shaderMatrix), RMatrix4_size);
	RD(UnpackBlock(thePtr, &theSHAD->shaderOpacity, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theSHAD->shaderBumpScale, Real_size), Real_size);
	RD(UnpackBlock(thePtr, &theSHAD->shaderFlags, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theSHAD->shaderReference, LongInt_size), LongInt_size);
	RD(UnpackBlock(thePtr, &theSHAD->shaderInterfaceSize, LongInt_size), LongInt_size);
	theSHAD->shaderInterface = (char*)AllocateMemoryBlock(theSHAD->shaderInterfaceSize);
	if (theSHAD->shaderInterface && theSHAD->shaderInterfaceSize && (theSHAD->shaderInterfaceSize <= theSize))
		RD(UnpackBlockForward(thePtr, theSHAD->shaderInterface, theSHAD->shaderInterfaceSize), theSHAD->shaderInterfaceSize);
	*thePtr += theSize;
}  /* UnpackSHAD */

void ReadGLEX(UFFile theFile, void* theData, UFSize theSize)
{
	GLEXPtr theGLEX = (GLEXPtr)theData;
	
	theGLEX->lightListSize = (unsigned long) theSize;/*Cast to limit light list to 2GB*/
	if (theGLEX->lightListSize > 0)
		theGLEX->lightList = (char*)AllocateMemoryBlock(theGLEX->lightListSize);
	else
		theGLEX->lightList = nil;
	if (theGLEX->lightList)
		ReadSmallBlockForward(theFile, theGLEX->lightList, theGLEX->lightListSize);
}  /* ReadGLEX */

void ReadFileGLEX(UFFile theFile, void* theData, UFSize theSize)
{
	GLEXPtr theGLEX = (GLEXPtr)theData;
	
	theGLEX->lightListSize = (unsigned long) theSize; /*Cast to limit light list to 2GB*/
	if (theGLEX->lightListSize > 0)
		theGLEX->lightList = (char*)AllocateMemoryBlock(theGLEX->lightListSize);
	else
		theGLEX->lightList = nil;
	if (theGLEX->lightList)
		UFRead(theFile, theGLEX->lightList, theGLEX->lightListSize);
}  /* ReadFileGLEX */

void WriteGLEX(UFFile theFile, GLEXPtr theGLEX)
{
	ulong theSize = ALIGN(theGLEX->lightListSize, 4);
	
	WriteSmallBlockForward(theFile, (void*)"GLEX", Type_size);
	WriteSmallBlock(theFile, &theSize, LongInt_size);
	if (theSize)
		WriteSmallBlockForward(theFile, theGLEX->lightList, theSize);
}  /* WriteGLEX */

void WriteFileGLEX(UFFile theFile, GLEXPtr theGLEX)
{
	ulong theSize = ALIGN(theGLEX->lightListSize, 4);
	
	UFWrite(theFile, (void*)"GLEX", Type_size);
	UFWriteEndian(theFile, &theSize, LongInt_size);
	if (theSize)
		UFWrite(theFile, theGLEX->lightList, theSize);
}  /* WriteFileGLEX */

void PackGLEX(char **thePtr, GLEXPtr theGLEX)
{
	ulong theSize = ALIGN(theGLEX->lightListSize, 4);
	
	PackBlockForward(thePtr, (void*)"GLEX", Type_size);
	PackBlock(thePtr, &theSize, LongInt_size);
	if (theSize)
		PackBlockForward(thePtr, theGLEX->lightList, theSize);
}  /* PackGLEX */

void UnpackGLEX(char **thePtr, void* theData, ulong theSize)
{
	GLEXPtr theGLEX = (GLEXPtr)theData;
	
	theGLEX->lightListSize = theSize;
	if (theGLEX->lightListSize > 0)
		theGLEX->lightList = (char*)AllocateMemoryBlock(theGLEX->lightListSize);
	else
		theGLEX->lightList = nil;
	if (theGLEX->lightList)
		UnpackBlockForward(thePtr, theGLEX->lightList, theGLEX->lightListSize);
}  /* UnpackGLEX */

void ReadMORF(UFFile theFile, void* theData, UFSize theSize)
{
	MORFPtr theMORF = (MORFPtr)theData;
	boolean theFlag = 1;
	
	theMORF->morphNumTargets = (unsigned long)(theSize / MorphTarget_size); /*Cast to limit morph list to 2GB*/
	theMORF->morphTargets = (MorphTargetRec*)AllocateMemoryBlock(theMORF->morphNumTargets * sizeof(MorphTargetRec));
	if (theMORF->morphTargets) {
		MorphTargetRec *theTarget = theMORF->morphTargets;
		ulong i = theMORF->morphNumTargets;
		
		while (i--) {
			RD(ReadSmallBlock(theFile, &theTarget->targetBlendValue, Real_size), Real_size);
			RD(ReadSmallBlock(theFile, &theTarget->targetBlurBlendValue, Real_size), Real_size);
			RD(ReadSmallBlock(theFile, &theTarget->targetFileRef, LongInt_size), LongInt_size);
			RD(ReadSmallBlock(theFile, &theTarget->targetVertexOffset, LongInt_size), LongInt_size);
			RD(ReadSmallBlock(theFile, &theTarget->targetNormalOffset, LongInt_size), LongInt_size);
			theTarget++;
		}
	}
	if (theSize > 0)
		SkipBytes(theFile, theSize);
}  /* ReadMORF */

void ReadFileMORF(UFFile theFile, void* theData, UFSize theSize)
{
	MORFPtr theMORF = (MORFPtr)theData;
	boolean theFlag = 1;
	
	theMORF->morphNumTargets = (unsigned long)(theSize / MorphTarget_size); /*Cast to limit light list to 2GB*/
	theMORF->morphTargets = (MorphTargetRec*)AllocateMemoryBlock(theMORF->morphNumTargets * sizeof(MorphTargetRec));
	if (theMORF->morphTargets) {
		MorphTargetRec *theTarget = theMORF->morphTargets;
		ulong i = theMORF->morphNumTargets;
		
		while (i--) {
			RD(UFReadEndian(theFile, &theTarget->targetBlendValue, Real_size), Real_size);
			RD(UFReadEndian(theFile, &theTarget->targetBlurBlendValue, Real_size), Real_size);
			RD(UFReadEndian(theFile, &theTarget->targetFileRef, LongInt_size), LongInt_size);
			RD(UFReadEndian(theFile, &theTarget->targetVertexOffset, LongInt_size), LongInt_size);
			RD(UFReadEndian(theFile, &theTarget->targetNormalOffset, LongInt_size), LongInt_size);
			theTarget++;
		}
	}
	if (theSize > 0)
		UFSetPos(theFile, UFGetPos(theFile) + theSize);
}  /* ReadFileMORF */

void WriteMORF(UFFile theFile, MORFPtr theMORF)
{
	ulong theSize = MorphTarget_size * theMORF->morphNumTargets;
	ulong i = theMORF->morphNumTargets;
	MorphTargetRec *theTarget = theMORF->morphTargets;

	WriteSmallBlockForward(theFile, (void*)"MORF", Type_size);
	WriteSmallBlock(theFile, &theSize, LongInt_size);
	while (i--) {
		WriteSmallBlock(theFile, &theTarget->targetBlendValue, Real_size);
		WriteSmallBlock(theFile, &theTarget->targetBlurBlendValue, Real_size);
		WriteSmallBlock(theFile, &theTarget->targetFileRef, LongInt_size);
		WriteSmallBlock(theFile, &theTarget->targetVertexOffset, LongInt_size);
		WriteSmallBlock(theFile, &theTarget->targetNormalOffset, LongInt_size);
		theTarget++;
	}
}  /* WriteMORF */

void WriteFileMORF(UFFile theFile, MORFPtr theMORF)
{
	ulong theSize = MorphTarget_size * theMORF->morphNumTargets;
	ulong i = theMORF->morphNumTargets;
	MorphTargetRec *theTarget = theMORF->morphTargets;

	UFWrite(theFile, (void*)"MORF", Type_size);
	UFWriteEndian(theFile, &theSize, LongInt_size);
	while (i--) {
		UFWriteEndian(theFile, &theTarget->targetBlendValue, Real_size);
		UFWriteEndian(theFile, &theTarget->targetBlurBlendValue, Real_size);
		UFWriteEndian(theFile, &theTarget->targetFileRef, LongInt_size);
		UFWriteEndian(theFile, &theTarget->targetVertexOffset, LongInt_size);
		UFWriteEndian(theFile, &theTarget->targetNormalOffset, LongInt_size);
		theTarget++;
	}
}  /* WriteFileMORF */

void PackMORF(char **thePtr, MORFPtr theMORF)
{
	ulong theSize = MorphTarget_size * theMORF->morphNumTargets;
	ulong i = theMORF->morphNumTargets;
	MorphTargetRec *theTarget = theMORF->morphTargets;

	PackBlockForward(thePtr, (void*)"MORF", Type_size);
	PackBlock(thePtr, &theSize, LongInt_size);
	while (i--) {
		PackBlock(thePtr, &theTarget->targetBlendValue, Real_size);
		PackBlock(thePtr, &theTarget->targetBlurBlendValue, Real_size);
		PackBlock(thePtr, &theTarget->targetFileRef, LongInt_size);
		PackBlock(thePtr, &theTarget->targetVertexOffset, LongInt_size);
		PackBlock(thePtr, &theTarget->targetNormalOffset, LongInt_size);
		theTarget++;
	}
}  /* PackMORF */

void UnpackMORF(char **thePtr, void* theData, ulong theSize)
{
	MORFPtr theMORF = (MORFPtr)theData;
	boolean theFlag = 1;
	
	theMORF->morphNumTargets = theSize / MorphTarget_size;
	theMORF->morphTargets = (MorphTargetRec*)AllocateMemoryBlock(theMORF->morphNumTargets * sizeof(MorphTargetRec));
	if (theMORF->morphTargets) {
		MorphTargetRec *theTarget = theMORF->morphTargets;
		ulong i = theMORF->morphNumTargets;
		
		while (i--) {
			RD(UnpackBlock(thePtr, &theTarget->targetBlendValue, Real_size), Real_size);
			RD(UnpackBlock(thePtr, &theTarget->targetBlurBlendValue, Real_size), Real_size);
			RD(UnpackBlock(thePtr, &theTarget->targetFileRef, LongInt_size), LongInt_size);
			RD(UnpackBlock(thePtr, &theTarget->targetVertexOffset, LongInt_size), LongInt_size);
			RD(UnpackBlock(thePtr, &theTarget->targetNormalOffset, LongInt_size), LongInt_size);
			theTarget++;
		}
	}
	*thePtr += theSize;
}  /* UnpackMORF */
