/*
 *		Copyright (c) 1987-1999 Play, Inc.
 *		This code is confidential and is a trade secret of Electric Image, Inc.
 */

#ifndef __FACTIO_h__
#define __FACTIO_h__

#ifndef __FACTStuff_h__
#include "FACTStuff.h"
#endif

#ifndef __MinStuff_h__
#include "MinStuff.h"
#endif

#ifndef __UniversalIO_h__
#include "UniversalIO.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define READEXTENDED12(f, e)		ReadExtended12(f, e)
#define READFILEEXTENDED12(f, e)	ReadFileExtended12(f, e)
#define WRITEEXTENDED12(f, e)		WriteExtended12(f, e)
#define WRITEFILEEXTENDED12(f, e)	WriteFileExtended12(f, e)
#define PACKEXTENDED12(e, p)		PackExtended12(e, p)
#define UNPACKEXTENDED12(e, p)		UnpackExtended12(e, p)

#define Boolean_size 1UL
#define Byte_size 1UL
#define Integer_size 2UL
#define LongInt_size 4UL
#define Type_size 4UL
#define Real_size 4UL
#define Double_size 8UL
#define Extended_size 12UL
#define Str31_size 32UL
#define Str63_size 64UL
#define Str255_size 256UL
#define IFF_size 8UL
#define FORM_size 4UL
#define ARGB_size 4UL
#define Rect_size 8UL
#define ImageHeader_size 6UL
#define ImageFrame_size 32UL
#define Surface_size 18UL
#define Facet_size 8UL
#define MinMax_size 8UL
#define Coordinate_size 12UL
#define Extent_size 24UL
#define DCoordinate_size 24UL
#define XCoordinate_size 36UL
#define Vector3_size 24UL
#define Matrix3_size 72UL
#define Vector4_size 32UL
#define Matrix4_size 128UL
#define XVector3_size 36UL
#define XMatrix3_size 108UL
#define XVector4_size 48UL
#define XMatrix4_size 192UL
#define RVector3_size 12UL
#define RMatrix3_size 36UL
#define RVector4_size 16UL
#define RMatrix4_size 64UL
#define GINF_size 858UL
#define GLNK_size 522UL
#define GSTR_size 144UL
#define GBLR_size 204UL
#define GOUT_size 28UL
#define GATR_size 212UL       // 188UL // 152UL
#define TMAP_size 988UL
#define TMEI_size 174UL
#define GIMN_size 468UL
#define GMN2_size 196UL
#define GBMN_size 140UL
#define TMMN_size 256UL
#define SHAD_size 88UL
#define GLEX_size 0UL
#define MorphTarget_size 20UL
#define MORF_size 0UL

/* Prototype for FACT file read routines */

typedef void (*FACTIORoutine)(UFFile theFile, void* theData, UFSize theSize);
typedef void (*FACTIOFileRoutine)(UFFile theFile, void* theData, UFSize theSize);
typedef void (*FACTIOUnpackRoutine)(char **thePtr, void* theData, ulong theSize);

/* Data packing and unpacking routines */

void PackBlockForward(char **thePtr, void *theBlock, ulong theSize);
void UnpackBlockForward(char **thePtr, void *theBlock, ulong theSize);
void PackBlockBackward(char **thePtr, void *theBlock, ulong theSize);
void UnpackBlockBackward(char **thePtr, void *theBlock, ulong theSize);

#ifdef __LITTLE_ENDIAN__
#define PackBlock PackBlockBackward
#define UnpackBlock UnpackBlockBackward
#else
#define PackBlock PackBlockForward
#define UnpackBlock UnpackBlockForward
#endif

void PackIFF(char **thePtr, ulong theSize, IFFType theType);
void UnpackIFF(char **thePtr, long *theSize, IFFType *theType);
void PackIFFBlock(char **thePtr, void *theBlock, ulong theSize, IFFType theType);
void PackFORM(char **thePtr, ulong theSize, IFFType theType);
void UnpackFORM(char **thePtr, IFFType *theType);

/* These routines are designed to facilitate reading and writing of missaligned data */
/* records and records containing imbedded 12 byte long double floating point numbers */

#define READ_ARGB(inputBuffer, theARGB)		READFORWARD4(inputBuffer, theARGB)
#define READFILE_ARGB(theFile, theARGB)		ReadFileBlock(theFile, theARGB, ARGB_size)
#define WRITE_ARGB(outputBuffer, theARGB)	WRITEFORWARD4(outputBuffer, theARGB)
#define WRITEFILE_ARGB(theFile, theARGB)	WriteFileBlock(theFile, theARGB, ARGB_size)
#define PACK_ARGB(theARGB, thePtr)			PackBlockForward(theARGB, ARGB_size, thePtr)
#define UNPACK_ARGB(theARGB, thePtr)		UnpackBlockForward(theARGB, ARGB_size, thePtr)

void ReadARGB(UFFile theFile, ARGB* theARGB);
void ReadFileARGB(UFFile theFile, ARGB* theARGB);
void WriteARGB(UFFile theFile, ARGB* theARGB);
void WriteFileARGB(UFFile theFile, ARGB* theARGB);
void PackARGB(char **thePtr, ARGB* theARGB);
void UnpackARGB(char **thePtr, ARGB* theARGB);

void ReadSRect(UFFile theFile, SRect* theRect);
void ReadFileSRect(UFFile theFile, SRect* theRect);
void WriteSRect(UFFile theFile, SRect* theRect);
void WriteFileSRect(UFFile theFile, SRect* theRect);
void PackSRect(char **thePtr, SRect* theRect);
void UnpackSRect(char **thePtr, SRect* theRect);

void ReadImageHeader(UFFile theFile, ImageHeaderPtr theImageHeader);
void ReadFileImageHeader(UFFile theFile, ImageHeaderPtr theImageHeader);
void WriteImageHeader(UFFile theFile, ImageHeaderPtr theImageHeader);
void WriteFileImageHeader(UFFile theFile, ImageHeaderPtr theImageHeader);
void PackImageHeader(char **thePtr, ImageHeaderPtr theImageHeader);
void UnpackImageHeader(char **thePtr, ImageHeaderPtr theImageHeader);

void ReadImageFrame(UFFile theFile, ImageFramePtr theImageFrame);
void ReadFileImageFrame(UFFile theFile, ImageFramePtr theImageFrame);
void WriteImageFrame(UFFile theFile, ImageFramePtr theImageFrame);
void WriteFileImageFrame(UFFile theFile, ImageFramePtr theImageFrame);
void PackImageFrame(char **thePtr, ImageFramePtr theImageFrame);
void UnpackImageFrame(char **thePtr, ImageFramePtr theImageFrame);

void ReadMinMax(UFFile theFile, MinMax *theMinMax);
void ReadFileMinMax(UFFile theFile, MinMax *theMinMax);
void WriteMinMax(UFFile theFile, MinMax *theMinMax);
void WriteFileMinMax(UFFile theFile, MinMax *theMinMax);
void PackMinMax(char **thePtr, MinMax *theMinMax);
void UnpackMinMax(char **thePtr, MinMax *theMinMax);

void ReadCoordinate(UFFile theFile, CoordPtr theCoord);
void ReadFileCoordinate(UFFile theFile, CoordPtr theCoord);
void WriteCoordinate(UFFile theFile, CoordPtr theCoord);
void WriteFileCoordinate(UFFile theFile, CoordPtr theCoord);
void PackCoordinate(char **thePtr, CoordPtr theCoord);
void UnpackCoordinate(char **thePtr, CoordPtr theCoord);

void ReadDCoordinate(UFFile theFile, DCoordPtr theCoord);
void ReadFileDCoordinate(UFFile theFile, DCoordPtr theCoord);
void WriteDCoordinate(UFFile theFile, DCoordPtr theCoord);
void WriteFileDCoordinate(UFFile theFile, DCoordPtr theCoord);
void PackDCoordinate(char **thePtr, DCoordPtr theCoord);
void UnpackDCoordinate(char **thePtr, DCoordPtr theCoord);

void ReadExtent(UFFile theFile, ExtentPtr theExtent);
void ReadFileExtent(UFFile theFile, ExtentPtr theExtent);
void WriteExtent(UFFile theFile, ExtentPtr theExtent);
void WriteFileExtent(UFFile theFile, ExtentPtr theExtent);
void PackExtent(char **thePtr, ExtentPtr theExtent);
void UnpackExtent(char **thePtr, ExtentPtr theExtent);

void ReadRMatrix3(UFFile theFile, RMatrix3Ptr theMatrix);
void ReadFileRMatrix3(UFFile theFile, RMatrix3Ptr theMatrix);
void WriteRMatrix3(UFFile theFile, RMatrix3Ptr theMatrix);
void WriteFileRMatrix3(UFFile theFile, RMatrix3Ptr theMatrix);
void PackRMatrix3(char **thePtr, RMatrix3Ptr theMatrix);
void UnpackRMatrix3(char **thePtr, RMatrix3Ptr theMatrix);

void ReadRMatrix4(UFFile theFile, RMatrix4Ptr theMatrix);
void ReadFileRMatrix4(UFFile theFile, RMatrix4Ptr theMatrix);
void WriteRMatrix4(UFFile theFile, RMatrix4Ptr theMatrix);
void WriteFileRMatrix4(UFFile theFile, RMatrix4Ptr theMatrix);
void PackRMatrix4(char **thePtr, RMatrix4Ptr theMatrix);
void UnpackRMatrix4(char **thePtr, RMatrix4Ptr theMatrix);

void ReadMatrix3(UFFile theFile, Matrix3Ptr theMatrix);
void ReadFileMatrix3(UFFile theFile, Matrix3Ptr theMatrix);
void WriteMatrix3(UFFile theFile, Matrix3Ptr theMatrix);
void WriteFileMatrix3(UFFile theFile, Matrix3Ptr theMatrix);
void PackMatrix3(char **thePtr, Matrix3Ptr theMatrix);
void UnpackMatrix3(char **thePtr, Matrix3Ptr theMatrix);

void ReadMatrix4(UFFile theFile, Matrix4Ptr theMatrix);
void ReadFileMatrix4(UFFile theFile, Matrix4Ptr theMatrix);
void WriteMatrix4(UFFile theFile, Matrix4Ptr theMatrix);
void WriteFileMatrix4(UFFile theFile, Matrix4Ptr theMatrix);
void PackMatrix4(char **thePtr, Matrix4Ptr theMatrix);
void UnpackMatrix4(char **thePtr, Matrix4Ptr theMatrix);

void ReadExtended12(UFFile theFile, double* theExtended);
void ReadFileExtended12(UFFile theFile, double* theExtended);
void WriteExtended12(UFFile theFile, double* theExtended);
void WriteFileExtended12(UFFile theFile, double* theExtended);
void PackExtended12(char **thePtr, double* theExtended);
void UnpackExtended12(char **thePtr, double* theExtended);

void ReadXMatrix3(UFFile theFile, XMatrix3Ptr theMatrix);
void ReadFileXMatrix3(UFFile theFile, XMatrix3Ptr theMatrix);
void WriteXMatrix3(UFFile theFile, XMatrix3Ptr theMatrix);
void WriteFileXMatrix3(UFFile theFile, XMatrix3Ptr theMatrix);
void PackXMatrix3(char **thePtr, XMatrix3Ptr theMatrix);
void UnpackXMatrix3(char **thePtr, XMatrix3Ptr theMatrix);

void ReadXMatrix4(UFFile theFile, XMatrix4Ptr theMatrix);
void ReadFileXMatrix4(UFFile theFile, XMatrix4Ptr theMatrix);
void WriteXMatrix4(UFFile theFile, XMatrix4Ptr theMatrix);
void WriteFileXMatrix4(UFFile theFile, XMatrix4Ptr theMatrix);
void PackXMatrix4(char **thePtr, XMatrix4Ptr theMatrix);
void UnpackXMatrix4(char **thePtr, XMatrix4Ptr theMatrix);

void ReadXCoordinate(UFFile theFile, XCoordPtr theCoord);
void ReadFileXCoordinate(UFFile theFile, XCoordPtr theCoord);
void WriteXCoordinate(UFFile theFile, XCoordPtr theCoord);
void WriteFileXCoordinate(UFFile theFile, XCoordPtr theCoord);
void PackXCoordinate(char **thePtr, XCoordPtr theCoord);
void UnpackXCoordinate(char **thePtr, XCoordPtr theCoord);

void ReadGINF(UFFile theFile, void* theData, UFSize theSize);
void ReadFileGINF(UFFile theFile, void* theData, UFSize theSize);
void WriteGINF(UFFile theFile, GINFPtr theGINF);
void WriteFileGINF(UFFile theFile, GINFPtr theGINF);
void PackGINF(char **thePtr, GINFPtr theGINF);
void UnpackGINF(char **thePtr, void* theData, ulong theSize);

void ReadGLNK(UFFile theFile, void* theData, ulong theSize);
void ReadFileGLNK(UFFile theFile, void* theData, ulong theSize);
void WriteGLNK(UFFile theFile, GLNKPtr theGLNK);
void WriteFileGLNK(UFFile theFile, GLNKPtr theGLNK);
void PackGLNK(char **thePtr, GLNKPtr theGLNK);
void UnpackGLNK(char **thePtr, void* theData, ulong theSize);

void ReadGSTR(UFFile theFile, void* theData, ulong theSize);
void ReadFileGSTR(UFFile theFile, void* theData, ulong theSize);
void WriteGSTR(UFFile theFile, GSTRPtr theGSTR);
void WriteFileGSTR(UFFile theFile, GSTRPtr theGSTR);
void PackGSTR(char **thePtr, GSTRPtr theGSTR);
void UnpackGSTR(char **thePtr, void* theData, ulong theSize);

void ReadGBLR(UFFile theFile, void* theData, UFSize theSize);
void ReadFileGBLR(UFFile theFile, void* theData, UFSize theSize);
void WriteGBLR(UFFile theFile, GBLRPtr theGBLR);
void WriteFileGBLR(UFFile theFile, GBLRPtr theGBLR);
void PackGBLR(char **thePtr, GBLRPtr theGBLR);
void UnpackGBLR(char **thePtr, void* theData, ulong theSize);

void ReadGOUT(UFFile theFile, void* theData, UFSize theSize);
void ReadFileGOUT(UFFile theFile, void* theData, UFSize theSize);
void WriteGOUT(UFFile theFile, GOUTPtr theGOUT);
void WriteFileGOUT(UFFile theFile, GOUTPtr theGOUT);
void PackGOUT(char **thePtr, GOUTPtr theGOUT);
void UnpackGOUT(char **thePtr, void* theData, ulong theSize);

void ReadGATR(UFFile theFile, void* theData, UFSize theSize);
void ReadFileGATR(UFFile theFile, void* theData, UFSize theSize);
void WriteGATR(UFFile theFile, GATRPtr theGATR, IFFType theType);
void WriteFileGATR(UFFile theFile, GATRPtr theGATR, IFFType theType);
void PackGATR(char **thePtr, GATRPtr theGATR, IFFType theType);
void UnpackGATR(char **thePtr, void* theData, ulong theSize);

void ReadTMAP(UFFile theFile, void* theData, UFSize theSize);
void ReadFileTMAP(UFFile theFile, void* theData, UFSize theSize);
void WriteTMAP(UFFile theFile, TMAPPtr theTMAP, IFFType theType);
void WriteFileTMAP(UFFile theFile, TMAPPtr theTMAP, IFFType theType);
void PackTMAP(char **thePtr, TMAPPtr theTMAP, IFFType theType);
void UnpackTMAP(char **thePtr, void* theData, ulong theSize);

void ReadTMEI(UFFile theFile, void* theData, ulong theSize);
void ReadFileTMEI(UFFile theFile, void* theData, ulong theSize);
void WriteTMEI(UFFile theFile, TMEIPtr theTMEI, IFFType theType);
void WriteFileTMEI(UFFile theFile, TMEIPtr theTMEI, IFFType theType);
void PackTMEI(char **thePtr, TMEIPtr theTMEI, IFFType theType);
void UnpackTMEI(char **thePtr, void* theData, ulong theSize);

void ReadGIMN(UFFile theFile, void* theData, ulong theSize);
void ReadFileGIMN(UFFile theFile, void* theData, ulong theSize);
void UnpackGIMN(char **thePtr, void* theData, ulong theSize);

void ReadGMN2(UFFile theFile, void* theData, UFSize theSize);
void ReadFileGMN2(UFFile theFile, void* theData, UFSize theSize);
void UnpackGMN2(char **thePtr, void* theData, ulong theSize);

void ReadGBMN(UFFile theFile, void* theData, UFSize theSize);
void ReadFileGBMN(UFFile theFile, void* theData, UFSize theSize);
void UnpackGBMN(char **thePtr, void* theData, ulong theSize);

void ReadTMMN(UFFile theFile, void* theData, UFSize theSize);
void ReadFileTMMN(UFFile theFile, void* theData, UFSize theSize);
void UnpackTMMN(char **thePtr, void* theData, ulong theSize);

void ReadSHAD(UFFile theFile, void* theData, UFSize theSize);
void ReadFileSHAD(UFFile theFile, void* theData, UFSize theSize);
void WriteSHAD(UFFile theFile, SHADPtr theSHAD);
void WriteFileSHAD(UFFile theFile, SHADPtr theSHAD);
void PackSHAD(char **thePtr, SHADPtr theSHAD);
void UnpackSHAD(char **thePtr, void* theData, ulong theSize);

void ReadGLEX(UFFile theFile, void* theData, UFSize theSize);
void ReadFileGLEX(UFFile theFile, void* theData, UFSize theSize);
void WriteGLEX(UFFile theFile, GLEXPtr theGLEX);
void WriteFileGLEX(UFFile theFile, GLEXPtr theGLEX);
void PackGLEX(char **thePtr, GLEXPtr theGLEX);
void UnpackGLEX(char **thePtr, void* theData, ulong theSize);

void ReadMORF(UFFile theFile, void* theData, UFSize theSize);
void ReadFileMORF(UFFile theFile, void* theData, UFSize theSize);
void WriteMORF(UFFile theFile, MORFPtr theMORF);
void WriteFileMORF(UFFile theFile, MORFPtr theMORF);
void PackMORF(char **thePtr, MORFPtr theMORF);
void UnpackMORF(char **thePtr, void* theData, ulong theSize);

#ifdef __cplusplus
}
#endif

#endif
