/*
 *		Copyright (c) 1987-1999 Play, Inc.
 *		This code is confidential and is a trade secret of Electric Image, Inc.
 */

#include "FPU.h"
#include "FACTStuff.h"

#if 0

/* The following functions are not currently used but are provided for future use on */
/* systems without a math library */

#define oneovertwopi (0.5 / PI)
#define piover4 (PI / 4.0)
#define fouroverpi (4.0 / PI)

static double atanc[] = {
	0.0,
	0.4636476090008061165,
	0.7853981633974483094,
	0.98279372324732906714,
	1.1071487177940905022,
	1.1902899496825317322,
	1.2490457723982544262,
	1.2924966677897852673,
	1.3258176636680324644
};

/* Sin(x)		Return sine, x in radians */

double Sin(double x)
{
	long sign;
	double y, r, z;
	
	sign = x < 0.0;
	x = (sign ? -x: x);
	
	if (x >	PI2)
		x -= (FASTAINT(x * oneovertwopi) * PI2);
	
	if (x >	PI) {
		x -= PI;
		sign = !sign;
	}
	
	if (x >	PI05)
		x = PI - x;
	
	if (x <	piover4) {
		y = x * fouroverpi;
		z = y * y;
		r = y * (((((((-0.202253129293E-13 * z + 0.69481520350522E-11) * z -
			0.17572474176170806E-8) * z + 0.313361688917325348E-6) * z -
			0.365762041821464001E-4) * z + 0.249039457019271628E-2) * z -
			0.0807455121882807815) * z + 0.785398163397448310);
	} else {
		y = (PI05 - x) * fouroverpi;
		z = y * y;
		r = ((((((-0.38577620372E-12 * z + 0.11500497024263E-9) * z -
			0.2461136382637005E-7) * z + 0.359086044588581953E-5) * z -
			0.325991886926687550E-3) * z + 0.0158543442438154109) * z -
			0.308425137534042452) * z + 1.0;
	}
	return sign ? -r : r;
}  /* Sin */

/* Cos(x)		Return cosine, x in radians */

double Cos(double x)
{
	x = (x < 0.0) ? -x : x;
	if (x > PI2)									/* Do range reduction here to limit */
		x -= (FASTAINT(x * oneovertwopi) * PI2);	/* roundoff on add of PI/2 */
	return fast_sin(x + PI05);
}  /* Cos */

/* Sqrt(x)	  Return square	root.  Initial guess, then Newton-Raphson refinement */

double Sqrt(double x)
{
//	double y0 = FREXP(x);
//	return y0 + 0.5 * y0 * (1.0 - x * y0 * y0);

	double c, cl, y;
	long n;

	if (x > 1.0E+10)
		return sqrt(x);
	else if (x <= 0.0)
		return 0.0;

	y = (0.154116 +	1.893872 * x) /	(1.0 + 1.047988	* x);

	c = (y - x / y) * 0.5;
	cl = 0.0;
	for (n = 25; c != cl &&	n--;) {
		y = y - c;
		cl = c;
		c = (y - x / y) * 0.5;
		if (c == cl) 
			return y;
		else {
			y = y - c;
			cl = c;
			c = (y - x / y) * 0.5;
		}
	}
	return y;
}  /* Sqrt */

/*  ATan(x)	  Return arctangent in radians, range -PI/2 to PI/2  */

double ATan(double x)
{
	long sign;
	long y;
	double a, b, z;
	
	sign = (x < 0.0);
	if (sign)
		x = -x;
		
	if (x >= 4.0) {
		x = 1.0 / x;
		y = 0;
		z = x *	x;
		b = ((((893025.0 * z + 49116375.0) * z + 425675250.0) * z + 1277025750.0) * z + 1550674125.0) * z + 654729075.0;
		a = PI05 - ((((13852575.0 * z + 216602100.0) * z + 891080190.0) * z + 1332431100.0) * z + 654729075.0) * x / b - atanc[y];
	} else {
		if (x < 0.25)
			y = 0;
		else {
			y = x + x;
			z = y * 0.5;
			x = (x - z) / (x * z + 1.0);
		}
		z = x *	x;
		b = ((((893025.0 * z + 49116375.0) * z + 425675250.0) * z + 1277025750.0) * z + 1550674125.0) * z + 654729075.0;
		a = ((((13852575.0 * z + 216602100.0) * z + 891080190.0) * z + 1332431100.0) * z + 654729075.0) * x / b + atanc[y];
	}
	return sign ? -a : a;
}  /* ATan */

/*  ATan2(y,x)	  Return arctangent in radians of y/x, range -PI to PI  */

double ATan2(double y, double x)
{
	double temp;
	
	if (x == 0.0)
		if (y == 0.0)   /*  Special case: atan2(0,0)	= 0  */
			return 0.0;
		else if (y > 0.0)
			return PI05;
		else
			return PINeg05;

	temp = ATAN(y / x);
	if (x <	0.0)
		if (y >= 0.0)
			temp += PI;
		else
			temp -= PI;

	return temp;
}  /* ATan2 */

/*  Pow(x)	  Return x^y  */

double Pow(double x, double y)
{
	return EXP(y * LOG(x));
}  /* Pow */
#endif

#if defined(__MACINTOSH__) && defined(powerc)

/* Tan(x)	  Return tangent, x in radians,	by identity */

double Tan(double x)
{
	return SIN(x) / COS(x);
}  /* Tan */

/* The following functions are only used on PowerPC Macintosh computers which are using the */
/* Motorola floating point library (which does not include these functions) */

/*  ASin(x)	  Return arcsine in radians of x  */

double ASin(double x)
{
	return ATAN2(x, SQRT(1.0 - x * x));
}  /* ASin */

/*  ACos(x)	  Return arccosine in radians of x  */

double ACos(double x)
{
	return PI05 - ATAN2(x, SQRT(1.0 - x * x));
}  /* ACos */

/*  SinH(x)	  Return hyperbolic sine in radians of x  */

double SinH(double x)
{
	return 0.5 * (EXP(x) - EXP(-x));
}  /* SinH */

/*  CosH(x)	  Return hyperbolic cosine in radians of x  */

double CosH(double x)
{
	return 0.5 * (EXP(x) + EXP(-x));
}  /* CosH */

/*  TanH(x)	  Return hyperbolic tangent in radians of x  */

double TanH(double x)
{
	double a = EXP(x);
	double b = EXP(-x);
	return (a - b) / (a + b);
}  /* TanH */

#endif

/* 8, 10, 12 Byte Floating Point Conversions */

void ConvertFloat12To10(void* the12, void* the10)
{
	char *the12Data = (char*)the12;
	char *the10Data = (char*)the10;
	
#ifdef __LITTLE_ENDIAN__
	the10Data[0] = the12Data[0];
	the10Data[1] = the12Data[1];
	the10Data[2] = the12Data[2];
	the10Data[3] = the12Data[3];
	the10Data[4] = the12Data[4];
	the10Data[5] = the12Data[5];
	the10Data[6] = the12Data[6];
	the10Data[7] = the12Data[7];
	the10Data[8] = the12Data[10];
	the10Data[9] = the12Data[11];
#else
	the10Data[0] = the12Data[0];
	the10Data[1] = the12Data[1];
	the10Data[2] = the12Data[4];
	the10Data[3] = the12Data[5];
	the10Data[4] = the12Data[6];
	the10Data[5] = the12Data[7];
	the10Data[6] = the12Data[8];
	the10Data[7] = the12Data[9];
	the10Data[8] = the12Data[10];
	the10Data[9] = the12Data[11];
#endif
}  /* ConvertFloat12To10 */

void ConvertFloat10To12(void* the10, void* the12)
{
	char *the12Data = (char*)the12;
	char *the10Data = (char*)the10;
	
#ifdef __LITTLE_ENDIAN__
	the12Data[0] = the10Data[0];
	the12Data[1] = the10Data[1];
	the12Data[2] = the10Data[2];
	the12Data[3] = the10Data[3];
	the12Data[4] = the10Data[4];
	the12Data[5] = the10Data[5];
	the12Data[6] = the10Data[6];
	the12Data[7] = the10Data[7];
	the12Data[8] = 0;
	the12Data[9] = 0;
	the12Data[10] = the10Data[8];
	the12Data[11] = the10Data[9];
#else
	the12Data[0] = the10Data[0];
	the12Data[1] = the10Data[1];
	the12Data[2] = 0;
	the12Data[3] = 0;
	the12Data[4] = the10Data[2];
	the12Data[5] = the10Data[3];
	the12Data[6] = the10Data[4];
	the12Data[7] = the10Data[5];
	the12Data[8] = the10Data[6];
	the12Data[9] = the10Data[7];
	the12Data[10] = the10Data[8];
	the12Data[11] = the10Data[9];
#endif
}  /* ConvertFloat10To12 */

/* Copyright (C) 1989-1991 Apple Computer, Inc.
 *
 * All rights reserved.
 *
 * Warranty Information
 *  Even though Apple has reviewed this software, Apple makes no warranty
 *  or representation, either express or implied, with respect to this
 *  software, its quality, accuracy, merchantability, or fitness for a
 *  particular purpose.  As a result, this software is provided "as is,"
 *  and you, its user, are assuming the entire risk as to its quality
 *  and accuracy.
 *
 * This code may be used and freely distributed as long as it includes
 * this copyright notice and the above warranty information.
 *
 * Machine-independent I/O routines for IEEE floating-point numbers.
 *
 * NaN's and infinities are converted to HUGE_VAL or HUGE, which
 * happens to be infinity on IEEE machines.  Unfortunately, it is
 * impossible to preserve NaN's in a machine-independent way.
 * Infinities are, however, preserved on IEEE machines.
 *
 * These routines have been tested on the following machines:
 *	Apple Macintosh, MPW 3.1 C compiler
 *	Apple Macintosh, THINK C compiler
 *	Apple Macintosh, Metrowerks C compiler
 *	Silicon Graphics IRIS, MIPS compiler
 *	Cray X/MP and Y/MP
 *	Digital Equipment VAX
 *	Sequent Balance (Multiprocesor 386)
 *	NeXT
 *
 *
 * Implemented by Malcolm Slaney and Ken Turkowski.
 *
 * Malcolm Slaney contributions during 1988-1990 include big- and little-
 * endian file I/O, conversion to and from Motorola's extended 80-bit
 * floating-point format, and conversions to and from IEEE single-
 * precision floating-point format.
 *
 * In 1991, Ken Turkowski implemented the conversions to and from
 * IEEE double-precision format, added more precision to the extended
 * conversions, and accommodated conversions involving +/- infinity,
 * NaN's, and denormalized numbers.
 */

/****************************************************************
 * double precision IEEE floating-point conversion routines.
 * Extended is an 80-bit number as defined by Motorola,
 * with a sign bit, 15 bits of exponent (offset 16383?),
 * and a 64-bit mantissa, with no hidden bit.
 ****************************************************************/

/* Note: To Do -- fix the code so that 12 byte native doubles are supported for 68881 compilation */

void ConvertFloat10To8(void* the10, void* the8)
{
	char* bytes = (char*)the10;
	double f;

#ifdef __LITTLE_ENDIAN__
	long sign = bytes[9] & 0x80;
	long expon = ((bytes[9] & 0x7F) << 8) | (bytes[8] & 0xFF);
	ulong hiMant = ((ulong)(bytes[7] & 0xFF) << 24) | ((ulong)(bytes[6] & 0xFF) << 16) | ((ulong)(bytes[5] & 0xFF) << 8) | ((ulong)(bytes[4] & 0xFF));
	ulong loMant = ((ulong)(bytes[3] & 0xFF) << 24) | ((ulong)(bytes[2] & 0xFF) << 16) | ((ulong)(bytes[1] & 0xFF) << 8) | ((ulong)(bytes[0] & 0xFF));
#else
	long sign = bytes[0] & 0x80;
	long expon = ((bytes[0] & 0x7F) << 8) | (bytes[1] & 0xFF);
	ulong hiMant = ((ulong)(bytes[2] & 0xFF) << 24) | ((ulong)(bytes[3] & 0xFF) << 16) | ((ulong)(bytes[4] & 0xFF) << 8) | ((ulong)(bytes[5] & 0xFF));
	ulong loMant = ((ulong)(bytes[6] & 0xFF) << 24) | ((ulong)(bytes[7] & 0xFF) << 16) | ((ulong)(bytes[8] & 0xFF) << 8) | ((ulong)(bytes[9] & 0xFF));
#endif
	if (expon == 0 && hiMant == 0 && loMant == 0)
		f = 0.0;
	else if (expon == 0x7FFF)	/* Infinity or NaN */
		f = HUGE_VAL;
	else {
		expon -= 16383;
		f = ldexp((double)hiMant, expon -= 31);
		f += ldexp((double)loMant, expon -= 32);
	}

    if (sign)
		*(double*)the8 = -f;
	else
		*(double*)the8 = f;
}  /* ConvertFloat10To8 */

void ConvertFloat8To10(void* the8, void* the10)
{
	double num = *(double*)the8;
	char* bytes = (char*)the10;
	long sign;
	int expon;
	double fMant, fsMant;
	ulong hiMant, loMant;

	if (num < 0) {
		sign = 0x8000;
		num *= -1;
	} else 
		sign = 0;

	if (num == 0) {
		expon = 0;
		hiMant = 0;
		loMant = 0;
	} else {
		fMant = frexp(num, &expon);
		if ((expon > 16384) || (fMant >= 1)) {	/* Infinity or NaN */
			expon = sign | 0x7FFF; 				/* infinity */
			hiMant = 0;
			loMant = 0;
		} else {								/* Finite */
			expon += 16382;
			if (expon < 0) {					/* denormalized */
				fMant = ldexp(fMant, expon);
				expon = 0;
			}
			expon |= sign;
			
			fMant = ldexp(fMant, 32);
			fsMant = floor(fMant);
			hiMant = fsMant;

			fMant = ldexp(fMant - fsMant, 32);
			fsMant = floor(fMant);
			loMant = fsMant;
		}
	}
	
#ifdef __LITTLE_ENDIAN__
	bytes[9] = expon >> 8;
	bytes[8] = expon;
	bytes[7] = hiMant >> 24;
	bytes[6] = hiMant >> 16;
	bytes[5] = hiMant >> 8;
	bytes[4] = hiMant;
	bytes[3] = loMant >> 24;
	bytes[2] = loMant >> 16;
	bytes[1] = loMant >> 8;
	bytes[0] = loMant;
#else
	bytes[0] = expon >> 8;
	bytes[1] = expon;
	bytes[2] = hiMant >> 24;
	bytes[3] = hiMant >> 16;
	bytes[4] = hiMant >> 8;
	bytes[5] = hiMant;
	bytes[6] = loMant >> 24;
	bytes[7] = loMant >> 16;
	bytes[8] = loMant >> 8;
	bytes[9] = loMant;
#endif
}  /* ConvertFloat8To10 */

void ConvertFloat12To8(void* the12, void* the8)
{
	char the10[10];
	
	ConvertFloat12To10(the12, the10);
	ConvertFloat10To8(the10, the8);
}  /* ConvertFloat12To8 */

void ConvertFloat8To12(void* the8, void* the12)
{
	char the10[10];
	
	ConvertFloat8To10(the8, the10);
	ConvertFloat10To12(the10, the12);
}  /* ConvertFloat8To12 */

/* Convert a binary hex constant to a float */

float HexFloat(long h)
{
	return *(float*)&h;
}  /* HexFloat */

/* Convert a float to its binary representation */

long FloatHex(float f)
{
	return *(long*)&f;
}  /* FloatHex */

boolean ValidateReal(float* theValue)
{
	RealRecordPtr r;
	
	r = (RealRecordPtr)theValue;
	if (r->l == (long)0x80000000) {
		r->l = 0;
		return 1;
	} else
		return (0x7F80 & r->i.i1) != 0x7F80;
}  /* ValidateReal */

boolean ValidateDouble(double* theValue)
{
	if (sizeof(double) == 8) {
		DoubleRecordPtr d;
		
		d = (DoubleRecordPtr)theValue;
		if ((d->l.l1 == (long)0x80000000) && (d->l.l2 == 0)) {
			d->l.l1 = 0;
			return 1;
		} else
			return (0x7FF0 & d->i.i1) != 0x7FF0;
	} else
		return 1;
}  /* ValidateDouble */

float FixReal(float theValue)
{
	RealRecordPtr r;
	
	r = (RealRecordPtr)&theValue;
	if ((r->l == (long)0x80000000) || ((0x7F80 & r->i.i1) == 0x7F80))
		r->l = 0;
	return r->r;
}  /* FixReal */

double FixDouble(double theValue)
{
	if (sizeof(double) == 8) {
		DoubleRecordPtr d;
		
		d = (DoubleRecordPtr)&theValue;
		if (((d->l.l1 == (long)0x80000000) && (d->l.l2 == 0)) || ((0x7FF0 & d->i.i1) == 0x7FF0))
			return 0.0;
		else
			return theValue;
	} else
		return theValue;
}  /* FixDouble */
